@@ -56,6 +56,9 @@
 
 /*
  * $Log: MemParse.hpp,v $
+ * Revision 1.7  2003/05/30 09:36:35  gareth
+ * Use new macros for iostream.h and std:: issues.
+ *
  * Revision 1.6  2003/02/05 18:53:23  tng
  * [Bug 11915] Utility for freeing memory.
  *
@@ -87,10 +90,16 @@
 // ---------------------------------------------------------------------------
 #include <string.h>
 #include <stdlib.h>
-#include <iostream.h>
 #include <xercesc/util/PlatformUtils.hpp>
 #include "MemParseHandlers.hpp"
 
+#if defined(XERCES_NEW_IOSTREAMS)
+#include <iostream>
+#else
+#include <iostream.h>
+#endif
+
+
 
 
 // ---------------------------------------------------------------------------
@@ -134,7 +143,7 @@ private :
     char*   fLocalForm;
 };
 
-inline ostream& operator<<(ostream& target, const StrX& toDump)
+inline XERCES_STD_QUALIFIER ostream& operator<<(XERCES_STD_QUALIFIER ostream& target, const StrX& toDump)
 {
     target << toDump.localForm();
     return target;
