@@ -56,6 +56,9 @@
 
 /*
  * $Log: SAX2Print.hpp,v $
+ * Revision 1.3  2003/05/30 09:36:36  gareth
+ * Use new macros for iostream.h and std:: issues.
+ *
  * Revision 1.2  2003/02/05 18:53:23  tng
  * [Bug 11915] Utility for freeing memory.
  *
@@ -69,7 +72,11 @@
 //  Includes for all the program files to see
 // ---------------------------------------------------------------------------
 #include <string.h>
+#if defined(XERCES_NEW_IOSTREAMS)
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include <stdlib.h>
 #include "SAX2PrintHandlers.hpp"
 
@@ -113,7 +120,7 @@ private :
     char*   fLocalForm;
 };
 
-inline ostream& operator<<(ostream& target, const StrX& toDump)
+inline XERCES_STD_QUALIFIER ostream& operator<<(XERCES_STD_QUALIFIER ostream& target, const StrX& toDump)
 {
     target << toDump.localForm();
     return target;
