@@ -1,4 +1,59 @@
 #!/usr/local/bin/perl5
+#
+# The Apache Software License, Version 1.1
+#
+# Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+# reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in
+#    the documentation and/or other materials provided with the
+#    distribution.
+#
+# 3. The end-user documentation included with the redistribution,
+#    if any, must include the following acknowledgment:
+#       "This product includes software developed by the
+#        Apache Software Foundation (http://www.apache.org/)."
+#    Alternately, this acknowledgment may appear in the software itself,
+#    if and wherever such third-party acknowledgments normally appear.
+#
+# 4. The names "Xerces" and "Apache Software Foundation" must
+#    not be used to endorse or promote products derived from this
+#    software without prior written permission. For written
+#    permission, please contact apache\@apache.org.
+#
+# 5. Products derived from this software may not be called "Apache",
+#    nor may "Apache" appear in their name, without prior written
+#    permission of the Apache Software Foundation.
+#
+# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
+# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
+# ====================================================================
+#
+# This software consists of voluntary contributions made by many
+# individuals on behalf of the Apache Software Foundation, and was
+# originally based on software copyright (c) 1999, International
+# Business Machines, Inc., http://www.ibm.com .  For more information
+# on the Apache Software Foundation, please see
+# <http://www.apache.org/>.
+#
 
 push(@INC, "/home/xml4c/bin", "/home/xml4c/bin/perl/perl-RUN/opt/perl5/lib", "/Development2/cupert/bin/perl/perl-RUN/opt/perl5/lib", "/Development/cupert/usr/local/perl/perl-RUN/opt/perl5/lib");
 
@@ -29,11 +84,11 @@ if (!length($XERCESCROOT) || !length($targetdir) || (length($opt_h) > 0) ) {
     print ("    -j suppress building of ICU (speeds up builds when debugging)\n");
     print ("    -h to get help on these commands\n\n");
     print ("Example: Under unix's\n");
-    print ("    perl packageBinaries.pl -s \$HOME/xerces-c-src2_3_0");
-    print (" -o \$HOME/xerces-c2_3_0-linux -c gcc -x g++ -m inmem -n fileonly -t native\n\n");
+    print ("    perl packageBinaries.pl -s \$HOME/xerces-c-src2_5_0");
+    print (" -o \$HOME/xerces-c2_5_0-linux -c gcc -x g++ -m inmem -n fileonly -t native\n\n");
     print ("Example: Under Windows\n");
-    print ("    perl packageBinaries.pl -s \\xerces-c-src2_3_0");
-    print (" -o\\xerces-c2_3_0-win32 [-n fileonly] [-t icu]\n\n");
+    print ("    perl packageBinaries.pl -s \\xerces-c-src2_5_0");
+    print (" -o\\xerces-c2_5_0-win32 [-n fileonly] [-t icu]\n\n");
     print ("Note:\n");
     print ("    Under Windows, by default the XercesLib project files is\n");
     print ("    configured to use Win32 resource file based message loader,\n");
@@ -47,11 +102,13 @@ if (!length($XERCESCROOT) || !length($targetdir) || (length($opt_h) > 0) ) {
 # As of Version 3, ICU is not a required component of XERCES-C
 $ICUROOT = $ENV{'ICUROOT'};
 $ICU_DATA = $ENV{'ICU_DATA'};
-if (!length($ICUROOT)) {
-    print "You have not defined your ICU install directory.\n";
+if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && (length($ICUROOT) == 0)) {
+    print "You have specified an ICU build but you have not defined your ICU install directory.\n";
     print "To build with ICU, you must set an environment variable called ICUROOT\n";
-    print "Proceeding to build XERCES-C without ICU...\n";
+    print "Cannot proceed any further.\n";
+    exit(-1);
 }
+$ICUIsPresent = (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0);
 
 # Check if the source directory exists or not
 if (!(-e $XERCESCROOT)) {
@@ -151,6 +208,9 @@ if ($platform eq "win64bit" )
     psystem ("mkdir $targetdir\\samples\\EnumVal");
     psystem ("mkdir $targetdir\\samples\\SEnumVal");
     psystem ("mkdir $targetdir\\samples\\CreateDOMDocument");
+    psystem ("mkdir $targetdir\\samples\\PSVIWriter");
+    psystem ("mkdir $targetdir\\samples\\SCMPrint");
+    psystem ("mkdir $targetdir\\scripts");
     psystem ("mkdir $targetdir\\doc");
     psystem ("mkdir $targetdir\\doc\\html");
     psystem ("mkdir $targetdir\\doc\\html\\apiDocs");
@@ -168,7 +228,7 @@ if ($platform eq "win64bit" )
     #
     #REVISIT: icu
     #
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
 
         print ("Building ICU from $ICUROOT ...\n");
 
@@ -241,6 +301,7 @@ if ($platform eq "win64bit" )
      qw'sax
 		sax2
         framework
+        framework\\psvi        
         dom
         dom\\deprecated
         internal
@@ -304,7 +365,7 @@ if ($platform eq "win64bit" )
     psystem("del  $targetdir\\include\\xercesc\\dom\\deprecated\\*Impl.hpp");
     psystem("del  $targetdir\\include\\xercesc\\dom\\deprecated\\DS*.hpp");
 
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem("xcopy /s /y $ICUROOT\\include\\* $targetdir\\include");
     }
 
@@ -315,16 +376,16 @@ if ($platform eq "win64bit" )
     psystem("copy /y $BUILDDIR\\*.dll $targetdir\\bin");
     psystem("copy /y $BUILDDIR\\*.exe $targetdir\\bin");
 
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
 
         # Copy the ICU dlls and libs
-        psystem("copy /y $ICUROOT\\bin\\icuuc24.dll $targetdir\\bin");
-        psystem("copy /y $ICUROOT\\bin\\icuuc24d.dll $targetdir\\bin");
+        psystem("copy /y $ICUROOT\\bin\\icuuc26.dll $targetdir\\bin");
+        psystem("copy /y $ICUROOT\\bin\\icuuc26d.dll $targetdir\\bin");
 
-        # it seems icudt24*.DLL is generated (upper case dll)
+        # it seems icudt26*.DLL is generated (upper case dll)
         # but just case, try lower case .dll as well
-        psystem("copy /y $ICUROOT\\bin\\icudt24*.DLL $targetdir\\bin");
-        psystem("copy /y $ICUROOT\\bin\\icudt24*.dll $targetdir\\bin");
+        psystem("copy /y $ICUROOT\\bin\\icudt26*.DLL $targetdir\\bin");
+        psystem("copy /y $ICUROOT\\bin\\icudt26*.dll $targetdir\\bin");
 
         psystem("copy /y $ICUROOT\\lib\\icudata.lib $targetdir\\lib");
 
@@ -384,9 +445,17 @@ if ($platform eq "win64bit" )
     psystem("del /f $targetdir\\samples\\SEnumVal\\Makefile.in");
     psystem("copy /y $XERCESCROOT\\samples\\CreateDOMDocument\\* $targetdir\\samples\\CreateDOMDocument");
     psystem("del /f $targetdir\\samples\\CreateDOMDocument\\Makefile.in");
-
+    psystem("copy /y $XERCESCROOT\\samples\\PSVIWriter\\* $targetdir\\samples\\PSVIWriter");
+    psystem("del /f $targetdir\\samples\\PSVIWriter\\Makefile.in");
+    psystem("copy /y $XERCESCROOT\\samples\\SCMPrint\\* $targetdir\\samples\\SCMPrint");
+    psystem("del /f $targetdir\\samples\\SCMPrint\\Makefile.in");
+        
     psystem("copy /y $XERCESCROOT\\samples\\data\\* $targetdir\\samples\\data");
 
+    # Populate the scripts directory
+    print ("\n \nCopying script files ...\n");
+    psystem("copy $XERCESCROOT\\scripts\\sanityTest* $targetdir\\scripts");
+
     # Populate the docs directory
     print ("\n \nCopying documentation ...\n");
     psystem("xcopy /s /y $XERCESCROOT\\doc\\* $targetdir\\doc");
@@ -394,7 +463,7 @@ if ($platform eq "win64bit" )
     psystem("copy $XERCESCROOT\\credits.txt $targetdir");
     psystem("copy $XERCESCROOT\\LICENSE.txt $targetdir");
 
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem("copy $XERCESCROOT\\license.html $targetdir");
         psystem("copy $XERCESCROOT\\XLicense.html $targetdir");
     }
@@ -502,6 +571,9 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     psystem ("mkdir $targetdir/samples/EnumVal");
     psystem ("mkdir $targetdir/samples/SEnumVal");
     psystem ("mkdir $targetdir/samples/CreateDOMDocument");
+    psystem ("mkdir $targetdir/samples/PSVIWriter");    
+    psystem ("mkdir $targetdir/samples/SCMPrint");        
+    psystem ("mkdir $targetdir/scripts");
     psystem ("mkdir $targetdir/doc");
     psystem ("mkdir $targetdir/doc/html");
     psystem ("mkdir $targetdir/doc/html/apiDocs");
@@ -520,7 +592,7 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     #
     #	ICU Build happens here, if one is required.
     #
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         print ("Building ICU from $ICUROOT ...\n");
 
         #Clean up all the dependency files, causes problems for nmake
@@ -551,15 +623,13 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
                 }	
                 psystem("type buildlog.txt");
             } elsif ($DevStudioVer eq "7.0") {
-                pchdir ("$ICUROOT/as_is/win32");
-                psystem("unzip msvc7.zip");
-                pchdir ("$ICUROOT/as_is/win32/vc7");
+                pchdir ("$ICUROOT/source/allinone");
 
-                psystem("devenv /rebuild Release /out buildlog.txt /project all allinone.sln");
-                psystem("type buildlog.txt");
+                psystem("devenv /rebuild Release /out buildlog_release.txt /project all allinone.sln");
+                psystem("type buildlog_release.txt");
 
-                psystem("devenv /rebuild debug /out buildlog.txt /project all allinone.sln");
-                psystem("type buildlog.txt");
+                psystem("devenv /rebuild debug /out buildlog_debug.txt /project all allinone.sln");
+                psystem("type buildlog_debug.txt");                            	
             }
         }
 
@@ -647,6 +717,7 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
      qw'sax
 		sax2
         framework
+        framework/psvi        
         dom
         dom/deprecated
         internal
@@ -710,7 +781,7 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     psystem ("rm  $targetdir/include/xercesc/dom/deprecated/*Impl.hpp");
     psystem ("rm  $targetdir/include/xercesc/dom/deprecated/DS*.hpp");
 
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem("cp -Rfv $ICUROOT/include/* $targetdir/include");
     }
 
@@ -721,15 +792,15 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     psystem("cp -fv $BUILDDIR/*.dll $targetdir/bin");
     psystem("cp -fv $BUILDDIR/*.exe $targetdir/bin");
 
-    if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && length($ICUROOT) > 0) {    	
+    if ($ICUIsPresent) {    	
         # Copy the ICU dlls and libs
-        psystem("cp -fv $ICUROOT/bin/icuuc24.dll $targetdir/bin");
-        psystem("cp -fv $ICUROOT/bin/icuuc24d.dll $targetdir/bin");
+        psystem("cp -fv $ICUROOT/bin/icuuc26.dll $targetdir/bin");
+        psystem("cp -fv $ICUROOT/bin/icuuc26d.dll $targetdir/bin");
 
-        # it seems icudt24*.DLL is generated (upper case dll)
+        # it seems icudt26*.DLL is generated (upper case dll)
         # but just case, try lower case .dll as well
-        psystem("cp -fv $ICUROOT/bin/icudt24*.DLL $targetdir/bin");
-        psystem("cp -fv $ICUROOT/bin/icudt24*.dll $targetdir/bin");
+        psystem("cp -fv $ICUROOT/bin/icudt26*.DLL $targetdir/bin");
+        psystem("cp -fv $ICUROOT/bin/icudt26*.dll $targetdir/bin");
 
         psystem("cp -fv $ICUROOT/lib/icudata.lib $targetdir/lib");
 
@@ -788,9 +859,17 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     psystem("rm -f $targetdir/samples/SEnumVal/Makefile");
     psystem("cp -Rfv $XERCESCROOT/samples/CreateDOMDocument/* $targetdir/samples/CreateDOMDocument");
     psystem("rm -f $targetdir/samples/CreateDOMDocument/Makefile");
-
+    psystem("cp -Rfv $XERCESCROOT/samples/PSVIWriter/* $targetdir/samples/PSVIWriter");
+    psystem("rm -f $targetdir/samples/PSVIWriter/Makefile");
+    psystem("cp -Rfv $XERCESCROOT/samples/SCMPrint/* $targetdir/samples/SCMPrint");
+    psystem("rm -f $targetdir/samples/SCMPrint/Makefile");
+        
     psystem("cp -Rfv $XERCESCROOT/samples/data/* $targetdir/samples/data");
 
+    # Populate the scripts directory
+    print ("\n\nCopying script files ...\n");
+    psystem("cp $XERCESCROOT/scripts/sanityTest* $targetdir/scripts");
+
     # Populate the docs directory
     print ("\n\nCopying documentation ...\n");
     psystem("cp -Rfv $XERCESCROOT/doc/* $targetdir/doc");
@@ -798,7 +877,7 @@ if ($platform =~ m/Windows/  || ($platform =~ m/CYGWIN/ && !($opt_c =~ m/gcc/)))
     psystem("cp $XERCESCROOT/credits.txt $targetdir");
     psystem("cp $XERCESCROOT/LICENSE.txt $targetdir");
 
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem("cp $XERCESCROOT/license.html $targetdir");
         psystem("cp $XERCESCROOT/XLicense.html $targetdir");
     }
@@ -938,7 +1017,7 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
         # Generally speaking, ICU must be built, before XML4C can be built, for ptx.
         # If this case causes problems, we can revisit it in the future. Right now,
         # we fail only if ICUROOT is defined but mh-ptx is not present.
-        if (length($ICUROOT)) {
+        if ($ICUIsPresent) {
             if (!(-e "$ICUROOT/source/config/mh-ptx")) {
                 print ("Error: Could not locate PTX-specific ICU files.\n");
                 print ("    The PTX-specific patches must be applied to both XML4C and ICU before a build can succeed.\n");
@@ -994,12 +1073,13 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     }
     psystem ("mkdir $targetdir/include");
     psystem ("mkdir $targetdir/include/xercesc");
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem ("mkdir $targetdir/include/unicode");
     }
     psystem ("mkdir $targetdir/include/xercesc/sax");
     psystem ("mkdir $targetdir/include/xercesc/sax2");
     psystem ("mkdir $targetdir/include/xercesc/framework");
+    psystem ("mkdir $targetdir/include/xercesc/framework/psvi");    
     psystem ("mkdir $targetdir/include/xercesc/internal");
     psystem ("mkdir $targetdir/include/xercesc/parsers");
     psystem ("mkdir $targetdir/include/xercesc/util");
@@ -1049,6 +1129,9 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     psystem ("mkdir $targetdir/samples/EnumVal");
     psystem ("mkdir $targetdir/samples/SEnumVal");
     psystem ("mkdir $targetdir/samples/CreateDOMDocument");
+    psystem ("mkdir $targetdir/samples/PSVIWriter");    
+    psystem ("mkdir $targetdir/samples/SCMPrint");        
+    psystem ("mkdir $targetdir/scripts");
     psystem ("mkdir $targetdir/doc");
     psystem ("mkdir $targetdir/doc/html");
     psystem ("mkdir $targetdir/doc/html/apiDocs");
@@ -1057,10 +1140,6 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     if (($opt_t =~ m/icu/i || $opt_m =~ m/icu/i) && !(length($opt_j) > 0))
     {
         print("\n\nBuild ICU with \'$opt_b\' bit ...\n");
-        if(length($ICUROOT) == 0) {
-           print("Error, ICUROOT not set, can not build ICU\n");
-           exit(-1);
-           }
 
         # First make the ICU files executable
         pchdir ("$ICUROOT/source");
@@ -1091,6 +1170,13 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
         # if ( ($platform =~ m/hp-/i) && ($opt_x eq 'CC') ) {
         #   system ("$MAKE");
         # }
+        
+        # on AIX, create '.a' version for building libxerces-c
+        if ( $platform eq "aix") {
+            psystem("ln -s $ICUROOT/lib/libicudata26.1.so $ICUROOT/lib/libicudata26.a");
+            psystem("ln -s $ICUROOT/lib/libicuuc26.1.so   $ICUROOT/lib/libicuuc26.a");
+        }
+        
     }
 
     # For ptx, ICUROOT must now be set to XMLINSTALL for further work.
@@ -1113,53 +1199,59 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
 
     psystem ("$MAKE clean");     # May want to comment this line out to speed up
     psystem ("$MAKE");
-
+                
     # Move ICU libs into lib dir, so samples will link.  This matches the structure of
     #   the eventual binary packaging, even though we are doing it in the build directory.
     #
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         pchdir ("$XERCESCROOT/lib");
 
         #
         # copy icudata dll
         # For ICU 2.4:
-        # on AIX, it is called libicudata24.0.so
-        # on Solaris/Linux, it is called libicudata.so.24.0
-        # on HP, it is called libicudata.s1.24.0
+        # on AIX, it is called libicudata26.1.so
+        # on Solaris/Linux, it is called libicudata.so.26.1
+        # on HP, it is called libicudata.sl.26.1
         #
         psystem("rm -f libicudata*");
-        psystem("cp -f $ICUROOT/lib/libicudata24.0.so .");
-        psystem("cp -f $ICUROOT/lib/libicudata.so.24.0 .");
-        psystem("cp -f $ICUROOT/lib/libicudata.sl.24.0 .");
+        psystem("cp -f $ICUROOT/lib/libicudata26.1.so .");
+        psystem("cp -f $ICUROOT/lib/libicudata.so.26.1 .");
+        psystem("cp -f $ICUROOT/lib/libicudata.sl.26.1 .");
+        
+        psystem("find . -name 'libicudata26.1.so' -exec ln -s {} libicudata.so \\;");
+        psystem("find . -name 'libicudata26.1.so' -exec ln -s {} libicudata26.so \\;");
 
-        psystem("find . -name 'libicudata24.0.so' -exec ln -s {} libicudata.so \\;");
-        psystem("find . -name 'libicudata24.0.so' -exec ln -s {} libicudata24.so \\;");
+        psystem("find . -name 'libicudata.so.26.1' -exec ln -s {} libicudata.so \\;");
+        psystem("find . -name 'libicudata.so.26.1' -exec ln -s {} libicudata.so.26 \\;");
 
-        psystem("find . -name 'libicudata.so.24.0' -exec ln -s {} libicudata.so \\;");
-        psystem("find . -name 'libicudata.so.24.0' -exec ln -s {} libicudata.so.24 \\;");
-
-        psystem("find . -name 'libicudata.sl.24.0' -exec ln -s {} libicudata.sl \\;");
-        psystem("find . -name 'libicudata.sl.24.0' -exec ln -s {} libicudata.sl.24 \\;");
+        psystem("find . -name 'libicudata.sl.26.1' -exec ln -s {} libicudata.sl \\;");
+        psystem("find . -name 'libicudata.sl.26.1' -exec ln -s {} libicudata.sl.26 \\;");
 
         #
         # copy icuuc dll
-        # on AIX, it is called libicuuc24.0.so
-        # on Solaris/Linux, it is called libicuuc.so.24.0
-        # on HP, it is called libicuuc.sl.24.0
+        # on AIX, it is called libicuuc26.1.so
+        # on Solaris/Linux, it is called libicuuc.so.26.1
+        # on HP, it is called libicuuc.sl.26.1
         #
         psystem("rm -f libicuuc*");
-        psystem("cp -f $ICUROOT/lib/libicuuc24.0.so .");
-        psystem("cp -f $ICUROOT/lib/libicuuc.so.24.0  .");
-        psystem("cp -f $ICUROOT/lib/libicuuc.sl.24.0  .");
+        psystem("cp -f $ICUROOT/lib/libicuuc26.1.so .");
+        psystem("cp -f $ICUROOT/lib/libicuuc.so.26.1  .");
+        psystem("cp -f $ICUROOT/lib/libicuuc.sl.26.1  .");
+        
+        psystem("find . -name 'libicuuc26.1.so' -exec ln -s {} libicuuc.so \\;");
+        psystem("find . -name 'libicuuc26.1.so' -exec ln -s {} libicuuc26.so \\;");
+        
+        psystem("find . -name 'libicuuc.so.26.1' -exec ln -s {} libicuuc.so \\;");
+        psystem("find . -name 'libicuuc.so.26.1' -exec ln -s {} libicuuc.so.26 \\;");
 
-        psystem("find . -name 'libicuuc24.0.so' -exec ln -s {} libicuuc.so \\;");
-        psystem("find . -name 'libicuuc24.0.so' -exec ln -s {} libicuuc24.so \\;");
-
-        psystem("find . -name 'libicuuc.so.24.0' -exec ln -s {} libicuuc.so \\;");
-        psystem("find . -name 'libicuuc.so.24.0' -exec ln -s {} libicuuc.so.24 \\;");
-
-        psystem("find . -name 'libicuuc.sl.24.0' -exec ln -s {} libicuuc.sl \\;");
-        psystem("find . -name 'libicuuc.sl.24.0' -exec ln -s {} libicuuc.sl.24 \\;");
+        psystem("find . -name 'libicuuc.sl.26.1' -exec ln -s {} libicuuc.sl \\;");
+        psystem("find . -name 'libicuuc.sl.26.1' -exec ln -s {} libicuuc.sl.26 \\;");
+        
+        # on AIX, copy '.a' version
+        if ( $platform eq "aix") {
+            psystem("ln -s libicudata26.so libicudata26.a");
+            psystem("ln -s libicuuc26.so   libicuuc26.a");
+        }        
     }
 
     # Now build the samples
@@ -1185,6 +1277,8 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     psystem("cp -Rf $XERCESCROOT/src/xercesc/sax/*.hpp $targetdir/include/xercesc/sax");
 	psystem("cp -Rf $XERCESCROOT/src/xercesc/sax2/*.hpp $targetdir/include/xercesc/sax2");
     psystem("cp -Rf $XERCESCROOT/src/xercesc/framework/*.hpp $targetdir/include/xercesc/framework");
+    psystem("cp -Rf $XERCESCROOT/src/xercesc/framework/psvi/*.hpp $targetdir/include/xercesc/framework/psvi");
+    psystem("cp -Rf $XERCESCROOT/src/xercesc/framework/psvi/*.c $targetdir/include/xercesc/framework/psvi");
     psystem("cp -Rf $XERCESCROOT/src/xercesc/dom/*.hpp $targetdir/include/xercesc/dom");
     psystem("cp -Rf $XERCESCROOT/src/xercesc/dom/deprecated/*.hpp $targetdir/include/xercesc/dom/deprecated");
 
@@ -1228,7 +1322,7 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     psystem("cp -Rf $XERCESCROOT/src/xercesc/validators/schema/*.hpp $targetdir/include/xercesc/validators/schema");
     psystem("cp -Rf $XERCESCROOT/src/xercesc/validators/schema/identity/*.hpp $targetdir/include/xercesc/validators/schema/identity");
 
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         print "\nICU files are being copied from \'$ICUROOT\'";
         psystem("cp -Rf $ICUROOT/include/* $targetdir/include");
     }
@@ -1243,29 +1337,35 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     pchdir ("$targetdir/lib");
     psystem("rm -f libxerces-c* ");
 
-    if ((-e "$XERCESCROOT/lib/libxerces-c.so.23.0" )) {
-        psystem("cp -f $XERCESCROOT/lib/libxerces-c.so.23.0 .");
-        psystem("ln -s libxerces-c.so.23.0 libxerces-c.so.23 ");
-        psystem("ln -s libxerces-c.so.23   libxerces-c.so    ");
+    if ((-e "$XERCESCROOT/lib/libxerces-c.so.25.0" )) {
+        psystem("cp -f $XERCESCROOT/lib/libxerces-c.so.25.0 .");
+        psystem("ln -s libxerces-c.so.25.0 libxerces-c.so.25 ");
+        psystem("ln -s libxerces-c.so.25   libxerces-c.so    ");
     }
 
-    if ((-e "$XERCESCROOT/lib/libxerces-c.sl.23.0" )) {
-        psystem("cp -f $XERCESCROOT/lib/libxerces-c.sl.23.0 .");
-        psystem("ln -s libxerces-c.sl.23.0 libxerces-c.sl.23 ");
-        psystem("ln -s libxerces-c.sl.23   libxerces-c.sl    ");
+    if ((-e "$XERCESCROOT/lib/libxerces-c.sl.25.0" )) {
+        psystem("cp -f $XERCESCROOT/lib/libxerces-c.sl.25.0 .");
+        psystem("ln -s libxerces-c.sl.25.0 libxerces-c.sl.25 ");
+        psystem("ln -s libxerces-c.sl.25   libxerces-c.sl    ");
     }
 
-    if ((-e "$XERCESCROOT/lib/libxerces-c23.0.so" )) {
-        psystem("cp -f $XERCESCROOT/lib/libxerces-c23.0.so .");
-        psystem("ln -s libxerces-c23.0.so libxerces-c23.so  ");
-        psystem("ln -s libxerces-c23.so   libxerces-c.so    ");
+    if ((-e "$XERCESCROOT/lib/libxerces-c25.0.so" )) {
+        psystem("cp -f $XERCESCROOT/lib/libxerces-c25.0.so .");
+        psystem("ln -s libxerces-c25.0.so libxerces-c25.so  ");
+        psystem("ln -s libxerces-c25.so   libxerces-c.so    ");
     }
 
+    if ((-e "$XERCESCROOT/lib/libxerces-c25.0.a" )) {
+        psystem("cp -f $XERCESCROOT/lib/libxerces-c25.0.a . ");
+        psystem("ln -s libxerces-c25.0.a  libxerces-c25.a ");
+        psystem("ln -s libxerces-c25.a    libxerces-c.a ");         
+    }
+        
 	# Mac OS X
-    if ((-e "$XERCESCROOT/lib/libxerces-c.23.0.dylib" )) {
-        psystem("cp -f $XERCESCROOT/lib/libxerces-c.23.0.dylib .");
-        psystem("ln -s libxerces-c.23.0.dylib libxerces-c.23.dylib ");
-        psystem("ln -s libxerces-c.23.dylib   libxerces-c.dylib    ");
+    if ((-e "$XERCESCROOT/lib/libxerces-c.25.0.dylib" )) {
+        psystem("cp -f $XERCESCROOT/lib/libxerces-c.25.0.dylib .");
+        psystem("ln -s libxerces-c.25.0.dylib libxerces-c.25.dylib ");
+        psystem("ln -s libxerces-c.25.dylib   libxerces-c.dylib    ");
     }
 
     # Populate the Message Catalog Files
@@ -1279,65 +1379,79 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     #
     # Create symbolic link for those ICU libraries
     #
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         pchdir ("$targetdir/lib");
 
         #
         # copy icudata dll
-        # on AIX, it is called libicudata24.0.so
-        # on Solaris/Linux, it is called libicudata.so.24.0
-        # on HP, it is called libicudata.s1.24.0
+        # on AIX, it is called libicudata26.1.so
+        # on Solaris/Linux, it is called libicudata.so.26.1
+        # on HP, it is called libicudata.sl.26.1
         #
         psystem("rm -f libicudata*");
-        psystem("cp -f $XERCESCROOT/lib/libicudata24.0.so .");
-        psystem("cp -f $XERCESCROOT/lib/libicudata.so.24.0 .");
-        psystem("cp -f $XERCESCROOT/lib/libicudata.sl.24.0 .");
+        psystem("cp -f $XERCESCROOT/lib/libicudata26.1.so .");
+        psystem("cp -f $XERCESCROOT/lib/libicudata.so.26.1 .");
+        psystem("cp -f $XERCESCROOT/lib/libicudata.sl.26.1 .");
 
-        psystem("find . -name 'libicudata24.0.so' -exec ln -s {} libicudata.so \\;");
-        psystem("find . -name 'libicudata24.0.so' -exec ln -s {} libicudata24.so \\;");
+        psystem("find . -name 'libicudata26.1.so' -exec ln -s {} libicudata.so \\;");
+        psystem("find . -name 'libicudata26.1.so' -exec ln -s {} libicudata26.so \\;");
 
-        psystem("find . -name 'libicudata.so.24.0' -exec ln -s {} libicudata.so \\;");
-        psystem("find . -name 'libicudata.so.24.0' -exec ln -s {} libicudata.so.24 \\;");
+        psystem("find . -name 'libicudata.so.26.1' -exec ln -s {} libicudata.so \\;");
+        psystem("find . -name 'libicudata.so.26.1' -exec ln -s {} libicudata.so.26 \\;");
 
-        psystem("find . -name 'libicudata.sl.24.0' -exec ln -s {} libicudata.sl \\;");
-        psystem("find . -name 'libicudata.sl.24.0' -exec ln -s {} libicudata.sl.24 \\;");
+        psystem("find . -name 'libicudata.sl.26.1' -exec ln -s {} libicudata.sl \\;");
+        psystem("find . -name 'libicudata.sl.26.1' -exec ln -s {} libicudata.sl.26 \\;");
 
         #
         # copy icuuc dll
-        # on AIX, it is called libicuuc24.0.so
-        # on Solaris/Linux, it is called libicuuc.so.24.0
-        # on HP, it is called libicuuc.sl.24.0
+        # on AIX, it is called libicuuc26.1.so
+        # on Solaris/Linux, it is called libicuuc.so.26.1
+        # on HP, it is called libicuuc.sl.26.1
         #
         psystem("rm -f libicuuc*");
-        psystem("cp -f $XERCESCROOT/lib/libicuuc24.0.so .");
-        psystem("cp -f $XERCESCROOT/lib/libicuuc.so.24.0  .");
-        psystem("cp -f $XERCESCROOT/lib/libicuuc.sl.24.0  .");
+        psystem("cp -f $XERCESCROOT/lib/libicuuc26.1.so .");
+        psystem("cp -f $XERCESCROOT/lib/libicuuc.so.26.1  .");
+        psystem("cp -f $XERCESCROOT/lib/libicuuc.sl.26.1  .");
 
-        psystem("find . -name 'libicuuc24.0.so' -exec ln -s {} libicuuc.so \\;");
-        psystem("find . -name 'libicuuc24.0.so' -exec ln -s {} libicuuc24.so \\;");
+        psystem("find . -name 'libicuuc26.1.so' -exec ln -s {} libicuuc.so \\;");
+        psystem("find . -name 'libicuuc26.1.so' -exec ln -s {} libicuuc26.so \\;");
 
-        psystem("find . -name 'libicuuc.so.24.0' -exec ln -s {} libicuuc.so \\;");
-        psystem("find . -name 'libicuuc.so.24.0' -exec ln -s {} libicuuc.so.24 \\;");
+        psystem("find . -name 'libicuuc.so.26.1' -exec ln -s {} libicuuc.so \\;");
+        psystem("find . -name 'libicuuc.so.26.1' -exec ln -s {} libicuuc.so.26 \\;");
 
-        psystem("find . -name 'libicuuc.sl.24.0' -exec ln -s {} libicuuc.sl \\;");
-        psystem("find . -name 'libicuuc.sl.24.0' -exec ln -s {} libicuuc.sl.24 \\;");
+        psystem("find . -name 'libicuuc.sl.26.1' -exec ln -s {} libicuuc.sl \\;");
+        psystem("find . -name 'libicuuc.sl.26.1' -exec ln -s {} libicuuc.sl.26 \\;");
 
+        # on AIX, copy '.a' version
+        if ( $platform eq "aix") {
+            psystem("ln -s libicudata26.so libicudata26.a");
+            psystem("ln -s libicuuc26.so   libicuuc26.a");
+        }        
+                
         # Copy the Resouce Bundle for ICUMsgLoader
         if ( $opt_m =~ m/icu/i) {
             print ("\n\nCopying ICU message bundles ...\n");        	
             psystem("cp -f $XERCESCROOT/msg/XercesMessages*.res $targetdir/msg");
            
-            psystem("cp -f $XERCESCROOT/lib/libXercesMessages23.0.so .");
-            psystem("find . -name 'libXercesMessages23.0.so' -exec ln -s {} libXercesMessages23.so \\;");
-            psystem("find . -name 'libXercesMessages23.so'   -exec ln -s {} libXercesMessages.so \\;");
+            psystem("cp -f $XERCESCROOT/lib/libXercesMessages25.0.so .");
+            psystem("find . -name 'libXercesMessages25.0.so' -exec ln -s {} libXercesMessages25.so \\;");
+            psystem("find . -name 'libXercesMessages25.so'   -exec ln -s {} libXercesMessages.so \\;");
                     
-            psystem("cp -f $XERCESCROOT/lib/libXercesMessages.so.23.0 .");
-            psystem("find . -name 'libXercesMessages.so.23.0' -exec ln -s {} libXercesMessages.so.23 \\;");
-            psystem("find . -name 'libXercesMessages.so.23'   -exec ln -s {} libXercesMessages.so \\;");
+            psystem("cp -f $XERCESCROOT/lib/libXercesMessages.so.25.0 .");
+            psystem("find . -name 'libXercesMessages.so.25.0' -exec ln -s {} libXercesMessages.so.25 \\;");
+            psystem("find . -name 'libXercesMessages.so.25'   -exec ln -s {} libXercesMessages.so \\;");
             
-            psystem("cp -f $XERCESCROOT/lib/libXercesMessages.sl.23.0 .");
-            psystem("find . -name 'libXercesMessages.sl.23.0' -exec ln -s {} libXercesMessages.sl.23 \\;");
-            psystem("find . -name 'libXercesMessages.sl.23'   -exec ln -s {} libXercesMessages.sl \\;");            
+            psystem("cp -f $XERCESCROOT/lib/libXercesMessages.sl.25.0 .");
+            psystem("find . -name 'libXercesMessages.sl.25.0' -exec ln -s {} libXercesMessages.sl.25 \\;");
+            psystem("find . -name 'libXercesMessages.sl.25'   -exec ln -s {} libXercesMessages.sl \\;");            
+
+            # on AIX            
+            if ( $platform eq "aix") {
+                psystem("cp -f $XERCESCROOT/lib/libXercesMessages25.0.a .");
+                psystem("ln -s libXercesMessages25.0.a libXercesMessages25.a ");
+                psystem("ln -s libXercesMessages25.a   libXercesMessages.a   ");
+            }        
+                    
         }        	
 
     }
@@ -1381,15 +1495,23 @@ if ( ($platform =~ m/AIX/i)   || ($platform =~ m/HP-UX/i) || ($platform =~ m/BeO
     psystem("rm -f $targetdir/samples/SEnumVal/Makefile");
     psystem("cp -Rf $XERCESCROOT/samples/CreateDOMDocument/* $targetdir/samples/CreateDOMDocument");
     psystem("rm -f $targetdir/samples/CreateDOMDocument/Makefile");
+    psystem("cp -Rf $XERCESCROOT/samples/PSVIWriter/* $targetdir/samples/PSVIWriter");
+    psystem("rm -f $targetdir/samples/PSVIWriter/Makefile");    
+    psystem("cp -Rf $XERCESCROOT/samples/SCMPrint/* $targetdir/samples/SCMPrint");
+    psystem("rm -f $targetdir/samples/SCMPrint/Makefile");        
     psystem("rm -f $targetdir/samples/Makefile");
 
+    # Populate the scripts directory
+    print ("\n\nCopying script files ...\n");
+    psystem("cp $XERCESCROOT/scripts/sanityTest* $targetdir/scripts");
+
     # Populate the docs directory
     print ("\n\nCopying documentation ...\n");
     psystem("cp -Rf $XERCESCROOT/doc/* $targetdir/doc");
     psystem("cp $XERCESCROOT/Readme.html $targetdir");
     psystem("cp $XERCESCROOT/credits.txt $targetdir");
     psystem("cp $XERCESCROOT/LICENSE.txt $targetdir");
-    if (length($ICUROOT) > 0) {
+    if ($ICUIsPresent) {
         psystem("cp $XERCESCROOT/license.html $targetdir");
         psystem("cp $XERCESCROOT/XLicense.html $targetdir");
     }
@@ -1457,7 +1579,15 @@ sub change_windows_project_for_ICU() {
         $line =~ s[/D "PROJ_XMLPARSER"][/I "$ICUROOT\\include" /D "PROJ_XMLPARSER"];
         $line =~ s[Debug/xerces-c_2D.lib"][Debug/xerces-c_2D.lib" /libpath:"$ICUROOT\\lib" /libpath:"$ICUROOT\\source\\data" /libpath:"$XERCESCROOT\\src\\xercesc\\util\\MsgLoaders\\ICU\\resources"];
         $line =~ s[Release/xerces-c_2.lib"][Release/xerces-c_2.lib" /libpath:"$ICUROOT\\lib" /libpath:"$ICUROOT\\source\\data" /libpath:"$XERCESCROOT\\src\\xercesc\\util\\MsgLoaders\\ICU\\resources"];       
-        $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib XercesMessages2_3_0.lib/g;
+       
+        if ($msgloader)
+        {
+            $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib XercesMessages2_5_0.lib/g;
+        }        
+        elsif ($transcoder)
+        {
+            $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib/g;
+        }
         
         if ($transcoder)
         {
@@ -1498,7 +1628,15 @@ sub change_windows_makefile_for_ICU() {
 
         $line =~ s[/D "PROJ_XMLPARSER"][/I "$ICUROOT\\include" /D "PROJ_XMLPARSER"];
         $line =~ s[/machine:IA64][/libpath:"$ICUROOT\\lib" /libpath:"$ICUROOT\\source\\data" /libpath:"$XERCESCROOT\\src\\xercesc\\util\\MsgLoaders\\ICU\\resources" /machine:IA64];
-        $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib XercesMessages2_3_0.lib/g;
+
+        if ($msgloader)
+        {
+            $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib XercesMessages2_5_0.lib/g;
+        }        
+        elsif ($transcoder)
+        {
+            $line =~ s/user32.lib/user32.lib $icuuc.lib icudata.lib/g;
+        }
             
         if ($transcoder) {
             $line =~ s/XML_USE_WIN32_TRANSCODER/XML_USE_ICU_TRANSCODER/g;
@@ -1537,7 +1675,15 @@ sub change_windows_project_for_ICU_VC7() {
             }
         $line =~ s/AdditionalIncludeDirectories=\"([^"]*)/AdditionalIncludeDirectories=\"$ICUROOT\\include;$1/;
         $line =~ s/AdditionalLibraryDirectories=\"([^"]*)/AdditionalLibraryDirectories=\"$ICUROOT\\lib;$ICUROOT\\source\\data;$XERCESCROOT\\src\\xercesc\\util\\MsgLoaders\\ICU\\resources;$1/;
-        $line =~ s/AdditionalDependencies=\"([^"]*)/AdditionalDependencies=\"$icuuc.lib icudata.lib XercesMessages2_3_0.lib $1/;
+        
+        if ($msgloader)
+        {
+            $line =~ s/AdditionalDependencies=\"([^"]*)/AdditionalDependencies=\"$icuuc.lib icudata.lib XercesMessages2_5_0.lib $1/;
+        }        
+        elsif ($transcoder)
+        {
+            $line =~ s/AdditionalDependencies=\"([^"]*)/AdditionalDependencies=\"$icuuc.lib icudata.lib $1/;
+        }
 
         if ($transcoder) {
             $line =~ s/XML_USE_WIN32_TRANSCODER/XML_USE_ICU_TRANSCODER/g;
