@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: DOMParser.cpp,v 1.18 2003/05/22 02:26:50 knoaman Exp $
+* $Id: DOMParser.cpp,v 1.31 2003/12/17 00:18:33 cargilld Exp $
 *
 */
 
@@ -77,6 +77,10 @@
 #include <xercesc/util/IOException.hpp>
 #include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/framework/XMLGrammarPool.hpp>
+#include <xercesc/framework/XMLSchemaDescription.hpp>
+#include <xercesc/util/Janitor.hpp>
+
 #include "DOMParser.hpp"
 #include "ElementImpl.hpp"
 #include "AttrImpl.hpp"
@@ -92,6 +96,8 @@
 
 #include <xercesc/validators/common/ContentSpecNode.hpp>
 #include <xercesc/validators/DTD/DTDAttDefList.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/util/XMLEntityResolver.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -99,8 +105,9 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  DOMParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-DOMParser::DOMParser( XMLValidator* const  valToAdopt
-                    , MemoryManager* const manager) :
+DOMParser::DOMParser( XMLValidator* const   valToAdopt
+                    , MemoryManager* const  manager
+                    , XMLGrammarPool* const gramPool) :
 
     fToCreateXMLDeclTypeNode(false)
     , fCreateEntityReferenceNodes(true)
@@ -108,7 +115,9 @@ DOMParser::DOMParser( XMLValidator* const  valToAdopt
     , fParseInProgress(false)
     , fWithinElement(false)
     , fEntityResolver(0)
+    , fXMLEntityResolver(0)
     , fErrorHandler(0)
+    , fPSVIHandler(0)
     , fNodeStack(0)
     , fScanner(0)
     , fDocumentType(0)
@@ -116,11 +125,16 @@ DOMParser::DOMParser( XMLValidator* const  valToAdopt
     , fURIStringPool(0)
     , fValidator(valToAdopt)
     , fMemoryManager(manager)
+    , fGrammarPool(gramPool)
 {
     try
     {
         initialize();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         cleanUp();
@@ -140,18 +154,17 @@ DOMParser::~DOMParser()
 void DOMParser::initialize()
 {
     // Create grammar resolver and URI string pool to pass to the scanner
-    fGrammarResolver = new (fMemoryManager) GrammarResolver(fMemoryManager);
-    fURIStringPool = new (fMemoryManager) XMLStringPool(109, fMemoryManager);
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fGrammarPool, fMemoryManager);
+    fURIStringPool = fGrammarResolver->getStringPool();
 
     //  Create a scanner and tell it what validator to use. Then set us
     //  as the document event handler so we can fill the DOM document.
-    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fMemoryManager);
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fGrammarResolver, fMemoryManager);
     fScanner->setDocHandler(this);
     fScanner->setDocTypeHandler(this);
-    fScanner->setGrammarResolver(fGrammarResolver);
     fScanner->setURIStringPool(fURIStringPool);
 
-    fNodeStack = new (fMemoryManager) ValueStackOf<DOM_Node>(64, fMemoryManager);
+    fNodeStack = new (fMemoryManager) ValueStackOf<DOM_Node>(64, fMemoryManager, true);
     this->reset();
 }
 
@@ -160,7 +173,8 @@ void DOMParser::cleanUp()
     delete fNodeStack;
     delete fScanner;
     delete fGrammarResolver;
-    delete fURIStringPool;
+    // grammar pool must do this
+    //delete fURIStringPool;
 
     if (fValidator)
         delete fValidator;
@@ -306,11 +320,35 @@ void DOMParser::setErrorHandler(ErrorHandler* const handler)
     }
 }
 
+void DOMParser::setPSVIHandler(PSVIHandler* const handler)
+{
+    fPSVIHandler = handler;
+    if (fPSVIHandler) {
+        fScanner->setPSVIHandler(fPSVIHandler);        
+    }
+    else {
+        fScanner->setPSVIHandler(0);       
+    }
+}
+
 void DOMParser::setEntityResolver(EntityResolver* const handler)
 {
     fEntityResolver = handler;
     if (fEntityResolver) {
         fScanner->setEntityHandler(this);
+        fXMLEntityResolver = 0;
+    }
+    else {
+        fScanner->setEntityHandler(0);
+    }
+}
+
+void DOMParser::setXMLEntityResolver(XMLEntityResolver* const handler)
+{
+    fXMLEntityResolver = handler;
+    if (fXMLEntityResolver) {
+        fEntityResolver = 0;
+        fScanner->setEntityHandler(this);
     }
     else {
         fScanner->setEntityHandler(0);
@@ -395,13 +433,13 @@ void DOMParser::useScanner(const XMLCh* const scannerName)
     (
         scannerName
         , fValidator
+        , fGrammarResolver
         , fMemoryManager
     );
 
     if (tempScanner) {
 
         tempScanner->setParseSettings(fScanner);
-        tempScanner->setGrammarResolver(fGrammarResolver);
         tempScanner->setURIStringPool(fURIStringPool);
         delete fScanner;
         fScanner = tempScanner;
@@ -415,7 +453,7 @@ void DOMParser::parse(const InputSource& source)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -423,7 +461,10 @@ void DOMParser::parse(const InputSource& source)
         fScanner->scanDocument(source);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -435,7 +476,7 @@ void DOMParser::parse(const XMLCh* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -443,7 +484,10 @@ void DOMParser::parse(const XMLCh* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -455,7 +499,7 @@ void DOMParser::parse(const char* const systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -463,7 +507,10 @@ void DOMParser::parse(const char* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -484,7 +531,7 @@ bool DOMParser::parseFirst( const   XMLCh* const    systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -497,7 +544,7 @@ bool DOMParser::parseFirst( const   char* const         systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -510,7 +557,7 @@ bool DOMParser::parseFirst( const   InputSource&    source
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(source, toFill);
 }
@@ -547,6 +594,7 @@ void DOMParser::error(  const   unsigned int                code
         , systemId
         , lineNum
         , colNum
+        , fMemoryManager
         );
 
     //
@@ -590,7 +638,19 @@ DOMParser::resolveEntity(const XMLCh* const publicId,
     return 0;
 }
 
-
+InputSource* DOMParser::resolveEntity(XMLResourceIdentifier* resourceIdentifier) 
+{
+    //
+    //  Just map it to the SAX entity resolver. If there is not one installed,
+    //  return a null pointer to cause the default resolution.
+    //
+    if (fEntityResolver)
+        return fEntityResolver->resolveEntity(resourceIdentifier->getPublicId(), 
+                                                resourceIdentifier->getSystemId());
+    if (fXMLEntityResolver)
+        return fXMLEntityResolver->resolveEntity(resourceIdentifier);
+    return 0;
+}
 
 // ---------------------------------------------------------------------------
 //  DOMParser: Implementation of XMLDocumentHandler interface
@@ -1132,9 +1192,9 @@ void DOMParser::endAttList
 		DOM_Element dom_elem = fDocument.createElement(elemDecl.getFullName());
 		ElementImpl* elem = (ElementImpl*)(dom_elem.fImpl);
 
-		while (defAttrs->hasMoreElements())
+        for(unsigned int i=0; i<defAttrs->getAttDefCount(); i++)
         {
-            attr = &defAttrs->nextElement();
+            attr = &defAttrs->getAttDef(i);
             if (attr->getValue() != null)
             {
                 if (fScanner->getDoNamespaces())
@@ -1347,7 +1407,7 @@ Grammar* DOMParser::loadGrammar(const char* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1356,8 +1416,10 @@ Grammar* DOMParser::loadGrammar(const char* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
-    catch(...)
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }    catch(...)
     {
         fParseInProgress = false;
         throw;
@@ -1372,7 +1434,7 @@ Grammar* DOMParser::loadGrammar(const XMLCh* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1381,7 +1443,10 @@ Grammar* DOMParser::loadGrammar(const XMLCh* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -1397,7 +1462,7 @@ Grammar* DOMParser::loadGrammar(const InputSource& source,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
    Grammar* grammar = 0;
     try
@@ -1406,7 +1471,10 @@ Grammar* DOMParser::loadGrammar(const InputSource& source,
         grammar = fScanner->loadGrammar(source, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
