@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMChildNode.hpp,v 1.2 2002/11/04 15:07:34 tng Exp $
+ * $Id: DOMChildNode.hpp,v 1.3 2004/01/29 11:44:26 cargilld Exp $
  */
 
 //
@@ -97,6 +97,12 @@ public:
     DOMNode * getNextSibling() const;
     DOMNode * getParentNode(const DOMNode *thisNode) const;
     DOMNode * getPreviousSibling(const DOMNode *thisNode) const;
+
+private:
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DOMChildNode & operator = (const DOMChildNode &);   
 };
 
 
