@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMDocumentImpl.cpp,v 1.43 2003/05/23 17:07:41 knoaman Exp $
+ * $Id: DOMDocumentImpl.cpp,v 1.52 2004/01/29 11:44:26 cargilld Exp $
  */
 #include "DOMDocumentImpl.hpp"
 #include "DOMCasts.hpp"
@@ -85,7 +85,7 @@
 #include <xercesc/dom/DOMImplementation.hpp>
 #include <xercesc/util/XMLChar.hpp>
 #include <xercesc/framework/MemoryManager.hpp>
-
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -103,32 +103,32 @@ XERCES_CPP_NAMESPACE_BEGIN
 DOMDocumentImpl::DOMDocumentImpl(MemoryManager* const manager)
     : fNode(this),
       fParent(this),
-      fCurrentBlock(0),
+      fNodeIDMap(0),
+      fActualEncoding(0),
+      fEncoding(0),
+      fStandalone(false),
+      fVersion(0),
+      fDocumentURI(0),
       fDOMConfiguration(0),
+      fUserDataTable(0),
+      fCurrentBlock(0),      
       fFreePtr(0),
       fFreeBytesRemaining(0),
+      fRecycleNodePtr(0),
+      fRecycleBufferPtr(0),
+      fNodeListPool(0),
       fDocType(0),
       fDocElement(0),
       fNamePool(0),
-      fNormalizer(0),
-      fNodeIDMap(0),
+      fNormalizer(0),      
       fRanges(0),
       fNodeIterators(0),
-      fChanges(0),
-      fNodeListPool(0),
-      fActualEncoding(0),
-      fEncoding(0),
-      fVersion(0),
-      fStandalone(false),
-      fDocumentURI(0),
-      fUserDataTable(0),
-      fRecycleNodePtr(0),
-      fRecycleBufferPtr(0),
       fMemoryManager(manager),
+      fChanges(0),      
       errorChecking(true)
 {
     fNamePool    = new (this) DOMStringPool(257, this);
-};
+}
 
 
 //DOM Level 2
@@ -138,28 +138,28 @@ DOMDocumentImpl::DOMDocumentImpl(const XMLCh *fNamespaceURI,
                                MemoryManager* const manager)
     : fNode(this),
       fParent(this),
-      fCurrentBlock(0),
+      fNodeIDMap(0),
+      fActualEncoding(0),
+      fEncoding(0),
+      fStandalone(false),
+      fVersion(0),
+      fDocumentURI(0),
       fDOMConfiguration(0),
+      fUserDataTable(0),
+      fCurrentBlock(0),      
       fFreePtr(0),
       fFreeBytesRemaining(0),
+      fRecycleNodePtr(0),
+      fRecycleBufferPtr(0),
+      fNodeListPool(0),
       fDocType(0),
       fDocElement(0),
       fNamePool(0),
-      fNormalizer(0),
-      fNodeIDMap(0),
+      fNormalizer(0),      
       fRanges(0),
       fNodeIterators(0),
-      fChanges(0),
-      fNodeListPool(0),
-      fActualEncoding(0),
-      fEncoding(0),
-      fVersion(0),
-      fStandalone(false),
-      fDocumentURI(0),
-      fUserDataTable(0),
-      fRecycleNodePtr(0),
-      fRecycleBufferPtr(0),
       fMemoryManager(manager),
+      fChanges(0),
       errorChecking(true)
 {
     fNamePool    = new (this) DOMStringPool(257, this);
@@ -171,6 +171,10 @@ DOMDocumentImpl::DOMDocumentImpl(const XMLCh *fNamespaceURI,
         else if (fNamespaceURI)
             throw DOMException(DOMException::NAMESPACE_ERR, 0);
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...) {
         this->deleteHeap();
         throw;
@@ -229,7 +233,7 @@ DOMDocumentImpl::~DOMDocumentImpl()
     //  Delete the heap for this document.  This uncerimoniously yanks the storage
     //      out from under all of the nodes in the document.  Destructors are NOT called.
     this->deleteHeap();
-};
+}
 
 
 DOMNode *DOMDocumentImpl::cloneNode(bool deep) const {
@@ -247,7 +251,7 @@ DOMNode *DOMDocumentImpl::cloneNode(bool deep) const {
 
     fNode.callUserDataHandlers(DOMUserDataHandler::NODE_CLONED, this, newdoc);
     return newdoc;
-};
+}
 
 
 const XMLCh * DOMDocumentImpl::getNodeName() const {
@@ -259,7 +263,7 @@ const XMLCh * DOMDocumentImpl::getNodeName() const {
 
 short DOMDocumentImpl::getNodeType() const {
     return DOMNode::DOCUMENT_NODE;
-};
+}
 
 
 // even though ownerDocument refers to this in this implementation
@@ -274,27 +278,27 @@ DOMAttr *DOMDocumentImpl::createAttribute(const XMLCh *nam)
     if(!nam || !isXMLName(nam))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     return new (this, DOMDocumentImpl::ATTR_OBJECT) DOMAttrImpl(this,nam);
-};
+}
 
 
 
 DOMCDATASection *DOMDocumentImpl::createCDATASection(const XMLCh *data) {
     return new (this, DOMDocumentImpl::CDATA_SECTION_OBJECT) DOMCDATASectionImpl(this,data);
-};
+}
 
 
 
 DOMComment *DOMDocumentImpl::createComment(const XMLCh *data)
 {
     return new (this, DOMDocumentImpl::COMMENT_OBJECT) DOMCommentImpl(this, data);
-};
+}
 
 
 
 DOMDocumentFragment *DOMDocumentImpl::createDocumentFragment()
 {
     return new (this, DOMDocumentImpl::DOCUMENT_FRAGMENT_OBJECT) DOMDocumentFragmentImpl(this);
-};
+}
 
 
 
@@ -305,7 +309,7 @@ DOMDocumentType *DOMDocumentImpl::createDocumentType(const XMLCh *nam)
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::DOCUMENT_TYPE_OBJECT) DOMDocumentTypeImpl(this, nam, false);
-};
+}
 
 
 
@@ -319,7 +323,7 @@ DOMDocumentType *
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::DOCUMENT_TYPE_OBJECT) DOMDocumentTypeImpl(this, qualifiedName, publicId, systemId, false);
-};
+}
 
 
 
@@ -329,13 +333,13 @@ DOMElement *DOMDocumentImpl::createElement(const XMLCh *tagName)
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
 
     return new (this, DOMDocumentImpl::ELEMENT_OBJECT) DOMElementImpl(this,tagName);
-};
+}
 
 
 DOMElement *DOMDocumentImpl::createElementNoCheck(const XMLCh *tagName)
 {
     return new (this, DOMDocumentImpl::ELEMENT_OBJECT) DOMElementImpl(this, tagName);
-};
+}
 
 
 
@@ -347,7 +351,7 @@ DOMEntity *DOMDocumentImpl::createEntity(const XMLCh *nam)
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::ENTITY_OBJECT) DOMEntityImpl(this, nam);
-};
+}
 
 
 
@@ -358,7 +362,7 @@ DOMEntityReference *DOMDocumentImpl::createEntityReference(const XMLCh *nam)
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::ENTITY_REFERENCE_OBJECT) DOMEntityReferenceImpl(this, nam);
-};
+}
 
 DOMEntityReference *DOMDocumentImpl::createEntityReferenceByParser(const XMLCh *nam)
 {
@@ -367,7 +371,7 @@ DOMEntityReference *DOMDocumentImpl::createEntityReferenceByParser(const XMLCh *
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::ENTITY_REFERENCE_OBJECT) DOMEntityReferenceImpl(this, nam, false);
-};
+}
 
 DOMNotation *DOMDocumentImpl::createNotation(const XMLCh *nam)
 {
@@ -376,7 +380,7 @@ DOMNotation *DOMDocumentImpl::createNotation(const XMLCh *nam)
         DOMException::INVALID_CHARACTER_ERR, 0);
 
     return new (this, DOMDocumentImpl::NOTATION_OBJECT) DOMNotationImpl(this, nam);
-};
+}
 
 
 
@@ -386,7 +390,7 @@ DOMProcessingInstruction *DOMDocumentImpl::createProcessingInstruction(
     if(!target || !isXMLName(target))
         throw DOMException(DOMException::INVALID_CHARACTER_ERR,0);
     return new (this, DOMDocumentImpl::PROCESSING_INSTRUCTION_OBJECT) DOMProcessingInstructionImpl(this,target,data);
-};
+}
 
 
 
@@ -394,7 +398,7 @@ DOMProcessingInstruction *DOMDocumentImpl::createProcessingInstruction(
 DOMText *DOMDocumentImpl::createTextNode(const XMLCh *data)
 {
     return new (this, DOMDocumentImpl::TEXT_OBJECT) DOMTextImpl(this,data);
-};
+}
 
 
 DOMNodeIterator* DOMDocumentImpl::createNodeIterator (
@@ -437,6 +441,29 @@ void DOMDocumentImpl::removeNodeIterator(DOMNodeIteratorImpl* nodeIterator)
     }
 }
 
+
+const DOMXPathExpression* DOMDocumentImpl::createExpression(const XMLCh *, const DOMXPathNSResolver *)
+{
+    throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
+    return 0;
+}
+
+const DOMXPathNSResolver* DOMDocumentImpl::createNSResolver(DOMNode *)
+
+{
+    throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
+    return 0;
+}
+
+void* DOMDocumentImpl::evaluate(const XMLCh *, DOMNode *, const DOMXPathNSResolver *, 
+                           unsigned short, void* ) 
+{
+    throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
+    return 0;
+}
+
+
+
 DOMTreeWalker* DOMDocumentImpl::createTreeWalker (DOMNode *root, unsigned long whatToShow, DOMNodeFilter* filter, bool entityReferenceExpansion)
 {
     if (!root) {
@@ -453,14 +480,14 @@ DOMTreeWalker* DOMDocumentImpl::createTreeWalker (DOMNode *root, unsigned long w
 DOMDocumentType *DOMDocumentImpl::getDoctype() const
 {
     return fDocType;
-};
+}
 
 
 
 DOMElement *DOMDocumentImpl::getDocumentElement() const
 {
     return fDocElement;
-};
+}
 
 
 
@@ -468,7 +495,7 @@ DOMNodeList *DOMDocumentImpl::getElementsByTagName(const XMLCh *tagname) const
 {
     // cast off the const of this because we will update the fNodeListPool
     return ((DOMDocumentImpl*)this)->getDeepNodeList(this,tagname);
-};
+}
 
 
 DOMImplementation   *DOMDocumentImpl::getImplementation() const {
@@ -499,7 +526,7 @@ DOMNode *DOMDocumentImpl::insertBefore(DOMNode *newChild, DOMNode *refChild)
         fDocType=(DOMDocumentType *)newChild;
 
     return newChild;
-};
+}
 
 
 DOMNode* DOMDocumentImpl::replaceChild(DOMNode *newChild, DOMNode *oldChild) {
@@ -521,6 +548,10 @@ DOMNode* DOMDocumentImpl::replaceChild(DOMNode *newChild, DOMNode *oldChild) {
         else
             return removeChild(oldChild);
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...) {
         fDocType = tempDocType;
         fDocElement = tempDocElement;
@@ -534,7 +565,7 @@ bool DOMDocumentImpl::isXMLName(const XMLCh *s)
         return XMLChar1_1::isValidName(s, XMLString::stringLen(s));
     else
         return XMLChar1_0::isValidName(s, XMLString::stringLen(s));
-};
+}
 
 
 
@@ -550,14 +581,14 @@ DOMNode *DOMDocumentImpl::removeChild(DOMNode *oldChild)
         fDocType=0;
 
     return oldChild;
-};
+}
 
 
 
 void DOMDocumentImpl::setNodeValue(const XMLCh *x)
 {
     fNode.setNodeValue(x);
-};
+}
 
 
 //Introduced in DOM Level 2
@@ -640,13 +671,13 @@ int DOMDocumentImpl::indexofQualifiedName(const XMLCh * qName)
 const XMLCh*     DOMDocumentImpl::getBaseURI() const
 {
 	  return fDocumentURI;
-};
+}
 
 
 DOMRange* DOMDocumentImpl::createRange()
 {
 
-    DOMRangeImpl* range = new (this) DOMRangeImpl(this);
+    DOMRangeImpl* range = new (this) DOMRangeImpl(this, fMemoryManager);
 
     if (fRanges == 0L) {
         //fRanges = new (this) Ranges(1, false);
@@ -715,7 +746,7 @@ bool DOMDocumentImpl::isKidOK(DOMNode *parent, DOMNode *child)
               kidOK[DOMNode::CDATA_SECTION_NODE] =
               kidOK[DOMNode::NOTATION_NODE] =
               0;
-      };
+      }
       int p=parent->getNodeType();
       int ch = child->getNodeType();
       return (kidOK[p] & 1<<ch) != 0;
@@ -729,43 +760,43 @@ void            DOMDocumentImpl::changed()
 
 int             DOMDocumentImpl::changes() const{
     return fChanges;
-};
+}
 
 
 
 //
 //    Delegation for functions inherited from DOMNode
 //
-           DOMNode*         DOMDocumentImpl::appendChild(DOMNode *newChild)          {return insertBefore(newChild, 0); };
-           DOMNamedNodeMap* DOMDocumentImpl::getAttributes() const                   {return fNode.getAttributes (); };
-           DOMNodeList*     DOMDocumentImpl::getChildNodes() const                   {return fParent.getChildNodes (); };
-           DOMNode*         DOMDocumentImpl::getFirstChild() const                   {return fParent.getFirstChild (); };
-           DOMNode*         DOMDocumentImpl::getLastChild() const                    {return fParent.getLastChild (); };
-     const XMLCh*           DOMDocumentImpl::getLocalName() const                    {return fNode.getLocalName (); };
-     const XMLCh*           DOMDocumentImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); };
-           DOMNode*         DOMDocumentImpl::getNextSibling() const                  {return fNode.getNextSibling (); };
-     const XMLCh*           DOMDocumentImpl::getNodeValue() const                    {return fNode.getNodeValue (); };
-     const XMLCh*           DOMDocumentImpl::getPrefix() const                       {return fNode.getPrefix (); };
-           DOMNode*         DOMDocumentImpl::getParentNode() const                   {return fNode.getParentNode (); };
-           DOMNode*         DOMDocumentImpl::getPreviousSibling() const              {return fNode.getPreviousSibling (); };
-           bool             DOMDocumentImpl::hasChildNodes() const                   {return fParent.hasChildNodes (); };
-           void             DOMDocumentImpl::normalize()                             {fParent.normalize (); };
+           DOMNode*         DOMDocumentImpl::appendChild(DOMNode *newChild)          {return insertBefore(newChild, 0); }
+           DOMNamedNodeMap* DOMDocumentImpl::getAttributes() const                   {return fNode.getAttributes (); }
+           DOMNodeList*     DOMDocumentImpl::getChildNodes() const                   {return fParent.getChildNodes (); }
+           DOMNode*         DOMDocumentImpl::getFirstChild() const                   {return fParent.getFirstChild (); }
+           DOMNode*         DOMDocumentImpl::getLastChild() const                    {return fParent.getLastChild (); }
+     const XMLCh*           DOMDocumentImpl::getLocalName() const                    {return fNode.getLocalName (); }
+     const XMLCh*           DOMDocumentImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); }
+           DOMNode*         DOMDocumentImpl::getNextSibling() const                  {return fNode.getNextSibling (); }
+     const XMLCh*           DOMDocumentImpl::getNodeValue() const                    {return fNode.getNodeValue (); }
+     const XMLCh*           DOMDocumentImpl::getPrefix() const                       {return fNode.getPrefix (); }
+           DOMNode*         DOMDocumentImpl::getParentNode() const                   {return fNode.getParentNode (); }
+           DOMNode*         DOMDocumentImpl::getPreviousSibling() const              {return fNode.getPreviousSibling (); }
+           bool             DOMDocumentImpl::hasChildNodes() const                   {return fParent.hasChildNodes (); }
+           void             DOMDocumentImpl::normalize()                             {fParent.normalize (); }
            bool             DOMDocumentImpl::isSupported(const XMLCh *feature, const XMLCh *version) const
-                                                                                     {return fNode.isSupported (feature, version); };
-           void             DOMDocumentImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); };
-           bool             DOMDocumentImpl::hasAttributes() const                   {return fNode.hasAttributes(); };
-           bool             DOMDocumentImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other);};
-           bool             DOMDocumentImpl::isEqualNode(const DOMNode* arg) const   {return fParent.isEqualNode(arg);};
+                                                                                     {return fNode.isSupported (feature, version); }
+           void             DOMDocumentImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); }
+           bool             DOMDocumentImpl::hasAttributes() const                   {return fNode.hasAttributes(); }
+           bool             DOMDocumentImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other);}
+           bool             DOMDocumentImpl::isEqualNode(const DOMNode* arg) const   {return fParent.isEqualNode(arg);}
            void*            DOMDocumentImpl::setUserData(const XMLCh* key, void* data, DOMUserDataHandler* handler)
-                                                                                     {return fNode.setUserData(key, data, handler); };
-           void*            DOMDocumentImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
-           short            DOMDocumentImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); };
-           const XMLCh*     DOMDocumentImpl::getTextContent() const                  {return fNode.getTextContent(); };
-           void             DOMDocumentImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
-           const XMLCh*     DOMDocumentImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
-           bool             DOMDocumentImpl::isDefaultNamespace(const XMLCh* namespaceURI) const {return fNode.isDefaultNamespace(namespaceURI); };
-           const XMLCh*     DOMDocumentImpl::lookupNamespaceURI(const XMLCh* prefix) const  {return fNode.lookupNamespaceURI(prefix); };
-           DOMNode*         DOMDocumentImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); };
+                                                                                     {return fNode.setUserData(key, data, handler); }
+           void*            DOMDocumentImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); }
+           short            DOMDocumentImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); }
+           const XMLCh*     DOMDocumentImpl::getTextContent() const                  {return fNode.getTextContent(); }
+           void             DOMDocumentImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); }
+           const XMLCh*     DOMDocumentImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); }
+           bool             DOMDocumentImpl::isDefaultNamespace(const XMLCh* namespaceURI) const {return fNode.isDefaultNamespace(namespaceURI); }
+           const XMLCh*     DOMDocumentImpl::lookupNamespaceURI(const XMLCh* prefix) const  {return fNode.lookupNamespaceURI(prefix); }
+           DOMNode*         DOMDocumentImpl::getInterface(const XMLCh* feature)      {return fNode.getInterface(feature); }
 
 
 
@@ -819,16 +850,9 @@ void *         DOMDocumentImpl::allocate(size_t amount)
 		size_t sizeOfHeader = XMLPlatformUtils::alignPointerForNewBlockAllocation(sizeof(void *));
 
 		//	Try to allocate the block
-        void* newBlock = 0;
-        try {
-            newBlock = fMemoryManager->allocate((sizeOfHeader + amount) * sizeof(char)); //new char[amount + sizeOfHeader];
-        }
-        catch (...) {
-            ThrowXML(RuntimeException, XMLExcepts::Out_Of_Memory);
-        }
-        if (!newBlock)
-           ThrowXML(RuntimeException, XMLExcepts::Out_Of_Memory);
-
+        void* newBlock;
+        newBlock = fMemoryManager->allocate((sizeOfHeader + amount) * sizeof(char)); //new char[amount + sizeOfHeader];
+        
 		//	Link it into the list beyond current block, as current block
 		//	is still being subdivided. If there is no current block
 		//	then track that we have no bytes to further divide.
@@ -858,16 +882,9 @@ void *         DOMDocumentImpl::allocate(size_t amount)
 		size_t sizeOfHeader = XMLPlatformUtils::alignPointerForNewBlockAllocation(sizeof(void *));
 
         // Get a new block from the system allocator.
-        void* newBlock = 0;
-        try {
-            newBlock = fMemoryManager->allocate(kHeapAllocSize * sizeof(char)); //new char[kHeapAllocSize];
-        }
-        catch (...) {
-            ThrowXML(RuntimeException, XMLExcepts::Out_Of_Memory);
-        }
-        if (!newBlock)
-           ThrowXML(RuntimeException, XMLExcepts::Out_Of_Memory);
-
+        void* newBlock;
+        newBlock = fMemoryManager->allocate(kHeapAllocSize * sizeof(char)); //new char[kHeapAllocSize];
+        
         *(void **)newBlock = fCurrentBlock;
         fCurrentBlock = newBlock;
         fFreePtr = (char *)newBlock + sizeOfHeader;
@@ -990,7 +1007,7 @@ void DOMDocumentImpl::setStrictErrorChecking(bool strictErrorChecking) {
     setErrorChecking(strictErrorChecking);
 }
 
-DOMNode* DOMDocumentImpl::adoptNode(DOMNode* source) {
+DOMNode* DOMDocumentImpl::adoptNode(DOMNode*) {
     throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
     return 0;
 }
@@ -1005,7 +1022,7 @@ void DOMDocumentImpl::normalizeDocument() {
 
 DOMConfiguration* DOMDocumentImpl::getDOMConfiguration() const {
     if(!fDOMConfiguration)
-        ((DOMDocumentImpl*)this)->fDOMConfiguration = new ((DOMDocumentImpl*)this) DOMConfigurationImpl();
+        ((DOMDocumentImpl*)this)->fDOMConfiguration = new ((DOMDocumentImpl*)this) DOMConfigurationImpl(fMemoryManager);
 
     return fDOMConfiguration;
 }
@@ -1248,7 +1265,7 @@ void DOMDocumentImpl::callUserDataHandlers(const DOMNodeImpl* n, DOMUserDataHand
         DOMNodeUserDataTable*  node_userDataTable = fUserDataTable->get((void*)n);
 
         if (node_userDataTable) {
-            RefHashTableOfEnumerator<DOMUserDataRecord> userDataEnum(node_userDataTable);
+            RefHashTableOfEnumerator<DOMUserDataRecord> userDataEnum(node_userDataTable, false, fMemoryManager);
 
             // walk through the entire node_userDataTable table
             while (userDataEnum.hasMoreElements()) {
@@ -1311,8 +1328,9 @@ void DOMDocumentImpl::release()
     DOMDocument* doc = (DOMDocument*) this;
     fNode.callUserDataHandlers(DOMUserDataHandler::NODE_DELETED, 0, 0);
 
-    // notify userdatahandler first
-    releaseDocNotifyUserData(this);
+    // notify userdatahandler first, if we have some
+    if (fUserDataTable)
+        releaseDocNotifyUserData(this);
 
     // release the docType in case it was created from heap
     if (fDocType) {
@@ -1322,21 +1340,26 @@ void DOMDocumentImpl::release()
 
     // delete the document memory pool
     delete doc;
-};
+}
 
 void DOMDocumentImpl::releaseDocNotifyUserData(DOMNode* object)
 {
     DOMNode *child = object->getFirstChild();
-    if (child != 0)
+    
+    while( child != 0)
     {
-        while( child != 0)
-        {
-            releaseDocNotifyUserData(child);
-            child = child->getNextSibling();
-        }
+            
+         DOMNamedNodeMap *attrlist=child->getAttributes(); 
+    
+         if(attrlist!=0) 
+             for(XMLSize_t i=0;i<attrlist->getLength();++i) 
+                 releaseDocNotifyUserData(attrlist->item(i)); 
+            
+        releaseDocNotifyUserData(child);
+        child = child->getNextSibling();
     }
-    else
-        castToNodeImpl(object)->callUserDataHandlers(DOMUserDataHandler::NODE_DELETED, 0, 0);
+
+    castToNodeImpl(object)->callUserDataHandlers(DOMUserDataHandler::NODE_DELETED, 0, 0);
 }
 
 void DOMDocumentImpl::release(DOMNode* object, NodeObjectType type)
