@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMDocumentImpl.hpp,v 1.21 2003/05/23 03:37:14 knoaman Exp $
+ * $Id: DOMDocumentImpl.hpp,v 1.23 2004/01/29 11:44:26 cargilld Exp $
  */
 
 //
@@ -107,6 +107,8 @@ class DOMRangeImpl;
 class DOMStringPool;
 class DOMBuffer;
 class MemoryManager;
+class XPathNSResolver;
+class XPathExpression;
 
 typedef RefVectorOf<DOMRangeImpl>        Ranges;
 typedef RefVectorOf<DOMNodeIteratorImpl>     NodeIterators;
@@ -193,6 +195,11 @@ public:
     virtual void                 removeRange(DOMRangeImpl* range); //non-standard api
     virtual void                 removeNodeIterator(DOMNodeIteratorImpl* nodeIterator); //non-standard api
 
+    virtual const DOMXPathExpression*    createExpression(const XMLCh *expression, const DOMXPathNSResolver *resolver);
+    virtual const DOMXPathNSResolver*    createNSResolver(DOMNode *nodeResolver);
+    virtual void* evaluate(const XMLCh *expression, DOMNode *contextNode, const DOMXPathNSResolver *resolver, 
+                           unsigned short type, void* result);
+
 
     // Extension to be called by the Parser
     DOMEntityReference*  createEntityReferenceByParser(const XMLCh * name);
@@ -320,6 +327,12 @@ private:
     //Internal helper functions
     virtual DOMNode*             importNode(DOMNode *source, bool deep, bool cloningNode);
 
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DOMDocumentImpl(const DOMDocumentImpl &);
+    DOMDocumentImpl & operator = (const DOMDocumentImpl &);
+
 private:
     // -----------------------------------------------------------------------
     //  data
