@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMElementImpl.hpp,v 1.6 2003/01/29 19:43:08 gareth Exp $
+ * $Id: DOMElementImpl.hpp,v 1.7 2004/01/29 11:44:26 cargilld Exp $
  */
 
 //
@@ -163,6 +163,11 @@ protected:
     // default attribute helper functions
     virtual void setupDefaultAttributes();
 
+private:
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------    
+    DOMElementImpl & operator = (const DOMElementImpl &);
 };
 
 XERCES_CPP_NAMESPACE_END
