@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMNamedNodeMapImpl.hpp,v 1.4 2002/11/04 15:07:34 tng Exp $
+ * $Id: DOMNamedNodeMapImpl.hpp,v 1.5 2004/01/29 11:44:26 cargilld Exp $
  */
 
 //
@@ -117,7 +117,10 @@ public:
     virtual DOMNode*        setNamedItemNS(DOMNode *arg);
     virtual DOMNode*        removeNamedItemNS(const XMLCh *namespaceURI,
 	                                           const XMLCh *localName);
-
+private:
+    // unimplemented
+    DOMNamedNodeMapImpl(const DOMNamedNodeMapImpl &);
+    DOMNamedNodeMapImpl & operator = (const DOMNamedNodeMapImpl &);
 };
 
 XERCES_CPP_NAMESPACE_END
