@@ -57,7 +57,7 @@
  */
 
 /*
- * $Id: DOMNodeVector.hpp,v 1.3 2002/11/04 15:07:34 tng Exp $
+ * $Id: DOMNodeVector.hpp,v 1.4 2004/01/29 11:44:27 cargilld Exp $
  */
 
 //
@@ -85,6 +85,10 @@ private:
     XMLSize_t      nextFreeSlot;
     void           init(DOMDocument *doc, XMLSize_t size);
     void           checkSpace();
+    
+    // unimplemented
+    DOMNodeVector ( const DOMNodeVector& toCopy);
+    DOMNodeVector& operator= (const DOMNodeVector& other);
 
 public:
     DOMNodeVector(DOMDocument *doc);
