@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMStringPool.hpp,v 1.4 2003/03/07 18:07:17 tng Exp $
+ * $Id: DOMStringPool.hpp,v 1.5 2004/01/29 11:44:27 cargilld Exp $
  */
 
 //
@@ -199,6 +199,12 @@ private :
     unsigned int    fIndex;
     unsigned int    fCapacity;
     DOMDocumentImpl* fDoc;
+
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DOMBuffer(const DOMBuffer &);
+    DOMBuffer & operator = (const DOMBuffer &);
 };
 
 XERCES_CPP_NAMESPACE_END
