@@ -129,18 +129,18 @@ LocalFileInputSource::LocalFileInputSource( const XMLCh* const basePath
     //  with the base path. If not, just take the relative path as the
     //  entire path.
     //
-    if (XMLPlatformUtils::isRelative(relativePath))
+    if (XMLPlatformUtils::isRelative(relativePath, manager))
     {
-        XMLCh* tmpBuf = XMLPlatformUtils::weavePaths(basePath, relativePath);
+        XMLCh* tmpBuf = XMLPlatformUtils::weavePaths(basePath, relativePath, manager);
         setSystemId(tmpBuf);
-        XMLPlatformUtils::fgMemoryManager->deallocate(tmpBuf); //delete [] tmpBuf;
+        manager->deallocate(tmpBuf); //delete [] tmpBuf;
     }
     else
     {
-        XMLCh* tmpBuf = XMLString::replicate(relativePath, getMemoryManager());
-        XMLPlatformUtils::removeDotSlash(tmpBuf);
+        XMLCh* tmpBuf = XMLString::replicate(relativePath, manager);
+        XMLPlatformUtils::removeDotSlash(tmpBuf, manager);
         setSystemId(tmpBuf);
-        getMemoryManager()->deallocate(tmpBuf);//delete [] tmpBuf;
+        manager->deallocate(tmpBuf);//delete [] tmpBuf;
     }
 
 }
@@ -155,13 +155,13 @@ LocalFileInputSource::LocalFileInputSource(const XMLCh* const filePath,
     //  working directory rules of the current platform. Else, just take
     //  it as is.
     //
-    if (XMLPlatformUtils::isRelative(filePath))
+    if (XMLPlatformUtils::isRelative(filePath, manager))
     {
-        XMLCh* curDir = XMLPlatformUtils::getCurrentDirectory(getMemoryManager());
+        XMLCh* curDir = XMLPlatformUtils::getCurrentDirectory(manager);
 
         int    curDirLen = XMLString::stringLen(curDir);
         int    filePathLen = XMLString::stringLen(filePath);
-        XMLCh* fullDir = (XMLCh*) getMemoryManager()->allocate
+        XMLCh* fullDir = (XMLCh*) manager->allocate
         (
             (curDirLen + filePathLen + 2) * sizeof(XMLCh)
         );//new XMLCh [ curDirLen + filePathLen + 2];
@@ -170,20 +170,20 @@ LocalFileInputSource::LocalFileInputSource(const XMLCh* const filePath,
         fullDir[curDirLen] = chForwardSlash;
         XMLString::copyString(&fullDir[curDirLen+1], filePath);
         
-        XMLPlatformUtils::removeDotSlash(fullDir);
-        XMLPlatformUtils::removeDotDotSlash(fullDir);
+        XMLPlatformUtils::removeDotSlash(fullDir, manager);
+        XMLPlatformUtils::removeDotDotSlash(fullDir, manager);
 
         setSystemId(fullDir);
 
-        getMemoryManager()->deallocate(curDir);//delete [] curDir;
-        getMemoryManager()->deallocate(fullDir);//delete [] fullDir;
+        manager->deallocate(curDir);//delete [] curDir;
+        manager->deallocate(fullDir);//delete [] fullDir;
     }
      else
     {
-        XMLCh* tmpBuf = XMLString::replicate(filePath, getMemoryManager());
-        XMLPlatformUtils::removeDotSlash(tmpBuf);
+        XMLCh* tmpBuf = XMLString::replicate(filePath, manager);
+        XMLPlatformUtils::removeDotSlash(tmpBuf, manager);
         setSystemId(tmpBuf);
-        getMemoryManager()->deallocate(tmpBuf);//delete [] tmpBuf;
+        manager->deallocate(tmpBuf);//delete [] tmpBuf;
     }
 
 }
@@ -198,7 +198,7 @@ LocalFileInputSource::~LocalFileInputSource()
 // ---------------------------------------------------------------------------
 BinInputStream* LocalFileInputSource::makeStream() const
 {
-    BinFileInputStream* retStrm = new (getMemoryManager()) BinFileInputStream(getSystemId());
+    BinFileInputStream* retStrm = new (getMemoryManager()) BinFileInputStream(getSystemId(), getMemoryManager());
     if (!retStrm->getIsOpen())
     {
         delete retStrm;
