@@ -55,8 +55,11 @@
  */
 
 /*
- * $Id: MemBufFormatTarget.cpp,v 1.7 2003/05/16 21:36:55 knoaman Exp $
+ * $Id: MemBufFormatTarget.cpp,v 1.8 2004/01/29 11:46:29 cargilld Exp $
  * $Log: MemBufFormatTarget.cpp,v $
+ * Revision 1.8  2004/01/29 11:46:29  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
  * Revision 1.7  2003/05/16 21:36:55  knoaman
  * Memory manager implementation: Modify constructors to pass in the memory manager.
  *
@@ -111,7 +114,7 @@ MemBufFormatTarget::~MemBufFormatTarget()
 
 void MemBufFormatTarget::writeChars(const XMLByte* const toWrite
                                   , const unsigned int   count
-                                  , XMLFormatter * const formatter)
+                                  , XMLFormatter * const)
 {
 
     if (count) {
