@@ -55,7 +55,7 @@
  */
 
 /**
- * $Id: XMLFormatter.cpp,v 1.12 2003/05/16 21:36:55 knoaman Exp $
+ * $Id: XMLFormatter.cpp,v 1.18 2003/12/17 00:18:33 cargilld Exp $
  */
 
 
@@ -120,10 +120,9 @@ static const XMLCh gEscapeChars[XMLFormatter::EscapeFlags_Count][kEscapeCount] =
         { chNull      , chNull       , chNull        , chNull       , chNull        , chNull }
     ,   { chAmpersand , chCloseAngle , chDoubleQuote , chOpenAngle  , chSingleQuote , chNull }
     ,   { chAmpersand , chOpenAngle  , chDoubleQuote , chNull       , chNull        , chNull }
-    ,   { chAmpersand , chOpenAngle  , chNull        , chNull       , chNull        , chNull }
+    ,   { chAmpersand , chOpenAngle  , chCloseAngle  , chNull       , chNull        , chNull }
 };
 
-
 // ---------------------------------------------------------------------------
 //  Local methods
 // ---------------------------------------------------------------------------
@@ -218,11 +217,12 @@ XMLFormatter::XMLFormatter( const   char* const             outEncoding
     if (!fXCoder)
     {
         fMemoryManager->deallocate(fOutEncoding); //delete [] fOutEncoding;
-        ThrowXML1
+        ThrowXMLwithMemMgr1
         (
             TranscodingException
             , XMLExcepts::Trans_CantCreateCvtrFor
             , outEncoding
+            , fMemoryManager
         );
     }
 
@@ -268,11 +268,12 @@ XMLFormatter::XMLFormatter( const   XMLCh* const            outEncoding
 
     if (!fXCoder)
     {
-        ThrowXML1
+        ThrowXMLwithMemMgr1
         (
             TranscodingException
             , XMLExcepts::Trans_CantCreateCvtrFor
             , outEncoding
+            , fMemoryManager
         );
     }
 
@@ -283,6 +284,116 @@ XMLFormatter::XMLFormatter( const   XMLCh* const            outEncoding
     fIsXML11 = XMLString::equals(docVersion, XMLUni::fgVersion1_1);
 }
 
+XMLFormatter::XMLFormatter( const   char* const             outEncoding
+                            ,       XMLFormatTarget* const  target
+                            , const EscapeFlags             escapeFlags
+                            , const UnRepFlags              unrepFlags
+                            ,       MemoryManager* const    manager)
+    : fEscapeFlags(escapeFlags)
+    , fOutEncoding(0)
+    , fTarget(target)
+    , fUnRepFlags(unrepFlags)
+    , fXCoder(0)  
+    , fAposRef(0)
+    , fAposLen(0)
+    , fAmpRef(0)    
+    , fAmpLen(0)    
+    , fGTRef(0)
+    , fGTLen(0)
+    , fLTRef(0)
+    , fLTLen(0)
+    , fQuoteRef(0)
+    , fQuoteLen(0) 
+    , fIsXML11(false)
+    , fMemoryManager(manager)
+{
+    // this constructor uses "1.0" for the docVersion
+
+    // Transcode the encoding string
+    fOutEncoding = XMLString::transcode(outEncoding, fMemoryManager);
+
+    // Try to create a transcoder for this encoding
+    XMLTransService::Codes resCode;
+    fXCoder = XMLPlatformUtils::fgTransService->makeNewTranscoderFor
+    (
+        fOutEncoding
+        , resCode
+        , kTmpBufSize
+        , fMemoryManager
+    );
+
+    if (!fXCoder)
+    {
+        fMemoryManager->deallocate(fOutEncoding); //delete [] fOutEncoding;
+        ThrowXMLwithMemMgr1
+        (
+            TranscodingException
+            , XMLExcepts::Trans_CantCreateCvtrFor
+            , outEncoding
+            , fMemoryManager
+        );
+    }
+
+    //XMLCh* const tmpDocVer = XMLString::transcode("1.0", fMemoryManager);
+    //ArrayJanitor<XMLCh> jname(tmpDocVer, fMemoryManager);
+    //fIsXML11 = XMLString::equals(tmpDocVer, XMLUni::fgVersion1_1);
+    fIsXML11 = false;  // docVersion 1.0 is not 1.1!
+}
+
+
+XMLFormatter::XMLFormatter( const   XMLCh* const            outEncoding
+                            ,       XMLFormatTarget* const  target
+                            , const EscapeFlags             escapeFlags
+                            , const UnRepFlags              unrepFlags
+                            ,       MemoryManager* const    manager)
+    : fEscapeFlags(escapeFlags)
+    , fOutEncoding(0)
+    , fTarget(target)
+    , fUnRepFlags(unrepFlags)
+    , fXCoder(0)  
+    , fAposRef(0)
+    , fAposLen(0)
+    , fAmpRef(0)    
+    , fAmpLen(0)    
+    , fGTRef(0)
+    , fGTLen(0)
+    , fLTRef(0)
+    , fLTLen(0)
+    , fQuoteRef(0)
+    , fQuoteLen(0) 
+    , fIsXML11(false)
+    , fMemoryManager(manager)
+{
+    // this constructor uses XMLUni::fgVersion1_0 for the docVersion
+
+    // Try to create a transcoder for this encoding
+    XMLTransService::Codes resCode;
+    fXCoder = XMLPlatformUtils::fgTransService->makeNewTranscoderFor
+    (
+        outEncoding
+        , resCode
+        , kTmpBufSize
+        , fMemoryManager
+    );
+
+    if (!fXCoder)
+    {
+        ThrowXMLwithMemMgr1
+        (
+            TranscodingException
+            , XMLExcepts::Trans_CantCreateCvtrFor
+            , outEncoding
+            , fMemoryManager
+        );
+    }
+
+    // Copy the encoding string
+    fOutEncoding = XMLString::replicate(outEncoding, fMemoryManager);
+
+    //fIsXML11 = XMLString::equals(docVersion, XMLUni::fgVersion1_1);
+    fIsXML11 = false;  // docVersion 1.0 is not 1.1!
+}
+
 XMLFormatter::~XMLFormatter()
 {
     fMemoryManager->deallocate(fAposRef); //delete [] fAposRef;
@@ -358,7 +469,7 @@ XMLFormatter::formatBuf(const   XMLCh* const    toFormat
             //  current source pointer and here all at once.
             //
             const XMLCh* tmpPtr = srcPtr;
-            while (!inEscapeList(actualEsc, *tmpPtr) && (tmpPtr < endPtr))
+            while ((tmpPtr < endPtr) && !inEscapeList(actualEsc, *tmpPtr))
                 tmpPtr++;
 
             //
@@ -488,7 +599,7 @@ void XMLFormatter::writeBOM(const XMLByte* const toFormat
 // ---------------------------------------------------------------------------
 //  XMLFormatter: Private helper methods
 // ---------------------------------------------------------------------------
-const void XMLFormatter::writeCharRef(const XMLCh &toWrite)
+void XMLFormatter::writeCharRef(const XMLCh &toWrite)
 {
     XMLCh tmpBuf[32];
     tmpBuf[0] = chAmpersand;
@@ -496,7 +607,7 @@ const void XMLFormatter::writeCharRef(const XMLCh &toWrite)
     tmpBuf[2] = chLatin_x;
 
     // Build a char ref for the current char
-    XMLString::binToText(toWrite, &tmpBuf[3], 8, 16);
+    XMLString::binToText(toWrite, &tmpBuf[3], 8, 16, fMemoryManager);
     const unsigned int bufLen = XMLString::stringLen(tmpBuf);
     tmpBuf[bufLen] = chSemiColon;
     tmpBuf[bufLen+1] = chNull;
@@ -509,6 +620,28 @@ const void XMLFormatter::writeCharRef(const XMLCh &toWrite)
 
 }
 
+void XMLFormatter::writeCharRef(unsigned long toWrite)
+{
+    XMLCh tmpBuf[32];
+    tmpBuf[0] = chAmpersand;
+    tmpBuf[1] = chPound;
+    tmpBuf[2] = chLatin_x;
+
+    // Build a char ref for the current char
+    XMLString::binToText(toWrite, &tmpBuf[3], 8, 16, fMemoryManager);
+    const unsigned int bufLen = XMLString::stringLen(tmpBuf);
+    tmpBuf[bufLen] = chSemiColon;
+    tmpBuf[bufLen+1] = chNull;
+
+    // write it out
+    formatBuf(tmpBuf
+            , bufLen + 1
+            , XMLFormatter::NoEscapes
+            , XMLFormatter::UnRep_Fail);
+
+}
+
+
 const XMLByte* XMLFormatter::getCharRef(unsigned int & count, 
                                         XMLByte*       &ref, 
                                         const XMLCh *  stdRef) 
@@ -586,7 +719,18 @@ void XMLFormatter::specialFormat(const  XMLCh* const    toFormat
             //
             while (srcPtr < endPtr)
             {
-                writeCharRef(*srcPtr);
+                if ((*srcPtr & 0xFC00) == 0xD800) {
+                    // we have encountered a surrogate, need to recombine before printing out					  
+                    // use writeCharRef that takes unsigned long to get values larger than
+                    // hex 0xFFFF printed.
+                    tmpPtr = srcPtr;
+                    tmpPtr++; // point at low surrogate
+                    writeCharRef((unsigned long) (0x10000+((*srcPtr-0xD800)<<10)+*tmpPtr-0xDC00));
+                    srcPtr++; // advance to low surrogate (will advance again below)
+                }
+                else {
+                    writeCharRef(*srcPtr);
+                }
 
                 // Move up the source pointer and break out if needed
                 srcPtr++;
