@@ -56,6 +56,15 @@
 
 /*
  * $Log: ElemStack.cpp,v $
+ * Revision 1.10  2003/12/17 00:18:34  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.9  2003/10/23 14:11:07  knoaman
+ * Fix memory leak.
+ *
+ * Revision 1.8  2003/10/22 20:22:30  knoaman
+ * Prepare for annotation support.
+ *
  * Revision 1.7  2003/05/18 14:02:04  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -165,6 +174,7 @@ ElemStack::ElemStack(MemoryManager* const manager) :
     , fXMLPoolId(0)
     , fXMLNSNamespaceId(0)
     , fXMLNSPoolId(0)
+    , fNamespaceMap(0)
     , fMemoryManager(manager)
 {
     // Do an initial allocation of the stack and zero it out
@@ -173,6 +183,8 @@ ElemStack::ElemStack(MemoryManager* const manager) :
         fStackCapacity * sizeof(StackElem*)
     );//new StackElem*[fStackCapacity];
     memset(fStack, 0, fStackCapacity * sizeof(StackElem*));
+
+    fNamespaceMap = new (fMemoryManager) ValueVectorOf<PrefMapElem*>(16, fMemoryManager);
 }
 
 ElemStack::~ElemStack()
@@ -194,6 +206,7 @@ ElemStack::~ElemStack()
 
     // Delete the stack array itself now
     fMemoryManager->deallocate(fStack);//delete [] fStack;
+    delete fNamespaceMap;
 }
 
 
@@ -272,7 +285,7 @@ const ElemStack::StackElem* ElemStack::popTop()
 {
     // Watch for an underflow error
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_StackUnderflow);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_StackUnderflow, fMemoryManager);
 
     fStackTop--;
     return fStack[fStackTop];
@@ -283,7 +296,7 @@ void
 ElemStack::setElement(XMLElementDecl* const toSet, const unsigned int readerNum)
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     fStack[fStackTop - 1]->fThisElement = toSet;
     fStack[fStackTop - 1]->fReaderNum = readerNum;
@@ -296,14 +309,14 @@ ElemStack::setElement(XMLElementDecl* const toSet, const unsigned int readerNum)
 unsigned int ElemStack::addChild(QName* const child, const bool toParent)
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     //
     //  If they want to add to the parent, then we have to have at least two
     //  elements on the stack.
     //
     if (toParent && (fStackTop < 2))
-        ThrowXML(NoSuchElementException, XMLExcepts::ElemStack_NoParentPushed);
+        ThrowXMLwithMemMgr(NoSuchElementException, XMLExcepts::ElemStack_NoParentPushed, fMemoryManager);
 
     // Get a convenience pointer to the stack top row
     StackElem* curRow = toParent
@@ -353,7 +366,7 @@ unsigned int ElemStack::addChild(QName* const child, const bool toParent)
 const ElemStack::StackElem* ElemStack::topElement() const
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     return fStack[fStackTop - 1];
 }
@@ -366,7 +379,7 @@ void ElemStack::addPrefix(  const   XMLCh* const    prefixToAdd
                             , const unsigned int    uriId)
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     // Get a convenience pointer to the stack top row
     StackElem* curRow = fStack[fStackTop - 1];
@@ -467,6 +480,32 @@ unsigned int ElemStack::mapPrefixToURI( const   XMLCh* const    prefixToMap
 }
 
 
+ValueVectorOf<PrefMapElem*>* ElemStack::getNamespaceMap() const
+{
+    fNamespaceMap->removeAllElements();
+
+    //  Start at the stack top and work backwards until we come to some
+    //  element that mapped this prefix.
+    int startAt = (int)(fStackTop - 1);
+    for (int index = startAt; index >= 0; index--)
+    {
+        // Get a convenience pointer to the current element
+        StackElem* curRow = fStack[index];
+
+        // If no prefixes mapped at this level, then go the next one
+        if (!curRow->fMapCount)
+            continue;
+
+        // Search the map at this level for the passed prefix
+        for (unsigned int mapIndex = 0; mapIndex < curRow->fMapCount; mapIndex++)
+        {
+            fNamespaceMap->addElement(&(curRow->fMap[mapIndex]));
+        }
+    }
+
+    return fNamespaceMap;
+}
+
 // ---------------------------------------------------------------------------
 //  ElemStack: Miscellaneous methods
 // ---------------------------------------------------------------------------
@@ -693,7 +732,7 @@ const WFElemStack::StackElem* WFElemStack::popTop()
 {
     // Watch for an underflow error
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_StackUnderflow);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_StackUnderflow, fMemoryManager);
 
     fStackTop--;
     return fStack[fStackTop];
@@ -706,7 +745,7 @@ WFElemStack::setElement(const XMLCh* const toSet,
                       const unsigned int readerNum)
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     if (toSetLen > fStack[fStackTop - 1]->fElemMaxLength) {
 
@@ -729,7 +768,7 @@ WFElemStack::setElement(const XMLCh* const toSet,
 const WFElemStack::StackElem* WFElemStack::topElement() const
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     return fStack[fStackTop - 1];
 }
@@ -742,7 +781,7 @@ void WFElemStack::addPrefix(  const   XMLCh* const    prefixToAdd
                               , const unsigned int    uriId)
 {
     if (!fStackTop)
-        ThrowXML(EmptyStackException, XMLExcepts::ElemStack_EmptyStack);
+        ThrowXMLwithMemMgr(EmptyStackException, XMLExcepts::ElemStack_EmptyStack, fMemoryManager);
 
     // Get a convenience pointer to the stack top row
     StackElem* curRow = fStack[fStackTop - 1];
