@@ -56,6 +56,12 @@
 
 /*
  * $Log: ReaderMgr.hpp,v $
+ * Revision 1.11  2004/01/29 11:46:30  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.10  2003/08/14 02:56:41  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.9  2003/05/16 21:36:57  knoaman
  * Memory manager implementation: Modify constructors to pass in the memory manager.
  *
@@ -316,6 +322,11 @@ private :
     const XMLReader* getLastExtEntity(const XMLEntityDecl*& itsEntity) const;
     bool popReader();
 
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    ReaderMgr(const ReaderMgr&);
+    ReaderMgr& operator=(const ReaderMgr&);
 
     // -----------------------------------------------------------------------
     //  Private data members
@@ -387,6 +398,16 @@ inline unsigned int ReaderMgr::getCurrentReaderNum() const
     return fCurReader->getReaderNum();
 }
 
+inline const XMLReader* ReaderMgr::getCurrentReader() const
+{
+    return fCurReader;
+}
+
+inline XMLReader* ReaderMgr::getCurrentReader()
+{
+    return fCurReader;
+}
+
 inline bool ReaderMgr::getName(XMLBuffer& toFill)
 {
     toFill.reset();
@@ -511,6 +532,12 @@ public :
     };
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------    
+    ThrowEOEJanitor(const ThrowEOEJanitor&);
+    ThrowEOEJanitor& operator=(const ThrowEOEJanitor&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
