@@ -55,7 +55,7 @@
  */
 
 /*
-  * $Id: WFXMLScanner.cpp,v 1.17 2003/11/01 20:24:22 amassari Exp $
+  * $Id: WFXMLScanner.cpp,v 1.20 2004/02/11 12:42:50 cargilld Exp $
  */
 
 
@@ -188,7 +188,7 @@ void WFXMLScanner::scanDocument(const InputSource& src)
         else
         {
             // Scan content, and tell it its not an external entity
-            if (scanContent(false))
+            if (scanContent())
             {
                 // That went ok, so scan for any miscellaneous stuff
                 if (!fReaderMgr.atEOF())
@@ -278,7 +278,7 @@ bool WFXMLScanner::scanNext(XMLPScanToken& token)
 {
     // Make sure this token is still legal
     if (!isLegalToken(token))
-        ThrowXML(RuntimeException, XMLExcepts::Scan_BadPScanToken);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Scan_BadPScanToken, fMemoryManager);
 
     // Find the next token and remember the reader id
     unsigned int orgReader;
@@ -561,9 +561,9 @@ void WFXMLScanner::scanReset(const InputSource& src)
 
     if (!newReader) {
         if (src.getIssueFatalErrorIfNotFound())
-            ThrowXML1(RuntimeException, XMLExcepts::Scan_CouldNotOpenSource, src.getSystemId());
+            ThrowXMLwithMemMgr1(RuntimeException, XMLExcepts::Scan_CouldNotOpenSource, src.getSystemId(), fMemoryManager);
         else
-            ThrowXML1(RuntimeException, XMLExcepts::Scan_CouldNotOpenSource_Warning, src.getSystemId());
+            ThrowXMLwithMemMgr1(RuntimeException, XMLExcepts::Scan_CouldNotOpenSource_Warning, src.getSystemId(), fMemoryManager);
     }
 
     // Push this read onto the reader manager
@@ -604,7 +604,7 @@ void WFXMLScanner::sendCharData(XMLBuffer& toSend)
 
 //  This method will kick off the scanning of the primary content of the
 //  document, i.e. the elements.
-bool WFXMLScanner::scanContent(const bool extEntity)
+bool WFXMLScanner::scanContent()
 {
     //  Go into a loop until we hit the end of the root element, or we fall
     //  out because there is no root element.
@@ -734,7 +734,7 @@ void WFXMLScanner::scanEndTag(bool& gotData)
     {
         emitError(XMLErrs::MoreEndThanStartTags);
         fReaderMgr.skipPastChar(chCloseAngle);
-        ThrowXML(RuntimeException, XMLExcepts::Scan_UnbalancedStartEnd);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Scan_UnbalancedStartEnd, fMemoryManager);
     }
 
     //  Pop the stack of the element we are supposed to be ending. Remember
@@ -943,7 +943,7 @@ bool WFXMLScanner::scanStartTag(bool& gotData)
 
             //  See if this attribute is declared more than one for this element.
             const XMLCh* attNameRawBuf = fAttNameBuf.getRawBuffer();
-            unsigned int attNameHash = XMLString::hash(attNameRawBuf, 109);
+            unsigned int attNameHash = XMLString::hash(attNameRawBuf, 109, fMemoryManager);
 
             if (attCount) {
 
@@ -1049,7 +1049,7 @@ bool WFXMLScanner::scanStartTag(bool& gotData)
         //  It was some special case character so do all of the checks and
         //  deal with it.
         if (!nextCh)
-            ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+            ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
 
         if (nextCh == chForwardSlash)
         {
@@ -1270,7 +1270,7 @@ bool WFXMLScanner::scanStartTagNS(bool& gotData)
 
             //  See if this attribute is declared more than one for this element.
             const XMLCh* attNameRawBuf = fAttNameBuf.getRawBuffer();
-            unsigned int attNameHash = XMLString::hash(attNameRawBuf, 109);
+            unsigned int attNameHash = XMLString::hash(attNameRawBuf, 109, fMemoryManager);
             if (attCount) {
 
                 for (unsigned int k=0; k < attCount; k++) {
@@ -1433,23 +1433,6 @@ bool WFXMLScanner::scanStartTagNS(bool& gotData)
                 }
             }
 
-            // check for duplicate namespace attributes:
-            // by checking for qualified names with the same local part and with prefixes 
-            // which have been bound to namespace names that are identical. 
-            XMLAttr* loopAttr;
-            for (unsigned int attrIndex=0; attrIndex < attCount; attrIndex++) {
-                loopAttr = fAttrList->elementAt(attrIndex);
-                if (curAtt->getURIId() == loopAttr->getURIId() &&
-                    XMLString::equals(curAtt->getName(), loopAttr->getName())) {
-                    emitError
-                    ( 
-                        XMLErrs::AttrAlreadyUsedInSTag
-                        , curAtt->getName()
-                        , elemDecl->getFullName()
-                    );
-                }
-            }  
-
             // increment attribute count
             attCount++;
 
@@ -1460,7 +1443,7 @@ bool WFXMLScanner::scanStartTagNS(bool& gotData)
         //  It was some special case character so do all of the checks and
         //  deal with it.
         if (!nextCh)
-            ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+            ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
 
         if (nextCh == chForwardSlash)
         {
@@ -1513,6 +1496,27 @@ bool WFXMLScanner::scanStartTagNS(bool& gotData)
         );
     }
 
+    // check for duplicate namespace attributes:
+    // by checking for qualified names with the same local part and with prefixes 
+    // which have been bound to namespace names that are identical. 
+    XMLAttr* loopAttr;
+    XMLAttr* curAtt;
+    for (unsigned int attrIndex=0; attrIndex < attCount-1; attrIndex++) {
+        loopAttr = fAttrList->elementAt(attrIndex);
+        for (unsigned int curAttrIndex = attrIndex+1; curAttrIndex < attCount; curAttrIndex++) {
+            curAtt = fAttrList->elementAt(curAttrIndex);
+            if (curAtt->getURIId() == loopAttr->getURIId() &&
+                XMLString::equals(curAtt->getName(), loopAttr->getName())) {
+                emitError
+                ( 
+                    XMLErrs::AttrAlreadyUsedInSTag
+                        , curAtt->getName()
+                        , elemDecl->getFullName()
+                );
+            }
+        }
+    }  
+
     // Resolve the qualified name to a URI.
     unsigned int uriId = resolvePrefix
     (
@@ -1656,7 +1660,7 @@ bool WFXMLScanner::scanAttValue(const XMLCh* const attrName
             nextCh = fReaderMgr.getNextChar();
 
             if (!nextCh)
-                ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+                ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
 
             // Check for our ending quote in the same entity
             if (nextCh == quoteCh)
@@ -1725,6 +1729,7 @@ bool WFXMLScanner::scanAttValue(const XMLCh* const attrName
                             , tmpBuf
                             , 8
                             , 16
+                            , fMemoryManager
                         );
                         emitError(XMLErrs::InvalidCharacterInAttrValue, attrName, tmpBuf);
                     }
@@ -1799,7 +1804,7 @@ void WFXMLScanner::scanCDSection()
         if (!nextCh)
         {
             emitError(XMLErrs::UnterminatedCDATASection);
-            ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+            ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
         }
 
         //  If this is a close square bracket it could be our closing
@@ -1869,6 +1874,7 @@ void WFXMLScanner::scanCDSection()
                             , tmpBuf
                             , 8
                             , 16
+                            , fMemoryManager
                         );
                         emitError(XMLErrs::InvalidCharacter, tmpBuf);
                         emittedError = true;
@@ -1997,6 +2003,7 @@ void WFXMLScanner::scanCharData(XMLBuffer& toUse)
                                 , tmpBuf
                                 , 8
                                 , 16
+                                , fMemoryManager
                             );
                             emitError(XMLErrs::InvalidCharacter, tmpBuf);
                         }
@@ -2066,7 +2073,7 @@ void WFXMLScanner::scanCharData(XMLBuffer& toUse)
 //  only makes any difference if the return value indicates the value was
 //  returned directly.
 XMLScanner::EntityExpRes
-WFXMLScanner::scanEntityRef(const bool    inAttVal
+WFXMLScanner::scanEntityRef(const bool
                             ,     XMLCh&  firstCh
                             ,     XMLCh&  secondCh
                             ,     bool&   escaped)
@@ -2131,7 +2138,7 @@ WFXMLScanner::scanEntityRef(const bool    inAttVal
     // how many entity references we've had
     if(fSecurityManager != 0 && ++fEntityExpansionCount > fEntityExpansionLimit) {
         XMLCh expLimStr[16];
-        XMLString::binToText(fEntityExpansionLimit, expLimStr, 15, 10);
+        XMLString::binToText(fEntityExpansionLimit, expLimStr, 15, 10, fMemoryManager);
         emitError
         ( 
             XMLErrs::EntityExpansionLimitExceeded
