@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLReader.cpp,v 1.16 2003/07/02 15:17:23 knoaman Exp $
+ * $Id: XMLReader.cpp,v 1.19 2004/01/29 11:46:30 cargilld Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -70,6 +70,7 @@
 #include <xercesc/util/TransService.hpp>
 #include <xercesc/util/XMLEBCDICTranscoder.hpp>
 #include <xercesc/util/XMLString.hpp>
+#include <xercesc/util/Janitor.hpp>
 
 
 XERCES_CPP_NAMESPACE_BEGIN
@@ -151,9 +152,9 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
     , fSource(source)
     , fSrcOfsBase(0)
     , fSrcOfsSupported(false)
-    , fCalculateSrcOfs(calculateSrcOfs)
-    , fStream(streamToAdopt)
+    , fCalculateSrcOfs(calculateSrcOfs)    
     , fSystemId(XMLString::replicate(sysId, manager))
+    , fStream(streamToAdopt)
     , fSwapped(false)
     , fThrowAtEnd(throwAtEnd)
     , fTranscoder(0)
@@ -179,11 +180,11 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
     if ((fEncoding < XMLRecognizer::Encodings_Min)
     ||  (fEncoding > XMLRecognizer::Encodings_Max))
     {
-        ThrowXML(RuntimeException, XMLExcepts::Reader_BadAutoEncoding);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Reader_BadAutoEncoding, fMemoryManager);
     }
     #endif
 
-    fEncodingStr = XMLString::replicate(XMLRecognizer::nameForEncoding(fEncoding), fMemoryManager);
+    fEncodingStr = XMLString::replicate(XMLRecognizer::nameForEncoding(fEncoding, fMemoryManager), fMemoryManager);
 
     // Check whether the fSwapped flag should be set or not
     checkForSwapped();
@@ -232,9 +233,9 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
     , fSource(source)
     , fSrcOfsBase(0)
     , fSrcOfsSupported(false)
-    , fCalculateSrcOfs(calculateSrcOfs)
-    , fStream(streamToAdopt)
+    , fCalculateSrcOfs(calculateSrcOfs)    
     , fSystemId(XMLString::replicate(sysId, manager))
+    , fStream(streamToAdopt)
     , fSwapped(false)
     , fThrowAtEnd(throwAtEnd)
     , fTranscoder(0)
@@ -299,11 +300,12 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
 
     if (!fTranscoder)
     {
-        ThrowXML1
+        ThrowXMLwithMemMgr1
         (
             TranscodingException
             , XMLExcepts::Trans_CantCreateCvtrFor
             , fEncodingStr
+            , fMemoryManager
         );
     }
 
@@ -352,9 +354,9 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
     , fSource(source)
     , fSrcOfsBase(0)
     , fSrcOfsSupported(false)
-    , fCalculateSrcOfs(calculateSrcOfs)
-    , fStream(streamToAdopt)
+    , fCalculateSrcOfs(calculateSrcOfs)    
     , fSystemId(XMLString::replicate(sysId, manager))
+    , fStream(streamToAdopt)
     , fSwapped(false)
     , fThrowAtEnd(throwAtEnd)
     , fTranscoder(0)
@@ -373,7 +375,7 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
     //  Use the passed encoding code
     //
     fEncoding = encodingEnum;
-    fEncodingStr = XMLString::replicate(XMLRecognizer::nameForEncoding(fEncoding), fMemoryManager);
+    fEncodingStr = XMLString::replicate(XMLRecognizer::nameForEncoding(fEncoding, fMemoryManager), fMemoryManager);
 
     // Check whether the fSwapped flag should be set or not
     checkForSwapped();
@@ -393,11 +395,12 @@ XMLReader::XMLReader(const  XMLCh* const          pubId
 
     if (!fTranscoder)
     {
-        ThrowXML1
+        ThrowXMLwithMemMgr1
         (
             TranscodingException
             , XMLExcepts::Trans_CantCreateCvtrFor
             , fEncodingStr
+            , fMemoryManager
         );
     }
 
@@ -434,7 +437,7 @@ XMLReader::~XMLReader()
 unsigned int XMLReader::getSrcOffset() const
 {
     if (!fSrcOfsSupported || !fCalculateSrcOfs)
-        ThrowXML(RuntimeException, XMLExcepts::Reader_SrcOfsNotSupported);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Reader_SrcOfsNotSupported, fMemoryManager);
 
     //
     //  Take the current source offset and add in the sizes that we've
@@ -475,7 +478,7 @@ bool XMLReader::refreshCharBuffer()
     if (!fTranscoder)
     {
         if (fEncoding == XMLRecognizer::EBCDIC)
-            ThrowXML(RuntimeException, XMLExcepts::Reader_EncodingStrRequired);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Reader_EncodingStrRequired, fMemoryManager);
 
         // Ask the transcoding service to make use a transcoder
         XMLTransService::Codes failReason;
@@ -489,11 +492,12 @@ bool XMLReader::refreshCharBuffer()
 
         if (!fTranscoder)
         {
-            ThrowXML1
+            ThrowXMLwithMemMgr1
             (
                 TranscodingException
                 , XMLExcepts::Trans_CantCreateCvtrFor
                 , fEncodingStr
+                , fMemoryManager
             );
         }
     }
@@ -1230,7 +1234,7 @@ bool XMLReader::setEncoding(const XMLCh* const newEncoding)
         );
 
         if (!fTranscoder)
-            ThrowXML1(TranscodingException, XMLExcepts::Trans_CantCreateCvtrFor, fEncodingStr);
+            ThrowXMLwithMemMgr1(TranscodingException, XMLExcepts::Trans_CantCreateCvtrFor, fEncodingStr, fMemoryManager);
     }
 
     // Update the base encoding member with the new base encoding found
@@ -1318,11 +1322,15 @@ void XMLReader::doInitDecode()
                 {
                     fCharsAvail = 0;
                     fRawBufIndex = 0;
-                    ThrowXML1
+                    fMemoryManager->deallocate(fPublicId);
+                    fMemoryManager->deallocate(fEncodingStr);
+                    ArrayJanitor<XMLCh> janValue(fSystemId, fMemoryManager);
+                    ThrowXMLwithMemMgr1
                     (
                         TranscodingException
                         , XMLExcepts::Reader_CouldNotDecodeFirstLine
                         , fSystemId
+                        , fMemoryManager
                     );
                 }
 
@@ -1393,11 +1401,15 @@ void XMLReader::doInitDecode()
                 {
                     fCharsAvail = 0;
                     fRawBufIndex = 0;
-                    ThrowXML1
+                    fMemoryManager->deallocate(fPublicId);
+                    fMemoryManager->deallocate(fEncodingStr);
+                    ArrayJanitor<XMLCh> janValue(fSystemId, fMemoryManager);
+                    ThrowXMLwithMemMgr1
                     (
                         TranscodingException
                         , XMLExcepts::Reader_CouldNotDecodeFirstLine
                         , fSystemId
+                        , fMemoryManager
                     );
                 }
             }
@@ -1509,7 +1521,10 @@ void XMLReader::doInitDecode()
 
         default :
             // It should never be anything else here
-            ThrowXML(TranscodingException, XMLExcepts::Reader_BadAutoEncoding);
+            fMemoryManager->deallocate(fPublicId);
+            fMemoryManager->deallocate(fEncodingStr);                    
+            fMemoryManager->deallocate(fSystemId);
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Reader_BadAutoEncoding, fMemoryManager);
             break;
     }
 
