@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLScanner.cpp,v 1.45 2003/05/18 14:02:04 knoaman Exp $
+ * $Id: XMLScanner.cpp,v 1.62 2004/01/29 11:46:30 cargilld Exp $
  */
 
 
@@ -63,6 +63,7 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/internal/XMLScanner.hpp>
+#include <xercesc/internal/ValidationContextImpl.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/Mutexes.hpp>
 #include <xercesc/util/RuntimeException.hpp>
@@ -78,6 +79,8 @@
 #include <xercesc/internal/EndOfEntityException.hpp>
 #include <xercesc/validators/DTD/DocTypeHandler.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/util/XMLResourceIdentifier.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -119,23 +122,14 @@ void XMLScanner::reinitScannerMutex()
 //
 static XMLMutex& gScannerMutex()
 {
-
-    if (!sScannerMutex)
+    if (!sRegistered)
     {
-        XMLMutex* tmpMutex = new XMLMutex;
-        if (XMLPlatformUtils::compareAndSwap((void**)&sScannerMutex, tmpMutex, 0))
-        {
-            // Someone beat us to it, so let's clean up ours
-            delete tmpMutex;
-        }
+        XMLMutexLock lockInit(XMLPlatformUtils::fgAtomicMutex);
 
-        // Now lock it and try to register it
-        XMLMutexLock lock(sScannerMutex);
-
-        // If we got here first, then register it and set the registered flag
         if (!sRegistered)
         {
-			scannerMutexCleanup.registerCleanup(XMLScanner::reinitScannerMutex);
+            sScannerMutex = new XMLMutex;
+            scannerMutexCleanup.registerCleanup(XMLScanner::reinitScannerMutex);
             sRegistered = true;
         }
     }
@@ -144,17 +138,20 @@ static XMLMutex& gScannerMutex()
 
 static XMLMsgLoader& gScannerMsgLoader()
 {
-    XMLMutexLock lockInit(&gScannerMutex());
-
-    // If we haven't loaded our message yet, then do that
     if (!gMsgLoader)
     {
-        gMsgLoader = XMLPlatformUtils::loadMsgSet(XMLUni::fgXMLErrDomain);
-        if (!gMsgLoader)
-            XMLPlatformUtils::panic(PanicHandler::Panic_CantLoadMsgDomain);
+        XMLMutexLock lockInit(&gScannerMutex());
 
-        // Register this object to be cleaned up at termination
-        cleanupMsgLoader.registerCleanup(XMLScanner::reinitMsgLoader);
+        // If we haven't loaded our message yet, then do that
+        if (!gMsgLoader)
+        {
+            gMsgLoader = XMLPlatformUtils::loadMsgSet(XMLUni::fgXMLErrDomain);
+            if (!gMsgLoader)
+                XMLPlatformUtils::panic(PanicHandler::Panic_CantLoadMsgDomain);
+
+            // Register this object to be cleaned up at termination
+            cleanupMsgLoader.registerCleanup(XMLScanner::reinitMsgLoader);
+        }
     }
 
     return *gMsgLoader;
@@ -164,9 +161,10 @@ static XMLMsgLoader& gScannerMsgLoader()
 //  XMLScanner: Constructors and Destructor
 // ---------------------------------------------------------------------------
 XMLScanner::XMLScanner(XMLValidator* const valToAdopt,
-                       MemoryManager* const manager) :
-
-    fCalculateSrcOfs(false)
+                       GrammarResolver* const grammarResolver,
+                       MemoryManager* const manager)
+    : fStandardUriConformant(false)
+    , fCalculateSrcOfs(false)
     , fDoNamespaces(false)
     , fExitOnFirstFatal(true)
     , fValidationConstraintFatal(false)
@@ -182,11 +180,17 @@ XMLScanner::XMLScanner(XMLValidator* const valToAdopt,
     , fLoadExternalDTD(true)
     , fNormalizeData(true)
     , fErrorCount(0)
+    , fEntityExpansionLimit(0)
+    , fEntityExpansionCount(0)
     , fEmptyNamespaceId(0)
     , fUnknownNamespaceId(0)
     , fXMLNamespaceId(0)
     , fXMLNSNamespaceId(0)
     , fSchemaNamespaceId(0)
+    , fUIntPool(0)
+    , fUIntPoolRow(0)
+    , fUIntPoolCol(0)
+    , fUIntPoolRowTotal(2)
     , fScannerId(0)
     , fSequenceId(0)
     , fAttrList(0)
@@ -195,18 +199,20 @@ XMLScanner::XMLScanner(XMLValidator* const valToAdopt,
     , fEntityHandler(0)
     , fErrorReporter(0)
     , fErrorHandler(0)
-    , fIDRefList(0)
+    , fPSVIHandler(0)
+    , fValidationContext(0)
+    , fEntityDeclPoolRetrieved(false)
     , fReaderMgr(manager)
     , fValidator(valToAdopt)
     , fValScheme(Val_Never)
-    , fGrammarResolver(0)
+    , fGrammarResolver(grammarResolver)
+    , fGrammarPoolMemoryManager(grammarResolver->getGrammarPoolMemoryManager())
     , fGrammar(0)
     , fRootGrammar(0)
     , fURIStringPool(0)
     , fRootElemName(0)
     , fExternalSchemaLocation(0)
-    , fExternalNoNamespaceSchemaLocation(0)
-    , fStandardUriConformant(false)
+    , fExternalNoNamespaceSchemaLocation(0)    
     , fSecurityManager(0)
     , fXMLVersion(XMLReader::XMLV1_0)
     , fMemoryManager(manager)
@@ -217,6 +223,7 @@ XMLScanner::XMLScanner(XMLValidator* const valToAdopt,
     , fQNameBuf(1023, manager)
     , fPrefixBuf(1023, manager)
     , fURIBuf(1023, manager)
+    , fElemStack(manager)   
 {
    commonInit();
 
@@ -231,9 +238,10 @@ XMLScanner::XMLScanner( XMLDocumentHandler* const  docHandler
                           , XMLEntityHandler* const  entityHandler
                           , XMLErrorReporter* const  errHandler
                           , XMLValidator* const      valToAdopt
-                          , MemoryManager* const     manager) :
-
-    fCalculateSrcOfs(false)
+                          , GrammarResolver* const   grammarResolver
+                          , MemoryManager* const     manager)
+    : fStandardUriConformant(false)
+    , fCalculateSrcOfs(false)
     , fDoNamespaces(false)
     , fExitOnFirstFatal(true)
     , fValidationConstraintFatal(false)
@@ -249,11 +257,17 @@ XMLScanner::XMLScanner( XMLDocumentHandler* const  docHandler
 	, fLoadExternalDTD(true)
     , fNormalizeData(true)
     , fErrorCount(0)
+    , fEntityExpansionLimit(0)
+    , fEntityExpansionCount(0)
     , fEmptyNamespaceId(0)
     , fUnknownNamespaceId(0)
     , fXMLNamespaceId(0)
     , fXMLNSNamespaceId(0)
     , fSchemaNamespaceId(0)
+    , fUIntPool(0)
+    , fUIntPoolRow(0)
+    , fUIntPoolCol(0)
+    , fUIntPoolRowTotal(2)
     , fScannerId(0)
     , fSequenceId(0)
     , fAttrList(0)
@@ -262,18 +276,20 @@ XMLScanner::XMLScanner( XMLDocumentHandler* const  docHandler
     , fEntityHandler(entityHandler)
     , fErrorReporter(errHandler)
     , fErrorHandler(0)
-    , fIDRefList(0)
+    , fPSVIHandler(0)
+    , fValidationContext(0)
+    , fEntityDeclPoolRetrieved(false)
     , fReaderMgr(manager)
     , fValidator(valToAdopt)
     , fValScheme(Val_Never)
-    , fGrammarResolver(0)
+    , fGrammarResolver(grammarResolver)
+    , fGrammarPoolMemoryManager(grammarResolver->getGrammarPoolMemoryManager())
     , fGrammar(0)
     , fRootGrammar(0)
     , fURIStringPool(0)
     , fRootElemName(0)
     , fExternalSchemaLocation(0)
-    , fExternalNoNamespaceSchemaLocation(0)
-    , fStandardUriConformant(false)
+    , fExternalNoNamespaceSchemaLocation(0)    
     , fSecurityManager(0)
     , fXMLVersion(XMLReader::XMLV1_0)
     , fMemoryManager(manager)
@@ -284,6 +300,7 @@ XMLScanner::XMLScanner( XMLDocumentHandler* const  docHandler
     , fQNameBuf(1023, manager)
     , fPrefixBuf(1023, manager)
     , fURIBuf(1023, manager)
+    , fElemStack(manager)
 {
    commonInit();
 
@@ -296,10 +313,16 @@ XMLScanner::XMLScanner( XMLDocumentHandler* const  docHandler
 XMLScanner::~XMLScanner()
 {
     delete fAttrList;
-    delete fIDRefList;
+    delete fValidationContext;
     fMemoryManager->deallocate(fRootElemName);//delete [] fRootElemName;
     fMemoryManager->deallocate(fExternalSchemaLocation);//delete [] fExternalSchemaLocation;
     fMemoryManager->deallocate(fExternalNoNamespaceSchemaLocation);//delete [] fExternalNoNamespaceSchemaLocation;
+    // delete fUIntPool
+    for (unsigned int i=0; i<=fUIntPoolRow; i++)
+    {
+        fMemoryManager->deallocate(fUIntPool[i]);
+    }
+    fMemoryManager->deallocate(fUIntPool);
 }
 
 
@@ -327,14 +350,52 @@ void XMLScanner::scanDocument(  const   XMLCh* const    systemId)
         //  Create a temporary URL. Since this is the primary document,
         //  it has to be fully qualified. If not, then assume we are just
         //  mistaking a file for a URL.
-        XMLURL tmpURL(systemId, fMemoryManager);
-        if (tmpURL.isRelative()) {
+        XMLURL tmpURL(fMemoryManager);
+
+        if (XMLURL::parse(systemId, tmpURL)) {
+
+            if (tmpURL.isRelative()) {
+                if (!fStandardUriConformant)
+                    srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
+                else {
+                    // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
+                    // emit the error directly
+                    MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent, fMemoryManager);
+                    fInException = true;
+                    emitError
+                    (
+                        XMLErrs::XMLException_Fatal
+                        , e.getType()
+                        , e.getMessage()
+                    );
+                    return;
+                }
+            }
+            else
+            {
+                if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
+                    MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL, fMemoryManager);
+                    fInException = true;
+                    emitError
+                    (
+                        XMLErrs::XMLException_Fatal
+                        , e.getType()
+                        , e.getMessage()
+                    );
+                    return;
+                }
+                srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
+            }
+        }
+        else {
+
             if (!fStandardUriConformant)
                 srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
             else {
-                // since this is the top of the try/catch, cannot call ThrowXML
+                // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
                 // emit the error directly
-                MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent);
+                // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
+                MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL, fMemoryManager);
                 fInException = true;
                 emitError
                 (
@@ -345,42 +406,7 @@ void XMLScanner::scanDocument(  const   XMLCh* const    systemId)
                 return;
             }
         }
-        else
-        {
-            if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
-                MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL);
-                fInException = true;
-                emitError
-                (
-                    XMLErrs::XMLException_Fatal
-                    , e.getType()
-                    , e.getMessage()
-                );
-                return;
-            }
-            srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
-        }
-
     }
-    catch(const MalformedURLException& e)
-    {
-        if (!fStandardUriConformant)
-            srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
-        else {
-            // since this is the top of the try/catch, cannot call ThrowXML
-            // emit the error directly
-            // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
-            fInException = true;
-            emitError
-            (
-                XMLErrs::XMLException_Fatal
-                , e.getType()
-                , e.getMessage()
-            );
-            return;
-        }
-    }
-
     catch(const XMLException& excToCatch)
     {
         //  For any other XMLException,
@@ -409,11 +435,6 @@ void XMLScanner::scanDocument(  const   XMLCh* const    systemId)
             );
         return;
     }
-    catch(...)
-    {
-        // Just rethrow this, since its not our problem
-        throw;
-    }
 
     Janitor<InputSource> janSrc(srcToUse);
     scanDocument(*srcToUse);
@@ -444,27 +465,48 @@ bool XMLScanner::scanFirst( const   XMLCh* const    systemId
         //  Create a temporary URL. Since this is the primary document,
         //  it has to be fully qualified. If not, then assume we are just
         //  mistaking a file for a URL.
-        XMLURL tmpURL(systemId, fMemoryManager);
-        if (tmpURL.isRelative()) {
-            if (!fStandardUriConformant)
-                srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
-            else {
-                // since this is the top of the try/catch, cannot call ThrowXML
-                // emit the error directly
-                MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent);
-                fInException = true;
-                emitError
-                (
-                    XMLErrs::XMLException_Fatal
-                    , e.getType()
-                    , e.getMessage()
-                );
-                return false;
+        XMLURL tmpURL(fMemoryManager);
+        if (XMLURL::parse(systemId, tmpURL)) {        
+            if (tmpURL.isRelative()) {
+                if (!fStandardUriConformant)
+                    srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
+                else {
+                    // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
+                    // emit the error directly
+                    MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent, fMemoryManager);
+                    fInException = true;
+                    emitError
+                    (
+                        XMLErrs::XMLException_Fatal
+                        , e.getType()
+                        , e.getMessage()
+                    );
+                    return false;
+                }
+            }
+            else
+            {
+                if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
+                    MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL, fMemoryManager);
+                    fInException = true;
+                    emitError
+                    (
+                        XMLErrs::XMLException_Fatal
+                        , e.getType()
+                        , e.getMessage()
+                    );
+                    return false;
+                }
+                srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
             }
         }
-        else
-        {
-            if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
+        else {
+            if (!fStandardUriConformant)
+                srcToUse = new (fMemoryManager) LocalFileInputSource(systemId,  fMemoryManager);
+            else {
+                // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
+                // emit the error directly
+                // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
                 MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL);
                 fInException = true;
                 emitError
@@ -475,25 +517,6 @@ bool XMLScanner::scanFirst( const   XMLCh* const    systemId
                 );
                 return false;
             }
-            srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
-        }
-    }
-    catch(const MalformedURLException& e)
-    {
-        if (!fStandardUriConformant)
-            srcToUse = new (fMemoryManager) LocalFileInputSource(systemId,  fMemoryManager);
-        else {
-            // since this is the top of the try/catch, cannot call ThrowXML
-            // emit the error directly
-            // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
-            fInException = true;
-            emitError
-            (
-                XMLErrs::XMLException_Fatal
-                , e.getType()
-                , e.getMessage()
-            );
-            return false;
         }
     }
     catch(const XMLException& excToCatch)
@@ -524,11 +547,6 @@ bool XMLScanner::scanFirst( const   XMLCh* const    systemId
             );
         return false;
     }
-    catch(...)
-    {
-        // Just rethrow this, since its not our problem
-        throw;
-    }
 
     Janitor<InputSource> janSrc(srcToUse);
     return scanFirst(*srcToUse, toFill);
@@ -619,6 +637,10 @@ bool XMLScanner::scanFirst( const   InputSource&    src
                     , excToCatch.getMessage()
                 );
         }
+        catch(const OutOfMemoryException&)
+        {
+            throw;
+        }
         catch(...)
         {
             // Reset and rethrow the user error
@@ -630,6 +652,10 @@ bool XMLScanner::scanFirst( const   InputSource&    src
         fReaderMgr.reset();
         return false;
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         // Reset and rethrow original error
@@ -647,7 +673,7 @@ void XMLScanner::scanReset(XMLPScanToken& token)
 {
     // Make sure this token is still legal
     if (!isLegalToken(token))
-        ThrowXML(RuntimeException, XMLExcepts::Scan_BadPScanToken);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Scan_BadPScanToken, fMemoryManager);
 
     // Reset the reader manager
     fReaderMgr.reset();
@@ -680,6 +706,8 @@ void XMLScanner::setParseSettings(XMLScanner* const refScanner)
     setExternalSchemaLocation(refScanner->getExternalSchemaLocation());
     setExternalNoNamespaceSchemaLocation(refScanner->getExternalNoNamespaceSchemaLocation());
     setValidationScheme(refScanner->getValidationScheme());
+    setSecurityManager(refScanner->getSecurityManager());
+    setPSVIHandler(refScanner->getPSVIHandler());
 }
 
 // ---------------------------------------------------------------------------
@@ -706,10 +734,16 @@ void XMLScanner::commonInit()
 
     //  Create the id ref list. This is used to enforce XML 1.0 ID ref
     //  semantics, i.e. all id refs must refer to elements that exist
-    fIDRefList = new (fMemoryManager) RefHashTableOf<XMLRefInfo>(109, fMemoryManager);
+    fValidationContext = new (fMemoryManager) ValidationContextImpl(fMemoryManager);
 
     //  Create the GrammarResolver
     //fGrammarResolver = new GrammarResolver();
+
+    // create initial, 64-element, fUIntPool
+    fUIntPool = (unsigned int **)fMemoryManager->allocate(sizeof(unsigned int *) *fUIntPoolRowTotal);
+    fUIntPool[0] = (unsigned int *)fMemoryManager->allocate(sizeof(unsigned int) << 6);
+    memset(fUIntPool[0], 0, sizeof(unsigned int) << 6);
+    fUIntPool[1] = 0;
 }
 
 
@@ -728,6 +762,13 @@ void XMLScanner::initValidator(XMLValidator* theValidator) {
 //  These methods are called whenever the scanner wants to emit an error.
 //  It handles getting the message loaded, doing token replacement, etc...
 //  and then calling the error handler, if its installed.
+bool XMLScanner::emitErrorWillThrowException(const XMLErrs::Codes toEmit)
+{
+    if (XMLErrs::isFatal(toEmit) && fExitOnFirstFatal && !fInException)
+        return true;
+    return false;
+}
+
 void XMLScanner::emitError(const XMLErrs::Codes toEmit)
 {
     // Bump the error count if it is not a warning
@@ -766,7 +807,7 @@ void XMLScanner::emitError(const XMLErrs::Codes toEmit)
     }
 
     // Bail out if its fatal an we are to give up on the first fatal error
-    if (XMLErrs::isFatal(toEmit) && fExitOnFirstFatal && !fInException)
+    if (emitErrorWillThrowException(toEmit))
         throw toEmit;
 }
 
@@ -787,7 +828,7 @@ void XMLScanner::emitError( const   XMLErrs::Codes    toEmit
         const unsigned int maxChars = 2047;
         XMLCh errText[maxChars + 1];
 
-        if (!gScannerMsgLoader().loadMsg(toEmit, errText, maxChars, text1, text2, text3, text4))
+        if (!gScannerMsgLoader().loadMsg(toEmit, errText, maxChars, text1, text2, text3, text4, fMemoryManager))
         {
                 // <TBD> Should probably load a default message here
         }
@@ -813,7 +854,7 @@ void XMLScanner::emitError( const   XMLErrs::Codes    toEmit
     }
 
     // Bail out if its fatal an we are to give up on the first fatal error
-    if (XMLErrs::isFatal(toEmit) && fExitOnFirstFatal && !fInException)
+    if (emitErrorWillThrowException(toEmit))
         throw toEmit;
 }
 
@@ -834,7 +875,7 @@ void XMLScanner::emitError( const   XMLErrs::Codes    toEmit
         const unsigned int maxChars = 2047;
         XMLCh errText[maxChars + 1];
 
-        if (!gScannerMsgLoader().loadMsg(toEmit, errText, maxChars, text1, text2, text3, text4))
+        if (!gScannerMsgLoader().loadMsg(toEmit, errText, maxChars, text1, text2, text3, text4, fMemoryManager))
         {
                 // <TBD> Should probably load a default message here
         }
@@ -860,7 +901,7 @@ void XMLScanner::emitError( const   XMLErrs::Codes    toEmit
     }
 
     // Bail out if its fatal an we are to give up on the first fatal error
-    if (XMLErrs::isFatal(toEmit) && fExitOnFirstFatal && !fInException)
+    if (emitErrorWillThrowException(toEmit))
         throw toEmit;
 }
 
@@ -1050,7 +1091,7 @@ void XMLScanner::scanPI()
             if (!nextCh)
             {
                 emitError(XMLErrs::UnterminatedPI);
-                ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+                ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
             }
 
             // Watch for potential terminating character
@@ -1086,6 +1127,7 @@ void XMLScanner::scanPI()
                         , tmpBuf
                         , 8
                         , 16
+                        , fMemoryManager
                     );
                     emitError(XMLErrs::InvalidCharacter, tmpBuf);
                 }
@@ -1559,7 +1601,11 @@ Grammar* XMLScanner::loadGrammar(const   XMLCh* const systemId
     InputSource* srcToUse = 0;
 
     if (fEntityHandler){
-        srcToUse = fEntityHandler->resolveEntity(XMLUni::fgZeroLenString, systemId);
+        ReaderMgr::LastExtEntityInfo lastInfo;
+        fReaderMgr.getLastExtEntityInfo(lastInfo);
+        XMLResourceIdentifier resourceIdentifier(XMLResourceIdentifier::ExternalEntity,
+                            systemId, 0, XMLUni::fgZeroLenString, lastInfo.systemId);
+        srcToUse = fEntityHandler->resolveEntity(&resourceIdentifier);
     }
 
     //  First we try to parse it as a URL. If that fails, we assume its
@@ -1571,28 +1617,52 @@ Grammar* XMLScanner::loadGrammar(const   XMLCh* const systemId
             //  Create a temporary URL. Since this is the primary document,
             //  it has to be fully qualified. If not, then assume we are just
             //  mistaking a file for a URL.
-            XMLURL tmpURL(systemId, fMemoryManager);
-            if (tmpURL.isRelative())
+            XMLURL tmpURL(fMemoryManager);
+
+            if (XMLURL::parse(systemId, tmpURL)) {            
+
+                if (tmpURL.isRelative())
+                {
+                    if (!fStandardUriConformant)
+                        srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
+                    else {
+                        // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
+                        // emit the error directly
+                        MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent, fMemoryManager);
+                        fInException = true;
+                        emitError
+                        (
+                            XMLErrs::XMLException_Fatal
+                            , e.getType()
+                            , e.getMessage()
+                        );
+                        return 0;
+                    }
+                }
+                else
+                {
+                    if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
+                        MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL, fMemoryManager);
+                        fInException = true;
+                        emitError
+                        (
+                            XMLErrs::XMLException_Fatal
+                            , e.getType()
+                            , e.getMessage()
+                        );
+                        return 0;
+                    }
+                    srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
+                }
+            }
+            else         
             {
                 if (!fStandardUriConformant)
                     srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
                 else {
-                    // since this is the top of the try/catch, cannot call ThrowXML
+                    // since this is the top of the try/catch, cannot call ThrowXMLwithMemMgr
                     // emit the error directly
-                    MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_NoProtocolPresent);
-                    fInException = true;
-                    emitError
-                    (
-                        XMLErrs::XMLException_Fatal
-                        , e.getType()
-                        , e.getMessage()
-                    );
-                    return 0;
-                }
-            }
-            else
-            {
-                if (fStandardUriConformant && tmpURL.hasInvalidChar()) {
+                    // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
                     MalformedURLException e(__FILE__, __LINE__, XMLExcepts::URL_MalformedURL);
                     fInException = true;
                     emitError
@@ -1603,25 +1673,6 @@ Grammar* XMLScanner::loadGrammar(const   XMLCh* const systemId
                     );
                     return 0;
                 }
-                srcToUse = new (fMemoryManager) URLInputSource(tmpURL, fMemoryManager);
-            }
-        }
-        catch(const MalformedURLException& e)
-        {
-            if (!fStandardUriConformant)
-                srcToUse = new (fMemoryManager) LocalFileInputSource(systemId, fMemoryManager);
-            else {
-                // since this is the top of the try/catch, cannot call ThrowXML
-                // emit the error directly
-                // lazy bypass ... since all MalformedURLException are fatal, no need to check the type
-                fInException = true;
-                emitError
-                (
-                    XMLErrs::XMLException_Fatal
-                    , e.getType()
-                    , e.getMessage()
-                );
-                return 0;
             }
         }
         catch(const XMLException& excToCatch)
@@ -1652,11 +1703,6 @@ Grammar* XMLScanner::loadGrammar(const   XMLCh* const systemId
                 );
                 return 0;
         }
-        catch(...)
-        {
-            // Just rethrow this, since its not our problem
-            throw;
-        }
     }
 
     Janitor<InputSource> janSrc(srcToUse);
@@ -1693,11 +1739,12 @@ void XMLScanner::setURIStringPool(XMLStringPool* const stringPool)
 //  This method is called after the content scan to insure that all the
 //  ID/IDREF attributes match up (i.e. that all IDREFs refer to IDs.) This is
 //  an XML 1.0 rule, so we can do here in the core.
+
 void XMLScanner::checkIDRefs()
 {
     //  Iterate the id ref list. If we find any entries here which are used
     //  but not declared, then that's an error.
-    RefHashTableOfEnumerator<XMLRefInfo> refEnum(fIDRefList);
+    RefHashTableOfEnumerator<XMLRefInfo> refEnum(fValidationContext->getIdRefList(), false, fMemoryManager);
     while (refEnum.hasMoreElements())
     {
         // Get a ref to the current element
@@ -1870,7 +1917,7 @@ bool XMLScanner::scanCharRef(XMLCh& toFill, XMLCh& second)
 
         // Watch for EOF
         if (!nextCh)
-            ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+            ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
 
         // Break out on the terminating semicolon
         if (nextCh == chSemiColon)
@@ -1917,6 +1964,12 @@ bool XMLScanner::scanCharRef(XMLCh& toFill, XMLCh& second)
         else
         {
             value = (value * radix) + nextVal;
+            // Guard against overflow.
+            if (value > 0x10FFFF) {
+                // Character reference was not in the valid range
+                emitError(XMLErrs::InvalidCharacterRef);
+                return false;
+            }
         }
 
         // Indicate that we got at least one good digit
@@ -1985,7 +2038,7 @@ void XMLScanner::scanComment()
         if (!nextCh)
         {
             emitError(XMLErrs::UnterminatedComment);
-            ThrowXML(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF);
+            ThrowXMLwithMemMgr(UnexpectedEOFException, XMLExcepts::Gen_UnexpectedEOF, fMemoryManager);
         }
 
         // Check for correct surrogate pairs
@@ -2013,6 +2066,7 @@ void XMLScanner::scanComment()
                     , tmpBuf
                     , 8
                     , 16
+                    , fMemoryManager
                 );
                 emitError(XMLErrs::InvalidCharacter, tmpBuf);
             }
@@ -2089,5 +2143,63 @@ XMLScanner::scanUpToWSOr(XMLBuffer& toFill, const XMLCh chEndChar)
     return toFill.getLen();
 }
 
+unsigned int *XMLScanner::getNewUIntPtr()
+{
+    // this method hands back a new pointer initialized to 0
+    unsigned int *retVal;
+    if(fUIntPoolCol < 64)
+    {
+        retVal = fUIntPool[fUIntPoolRow]+fUIntPoolCol;
+        fUIntPoolCol++;
+        return retVal;
+    }
+    // time to grow the pool...
+    if(fUIntPoolRow+1 == fUIntPoolRowTotal)
+    {
+        // and time to add some space for new rows:
+        fUIntPoolRowTotal <<= 1;
+        unsigned int **newArray = (unsigned int **)fMemoryManager->allocate(sizeof(unsigned int *) * fUIntPoolRowTotal );
+        memcpy(newArray, fUIntPool, (fUIntPoolRow+1) * sizeof(unsigned int *));
+        fMemoryManager->deallocate(fUIntPool);
+        fUIntPool = newArray;
+        // need to 0 out new elements we won't need:
+        for (unsigned int i=fUIntPoolRow+2; i<fUIntPoolRowTotal; i++)
+            fUIntPool[i] = 0;
+    }
+    // now to add a new row; we just made sure we have space
+    fUIntPoolRow++;
+    fUIntPool[fUIntPoolRow] = (unsigned int *)fMemoryManager->allocate(sizeof(unsigned int) << 6);
+    memset(fUIntPool[fUIntPoolRow], 0, sizeof(unsigned int) << 6);
+    // point to next element
+    fUIntPoolCol = 1; 
+    return fUIntPool[fUIntPoolRow];
+}
+
+void XMLScanner::resetUIntPool()
+{
+    // to reuse the unsigned int pool--and the hashtables that use it--
+    // simply reinitialize everything to 0's
+    for(unsigned int i = 0; i<= fUIntPoolRow; i++)
+        memset(fUIntPool[i], 0, sizeof(unsigned int) << 6);
+}
+
+void XMLScanner::recreateUIntPool()
+{
+    // this allows a bloated unsigned int pool to be dispensed with
+
+    // first, delete old fUIntPool
+    for (unsigned int i=0; i<=fUIntPoolRow; i++)
+    {
+        fMemoryManager->deallocate(fUIntPool[i]);
+    }
+    fMemoryManager->deallocate(fUIntPool);
+
+    fUIntPoolRow = fUIntPoolCol = 0;
+    fUIntPoolRowTotal = 2;
+    fUIntPool = (unsigned int **)fMemoryManager->allocate(sizeof(unsigned int *) * fUIntPoolRowTotal);
+    fUIntPool[0] = (unsigned int *)fMemoryManager->allocate(sizeof(unsigned int) << 6);
+    memset(fUIntPool[fUIntPoolRow], 0, sizeof(unsigned int) << 6);
+    fUIntPool[1] = 0;
+}
 
 XERCES_CPP_NAMESPACE_END
