@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: DOMBuilderImpl.cpp,v 1.24 2003/05/16 21:36:59 knoaman Exp $
+* $Id: DOMBuilderImpl.cpp,v 1.34 2004/01/29 11:46:32 cargilld Exp $
 *
 */
 
@@ -78,7 +78,12 @@
 #include <xercesc/sax/SAXParseException.hpp>
 #include <xercesc/internal/XMLScanner.hpp>
 #include <xercesc/framework/Wrapper4DOMInputSource.hpp>
+#include <xercesc/framework/XMLGrammarPool.hpp>
+#include <xercesc/framework/XMLSchemaDescription.hpp>
+#include <xercesc/util/Janitor.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/util/XMLEntityResolver.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -87,14 +92,16 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  DOMBuilderImpl: Constructors and Destructor
 // ---------------------------------------------------------------------------
-DOMBuilderImpl::DOMBuilderImpl( XMLValidator* const  valToAdopt
-                              , MemoryManager* const manager) :
+DOMBuilderImpl::DOMBuilderImpl( XMLValidator* const   valToAdopt
+                              , MemoryManager* const  manager
+                              , XMLGrammarPool* const gramPool) :
 
-AbstractDOMParser(valToAdopt, manager)
+AbstractDOMParser(valToAdopt, manager, gramPool)
 , fAutoValidation(false)
 , fValidation(false)
-, fErrorHandler(0)
 , fEntityResolver(0)
+, fXMLEntityResolver(0)
+, fErrorHandler(0)
 , fFilter(0)
 , fCharsetOverridesXMLEncoding(true)
 , fUserAdoptsDocument(false)
@@ -128,13 +135,26 @@ void DOMBuilderImpl::setEntityResolver(DOMEntityResolver* const handler)
     fEntityResolver = handler;
     if (fEntityResolver) {
         getScanner()->setEntityHandler(this);
+        fXMLEntityResolver = 0;
     }
     else {
         getScanner()->setEntityHandler(0);
     }
 }
 
-void DOMBuilderImpl::setFilter(DOMBuilderFilter* const filter)
+void DOMBuilderImpl::setXMLEntityResolver(XMLEntityResolver* const handler)
+{
+    fXMLEntityResolver = handler;
+    if (fXMLEntityResolver) {
+        getScanner()->setEntityHandler(this);
+        fEntityResolver = 0;
+    }
+    else {
+        getScanner()->setEntityHandler(0);
+    }
+}
+
+void DOMBuilderImpl::setFilter(DOMBuilderFilter* const)
 {
     throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
 }
@@ -398,6 +418,11 @@ void DOMBuilderImpl::setProperty(const XMLCh* const name, void* value)
     {
         AbstractDOMParser::useScanner((const XMLCh*) value);
     }
+    else if (XMLString::equals(name, XMLUni::fgXercesParserUseDocumentFromImplementation))
+    {
+        useImplementation((const XMLCh*) value);
+    }
+
     else
       throw DOMException(DOMException::NOT_FOUND_ERR, 0);
 }
@@ -460,9 +485,9 @@ DOMDocument* DOMBuilderImpl::parseURI(const char* const systemId)
         return getDocument();
 }
 
-void DOMBuilderImpl::parseWithContext(const DOMInputSource& source,
-                                      DOMNode* const contextNode,
-                                      const short action)
+void DOMBuilderImpl::parseWithContext(const DOMInputSource&,
+                                      DOMNode* const,
+                                      const short)
 {
     throw DOMException(DOMException::NOT_SUPPORTED_ERR, 0);
 }
@@ -472,11 +497,11 @@ void DOMBuilderImpl::parseWithContext(const DOMInputSource& source,
 //  DOMBuilderImpl: Implementation of the XMLErrorReporter interface
 // ---------------------------------------------------------------------------
 void DOMBuilderImpl::error( const   unsigned int                code
-                            , const XMLCh* const                msgDomain
+                            , const XMLCh* const
                             , const XMLErrorReporter::ErrTypes  errType
                             , const XMLCh* const                errorText
                             , const XMLCh* const                systemId
-                            , const XMLCh* const                publicId
+                            , const XMLCh* const
                             , const XMLSSize_t                  lineNum
                             , const XMLSSize_t                  colNum)
 {
@@ -520,12 +545,33 @@ DOMBuilderImpl::resolveEntity(const XMLCh* const publicId,
         DOMInputSource* is = fEntityResolver->resolveEntity(publicId, systemId, baseURI);
 
         if (is)
-            return new Wrapper4DOMInputSource(is, true, getMemoryManager());
+            return new (getMemoryManager()) Wrapper4DOMInputSource(is, true, getMemoryManager());
     }
 
     return 0;
 }
 
+InputSource*
+DOMBuilderImpl::resolveEntity( XMLResourceIdentifier* resourceIdentifier )
+{
+    //
+    //  Just map it to the SAX entity resolver. If there is not one installed,
+    //  return a null pointer to cause the default resolution.
+    //
+    if (fEntityResolver) {
+        DOMInputSource* is = fEntityResolver->resolveEntity(resourceIdentifier->getPublicId(),
+                                                            resourceIdentifier->getSystemId(), 
+                                                            resourceIdentifier->getBaseURI());
+        if (is)
+            return new (getMemoryManager()) Wrapper4DOMInputSource(is, true, getMemoryManager());    
+    }
+    if (fXMLEntityResolver) {
+        return(fXMLEntityResolver->resolveEntity(resourceIdentifier));    
+    }
+    
+    return 0;
+}
+
 // ---------------------------------------------------------------------------
 //  DOMBuilderImpl: Grammar preparsing methods
 // ---------------------------------------------------------------------------
@@ -535,24 +581,33 @@ Grammar* DOMBuilderImpl::loadGrammar(const char* const systemId,
 {
     // Avoid multiple entrance
     if (getParseInProgress())
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
 	Grammar* grammar = 0;
     try
     {
         setParseInProgress(true);
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(0);
         grammar = getScanner()->loadGrammar(systemId, grammarType, toCache);
 
         // Release DOM tree - DTD
         DOMDocument* doc = adoptDocument();
         if (doc)
             doc->release();
-
+        
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
         throw;
     }
@@ -566,12 +621,14 @@ Grammar* DOMBuilderImpl::loadGrammar(const XMLCh* const systemId,
 {
     // Avoid multiple entrance
     if (getParseInProgress())
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
     {
         setParseInProgress(true);
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(0);
         grammar = getScanner()->loadGrammar(systemId, grammarType, toCache);
 
         // Release DOM tree - DTD
@@ -579,11 +636,18 @@ Grammar* DOMBuilderImpl::loadGrammar(const XMLCh* const systemId,
         if (doc)
             doc->release();
 
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
         throw;
     }
@@ -597,7 +661,7 @@ Grammar* DOMBuilderImpl::loadGrammar(const DOMInputSource& source,
 {
     // Avoid multiple entrance
     if (getParseInProgress())
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -605,6 +669,8 @@ Grammar* DOMBuilderImpl::loadGrammar(const DOMInputSource& source,
         Wrapper4DOMInputSource isWrapper((DOMInputSource*) &source, false, getMemoryManager());
 
         setParseInProgress(true);
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(0);
         grammar = getScanner()->loadGrammar(isWrapper, grammarType, toCache);
 
         // Release DOM tree - DTD
@@ -612,11 +678,18 @@ Grammar* DOMBuilderImpl::loadGrammar(const DOMInputSource& source,
         if (doc)
             doc->release();
 
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
+        if (grammarType == Grammar::DTDGrammarType) 
+            getScanner()->setDocTypeHandler(this);
         setParseInProgress(false);
         throw;
     }
