@@ -56,6 +56,46 @@
 
 /*
  * $Log: SAX2XMLReaderImpl.cpp,v $
+ * Revision 1.33  2004/01/29 11:46:32  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.32  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.31  2003/11/21 22:38:50  neilg
+ * Enable grammar pools and grammar resolvers to manufacture
+ * XSModels.  This also cleans up handling in the
+ * parser classes by eliminating the need to tell
+ * the grammar pool that schema compoments need to be produced.
+ * Thanks to David Cargill.
+ *
+ * Revision 1.30  2003/11/06 15:30:07  neilg
+ * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
+ *
+ * Revision 1.29  2003/10/30 21:37:31  knoaman
+ * Enhanced Entity Resolver Support. Thanks to David Cargill.
+ *
+ * Revision 1.28  2003/10/01 16:32:38  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.27  2003/09/16 18:30:54  neilg
+ * make Grammar pool be responsible for creating and owning URI string pools.  This is one more step towards having grammars be independent of the parsers involved in their creation
+ *
+ * Revision 1.26  2003/08/13 15:43:24  knoaman
+ * Use memory manager when creating SAX exceptions.
+ *
+ * Revision 1.25  2003/07/31 17:05:48  peiyongz
+ * using getGrammar(URI)
+ *
+ * Revision 1.24  2003/07/10 19:48:24  peiyongz
+ * Stateless Grammar: Initialize scanner with grammarResolver,
+ *
+ * Revision 1.23  2003/06/25 22:36:46  peiyongz
+ * to use new GrammarResolver::getGrammar()
+ *
+ * Revision 1.22  2003/06/20 18:55:54  peiyongz
+ * Stateless Grammar Pool :: Part I
+ *
  * Revision 1.21  2003/05/18 14:02:05  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -282,6 +322,10 @@
 #include <xercesc/internal/XMLScannerResolver.hpp>
 #include <xercesc/parsers/SAX2XMLReaderImpl.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/framework/XMLGrammarPool.hpp>
+#include <xercesc/framework/XMLSchemaDescription.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/util/XMLEntityResolver.hpp>
 #include <string.h>
 
 XERCES_CPP_NAMESPACE_BEGIN
@@ -292,7 +336,8 @@ const XMLCh gDTDEntityStr[] =
     chOpenSquare, chLatin_d, chLatin_t, chLatin_d, chCloseSquare, chNull
 };
 
-SAX2XMLReaderImpl::SAX2XMLReaderImpl(MemoryManager* const manager) :
+SAX2XMLReaderImpl::SAX2XMLReaderImpl(MemoryManager* const  manager
+                                   , XMLGrammarPool* const gramPool):
 
     fNamespacePrefix(false)
     , fAutoValidation(false)
@@ -308,7 +353,9 @@ SAX2XMLReaderImpl::SAX2XMLReaderImpl(MemoryManager* const manager) :
     , fPrefixCounts(0)
     , fDTDHandler(0)
     , fEntityResolver(0)
+    , fXMLEntityResolver(0)
     , fErrorHandler(0)
+    , fPSVIHandler(0)
     , fLexicalHandler(0)
     , fDeclHandler(0)
     , fAdvDHList(0)
@@ -317,12 +364,17 @@ SAX2XMLReaderImpl::SAX2XMLReaderImpl(MemoryManager* const manager) :
     , fURIStringPool(0)
     , fValidator(0)
     , fMemoryManager(manager)
+    , fGrammarPool(gramPool)
     , fStringBuffers(manager)
 {
     try
     {
         initialize();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         cleanUp();
@@ -341,13 +393,12 @@ SAX2XMLReaderImpl::~SAX2XMLReaderImpl()
 void SAX2XMLReaderImpl::initialize()
 {
     // Create grammar resolver and string pool that we pass to the scanner
-    fGrammarResolver = new (fMemoryManager) GrammarResolver(fMemoryManager);
-    fURIStringPool = new (fMemoryManager) XMLStringPool(109, fMemoryManager);
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fGrammarPool, fMemoryManager);
+    fURIStringPool = fGrammarResolver->getStringPool();
 
     //  Create a scanner and tell it what validator to use. Then set us
     //  as the document event handler so we can fill the DOM document.
-    fScanner = XMLScannerResolver::getDefaultScanner(0, fMemoryManager);
-    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner = XMLScannerResolver::getDefaultScanner(0, fGrammarResolver, fMemoryManager);
     fScanner->setURIStringPool(fURIStringPool);
 
     // Create the initial advanced handler list array and zero it out
@@ -377,7 +428,8 @@ void SAX2XMLReaderImpl::cleanUp()
     delete fTempAttrVec;
     delete fPrefixCounts;
     delete fGrammarResolver;
-    delete fURIStringPool;
+    // grammar pool must do this
+    //delete fURIStringPool;
 }
 
 // ---------------------------------------------------------------------------
@@ -544,6 +596,16 @@ void SAX2XMLReaderImpl::setErrorHandler(ErrorHandler* const handler)
     }
 }
 
+void SAX2XMLReaderImpl::setPSVIHandler(PSVIHandler* const handler)
+{
+    fPSVIHandler = handler;
+    if (fPSVIHandler) {
+        fScanner->setPSVIHandler(fPSVIHandler);        
+    }
+    else {
+        fScanner->setPSVIHandler(0);       
+    }
+}
 
 void SAX2XMLReaderImpl::setLexicalHandler(LexicalHandler* const handler)
 {
@@ -569,6 +631,19 @@ void SAX2XMLReaderImpl::setEntityResolver(EntityResolver* const resolver)
     fEntityResolver = resolver;
     if (fEntityResolver) {
         fScanner->setEntityHandler(this);
+        fXMLEntityResolver = 0;
+    }
+    else {
+        fScanner->setEntityHandler(0);
+    }
+}
+
+void SAX2XMLReaderImpl::setXMLEntityResolver(XMLEntityResolver* const resolver)
+{
+    fXMLEntityResolver = resolver;
+    if (fXMLEntityResolver) {
+        fScanner->setEntityHandler(this);
+        fEntityResolver = 0;
     }
     else {
         fScanner->setEntityHandler(0);
@@ -589,7 +664,7 @@ void SAX2XMLReaderImpl::parse (const   InputSource&    source)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -597,7 +672,10 @@ void SAX2XMLReaderImpl::parse (const   InputSource&    source)
         fScanner->scanDocument(source);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -609,7 +687,7 @@ void SAX2XMLReaderImpl::parse (const   XMLCh* const    systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -617,7 +695,10 @@ void SAX2XMLReaderImpl::parse (const   XMLCh* const    systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -629,7 +710,7 @@ void SAX2XMLReaderImpl::parse (const   char* const     systemId)
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     try
     {
@@ -637,7 +718,10 @@ void SAX2XMLReaderImpl::parse (const   char* const     systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         fParseInProgress = false;
@@ -656,7 +740,7 @@ bool SAX2XMLReaderImpl::parseFirst( const   XMLCh* const    systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -669,7 +753,7 @@ bool SAX2XMLReaderImpl::parseFirst( const   char* const     systemId
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(systemId, toFill);
 }
@@ -682,7 +766,7 @@ bool SAX2XMLReaderImpl::parseFirst( const   InputSource&    source
     //  is in progress.
     //
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     return fScanner->scanFirst(source, toFill);
 }
@@ -1100,7 +1184,7 @@ void SAX2XMLReaderImpl::attDef( const   DTDElementDecl& elemDecl
         if (defAttType == XMLAttDef::Fixed ||
             defAttType == XMLAttDef::Implied ||
             defAttType == XMLAttDef::Required) {
-            defAttTypeStr = attDef.getDefAttTypeString(defAttType);
+            defAttTypeStr = attDef.getDefAttTypeString(defAttType, fMemoryManager);
         }
 
         if (isEnumeration) {
@@ -1129,7 +1213,7 @@ void SAX2XMLReaderImpl::attDef( const   DTDElementDecl& elemDecl
         fDeclHandler->attributeDecl(elemDecl.getFullName(),
                                     attDef.getFullName(),
                                     (isEnumeration) ? enumBuf.getRawBuffer()
-                                                    : attDef.getAttTypeString(attDef.getType()),
+                                                    : attDef.getAttTypeString(attDef.getType(), fMemoryManager),
                                     defAttTypeStr,
                                     attDef.getValue());
     }
@@ -1170,8 +1254,8 @@ void SAX2XMLReaderImpl::doctypePI(  const   XMLCh* const
 }
 
 
-void SAX2XMLReaderImpl::doctypeWhitespace(  const   XMLCh* const    chars
-                                    , const unsigned int    length)
+void SAX2XMLReaderImpl::doctypeWhitespace(  const   XMLCh* const
+                                    , const unsigned int)
 {
     // Unused by SAX DTDHandler interface at this time
 }
@@ -1348,7 +1432,7 @@ void SAX2XMLReaderImpl::resetEntities()
 
 InputSource* SAX2XMLReaderImpl::resolveEntity(   const   XMLCh* const    publicId
                                                , const   XMLCh* const    systemId
-                                               , const   XMLCh* const    baseURI)
+                                               , const   XMLCh* const)
 {
     // Just map to the SAX entity resolver handler
     if (fEntityResolver)
@@ -1356,7 +1440,21 @@ InputSource* SAX2XMLReaderImpl::resolveEntity(   const   XMLCh* const    publicI
     return 0;
 }
 
+InputSource* SAX2XMLReaderImpl::resolveEntity(XMLResourceIdentifier* resourceIdentifier)
+{
+    //
+    //  Just map it to the SAX entity resolver. If there is not one installed,
+    //  return a null pointer to cause the default resolution.
+    //
+    if (fEntityResolver)
+        return fEntityResolver->resolveEntity(resourceIdentifier->getPublicId(), 
+                                                resourceIdentifier->getSystemId());
+    if (fXMLEntityResolver)
+        return fXMLEntityResolver->resolveEntity(resourceIdentifier);
 
+    return 0;
+}
+ 
 void SAX2XMLReaderImpl::startInputSource(const InputSource&)
 {
     // Nothing to do for this one
@@ -1372,8 +1470,8 @@ void SAX2XMLReaderImpl::resetErrors()
 }
 
 
-void SAX2XMLReaderImpl::error(  const   unsigned int                code
-                        , const XMLCh* const                msgDomain
+void SAX2XMLReaderImpl::error(  const   unsigned int
+                        , const XMLCh* const
                         , const XMLErrorReporter::ErrTypes  errType
                         , const XMLCh* const                errorText
                         , const XMLCh* const                systemId
@@ -1388,6 +1486,7 @@ void SAX2XMLReaderImpl::error(  const   unsigned int                code
         , systemId
         , lineNum
         , colNum
+        , fMemoryManager
     );
 
     if (!fErrorHandler)
@@ -1415,7 +1514,7 @@ void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
 {
 
     if (fParseInProgress)
-        throw SAXNotSupportedException("Feature modification is not supported during parse.");
+        throw SAXNotSupportedException("Feature modification is not supported during parse.", fMemoryManager);
 	
     if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpaces) == 0)
     {
@@ -1489,7 +1588,7 @@ void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
         fScanner->setStandardUriConformant(value);
     }
     else
-       throw SAXNotRecognizedException("Unknown Feature");
+       throw SAXNotRecognizedException("Unknown Feature", fMemoryManager);
 }
 
 bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
@@ -1521,7 +1620,7 @@ bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
     else if (XMLString::compareIString(name, XMLUni::fgXercesStandardUriConformant) == 0)
         return fScanner->getStandardUriConformant();
     else
-       throw SAXNotRecognizedException("Unknown Feature");
+       throw SAXNotRecognizedException("Unknown Feature", fMemoryManager);
 
     return false;
 }
@@ -1529,7 +1628,7 @@ bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
 void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
 {
 	if (fParseInProgress)
-		throw SAXNotSupportedException("Property modification is not supported during parse.");
+		throw SAXNotSupportedException("Property modification is not supported during parse.", fMemoryManager);
 
 	if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalSchemaLocation) == 0)
 	{
@@ -1549,20 +1648,20 @@ void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
         (
             (const XMLCh*) value
             , fValidator
+            , fGrammarResolver
             , fMemoryManager
         );
 
         if (tempScanner) {
 
             tempScanner->setParseSettings(fScanner);
-            tempScanner->setGrammarResolver(fGrammarResolver);
             tempScanner->setURIStringPool(fURIStringPool);
             delete fScanner;
             fScanner = tempScanner;
         }
     }
     else
-       throw SAXNotRecognizedException("Unknown Property");
+       throw SAXNotRecognizedException("Unknown Property", fMemoryManager);
 }
 
 
@@ -1577,7 +1676,7 @@ void* SAX2XMLReaderImpl::getProperty(const XMLCh* const name) const
     else if (XMLString::equals(name, XMLUni::fgXercesScannerName))
         return (void*)fScanner->getName();
     else
-        throw SAXNotRecognizedException("Unknown Property");
+        throw SAXNotRecognizedException("Unknown Property", fMemoryManager);
     return 0;
 }
 
@@ -1626,7 +1725,7 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const char* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1635,7 +1734,10 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const char* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -1651,7 +1753,7 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const XMLCh* const systemId,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1660,7 +1762,10 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const XMLCh* const systemId,
         grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -1676,7 +1781,7 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const InputSource& source,
 {
     // Avoid multiple entrance
     if (fParseInProgress)
-        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+        ThrowXMLwithMemMgr(IOException, XMLExcepts::Gen_ParseInProgress, fMemoryManager);
 
     Grammar* grammar = 0;
     try
@@ -1685,7 +1790,10 @@ Grammar* SAX2XMLReaderImpl::loadGrammar(const InputSource& source,
         grammar = fScanner->loadGrammar(source, grammarType, toCache);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
