@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XercesDOMParser.hpp,v 1.14 2003/05/15 18:26:50 knoaman Exp $
+ * $Id: XercesDOMParser.hpp,v 1.18 2004/01/29 11:46:32 cargilld Exp $
  *
  */
 
@@ -71,7 +71,8 @@ XERCES_CPP_NAMESPACE_BEGIN
 class EntityResolver;
 class ErrorHandler;
 class Grammar;
-
+class XMLEntityResolver;
+class XMLResourceIdentifier;
 
  /**
   * This class implements the Document Object Model (DOM) interface.
@@ -98,13 +99,20 @@ public :
       * validation. If you don't provide a validator, a default one will
       * be created for you in the scanner.
       *
+      * @param gramPool   Pointer to the grammar pool instance from 
+      *                   external application.
+      *                   The parser does NOT own it.
+      *
       * @param valToAdopt Pointer to the validator instance to use. The
       *                   parser is responsible for freeing the memory.
+      * @param  manager   Pointer to the memory manager to be used to
+      *                   allocate objects.
       */
     XercesDOMParser
     (
-          XMLValidator* const  valToAdopt = 0
-        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+          XMLValidator* const   valToAdopt = 0
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
+        , XMLGrammarPool* const gramPool = 0        
     );
 
     /**
@@ -158,6 +166,26 @@ public :
       */
     const EntityResolver* getEntityResolver() const;
 
+    /**
+      * Get a pointer to the entity resolver
+      *
+      * This method returns the installed entity resolver.  If no resolver
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return The pointer to the installed entity resolver object.
+      */
+    XMLEntityResolver* getXMLEntityResolver();
+ 
+    /**
+      * Get a const pointer to the entity resolver
+      *
+      * This method returns the installed entity resolver. If no resolver
+      * has been installed, then it will be a zero pointer.
+      *
+      * @return A const pointer to the installed entity resolver object.
+      */
+    const XMLEntityResolver* getXMLEntityResolver() const;
+
     /** Get the 'Grammar caching' flag
       *
       * This method returns the state of the parser's grammar caching when
@@ -244,8 +272,9 @@ public :
       * can trap and potentially redirect references to external
       * entities.
       *
-      * <i>Any previously set resolver is merely dropped, since the parser
-      * does not own them.</i>
+      * <i>Any previously set entity resolver is merely dropped, since the parser
+      * does not own them.  If both setEntityResolver and setXMLEntityResolver
+      * are called, then the last one is used.</i>
       *
       * @param handler  A const pointer to the user supplied entity
       *                 resolver.
@@ -254,6 +283,25 @@ public :
       */
     void setEntityResolver(EntityResolver* const handler);
 
+    /**
+      * Set the entity resolver
+      *
+      * This method allows applications to install their own entity
+      * resolver. By installing an entity resolver, the applications
+      * can trap and potentially redirect references to external
+      * entities.
+      *
+      * <i>Any previously set entity resolver is merely dropped, since the parser
+      * does not own them.  If both setEntityResolver and setXMLEntityResolver
+      * are called, then the last one set is used.</i>
+      *
+      * @param handler  A const pointer to the user supplied entity
+      *                 resolver.
+      *
+      * @see #getXMLEntityResolver
+      */
+    void setXMLEntityResolver(XMLEntityResolver* const handler);
+
     /** Set the 'Grammar caching' flag
       *
       * This method allows users to enable or disable caching of grammar when
@@ -442,6 +490,8 @@ public :
       * implement 'redirection' via this callback. This method is also
       * borrowed from the SAX specification.
       *
+      * @deprecated This method is no longer called (the other resolveEntity one is).
+      *
       * @param publicId A const pointer to a Unicode string representing the
       *                 public id of the entity just parsed.
       * @param systemId A const pointer to a Unicode string representing the
@@ -463,6 +513,27 @@ public :
         , const XMLCh* const    baseURI = 0
     );
 
+    /** Resolve a public/system id
+      *
+      * This method allows a user installed entity handler to further
+      * process any pointers to external entities. The applications can
+      * implement 'redirection' via this callback.  
+      *
+      * @param resourceIdentifier An object containing the type of
+      *        resource to be resolved and the associated data members
+      *        corresponding to this type.
+      * @return The value returned by the user installed resolveEntity
+      *         method or NULL otherwise to indicate no processing was done.
+      *         The returned InputSource is owned by the parser which is
+      *         responsible to clean up the memory.
+      * @see XMLEntityHandler
+      * @see XMLEntityResolver
+      */
+    virtual InputSource* resolveEntity
+    (
+        XMLResourceIdentifier* resourceIdentifier
+    );
+
     /** Handle a 'start input source' event
       *
       * This method is used to indicate the start of parsing an external
@@ -582,6 +653,12 @@ public :
 
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    XercesDOMParser(const XercesDOMParser&);
+    XercesDOMParser& operator=(const XercesDOMParser&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -592,6 +669,7 @@ private :
     //      The installed SAX error handler, if any. Null if none.
     //-----------------------------------------------------------------------
     EntityResolver*          fEntityResolver;
+    XMLEntityResolver*       fXMLEntityResolver;
     ErrorHandler*            fErrorHandler;
 };
 
@@ -645,6 +723,15 @@ inline const EntityResolver* XercesDOMParser::getEntityResolver() const
     return fEntityResolver;
 }
 
+inline XMLEntityResolver* XercesDOMParser::getXMLEntityResolver()
+{
+    return fXMLEntityResolver;
+}
+
+inline const XMLEntityResolver* XercesDOMParser::getXMLEntityResolver() const
+{
+    return fXMLEntityResolver;
+}
 
 XERCES_CPP_NAMESPACE_END
 
