@@ -56,6 +56,15 @@
 
 /*
  * $Log: DefaultHandler.hpp,v $
+ * Revision 1.7  2004/01/29 11:46:32  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.6  2003/12/02 14:33:13  amassari
+ * Don't use the word "exception" as a variable name, as VC 7.1 complains about it
+ *
+ * Revision 1.5  2002/11/04 14:55:45  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.4  2002/08/20 16:53:56  tng
  * [Bug 6251] Info during compilation.
  *
@@ -103,6 +112,8 @@
 #include <xercesc/sax/ErrorHandler.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class Locator;
 class Attributes;
 
@@ -407,13 +418,13 @@ public:
     * for each error, such as inserting the message in a log file or
     * printing it to the console.</p>
     *
-    * @param exception The warning information encoded as an exception.
+    * @param exc The warning information encoded as an exception.
     * @exception SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see ErrorHandler#warning
     * @see SAXParseException#SAXParseException
     */
-    virtual void error(const SAXParseException& exception);
+    virtual void error(const SAXParseException& exc);
 
   /**
     * Report a fatal XML parsing error.
@@ -426,13 +437,13 @@ public:
     * method is invoked, since the document is no longer reliable, and
     * the parser may no longer report parsing events.</p>
     *
-    * @param exception The error information encoded as an exception.
+    * @param exc The error information encoded as an exception.
     * @exception SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see ErrorHandler#fatalError
     * @see SAXParseException#SAXParseException
     */
-    virtual void fatalError(const SAXParseException& exception);
+    virtual void fatalError(const SAXParseException& exc);
 
   /**
     * Receive notification of a parser warning.
@@ -442,13 +453,13 @@ public:
     * for each warning, such as inserting the message in a log file or
     * printing it to the console.</p>
     *
-    * @param exception The warning information encoded as an exception.
+    * @param exc The warning information encoded as an exception.
     * @exception SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see ErrorHandler#warning
     * @see SAXParseException#SAXParseException
     */
-    virtual void warning(const SAXParseException& exception);
+    virtual void warning(const SAXParseException& exc);
 
     /**
     * Reset the Error handler object on its reuse
@@ -702,6 +713,16 @@ public:
     );
 
     //@}
+
+    DefaultHandler() {};
+    virtual ~DefaultHandler() {};
+
+private:
+	// -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DefaultHandler(const DefaultHandler&);
+    DefaultHandler& operator=(const DefaultHandler&);    
 };
 
 
@@ -727,9 +748,9 @@ inline void DefaultHandler::error(const SAXParseException&)
 {
 }
 
-inline void DefaultHandler::fatalError(const SAXParseException& exception)
+inline void DefaultHandler::fatalError(const SAXParseException& exc)
 {
-    throw exception;
+    throw exc;
 }
 
 inline void
@@ -866,4 +887,6 @@ inline void DefaultHandler::internalEntityDecl(const XMLCh* const,
 {
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif // ! DEFAULTHANDLER_HPP
