@@ -56,6 +56,9 @@
 
 /*
  * $Log: CSetDefs.hpp,v $
+ * Revision 1.7  2003/05/29 11:18:37  gareth
+ * Added macros in so we can determine whether to do things like iostream as opposed to iostream.h and whether to use std:: or not.
+ *
  * Revision 1.6  2002/11/05 21:43:55  tng
  * Turn on C++ Namespace support only if using AIX xlC version 4 or higher.
  *
@@ -193,6 +196,10 @@ typedef int             XMLInt32;
 #define XERCES_DEBUG
 #endif
 
+#if __IBMCPP__ >= 400
+#define XERCES_NEW_IOSTREAMS
+#define XERCES_STD_NAMESPACE
+#endif
 
 // ---------------------------------------------------------------------------
 //  Provide some common string ops that are different/notavail on CSet
