@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: SocketNetAccessor.cpp,v 1.3 2003/05/17 05:54:18 knoaman Exp $
+ * $Id: SocketNetAccessor.cpp,v 1.5 2004/01/29 11:51:20 cargilld Exp $
  */
 
 #include <xercesc/util/XMLUniDefs.hpp>
@@ -97,8 +97,7 @@ BinInputStream* SocketNetAccessor::makeNew(const XMLURL&  urlSource)
         {
             UnixHTTPURLInputStream* retStrm =
                 new (urlSource.getMemoryManager()) UnixHTTPURLInputStream(urlSource);
-            return retStrm;
-            break;
+            return retStrm;            
         }
 
         //
@@ -106,7 +105,7 @@ BinInputStream* SocketNetAccessor::makeNew(const XMLURL&  urlSource)
         // unsupported protocol exception for the others.
         //
         default :
-            ThrowXML(MalformedURLException, XMLExcepts::URL_UnsupportedProto);
+            ThrowXMLwithMemMgr(MalformedURLException, XMLExcepts::URL_UnsupportedProto, urlSource.getMemoryManager());
             break;
     }
     return 0;
