@@ -56,6 +56,16 @@
 
 /*
  * $Log: BinHTTPURLInputStream.cpp,v $
+ * Revision 1.7  2003/12/17 13:58:02  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.6  2003/12/17 00:18:37  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.5  2003/12/02 14:12:20  amassari
+ * Make the code compilable on Windows when UNICODE is defined (bug#16055)
+ *
  * Revision 1.4  2003/05/17 05:54:18  knoaman
  * Update NetAccessors to use the memory manager.
  *
@@ -119,8 +129,6 @@
  */
 
 
-#define _WINSOCKAPI_
-
 #define INCL_WINSOCK_API_TYPEDEFS 1
 #include <winsock2.h>
 #include <windows.h>
@@ -130,7 +138,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLNetAccessor.hpp>
 #include <xercesc/util/NetAccessors/WinSock/BinHTTPURLInputStream.hpp>
@@ -157,7 +164,7 @@ LPFN_WSACLEANUP gWSACleanup = NULL;
 bool BinHTTPURLInputStream::fInitialized = false;
 XMLMutex* BinHTTPURLInputStream::fInitMutex = 0;
 
-void BinHTTPURLInputStream::Initialize() {
+void BinHTTPURLInputStream::Initialize(MemoryManager* const manager) {
     //
     // Initialize the WinSock library here.
     //
@@ -168,21 +175,21 @@ void BinHTTPURLInputStream::Initialize() {
 	if(gWinsockLib == NULL) {
 		gWinsockLib = LoadLibrary(_T("WSOCK32"));
 		if(gWinsockLib == NULL) {
-			ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+			ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
 		}
 		else {
-			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,_T("WSAStartup"));
-			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,_T("WSACleanup"));
-			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,_T("gethostbyname"));
-			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,_T("inet_addr"));
-			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,_T("gethostbyaddr"));
-			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,_T("htons"));
-			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,_T("socket"));
-			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,_T("connect"));
-			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,_T("send"));
-			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,_T("recv"));
-			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,_T("shutdown"));
-			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,_T("closesocket"));
+			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,"WSAStartup");
+			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,"WSACleanup");
+			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,"gethostbyname");
+			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,"inet_addr");
+			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,"gethostbyaddr");
+			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,"htons");
+			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,"socket");
+			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,"connect");
+			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,"send");
+			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,"recv");
+			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,"shutdown");
+			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,"closesocket");
 
 			if(startup == NULL ||
 				gWSACleanup == NULL ||
@@ -199,7 +206,7 @@ void BinHTTPURLInputStream::Initialize() {
 			{
 				gWSACleanup = NULL;
 				Cleanup();
-				ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+				ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
 			}
 		}
 	}
@@ -208,7 +215,7 @@ void BinHTTPURLInputStream::Initialize() {
     if (err != 0)
     {
         // Call WSAGetLastError() to get the last error.
-        ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+        ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
     }
     fInitialized = true;
 }
@@ -297,7 +304,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     {
         if (!fInitMutex)
         {
-            XMLMutex* tmpMutex = new XMLMutex;
+            XMLMutex* tmpMutex = new XMLMutex();
             if (XMLPlatformUtils::compareAndSwap((void**)&fInitMutex, tmpMutex, 0))
             {
                 // Someone beat us to it, so let's clean up ours
@@ -307,10 +314,11 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
          XMLMutexLock lock(fInitMutex);
          if (!fInitialized)
          {
-             Initialize();
+             Initialize(urlSource.getMemoryManager());
          }
     }
 
+    fMemoryManager = urlSource.getMemoryManager();
     //
     // Pull all of the parts of the URL out of th urlSource object, and transcode them
     //   and transcode them back to ASCII.
@@ -350,16 +358,16 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         if (numAddress == INADDR_NONE)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML1(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution, hostName);
+            ThrowXMLwithMemMgr1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName, fMemoryManager);
         }
         if ((hostEntPtr =
                 gethostbyaddr((const char *) &numAddress,
                               sizeof(unsigned long), AF_INET)) == NULL)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML1(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution, hostName);
+            ThrowXMLwithMemMgr1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName, fMemoryManager);
         }
     }
 
@@ -372,15 +380,15 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (s == INVALID_SOCKET)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_CreateSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_CreateSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     if (connect(s, (struct sockaddr *) &sa, sizeof(sa)) == SOCKET_ERROR)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_ConnSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_ConnSocket, urlSource.getURLText(), fMemoryManager);
     }
 
 
@@ -427,8 +435,8 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if ((aLent = send(s, fBuffer, lent, 0)) != lent)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText(), fMemoryManager);
     }
 
 
@@ -440,7 +448,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (aLent == SOCKET_ERROR || aLent == 0)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     fBufferEnd = fBuffer+aLent;
@@ -475,7 +483,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
                 if (aLent == SOCKET_ERROR || aLent == 0)
                 {
                     // Call WSAGetLastError() to get the error number.
-                    ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+                    ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
                 }
                 fBufferEnd = fBufferEnd + aLent;
                 *fBufferEnd = 0;
@@ -488,13 +496,13 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     char *p = strstr(fBuffer, "HTTP");
     if (p == 0)
     {
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     p = strchr(p, ' ');
     if (p == 0)
     {
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     int httpResponse = atoi(p);
@@ -503,7 +511,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         // Most likely a 404 Not Found error.
         //   Should recognize and handle the forwarding responses.
         //
-        ThrowXML1(NetAccessorException, XMLExcepts::File_CouldNotOpenFile, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::File_CouldNotOpenFile, urlSource.getURLText(), fMemoryManager);
     }
 
     fSocketHandle = (unsigned int) s;
@@ -543,7 +551,7 @@ unsigned int BinHTTPURLInputStream::readBytes(XMLByte* const    toFill
         if (len == SOCKET_ERROR)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+            ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, fMemoryManager);
         }
     }
 
