@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,35 @@
 
 /*
  * $Log: FreeBSDPlatformUtils.cpp,v $
+ * Revision 1.22  2004/01/29 11:51:20  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.21  2004/01/13 16:34:21  cargilld
+ * Misc memory management changes.
+ *
+ * Revision 1.20  2004/01/06 17:31:20  neilg
+ * fix static initialization problems, bug 28517; thanks to Reid Spencer
+ *
+ * Revision 1.19  2003/12/24 15:24:14  cargilld
+ * More updates to memory management so that the static memory manager.
+ *
+ * Revision 1.18  2003/12/19 23:02:25  cargilld
+ * More memory management updates.
+ *
+ * Revision 1.17  2003/12/17 15:16:10  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.16  2003/12/17 13:58:02  cargilld
+ * Platform update for memory management so that the static memory manager (one
+ * used to call Initialize) is only for static data.
+ *
+ * Revision 1.15  2003/12/02 15:04:52  amassari
+ * openFile was opening the file for read and write, while only read was required (bug#20169)
+ *
+ * Revision 1.14  2003/10/01 16:32:39  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
  * Revision 1.13  2003/05/15 18:37:47  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -136,6 +165,7 @@
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
 #include    <xercesc/util/PanicHandler.hpp>
+#include    <xercesc/util/OutOfMemoryException.hpp>
 
 #if defined(XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -195,7 +225,10 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
         retVal = new InMemMsgLoader(msgDomain);
 #endif
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         panic(PanicHandler::Panic_CantLoadMsgDomain);
@@ -239,121 +272,129 @@ void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
 
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile
+                                          , MemoryManager* const manager)
 {
     if (theFile == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
     off_t curPos = ftell( (FILE*)theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize, manager);
 
     return (unsigned int)curPos;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+void XMLPlatformUtils::closeFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     if (theFile == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
     if (fclose((FILE*)theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile, manager);
 }
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile
+                                        , MemoryManager* const manager)
 {
     if (theFile == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
     // Get the current position
     off_t curPos = ftell((FILE*) theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos, manager);
 
     // Seek to the end and save that value for return
     if (fseek((FILE*) theFile, 0, SEEK_END))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     off_t retVal = ftell((FILE*)theFile);
     if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     // And put the pointer back
     if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos, manager);
 
     return (unsigned int)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName
+                                      , MemoryManager* const manager)
 {
     if (fileName == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
-    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
-    FileHandle retVal = (FileHandle)fopen( tmpFileName , "r+" );
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		    XMLExcepts::CPtr_PointerIsZero, manager);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
+    FileHandle retVal = (FileHandle)fopen( tmpFileName , "r" );
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+FileHandle XMLPlatformUtils::openFile(const char* const fileName
+                                      , MemoryManager* const manager)
 {
     if (fileName == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
-    FileHandle retVal = (FileHandle)fopen( fileName , "r+" );
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
+    FileHandle retVal = (FileHandle)fopen( fileName , "r" );
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                             , MemoryManager* const manager)
 {
     if (fileName == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
-    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
     return fopen( tmpFileName , "w" );
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                             , MemoryManager* const manager)
 {
     if (fileName == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
     return fopen( fileName , "w" );
 }
 
-FileHandle XMLPlatformUtils::openStdInHandle()
+FileHandle XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
     int nfd = dup(0);
     if (nfd == -1)
-	ThrowXML(XMLPlatformUtilsException,
-		XMLExcepts::File_CouldNotDupHandle);
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		    XMLExcepts::File_CouldNotDupHandle, manager);
     return (FileHandle) fdopen(nfd, "r");
 }
 
 unsigned int
 XMLPlatformUtils::readFileBuffer( FileHandle          theFile
                                 , const unsigned int  toRead
-                                , XMLByte* const      toFill)
+                                , XMLByte* const      toFill
+                                , MemoryManager* const manager)
 {
     if ( !theFile || !toFill )
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		    XMLExcepts::CPtr_PointerIsZero, manager);
     if (toRead == 0)
 	return 0;
     size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
 
     if(ferror((FILE*)theFile))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile, manager);
     }
 
     return (unsigned int)noOfItemsRead;
@@ -361,13 +402,14 @@ XMLPlatformUtils::readFileBuffer( FileHandle          theFile
 
 void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
 					, long                  toWrite
-					, const XMLByte* const  toFlush )
+					, const XMLByte* const  toFlush 
+                    , MemoryManager* const  manager)
 {
     if ( !theFile || !toFlush )
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		    XMLExcepts::CPtr_PointerIsZero, manager);
     if ( toWrite <= 0 )
-	return;
+        return;
 
     const XMLByte* tmpFlush = (const XMLByte*) toFlush;
     size_t bytesWritten = 0;
@@ -378,8 +420,8 @@ void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
 
         if(ferror((FILE*)theFile))
 		{
-			ThrowXML(XMLPlatformUtilsException,
-				XMLExcepts::File_CouldNotWriteToFile);
+			ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+				XMLExcepts::File_CouldNotWriteToFile, manager);
 		}
 
         if (bytesWritten < (size_t) toWrite) //incomplete write
@@ -393,15 +435,16 @@ void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
     }
 }
 
-void XMLPlatformUtils::resetFile(FileHandle theFile)
+void XMLPlatformUtils::resetFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     if (theFile == NULL)
-	ThrowXML(XMLPlatformUtilsException,
-		 XMLExcepts::CPtr_PointerIsZero);
+	ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero, manager);
     // Seek to the start of the file
     if (fseek((FILE*)theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile, manager);
 }
 
 
@@ -417,8 +460,8 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
-    ArrayJanitor<char> janText(newSrc, fgMemoryManager);
+    char* newSrc = XMLString::transcode(srcPath, manager);
+    ArrayJanitor<char> janText(newSrc, manager);
 
     // Use a local buffer that is big enough for the largest legal path
     char absPath[PATH_MAX + 1];
@@ -427,12 +470,13 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
 
     if (!retPath)
     {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
     return XMLString::transcode(absPath, manager);
 }
 
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                                  , MemoryManager* const manager)
 {
     // Check for pathological case of empty path
     if (!toCheck[0])
@@ -452,13 +496,13 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
 
 XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    char  dirBuf[PATH_MAX + 1];
+    char  dirBuf[PATH_MAX + 2];
     char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
     if (!curDir)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
 
     return XMLString::transcode(curDir, manager);
@@ -491,7 +535,7 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
 //  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
 
-static XMLMutex atomicOpsMutex;
+static XMLMutex *atomicOpsMutex = 0;
 
 void XMLPlatformUtils::platformInit()
 {
@@ -500,8 +544,12 @@ void XMLPlatformUtils::platformInit()
     // Normally, mutexes are created on first use, but there is a
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
-    if (atomicOpsMutex.fHandle == 0)
-        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if(!atomicOpsMutex)
+    {
+        atomicOpsMutex = new (fgMemoryManager) XMLMutex();
+        if (atomicOpsMutex->fHandle == 0)
+            atomicOpsMutex->fHandle = XMLPlatformUtils::makeMutex();
+    }
 }
 
 void* XMLPlatformUtils::makeMutex()
@@ -512,8 +560,8 @@ void* XMLPlatformUtils::makeMutex()
     pthread_mutexattr_settype(attr, PTHREAD_MUTEX_RECURSIVE);
     if (pthread_mutex_init(mutex, attr))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Mutex_CouldNotCreate);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::Mutex_CouldNotCreate, fgMemoryManager);
     }
     pthread_mutexattr_destroy(attr);
     delete attr;
@@ -527,8 +575,8 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
     {
         if (pthread_mutex_destroy((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotDestroy);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotDestroy, fgMemoryManager);
         }
         delete (pthread_mutex_t*)mtxHandle;
     }
@@ -541,8 +589,8 @@ void XMLPlatformUtils::lockMutex(void* const mtxHandle)
     {
         if (pthread_mutex_lock((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotLock);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotLock, fgMemoryManager);
         }
     }
 }
@@ -554,8 +602,8 @@ void XMLPlatformUtils::unlockMutex(void* const mtxHandle)
     {
         if (pthread_mutex_unlock((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotUnlock);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotUnlock, fgMemoryManager);
         }
     }
 }
@@ -569,7 +617,7 @@ void* XMLPlatformUtils::compareAndSwap(void**            toFill
                                      , const void* const newValue
                                      , const void* const toCompare)
 {
-    XMLMutexLock lockMutex(&atomicOpsMutex);
+    XMLMutexLock lockMutex(atomicOpsMutex);
 
     void *retVal = *toFill;
     if (*toFill == toCompare)
@@ -580,14 +628,14 @@ void* XMLPlatformUtils::compareAndSwap(void**            toFill
 
 int XMLPlatformUtils::atomicIncrement(int &location)
 {
-    XMLMutexLock localLock(&atomicOpsMutex);
+    XMLMutexLock localLock(atomicOpsMutex);
 
     return ++location;
 }
 
 int XMLPlatformUtils::atomicDecrement(int &location)
 {
-    XMLMutexLock localLock(&atomicOpsMutex);
+    XMLMutexLock localLock(atomicOpsMutex);
 
     return --location;
 }
@@ -641,8 +689,10 @@ void XMLPlatformUtils::platformTerm()
 {
 #if !defined(APP_NO_THREADS)
     // delete the mutex we created
-	closeMutex(atomicOpsMutex.fHandle);
-	atomicOpsMutex.fHandle = 0;
+	closeMutex(atomicOpsMutex->fHandle);
+	atomicOpsMutex->fHandle = 0;
+    delete atomicOpsMutex;
+    atomicOpsMutex = 0;
 #endif
 }
 
