@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: MacOSPlatformUtils.hpp,v 1.9 2003/05/21 02:43:07 jberry Exp $
+ * $Id: MacOSPlatformUtils.hpp,v 1.11 2003/08/27 16:41:56 jberry Exp $
  */
 
 #pragma once
@@ -98,16 +98,18 @@ XMLUTIL_EXPORT XMLMacAbstractFile* XMLMakeMacFile(void);
 //	Convert fom FSRef/FSSpec to a Unicode character string path.
 //	Note that you'll need to delete [] that string after you're done with it!
 XMLUTIL_EXPORT XMLCh*	XMLCreateFullPathFromFSRef(const FSRef& startingRef,
-                            MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+                            MemoryManager* const manager = XMLPlatformUtils::fgArrayMemoryManager);
 XMLUTIL_EXPORT XMLCh*	XMLCreateFullPathFromFSSpec(const FSSpec& startingSpec,
-                            MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+                            MemoryManager* const manager = XMLPlatformUtils::fgArrayMemoryManager);
 
 //	Convert from path to FSRef/FSSpec
 //	You retain ownership of the pathName.
+//	Note: in the general case, these routines will fail if the specified file
+//	      does not exist when the routine is called.
 XMLUTIL_EXPORT bool	XMLParsePathToFSRef(const XMLCh* const pathName, FSRef& ref,
-                            MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+                            MemoryManager* const manager = XMLPlatformUtils::fgArrayMemoryManager);
 XMLUTIL_EXPORT bool	XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec,
-                            MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
+                            MemoryManager* const manager = XMLPlatformUtils::fgArrayMemoryManager);
 
 //	These routines copy characters between their representation in the Unicode Converter
 //	and the representation used by XMLCh. Until a recent change in Xerces, these were
@@ -136,7 +138,6 @@ extern bool gHasHFSPlusAPIs;
 extern bool gHasFSPathAPIs;
 extern bool gPathAPIsUsePosixPaths;
 extern bool gHasMPAPIs;
-extern bool gMacOSX;
 extern bool gUsePosixFiles;
 
 XERCES_CPP_NAMESPACE_END
