@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  * 
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -89,6 +89,7 @@
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
 #include    <xercesc/util/PanicHandler.hpp>
+#include    <xercesc/util/OutOfMemoryException.hpp>
 
 #if defined(XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -127,8 +128,8 @@ static void WriteCharStr( FILE* stream, const char* const toWrite)
 
 static void WriteUStrStdErr( const XMLCh* const toWrite)
 {
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
+    char* tmpVal = XMLString::transcode(toWrite, XMLPlatformUtils::fgMemoryManager);
+    ArrayJanitor<char> janText(tmpVal, XMLPlatformUtils::fgMemoryManager);
     if (fputs(tmpVal, stderr) == EOF)
     {
        ThrowXML(XMLPlatformUtilsException,
@@ -138,8 +139,8 @@ static void WriteUStrStdErr( const XMLCh* const toWrite)
 
 static void WriteUStrStdOut( const XMLCh* const toWrite)
  {
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
+    char* tmpVal = XMLString::transcode(toWrite, XMLPlatformUtils::fgMemoryManager);
+    ArrayJanitor<char> janText(tmpVal, XMLPlatformUtils::fgMemoryManager);
     if (fputs(tmpVal, stdout) == EOF)
     {
         ThrowXML(XMLPlatformUtilsException,
@@ -177,7 +178,10 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
         retVal = new InMemMsgLoader(msgDomain);
 #endif
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         panic(PanicHandler::Panic_CantLoadMsgDomain);
@@ -220,66 +224,70 @@ void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
 
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile
+                                          , MemoryManager* const manager)
 {
     if (theFile == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     int curPos = ftell( (FILE*)theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize, manager);
 
     return (unsigned int)curPos;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+void XMLPlatformUtils::closeFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     if (theFile == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     if (fclose((FILE*)theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile, manager);
 }
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile
+                                        , MemoryManager* const manager)
 {
     if (theFile == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     // Get the current position
     long  int curPos = ftell((FILE*) theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos, manager);
 
     // Seek to the end and save that value for return
      if (fseek((FILE*) theFile, 0, SEEK_END))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     long int retVal = ftell((FILE*)theFile);
     if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     // And put the pointer back
 
     if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos, manager);
 
     return (unsigned int)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName
+                                      , MemoryManager* const manager)
 {
     if (fileName == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
     FileHandle retVal = (FILE*)fopen( tmpFileName , "r+" );
 
     if (retVal == NULL)
@@ -287,11 +295,12 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+FileHandle XMLPlatformUtils::openFile(const char* const fileName
+                                      , MemoryManager* const manager)
 {
     if (fileName == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     FileHandle retVal = (FILE*)fopen( fileName , "r+" );
 
     if (retVal == NULL)
@@ -299,13 +308,14 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                             , MemoryManager* const manager)
 {
     if (fileName == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
 
     FileHandle retVal = (FILE*)fopen( tmpFileName, "r+" );
     if (retVal == NULL)
@@ -313,11 +323,12 @@ FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                             , MemoryManager* const manager)
 {
     if (fileName == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     FileHandle retVal = (FILE*)fopen( fileName, "r+" );
 
     if (retVal == NULL)
@@ -325,33 +336,34 @@ FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
     return retVal;
 }
 
-FileHandle XMLPlatformUtils::openStdInHandle()
+FileHandle XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
     int nfd = dup(0);
     if (nfd == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                XMLExcepts::File_CouldNotDupHandle);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                XMLExcepts::File_CouldNotDupHandle, manager);
     return (FileHandle)fdopen(dup(0), "r");
 }
 
 
 
 unsigned int
-XMLPlatformUtils::readFileBuffer( FileHandle          theFile
-                                , const unsigned int  toRead
-                                , XMLByte* const      toFill)
+XMLPlatformUtils::readFileBuffer( FileHandle           theFile
+                                , const unsigned int   toRead
+                                , XMLByte* const       toFill
+                                , MemoryManager* const manager)
 {
     if ( !theFile || !toFill )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     if (toRead == 0)
         return 0;
     size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
 
     if(ferror((FILE*)theFile))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile, manager);
     }
 
     return (unsigned int)noOfItemsRead;
@@ -359,11 +371,12 @@ XMLPlatformUtils::readFileBuffer( FileHandle          theFile
 
 void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                         , long                  toWrite
-                                        , const XMLByte* const  toFlush )
+                                        , const XMLByte* const  toFlush 
+                                        , MemoryManager* const  manager)
 {
     if ( !theFile || !toFlush )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     if ( toWrite <= 0 )
         return;
 
@@ -377,11 +390,11 @@ void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
         if(ferror((FILE*)theFile))
         {
 #if 0
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::File_CouldNotWriteToFile);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::File_CouldNotWriteToFile, manager);
 #else
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::File_CouldNotReadFromFile);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::File_CouldNotReadFromFile, manager);
 #endif
         }
 
@@ -396,15 +409,16 @@ void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
     }
 }
 
-void XMLPlatformUtils::resetFile(FileHandle theFile)
+void XMLPlatformUtils::resetFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     if (theFile == NULL)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::CPtr_PointerIsZero);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::CPtr_PointerIsZero, manager);
     // Seek to the start of the file
     if (fseek((FILE*)theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile, manager);
 }
 
 
@@ -421,7 +435,8 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
     return (unsigned long)(t.tv_sec*1000 + t.tv_usec);
 }
 
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
 
     //
@@ -429,23 +444,23 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
+    char* newSrc = XMLString::transcode(srcPath, manager);
+    ArrayJanitor<char> janText(newSrc, manager);
 
     // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[1024];
+    char absPath[1024];
     // get the absolute path 
-    char* retPath = realpath(newSrc, absPath);  
-    ArrayJanitor<char> janText2(retPath);
+    char* retPath = realpath(newSrc, absPath);
     
     if (!retPath)
     {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(absPath, manager);
 }
 
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                                  , MemoryManager* const manager)
 {
     // Check for pathological case of empty path
     if (!toCheck[0])
@@ -463,18 +478,18 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-XMLCh* XMLPlatformUtils::getCurrentDirectory()
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    char  dirBuf[PATH_MAX + 1];
+    char  dirBuf[PATH_MAX + 2];
     char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
     if (!curDir)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
 
-    return XMLString::transcode(curDir);
+    return XMLString::transcode(curDir, manager);
 }
 
 inline bool XMLPlatformUtils::isAnySlash(XMLCh c)
@@ -493,7 +508,7 @@ inline bool XMLPlatformUtils::isAnySlash(XMLCh c)
 //  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
 
-static XMLMutex atomicOpsMutex;
+static XMLMutex *atomicOpsMutex = 0;
 
 void XMLPlatformUtils::platformInit()
 {
@@ -503,8 +518,12 @@ void XMLPlatformUtils::platformInit()
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
 
-    if (atomicOpsMutex.fHandle == 0)
-        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if(!atomicOpsMutex)
+    {
+        atomicOpsMutex = new (fgMemoryManager) XMLMutex();
+        if (atomicOpsMutex->fHandle == 0)
+            atomicOpsMutex->fHandle = XMLPlatformUtils::makeMutex();
+    }
 }
 
 void* XMLPlatformUtils::makeMutex()
@@ -515,8 +534,8 @@ void* XMLPlatformUtils::makeMutex()
     pthread_mutexattr_settype(attr, PTHREAD_MUTEX_RECURSIVE);
     if (pthread_mutex_init(mutex, attr))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Mutex_CouldNotCreate);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::Mutex_CouldNotCreate, fgMemoryManager);
     }
     pthread_mutexattr_destroy(attr);
     delete attr;
@@ -530,8 +549,8 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
     {
         if (pthread_mutex_destroy((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotDestroy);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotDestroy, fgMemoryManager);
         }
         delete (pthread_mutex_t*)mtxHandle;
     }
@@ -544,8 +563,8 @@ void XMLPlatformUtils::lockMutex(void* const mtxHandle)
     {
         if (pthread_mutex_lock((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotLock);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotLock, fgMemoryManager);
         }
     }
 }
@@ -557,8 +576,8 @@ void XMLPlatformUtils::unlockMutex(void* const mtxHandle)
     {
         if (pthread_mutex_unlock((pthread_mutex_t*) mtxHandle))
         {
-            ThrowXML(XMLPlatformUtilsException,
-                     XMLExcepts::Mutex_CouldNotUnlock);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                     XMLExcepts::Mutex_CouldNotUnlock, fgMemoryManager);
         }
     }
 }
@@ -572,7 +591,7 @@ void* XMLPlatformUtils::compareAndSwap(void**            toFill
                                      , const void* const newValue
                                      , const void* const toCompare)
 {
-    XMLMutexLock lockMutex(&atomicOpsMutex);
+    XMLMutexLock lockMutex(atomicOpsMutex);
 
     void *retVal = *toFill;
     if (*toFill == toCompare)
@@ -583,14 +602,14 @@ void* XMLPlatformUtils::compareAndSwap(void**            toFill
 
 int XMLPlatformUtils::atomicIncrement(int &location)
 {
-    XMLMutexLock localLock(&atomicOpsMutex);
+    XMLMutexLock localLock(atomicOpsMutex);
 
     return ++location;
 }
 
 int XMLPlatformUtils::atomicDecrement(int &location)
 {
-    XMLMutexLock localLock(&atomicOpsMutex);
+    XMLMutexLock localLock(atomicOpsMutex);
 
     return --location;
 }
@@ -644,8 +663,10 @@ void XMLPlatformUtils::platformTerm()
 {
 #if !defined(APP_NO_THREADS)
     // delete the mutex we created
-        closeMutex(atomicOpsMutex.fHandle);
-        atomicOpsMutex.fHandle = 0;
+        closeMutex(atomicOpsMutex->fHandle);
+        atomicOpsMutex->fHandle = 0;
+        delete atomicOpsMutex;
+        atomicOpsMutex = 0;
 #endif
 }
 
