@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: SolarisPlatformUtils.cpp,v 1.14 2003/05/15 18:37:49 knoaman Exp $
+ * $Id: SolarisPlatformUtils.cpp,v 1.25 2004/01/29 11:51:20 cargilld Exp $
  */
 
 
@@ -82,6 +82,9 @@
 #include    <link.h>
 #include    <limits.h>
 #include    <dlfcn.h>
+#include    <sys/types.h>
+#include    <sys/stat.h>
+#include    <fcntl.h>
 #include    <xercesc/util/Janitor.hpp>
 #include    <xercesc/util/PlatformUtils.hpp>
 #include    <xercesc/util/RuntimeException.hpp>
@@ -90,6 +93,7 @@
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
 #include    <xercesc/util/PanicHandler.hpp>
+#include    <xercesc/util/OutOfMemoryException.hpp>
 
 #if defined (XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -150,6 +154,10 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
         retVal = new InMemMsgLoader(msgDomain);
 #endif
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         panic(PanicHandler::Panic_CantLoadMsgDomain);
@@ -193,94 +201,102 @@ void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile
+                                          , MemoryManager* const manager)
 {
     // Get the current position
-    int curPos = ftell( (FILE*)theFile);
+    int curPos = tell( (int)theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize, manager);
 
     return (unsigned int)curPos;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+void XMLPlatformUtils::closeFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
-    if (fclose((FILE*) theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
+    if (close((int) theFile))
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile, manager);
 }
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile
+                                        , MemoryManager* const manager)
 {
     // Get the current position
-    long  int curPos = ftell((FILE*) theFile);
+    long  int curPos = tell((int) theFile);
     if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos, manager);
 
     // Seek to the end and save that value for return
-    if (fseek( (FILE*) theFile, 0, SEEK_END) )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+    if (lseek( (int) theFile, 0, SEEK_END) == (off_t)-1)
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
-    long int retVal = ftell((FILE*) theFile);
+    long int retVal = tell((int) theFile);
     if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd, manager);
 
     // And put the pointer back
-    if (fseek((FILE*) theFile, curPos, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
+    if (lseek((int) theFile, curPos, SEEK_SET) == (off_t)-1)
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos, manager);
 
     return (unsigned int)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+FileHandle XMLPlatformUtils::openFile(const char* const fileName
+                                      , MemoryManager* const manager)
 {
-    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
+    int retVal = open( fileName , O_RDONLY | O_LARGEFILE);
 
-    if (retVal == NULL)
+    if (retVal == -1)
         return 0;
-    return retVal;
+    return (FileHandle)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName
+                                      , MemoryManager* const manager)
 {
-    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
-    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
+    int retVal = open( tmpFileName , O_RDONLY | O_LARGEFILE);
 
-    if (retVal == NULL)
+    if (retVal == -1)
         return 0;
-    return retVal;
+    return (FileHandle)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                             , MemoryManager* const manager)
 {
-    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
-    return fopen( tmpFileName , "wb" );
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
+    return (FileHandle)open( tmpFileName , O_WRONLY | O_CREAT | O_LARGEFILE, 0666);
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                             , MemoryManager* const manager)
 {
-    return fopen( fileName , "wb" );
+    return (FileHandle)open( fileName , O_WRONLY | O_CREAT | O_LARGEFILE, 0666);
 }
 
 unsigned int
 XMLPlatformUtils::readFileBuffer(FileHandle              theFile
                                , const unsigned int      toRead
-                               , XMLByte* const          toFill)
+                               , XMLByte* const          toFill
+                               , MemoryManager* const    manager)
 {
-    size_t noOfItemsRead =
-               fread((void*) toFill, 1, toRead, (FILE*) theFile);
+    ssize_t noOfItemsRead =
+      read((int) theFile, (void*) toFill, toRead);
 
-    if(ferror((FILE*) theFile))
+    if(noOfItemsRead == -1)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile, manager);
     }
 
     return (unsigned int) noOfItemsRead;
@@ -289,7 +305,8 @@ XMLPlatformUtils::readFileBuffer(FileHandle              theFile
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)
+                                   , const XMLByte* const  toFlush
+                                   , MemoryManager*  const manager)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -297,15 +314,15 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
         return;
 
     const XMLByte* tmpFlush = (const XMLByte*) toFlush;
-    size_t bytesWritten = 0;
+    ssize_t bytesWritten = 0;
 
     while (true)
     {
-        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+      bytesWritten=write((int)theFile, (const void *)tmpFlush, toWrite);
 
-        if(ferror((FILE*)theFile))
+        if(bytesWritten == -1)
         {
-            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile, manager);
         }
 
         if (bytesWritten < toWrite) //incomplete write
@@ -321,12 +338,13 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
     return;
 }
 
-void XMLPlatformUtils::resetFile(FileHandle theFile)
+void XMLPlatformUtils::resetFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
     // Seek to the start of the file
-    if (fseek((FILE*) theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
+    if (lseek((int) theFile, 0, SEEK_SET) == -1)
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile, manager);
 }
 
 
@@ -341,8 +359,8 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
-    ArrayJanitor<char> janText(newSrc, fgMemoryManager);
+    char* newSrc = XMLString::transcode(srcPath, manager);
+    ArrayJanitor<char> janText(newSrc, manager);
 
     // Use a local buffer that is big enough for the largest legal path
     char absPath[PATH_MAX + 1];
@@ -351,12 +369,13 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
 
     if (!retPath)
     {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
     return XMLString::transcode(absPath, manager);
 }
 
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                                  , MemoryManager* const manager)
 {
     // Check for pathological case of empty path
     if (!toCheck[0])
@@ -376,13 +395,13 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
 
 XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    char  dirBuf[PATH_MAX + 1];
+    char  dirBuf[PATH_MAX + 2];
     char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
     if (!curDir)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
 
     return XMLString::transcode(curDir, manager);
@@ -454,15 +473,15 @@ public:
 
     RecursiveMutex() {
                if (pthread_mutex_init(&mutex, NULL))
-                ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate);
+                   ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate, XMLPlatformUtils::fgMemoryManager);
                        recursionCount = 0;
                        tid = 0;
-                     };
+                     }
 
     ~RecursiveMutex() {
             if (pthread_mutex_destroy(&mutex))
-                ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
-                      };
+                ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy, XMLPlatformUtils::fgMemoryManager);
+                      }
 
      void lock()      {
               if (pthread_equal(tid, pthread_self()))
@@ -471,10 +490,10 @@ public:
                   return;
               }
               if (pthread_mutex_lock(&mutex) != 0)
-                  ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock);
+                  ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock, XMLPlatformUtils::fgMemoryManager);
               tid = pthread_self();
               recursionCount = 1;
-              };
+              }
 
 
      void unlock()    {
@@ -482,9 +501,9 @@ public:
                               return;
 
               if (pthread_mutex_unlock(&mutex) != 0)
-                  ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock);
+                  ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock, XMLPlatformUtils::fgMemoryManager);
                           tid = 0;
-                       };
+                       }
    };
 #endif // ifndef XML_USE_DCE
 
@@ -494,23 +513,23 @@ void* XMLPlatformUtils::makeMutex()
     pthread_mutex_t* mutex = new pthread_mutex_t;
     if (mutex ==  NULL)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Mutex_CouldNotCreate);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::Mutex_CouldNotCreate, fgMemoryManager);
     }
     pthread_mutexattr_t attr;
     pthread_mutexattr_create(&attr);
     pthread_mutexattr_setkind_np(&attr, MUTEX_RECURSIVE_NP);
     if (pthread_mutex_init(mutex, attr))
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Mutex_CouldNotCreate);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::Mutex_CouldNotCreate, fgMemoryManager);
     }
     pthread_mutexattr_delete(&attr);
     return (void*)(mutex);
 #else
     return new RecursiveMutex;
 #endif
-};
+}
 
 
 void XMLPlatformUtils::closeMutex(void* const mtxHandle)
@@ -524,7 +543,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
     RecursiveMutex *rm = (RecursiveMutex *)mtxHandle;
 #endif
     delete rm;
-};
+}
 
 
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
@@ -657,9 +676,9 @@ int XMLPlatformUtils::atomicDecrement(int &location)
 
 
 
-FileHandle XMLPlatformUtils::openStdInHandle()
+FileHandle XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
-        return (FileHandle)fdopen(dup(0), "rb");
+        return (FileHandle)dup(0);
 }
 
 void XMLPlatformUtils::platformTerm()
