@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Tru64PlatformUtils.cpp,v 1.11 2003/10/01 16:32:40 neilg Exp $
+ * $Id: Tru64PlatformUtils.cpp,v 1.17 2004/01/29 11:51:21 cargilld Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -198,51 +198,55 @@ void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile
+                                          , MemoryManager* const manager)
 {
   // Get the current position
   int curPos = ftell( (FILE*)theFile);
   if (curPos == -1)
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotGetSize);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotGetSize, manager);
 
   return (unsigned int)curPos;
 }
 
-void XMLPlatformUtils::closeFile(FileHandle theFile)
+void XMLPlatformUtils::closeFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
   if (fclose((FILE*) theFile))
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotCloseFile);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotCloseFile, manager);
 }
 
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile
+                                        , MemoryManager* const manager)
 {
   // Get the current position
   long  int curPos = ftell((FILE*) theFile);
   if (curPos == -1)
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotGetCurPos);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotGetCurPos, manager);
 
   // Seek to the end and save that value for return
   if (fseek( (FILE*) theFile, 0, SEEK_END) )
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotSeekToEnd);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotSeekToEnd, manager);
 
   long int retVal = ftell((FILE*) theFile);
   if (retVal == -1)
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotSeekToEnd);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotSeekToEnd, manager);
 
   // And put the pointer back
   if (fseek((FILE*) theFile, curPos, SEEK_SET))
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotSeekToPos);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotSeekToPos, manager);
 
   return (unsigned int)retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+FileHandle XMLPlatformUtils::openFile(const char* const fileName
+                                      , MemoryManager* const manager)
 {
   FileHandle retVal = (FILE*)fopen( fileName , "rb" );
 
@@ -251,10 +255,11 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
   return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName
+                                      , MemoryManager* const manager)
 {
-  const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-  ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+  const char* tmpFileName = XMLString::transcode(fileName, manager);
+  ArrayJanitor<char> janText((char*)tmpFileName, manager);
   FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
 
   if (retVal == NULL)
@@ -262,29 +267,32 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
   return retVal;
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                             , MemoryManager* const manager)
 {
-    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
-    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+    const char* tmpFileName = XMLString::transcode(fileName, manager);
+    ArrayJanitor<char> janText((char*)tmpFileName, manager);
     return fopen( tmpFileName , "wb" );
 }
 
-FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                             , MemoryManager* const manager)
 {
     return fopen( fileName , "wb" );
 }
 
 unsigned int XMLPlatformUtils::readFileBuffer(FileHandle theFile,
 					      const unsigned int toRead,
-					      XMLByte* const toFill)
+					      XMLByte* const toFill
+                          , MemoryManager* const manager)
 {
   size_t noOfItemsRead =
     fread((void*) toFill, 1, toRead, (FILE*) theFile);
 
   if(ferror((FILE*) theFile))
   {
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotReadFromFile);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotReadFromFile, manager);
   }
 
   return (unsigned int) noOfItemsRead;
@@ -293,7 +301,8 @@ unsigned int XMLPlatformUtils::readFileBuffer(FileHandle theFile,
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)
+                                   , const XMLByte* const  toFlush
+                                   , MemoryManager* const  manager)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -309,7 +318,7 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
 
         if(ferror((FILE*)theFile))
         {
-            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+            ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile, manager);
         }
 
         if (bytesWritten < toWrite) //incomplete write
@@ -325,12 +334,13 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
     return;
 }
 
-void XMLPlatformUtils::resetFile(FileHandle theFile)
+void XMLPlatformUtils::resetFile(FileHandle theFile
+                                 , MemoryManager* const manager)
 {
   // Seek to the start of the file
   if (fseek((FILE*) theFile, 0, SEEK_SET))
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotResetFile);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotResetFile, manager);
 }
 
 
@@ -345,8 +355,8 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
   //  so we know that its not some pathological freaky path. It comes in
   //  in native format, and goes out as Unicode always
   //
-  char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
-  ArrayJanitor<char> janText(newSrc, fgMemoryManager);
+  char* newSrc = XMLString::transcode(srcPath, manager);
+  ArrayJanitor<char> janText(newSrc, manager);
 
   // Use a local buffer that is big enough for the largest legal path
   char absPath[PATH_MAX + 1];
@@ -355,13 +365,14 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
 
   if (!retPath)
   {
-    ThrowXML(XMLPlatformUtilsException,
-	     XMLExcepts::File_CouldNotGetBasePathName);
+    ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	     XMLExcepts::File_CouldNotGetBasePathName, manager);
   }
   return XMLString::transcode(absPath, manager);
 }
 
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                                  , MemoryManager* const manager)
 {
   // Check for pathological case of empty path
   if (!toCheck[0])
@@ -381,13 +392,13 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
 
 XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    char  dirBuf[PATH_MAX + 1];
+    char  dirBuf[PATH_MAX + 2];
     char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
     if (!curDir)
     {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetBasePathName);
+        ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName, manager);
     }
 
     return XMLString::transcode(curDir, manager);
@@ -453,16 +464,16 @@ public:
   RecursiveMutex()
   {
     if (pthread_mutex_init(&mutex, NULL))
-      ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate);
+      ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotCreate, XMLPlatformUtils::fgMemoryManager);
     recursionCount = 0;
     tid = 0;
-  };
+  }
 
   ~RecursiveMutex()
   {
     if (pthread_mutex_destroy(&mutex))
-      ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy);
-  };
+      ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotDestroy, XMLPlatformUtils::fgMemoryManager);
+  }
 
   void lock()
   {
@@ -472,10 +483,10 @@ public:
       return;
     }
     if (pthread_mutex_lock(&mutex) != 0)
-      ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock);
+      ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotLock, XMLPlatformUtils::fgMemoryManager);
     tid = pthread_self();
     recursionCount = 1;
-  };
+  }
 
 
   void unlock()
@@ -484,15 +495,15 @@ public:
       return;
 
     if (pthread_mutex_unlock(&mutex) != 0)
-      ThrowXML(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock);
+      ThrowXMLwithMemMgr(XMLPlatformUtilsException, XMLExcepts::Mutex_CouldNotUnlock, XMLPlatformUtils::fgMemoryManager);
     tid = 0;
-  };
+  }
 };
 
 void* XMLPlatformUtils::makeMutex()
 {
   return new RecursiveMutex;
-};
+}
 
 
 void XMLPlatformUtils::closeMutex(void* const mtxHandle)
@@ -501,7 +512,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
     return;
   RecursiveMutex *rm = (RecursiveMutex *)mtxHandle;
   delete rm;
-};
+}
 
 
 void XMLPlatformUtils::lockMutex(void* const mtxHandle)
@@ -625,7 +636,7 @@ int XMLPlatformUtils::atomicDecrement(int &location)
 
 
 
-FileHandle XMLPlatformUtils::openStdInHandle()
+FileHandle XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
   return (FileHandle)fdopen(dup(0), "rb");
 }
