@@ -56,6 +56,12 @@
 
 /*
  * $Log: TransService.hpp,v $
+ * Revision 1.12  2004/01/29 11:48:46  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.11  2003/12/24 15:24:13  cargilld
+ * More updates to memory management so that the static memory manager.
+ *
  * Revision 1.10  2003/11/24 19:52:06  neilg
  * allow classes derived from XMLTransService to tailor the intrinsic maps to their taste.
  *
@@ -485,6 +491,9 @@ protected :
     // -----------------------------------------------------------------------
     //  Protected helper methods
     // -----------------------------------------------------------------------
+    // As the body of this function is commented out it could be removed.
+    // However, currently all calls to it are guarded by #if defined(XERCES_DEBUG)
+    // so will leave it for now.
     void checkBlockSize(const unsigned int toCheck);
 
 
@@ -536,9 +545,11 @@ public :
     //          its assumed that the buffer is physically one char or byte
     //          larger.
     // -----------------------------------------------------------------------
-    virtual unsigned int calcRequiredSize(const char* const srcText) = 0;
+    virtual unsigned int calcRequiredSize(const char* const srcText
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
 
-    virtual unsigned int calcRequiredSize(const XMLCh* const srcText) = 0;
+    virtual unsigned int calcRequiredSize(const XMLCh* const srcText
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
 
     virtual char* transcode(const XMLCh* const toTranscode) = 0;
     virtual char* transcode(const XMLCh* const toTranscode,
@@ -553,6 +564,7 @@ public :
         const   char* const     toTranscode
         ,       XMLCh* const    toFill
         , const unsigned int    maxChars
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     ) = 0;
 
     virtual bool transcode
@@ -560,6 +572,7 @@ public :
         const   XMLCh* const    toTranscode
         ,       char* const     toFill
         , const unsigned int    maxBytes
+        , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     ) = 0;
 
 
