@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,7 +57,7 @@
 /**
  * @01A D998714.1 V5R2M0    100301   Swan    :Fix error return flags
  * @02A           V5R2M0    200419   jrhansen : support lowercase function
- * $Id: Iconv400TransService.cpp,v 1.5 2002/05/21 20:36:33 tng Exp $
+ * $Id: Iconv400TransService.cpp,v 1.12 2004/01/29 11:51:21 cargilld Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -74,6 +74,8 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Janitor.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  Local functions
 // ---------------------------------------------------------------------------
@@ -82,13 +84,17 @@
 //  When XMLCh and ICU's UChar are not the same size, we have to do a temp
 //  conversion of all strings. These local helper methods make that easier.
 //
-static UChar* convertToUChar(   const   XMLCh* const    toConvert
-                                , const unsigned int    srcLen = 0)
+static UChar* convertToUChar( const   XMLCh* const toConvert
+                            , const unsigned int   srcLen = 0
+                            , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager)
 {
     const unsigned int actualLen = srcLen
                                    ? srcLen : XMLString::stringLen(toConvert);
 
-    UChar* tmpBuf = new UChar[srcLen + 1];
+    UChar* tmpBuf = (UChar*) manager->allocate
+    (
+        (srcLen + 1) * sizeof(UChar)
+    );//new UChar[srcLen + 1];
     const XMLCh* srcPtr = toConvert;
     UChar* outPtr = tmpBuf;
     while (*srcPtr)
@@ -314,7 +320,8 @@ XMLCh Iconv400TransService::toUnicodeLower(XMLCh comp1) const
 XMLTranscoder*
 Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            encodingName
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
     UErrorCode uerr = U_ZERO_ERROR;
     UConverter* converter = ucnv_openU(encodingName, &uerr);
@@ -323,7 +330,7 @@ Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            enc
         resValue = XMLTransService::UnsupportedEncoding;
         return 0;
     }
-    return new Iconv400Transcoder(encodingName, converter, blockSize);
+    return new (manager) Iconv400Transcoder(encodingName, converter, blockSize, manager);
 }
 
 
@@ -332,18 +339,19 @@ Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            enc
 // ---------------------------------------------------------------------------
 //  IconvTranscoder: Constructors and Destructor
 // ---------------------------------------------------------------------------
-Iconv400Transcoder::Iconv400Transcoder(const  XMLCh* const        encodingName
-                            ,       UConverter* const   toAdopt
-                            , const unsigned int        blockSize) :
+Iconv400Transcoder::Iconv400Transcoder( const XMLCh* const         encodingName
+                                      ,       UConverter* const    toAdopt
+                                      , const unsigned int         blockSize
+                                      ,       MemoryManager* const manager) :
 
-    XMLTranscoder(encodingName, blockSize)
+    XMLTranscoder(encodingName, blockSize, manager)
     , fConverter(toAdopt)
     , fFixed(false)
     , fSrcOffsets(0)
 {
     // If there is a block size, then allocate our source offset array
     if (blockSize)
-        fSrcOffsets = new long[blockSize];
+        fSrcOffsets = (long*) manager->allocate(blockSize * sizeof(long));//new long[blockSize];
 
     // Remember if its a fixed size encoding
     fFixed = (ucnv_getMaxCharSize(fConverter) == ucnv_getMinCharSize(fConverter));
@@ -351,7 +359,7 @@ Iconv400Transcoder::Iconv400Transcoder(const  XMLCh* const        encodingName
 
 Iconv400Transcoder::~Iconv400Transcoder()
 {
-    delete [] fSrcOffsets;
+    getMemoryManager()->deallocate(fSrcOffsets);//delete [] fSrcOffsets;
 
     // If there is a converter, ask Iconv400 to clean it up
     if (fConverter)
@@ -363,124 +371,6 @@ Iconv400Transcoder::~Iconv400Transcoder()
 }
 
 
-
-XMLCh Iconv400Transcoder::transcodeOne(  const   XMLByte* const  srcData
-                                    , const unsigned int    srcBytes
-                                    ,       unsigned int&   bytesEaten)
-{
-    // Check for stupid stuff
-    if (!srcBytes)
-        return 0;
-
-    UErrorCode err = U_ZERO_ERROR;
-    const XMLByte* startSrc = srcData;
-    const XMLCh chRet = ucnv_getNextUChar
-    (
-        fConverter
-        , (const char**)&startSrc
-        , (const char*)((srcData + srcBytes) - 1)
-        , &err
-    );
-
-    // Bail out if an error
-    if (U_FAILURE(err))
-        return 0;
-
-    // Calculate the bytes eaten and return the char
-    bytesEaten = startSrc - srcData;
-    return chRet;
-}
-
-
-unsigned int
-Iconv400Transcoder::transcodeXML(const   XMLByte* const          srcData
-                            , const unsigned int            srcCount
-                            ,       XMLCh* const            toFill
-                            , const unsigned int            maxChars
-                            ,       unsigned int&           bytesEaten
-                            ,       unsigned char* const    charSizes)
-{
-    // If debugging, insure the block size is legal
-    #if defined(XERCES_DEBUG)
-    checkBlockSize(maxChars);
-    #endif
-
-    // Set up pointers to the source and destination buffers.
-    UChar*          startTarget = toFill;
-    const XMLByte*  startSrc = srcData;
-    const XMLByte*  endSrc = srcData + srcCount;
-
-    //
-    //  Transoode the buffer.  Buffer overflow errors are normal, occuring
-    //  when the raw input buffer holds more characters than will fit in
-    //  the Unicode output buffer.
-    //
-    UErrorCode  err = U_ZERO_ERROR;
-    ucnv_toUnicode
-    (
-        fConverter
-        , &startTarget
-        , toFill + maxChars
-        , (const char**)&startSrc
-        , (const char*)endSrc
-        , (fFixed ? 0 : fSrcOffsets)
-        , false
-        , &err
-    );
-
-    if ((err != U_ZERO_ERROR) && (err != U_BUFFER_OVERFLOW_ERROR))
-        ThrowXML(TranscodingException, XMLExcepts::Trans_Unrepresentable);
-
-    // Calculate the bytes eaten and store in caller's param
-    bytesEaten = startSrc - srcData;
-
-    // And the characters decoded
-    const unsigned int charsDecoded = startTarget - toFill;
-
-    //
-    //  Translate the array of char offsets into an array of character
-    //  sizes, which is what the transcoder interface semantics requires.
-    //  If its fixed, then we can optimize it.
-    //
-    if (fFixed)
-    {
-        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);;
-        memset(charSizes, fillSize, maxChars);
-    }
-     else
-    {
-        //
-        //  We have to convert the series of offsets into a series of
-        //  sizes. If just one char was decoded, then its the total bytes
-        //  eaten. Otherwise, do a loop and subtract out each element from
-        //  its previous element.
-        //
-        if (charsDecoded == 1)
-        {
-            charSizes[0] = (unsigned char)bytesEaten;
-        }
-         else
-        {
-            //  ICU does not return an extra element to allow us to figure
-            //  out the last char size, so we have to compute it from the
-            //  total bytes used.
-            unsigned int index;
-            for (index = 0; index < charsDecoded - 1; index++)
-            {
-                charSizes[index] = (unsigned char)(fSrcOffsets[index + 1]
-                                                    - fSrcOffsets[index]);
-            }
-            if( charsDecoded > 0 ) {
-                charSizes[charsDecoded - 1] = (unsigned char)(bytesEaten
-                                              - fSrcOffsets[charsDecoded - 1]);
-            }
-        }
-    }
-
-    // Return the chars we put into the target buffer
-    return charsDecoded;
-}
-
 // ---------------------------------------------------------------------------
 //  Iconv400Transcoder: The virtual transcoder API
 // ---------------------------------------------------------------------------
@@ -507,7 +397,10 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     if (sizeof(XMLCh) == sizeof(UChar))
         startTarget = (UChar*)toFill;
      else
-        startTarget = new UChar[maxChars];
+        startTarget = (UChar*) getMemoryManager()->allocate
+        (
+            maxChars * sizeof(UChar)
+        );//new UChar[maxChars];
     UChar* orgTarget = startTarget;
 
     //
@@ -531,23 +424,24 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     if ((err != U_ZERO_ERROR) && (err != U_INDEX_OUTOFBOUNDS_ERROR))
     {
         if (orgTarget != (UChar*)toFill)
-            delete [] orgTarget;
+            getMemoryManager()->deallocate(orgTarget);//delete [] orgTarget;
 
         if (fFixed)
         {
-            XMLCh tmpBuf[16];
-            XMLString::binToText((unsigned int)(*startTarget), tmpBuf, 16, 16);
-            ThrowXML2
+            XMLCh tmpBuf[17];
+            XMLString::binToText((unsigned int)(*startTarget), tmpBuf, 16, 16, getMemoryManager());
+            ThrowXMLwithMemMgr2
             (
                 TranscodingException
                 , XMLExcepts::Trans_BadSrcCP
                 , tmpBuf
                 , getEncodingName()
+                , getMemoryManager()
             );
         }
          else
         {
-            ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
         }
     }
 
@@ -564,7 +458,7 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     //
     if (fFixed)
     {
-        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);;
+        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);
         memset(charSizes, fillSize, maxChars);
     }
      else
@@ -609,7 +503,7 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
             *outPtr++ = XMLCh(*startTarget++);
 
         // And delete the temp buffer
-        delete [] orgTarget;
+        getMemoryManager()->deallocate(orgTarget);//delete [] orgTarget;
     }
 
     // Return the chars we put into the target buffer
@@ -638,10 +532,10 @@ Iconv400Transcoder::transcodeTo( const   XMLCh* const    srcData
     }
      else
     {
-        tmpBufPtr = convertToUChar(srcData, srcCount);
+        tmpBufPtr = convertToUChar(srcData, srcCount, getMemoryManager());
         srcPtr = tmpBufPtr;
     }
-    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr);
+    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr, getMemoryManager());
 
     //
     //  Set the appropriate callback so that it will either fail or use
@@ -672,14 +566,15 @@ Iconv400Transcoder::transcodeTo( const   XMLCh* const    srcData
 
     if (err)   /*@01A*/
     {
-        XMLCh tmpBuf[16];
-        XMLString::binToText((unsigned int)*startSrc, tmpBuf, 16, 16);
-        ThrowXML2
+        XMLCh tmpBuf[17];
+        XMLString::binToText((unsigned int)*startSrc, tmpBuf, 16, 16, getMemoryManager());
+        ThrowXMLwithMemMgr2
         (
             TranscodingException
             , XMLExcepts::Trans_Unrepresentable
             , tmpBuf
             , getEncodingName()
+            , getMemoryManager()
         );
     }
 
@@ -760,7 +655,8 @@ Iconv400LCPTranscoder::~Iconv400LCPTranscoder()
 // ---------------------------------------------------------------------------
 //  Iconv400LCPTranscoder: Constructors and Destructor
 // ---------------------------------------------------------------------------
-unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
+unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText
+                                                     , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -787,7 +683,8 @@ unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
     return (unsigned int)targetCap;
 }
 
-unsigned int Iconv400LCPTranscoder::calcRequiredSize(const char* const srcText)
+unsigned int Iconv400LCPTranscoder::calcRequiredSize(const char* const srcText
+                                                     , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -885,6 +782,76 @@ char* Iconv400LCPTranscoder::transcode(const XMLCh* const toTranscode)
     return retBuf;
 }
 
+char* Iconv400LCPTranscoder::transcode(const XMLCh* const toTranscode,
+                                       MemoryManager* const manager)
+{
+    char* retBuf = 0;
+
+    // Check for a couple of special cases
+    if (!toTranscode)
+        return 0;
+
+    if (!*toTranscode)
+    {
+        retBuf = (char*) manager->allocate(sizeof(char));//new char[1];
+        retBuf[0] = 0;
+        return retBuf;
+    }
+
+    // Caculate a return buffer size not too big, but less likely to overflow
+    int32_t targetLen = (int32_t)(u_strlen(toTranscode) * 1.25);
+
+    // Allocate the return buffer
+    retBuf = (char*) manager->allocate((targetLen + 1) * sizeof(char));//new char[targetLen + 1];
+
+    // Lock now while we call the converter.
+    UErrorCode err = U_ZERO_ERROR;
+    int32_t targetCap;
+    {
+        XMLMutexLock lockConverter(&fMutex);
+
+        //Convert the Unicode string to char*
+        targetCap = ucnv_fromUChars
+        (
+            fConverter
+            , retBuf
+            , targetLen + 1
+            , toTranscode
+            , &err
+        );
+    }
+
+    // If targetLen is not enough then buffer overflow might occur
+    if (err == U_BUFFER_OVERFLOW_ERROR)
+    {
+        // Reset the error, delete the old buffer, allocate a new one, and try again
+        err = U_ZERO_ERROR;
+        manager->deallocate(retBuf);//delete [] retBuf;
+        retBuf = (char*) manager->allocate(targetCap * sizeof(char));//new char[targetCap];
+
+        // Lock again before we retry
+        XMLMutexLock lockConverter(&fMutex);
+        targetCap = ucnv_fromUChars
+        (
+            fConverter
+            , retBuf
+            , targetCap
+            , toTranscode
+            , &err
+        );
+    }
+
+    if (U_FAILURE(err))
+    {
+        manager->deallocate(retBuf);//delete [] retBuf;
+        return 0;
+    }
+
+    // Cap it off and return
+    retBuf[targetCap] = 0;
+    return retBuf;
+}
+
 XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode)
 {
     // Watch for a few pyscho corner cases
@@ -956,10 +923,83 @@ XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode)
     return retVal;
 }
 
+XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode,
+                                        MemoryManager* const manager)
+{
+    // Watch for a few pyscho corner cases
+    if (!toTranscode)
+        return 0;
+
+    XMLCh* retVal = 0;
+    if (!*toTranscode)
+    {
+        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh));//new XMLCh[1];
+        retVal[0] = 0;
+        return retVal;
+    }
+
+    //
+    //  Get the length of the string to transcode. The Unicode string will
+    //  almost always be no more chars than were in the source, so this is
+    //  the best guess as to the storage needed.
+    //
+    const int32_t srcLen = (int32_t)strlen(toTranscode);
+    // Allocate unicode string of equivalent length in unicode bytes
+    retVal = (XMLCh*) manager->allocate((srcLen + 1) * sizeof(XMLCh));//new XMLCh[srcLen+1];
+
+    // Now lock while we do these calculations
+    UErrorCode err = U_ZERO_ERROR;
+    {
+        XMLMutexLock lockConverter(&fMutex);
+
+        //
+        //  Here we don't know what the target length will be so use 0 and
+        //  expect an U_BUFFER_OVERFLOW_ERROR in which case it'd get resolved
+        //  by the correct capacity value.
+        //
+        int32_t targetCap;
+        targetCap = ucnv_toUChars
+        (
+            fConverter
+            , retVal
+            , srcLen+1
+            , toTranscode
+            , srcLen
+            , &err
+        );
+
+        if (err != U_BUFFER_OVERFLOW_ERROR)
+	{
+
+        err = U_ZERO_ERROR;
+        retVal = (XMLCh*) manager->allocate(targetCap * sizeof(XMLCh));//new XMLCh[targetCap];
+        ucnv_toUChars
+        (
+            fConverter
+            , retVal
+            , targetCap
+            , toTranscode
+            , srcLen
+            , &err
+           );
+	 }
+   }
+
+    if (U_FAILURE(err))
+    {
+        // Clean up if we got anything allocated
+        manager->deallocate(retVal);//delete [] retVal;
+        return 0;
+    }
+
+    return retVal;
+}
+
 
 bool Iconv400LCPTranscoder::transcode(const  char* const     toTranscode
                                 ,       XMLCh* const    toFill
-                                , const unsigned int    maxChars)
+                                , const unsigned int    maxChars
+                                , MemoryManager* const  manager)
 {
     // Check for a couple of psycho corner cases
     if (!toTranscode || !maxChars)
@@ -999,7 +1039,8 @@ bool Iconv400LCPTranscoder::transcode(const  char* const     toTranscode
 
 bool Iconv400LCPTranscoder::transcode(   const   XMLCh* const    toTranscode
                                     ,       char* const     toFill
-                                    , const unsigned int    maxChars)
+                                    , const unsigned int    maxChars
+                                    , MemoryManager* const  manager)
 {
     // Watch for a few psycho corner cases
     if (!toTranscode || !maxChars)
@@ -1036,3 +1077,5 @@ bool Iconv400LCPTranscoder::transcode(   const   XMLCh* const    toTranscode
     return true;
 }
 
+XERCES_CPP_NAMESPACE_END
+
