@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- *	$Id: MacOSUnicodeConverter.cpp,v 1.15 2003/05/22 14:42:32 jberry Exp $
+ *	$Id: MacOSUnicodeConverter.cpp,v 1.20 2004/01/13 16:34:22 cargilld Exp $
  */
 
 
@@ -655,13 +655,11 @@ MacOSTranscoder::transcodeFrom(  const  XMLByte* const			srcData
     	&bytesProduced,
     	reinterpret_cast<UniCharArrayPtr>(toFill));
 		
-	if (status == kTECUsedFallbacksStatus)
+	if (status == kTECOutputBufferFullStatus || status == kTECUsedFallbacksStatus)
 		status = noErr;
     	
     if (status != noErr)
-    {
         ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
-    }
 	
 	std::size_t charsProduced = bytesProduced / sizeof(UniChar);
 	
@@ -763,7 +761,7 @@ MacOSTranscoder::transcodeTo(const  XMLCh* const    srcData
     {
     	if (status == kTECUnmappableElementErr && options == UnRep_Throw)
     	{
-    		XMLCh tmpBuf[16];
+    		XMLCh tmpBuf[17];
             XMLString::binToText((unsigned int)&srcData[totalCharsConsumed], tmpBuf, 16, 16);
             ThrowXML2
             (
@@ -773,14 +771,6 @@ MacOSTranscoder::transcodeTo(const  XMLCh* const    srcData
                 , getEncodingName()
             );
     	}
-    	
-    	// These were removed because we actually have succeeded in transcoding part of
-    	// the string; this logic should probably be changed to deal with non-transcodable
-    	// text somehow; if options != UnRep_Throw, and we zero these values, we get in
-    	// to an infinite loop and hang the machine
-    	
-    	//totalCharsConsumed = 0;
-    	//totalCharsProduced = 0;
     }
     	
     charsEaten = totalCharsConsumed;
@@ -870,7 +860,8 @@ MacOSLCPTranscoder::~MacOSLCPTranscoder()
 //	extra buffering to avoid this result.
 // ---------------------------------------------------------------------------
 unsigned int
-MacOSLCPTranscoder::calcRequiredSize(const char* const srcText)
+MacOSLCPTranscoder::calcRequiredSize(const char* const srcText
+                                     , MemoryManager* const manager)
 {
 	if (!srcText)
 		return 0;
@@ -937,7 +928,8 @@ MacOSLCPTranscoder::calcRequiredSize(const char* const srcText)
 //	extra buffering to avoid this result.
 // ---------------------------------------------------------------------------
 unsigned int
-MacOSLCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
+MacOSLCPTranscoder::calcRequiredSize(const XMLCh* const srcText
+                                     , MemoryManager* const manager)
 {
 	if (!srcText)
 		return 0;
@@ -1015,8 +1007,9 @@ MacOSLCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
 char*
 MacOSLCPTranscoder::transcode(const XMLCh* const srcText)
 {
-	//	Transcode using the static memory manaager
-	return transcode(srcText, XMLPlatformUtils::fgMemoryManager);
+	//	Transcode using a memory manager that allocates
+	//	memory using new[].
+	return transcode(srcText, XMLPlatformUtils::fgArrayMemoryManager);
 }
 
 
@@ -1134,8 +1127,9 @@ MacOSLCPTranscoder::transcode(const XMLCh* const srcText,
 XMLCh*
 MacOSLCPTranscoder::transcode(const char* const srcText)
 {
-	//	Transcode using the static memory manaager
-	return transcode(srcText, XMLPlatformUtils::fgMemoryManager);
+	//	Transcode using a memory manager that allocates
+	//	memory using new[].
+	return transcode(srcText, XMLPlatformUtils::fgArrayMemoryManager);
 }
 
 
@@ -1240,7 +1234,8 @@ MacOSLCPTranscoder::transcode(const char* const srcText,
 bool
 MacOSLCPTranscoder::transcode( 		 const   char* const	toTranscode
                                     ,       XMLCh* const    toFill
-                                    , const unsigned int    maxChars)
+                                    , const unsigned int    maxChars
+                                    , MemoryManager* const  manager)
 {
     // toFill must contain space for maxChars XMLCh characters + 1 (for terminating NULL).
 
@@ -1303,7 +1298,8 @@ MacOSLCPTranscoder::transcode( 		 const   char* const	toTranscode
 bool
 MacOSLCPTranscoder::transcode( 		const   XMLCh* const    toTranscode
                                     ,       char* const     toFill
-                                    , const unsigned int    maxChars)
+                                    , const unsigned int    maxChars
+                                    , MemoryManager* const  manager)
 {
     //	toFill must contain space for maxChars bytes characters + 1 (for terminating NULL).
 	const XMLCh* src = toTranscode;
