@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Win32TransService.cpp,v 1.14 2003/05/25 21:39:57 knoaman Exp $
+ * $Id: Win32TransService.cpp,v 1.21 2004/01/19 16:06:56 amassari Exp $
  */
 
 
@@ -668,19 +668,20 @@ Win32Transcoder::transcodeFrom( const   XMLByte* const      srcData
         {
             if (toEat == 1)
             {
-                XMLCh tmpBuf[16];
-                XMLString::binToText((unsigned int)(*inPtr), tmpBuf, 16, 16);
-                ThrowXML2
+                XMLCh tmpBuf[17];
+                XMLString::binToText((unsigned int)(*inPtr), tmpBuf, 16, 16, getMemoryManager());
+                ThrowXMLwithMemMgr2
                 (
                     TranscodingException
                     , XMLExcepts::Trans_BadSrcCP
                     , tmpBuf
                     , getEncodingName()
+                    , getMemoryManager()
                 );
             }
              else
             {
-                ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+                ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
             }
         }
 
@@ -749,14 +750,15 @@ Win32Transcoder::transcodeTo(const  XMLCh* const    srcData
         //
         if (usedDef && (options == UnRep_Throw))
         {
-            XMLCh tmpBuf[16];
-            XMLString::binToText((unsigned int)*srcPtr, tmpBuf, 16, 16);
-            ThrowXML2
+            XMLCh tmpBuf[17];
+            XMLString::binToText((unsigned int)*srcPtr, tmpBuf, 16, 16, getMemoryManager());
+            ThrowXMLwithMemMgr2
             (
                 TranscodingException
                 , XMLExcepts::Trans_Unrepresentable
                 , tmpBuf
                 , getEncodingName()
+                , getMemoryManager()
             );
         }
 
@@ -841,40 +843,23 @@ Win32LCPTranscoder::~Win32LCPTranscoder()
 // ---------------------------------------------------------------------------
 //  Win32LCPTranscoder: Implementation of the virtual transcoder interface
 // ---------------------------------------------------------------------------
-unsigned int Win32LCPTranscoder::calcRequiredSize(const char* const srcText)
+unsigned int Win32LCPTranscoder::calcRequiredSize(const char* const srcText
+                                                  , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
 
-    unsigned charLen = ::mblen(srcText, MB_CUR_MAX);
-    if (charLen == -1)
-        return 0;
-    else if (charLen != 0)
-        charLen = strlen(srcText)/charLen;
-
-    if (charLen == -1)
-        return 0;
-    return charLen;
+    return ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, srcText, -1, NULL, 0);
 }
 
 
-unsigned int Win32LCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
+unsigned int Win32LCPTranscoder::calcRequiredSize(const XMLCh* const srcText
+                                                  , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
 
-    unsigned int retVal = -1;
-    
-    #if defined(XML_METROWERKS)
-    	const unsigned int srcLen = ::wcslen(srcText);
-    	retVal = ::wcsmbslen(srcText, srcLen);
-    #else
-     	retVal = ::wcstombs(0, srcText, 0);
-    #endif
-    
-    if (retVal == (unsigned int)-1)
-        return 0;
-    return retVal;
+    return ::WideCharToMultiByte(CP_ACP, 0, srcText, -1, NULL, 0, NULL, NULL);
 }
 
 // Return value using global operator new
@@ -888,21 +873,11 @@ char* Win32LCPTranscoder::transcode(const XMLCh* const toTranscode)
     if (*toTranscode)
     {
         // Calc the needed size
-        unsigned int neededLen;
-        
-       #if defined(XML_METROWERKS)
-	    	const unsigned int srcLen = ::wcslen(toTranscode);
-	    	neededLen = ::wcsmbslen(toTranscode, srcLen);
-	    #else
-	     	neededLen = ::wcstombs(0, toTranscode, 0);
-	    #endif
-
-        if (neededLen == (unsigned int)-1)
-            return 0;
+        const unsigned int neededLen = calcRequiredSize(toTranscode);
 
         // Allocate a buffer of that size plus one for the null and transcode
         retVal = new char[neededLen + 1];
-        ::wcstombs(retVal, toTranscode, neededLen + 1);
+        ::WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)toTranscode, -1, retVal, neededLen+1, NULL, NULL);
 
         // And cap it off anyway just to make sure
         retVal[neededLen] = 0;
@@ -925,13 +900,11 @@ char* Win32LCPTranscoder::transcode(const XMLCh* const toTranscode,
     if (*toTranscode)
     {
         // Calc the needed size
-        const unsigned int neededLen = ::wcstombs(0, toTranscode, 0);
-        if (neededLen == (unsigned int)-1)
-            return 0;
+        const unsigned int neededLen = calcRequiredSize(toTranscode, manager);
 
         // Allocate a buffer of that size plus one for the null and transcode
         retVal = (char*) manager->allocate((neededLen + 1) * sizeof(char)); //new char[neededLen + 1];
-        ::wcstombs(retVal, toTranscode, neededLen + 1);
+        ::WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)toTranscode, -1, retVal, neededLen+1, NULL, NULL);
 
         // And cap it off anyway just to make sure
         retVal[neededLen] = 0;
@@ -965,7 +938,7 @@ XMLCh* Win32LCPTranscoder::transcode(const char* const toTranscode)
 
         // Allocate a buffer of that size plus one for the null and transcode
         retVal = new XMLCh[neededLen + 1];
-        ::mbstowcs(retVal, toTranscode, neededLen + 1);
+        ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, toTranscode, -1, (LPWSTR)retVal, neededLen + 1);
 
         // Cap it off just to make sure. We are so paranoid!
         retVal[neededLen] = 0;
@@ -988,7 +961,7 @@ XMLCh* Win32LCPTranscoder::transcode(const char* const toTranscode,
     if (*toTranscode)
     {
         // Calculate the buffer size required
-        const unsigned int neededLen = calcRequiredSize(toTranscode);
+        const unsigned int neededLen = calcRequiredSize(toTranscode, manager);
         if (neededLen == 0)
         {
             retVal = (XMLCh*) manager->allocate(sizeof(XMLCh)); //new XMLCh[1];
@@ -998,14 +971,14 @@ XMLCh* Win32LCPTranscoder::transcode(const char* const toTranscode,
 
         // Allocate a buffer of that size plus one for the null and transcode
         retVal = (XMLCh*) manager->allocate((neededLen + 1) * sizeof(XMLCh)); //new XMLCh[neededLen + 1];
-        ::mbstowcs(retVal, toTranscode, neededLen + 1);
+        ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, toTranscode, -1, (LPWSTR)retVal, neededLen + 1);
 
         // Cap it off just to make sure. We are so paranoid!
         retVal[neededLen] = 0;
     }
      else
     {
-        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh*)); //new XMLCh[1];
+        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh)); //new XMLCh[1];
         retVal[0] = 0;
     }
     return retVal;
@@ -1014,7 +987,8 @@ XMLCh* Win32LCPTranscoder::transcode(const char* const toTranscode,
 
 bool Win32LCPTranscoder::transcode( const   char* const     toTranscode
                                     ,       XMLCh* const    toFill
-                                    , const unsigned int    maxChars)
+                                    , const unsigned int    maxChars
+                                    , MemoryManager* const  manager)
 {
     // Check for a couple of psycho corner cases
     if (!toTranscode || !maxChars)
@@ -1030,7 +1004,7 @@ bool Win32LCPTranscoder::transcode( const   char* const     toTranscode
     }
 
     // This one has a fixed size output, so try it and if it fails it fails
-    if (::mbstowcs(toFill, toTranscode, maxChars + 1) == size_t(-1))
+    if ( 0 == ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, toTranscode, -1, (LPWSTR)toFill, maxChars + 1) )
         return false;
     return true;
 }
@@ -1038,7 +1012,8 @@ bool Win32LCPTranscoder::transcode( const   char* const     toTranscode
 
 bool Win32LCPTranscoder::transcode( const   XMLCh* const    toTranscode
                                     ,       char* const     toFill
-                                    , const unsigned int    maxBytes)
+                                    , const unsigned int    maxBytes
+                                    , MemoryManager* const  manager)
 {
     // Watch for a couple of pyscho corner cases
     if (!toTranscode || !maxBytes)
@@ -1054,7 +1029,7 @@ bool Win32LCPTranscoder::transcode( const   XMLCh* const    toTranscode
     }
 
     // This one has a fixed size output, so try it and if it fails it fails
-    if (::wcstombs(toFill, toTranscode, maxBytes + 1) == size_t(-1))
+    if ( 0 == ::WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)toTranscode, -1, toFill, maxBytes + 1, NULL, NULL) )
         return false;
 
     // Cap it off just in case
