@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,8 +55,47 @@
  */
 
 /*
- * $Id: XMLDateTime.cpp,v 1.12 2003/05/22 02:10:52 knoaman Exp $
+ * $Id: XMLDateTime.cpp,v 1.25 2004/01/29 11:48:47 cargilld Exp $
  * $Log: XMLDateTime.cpp,v $
+ * Revision 1.25  2004/01/29 11:48:47  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.24  2004/01/25 23:23:26  jberry
+ * Step around CodeWarrior compiler warning
+ *
+ * Revision 1.23  2004/01/13 19:50:56  peiyongz
+ * remove parseContent()
+ *
+ * Revision 1.22  2004/01/13 16:34:20  cargilld
+ * Misc memory management changes.
+ *
+ * Revision 1.21  2004/01/03 00:03:18  peiyongz
+ * parseContent
+ *
+ * Revision 1.20  2003/12/31 02:34:11  neilg
+ * enable production of canonical representations for dates with negative years, or years >9999
+ *
+ * Revision 1.19  2003/12/17 00:18:35  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.18  2003/12/16 22:48:52  peiyongz
+ * exception thrown upon invalid number, thanks Gareth Reakes.
+ *
+ * Revision 1.17  2003/12/11 21:38:12  peiyongz
+ * support for Canonical Representation for Datatype
+ *
+ * Revision 1.16  2003/09/25 22:24:28  peiyongz
+ * Using writeString/readString
+ *
+ * Revision 1.15  2003/09/25 15:22:54  peiyongz
+ * Solve HP complier error
+ *
+ * Revision 1.14  2003/09/23 18:16:07  peiyongz
+ * Inplementation for Serialization/Deserialization
+ *
+ * Revision 1.13  2003/08/14 02:57:27  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.12  2003/05/22 02:10:52  knoaman
  * Default the memory manager.
  *
@@ -113,10 +152,12 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <stdlib.h>
+#include <assert.h>
 #include <xercesc/util/XMLDateTime.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/NumberFormatException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -364,8 +405,8 @@ void XMLDateTime::addDuration(XMLDateTime*             fNewDate
     fNewDate->fValue[utc] = UTC_STD;
 }
 
-int XMLDateTime::compareResult(short resultA
-                             , short resultB
+int XMLDateTime::compareResult(int resultA
+                             , int resultB
                              , bool strict)
 {
 
@@ -408,7 +449,7 @@ int XMLDateTime::compare(const XMLDateTime* const pDate1
         return XMLDateTime::compareOrder(pDate1, pDate2);
     }
 
-    short c1, c2;
+    int c1, c2;
 
     if ( pDate1->isNormalized())
     {
@@ -444,6 +485,7 @@ int XMLDateTime::compareResult(const XMLDateTime* const pDate1
 
 int XMLDateTime::compareOrder(const XMLDateTime* const lValue
                             , const XMLDateTime* const rValue)
+                            //, MemoryManager* const memMgr)
 {
     //
     // If any of the them is not normalized() yet,
@@ -474,13 +516,16 @@ int XMLDateTime::compareOrder(const XMLDateTime* const lValue
 //  ctor and dtor
 // ---------------------------------------------------------------------------
 XMLDateTime::~XMLDateTime()
-{
+{    
     if (fBuffer)
-        fMemoryManager->deallocate(fBuffer);//delete[] fBuffer;
+        fMemoryManager->deallocate(fBuffer);//delete[] fBuffer;  
 }
 
 XMLDateTime::XMLDateTime(MemoryManager* const manager)
-: fBuffer(0)
+: fStart(0)
+, fEnd(0)
+, fBufferMaxLen(0)
+, fBuffer(0)
 , fMemoryManager(manager)
 {
     reset();
@@ -488,7 +533,10 @@ XMLDateTime::XMLDateTime(MemoryManager* const manager)
 
 XMLDateTime::XMLDateTime(const XMLCh* const aString,
                          MemoryManager* const manager)
-: fBuffer(0)
+: fStart(0)
+, fEnd(0)
+, fBufferMaxLen(0)
+, fBuffer(0)
 , fMemoryManager(manager)
 {
     setBuffer(aString);
@@ -499,7 +547,8 @@ XMLDateTime::XMLDateTime(const XMLCh* const aString,
 // -----------------------------------------------------------------------
 
 XMLDateTime::XMLDateTime(const XMLDateTime &toCopy)
-: fBuffer(0)
+: fBufferMaxLen(0)
+, fBuffer(0)
 , fMemoryManager(toCopy.fMemoryManager)
 {
     copy(toCopy);
@@ -567,9 +616,10 @@ void XMLDateTime::parseDateTime()
 
     //fStart is supposed to point to 'T'
     if (fBuffer[fStart++] != DATETIME_SEPARATOR)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dt_missingT
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 
     getTime();
     validateDateTime();
@@ -615,9 +665,10 @@ void XMLDateTime::parseDay()
         fBuffer[1] != DATE_SEPARATOR ||
         fBuffer[2] != DATE_SEPARATOR  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gDay_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //initialize values
@@ -630,9 +681,10 @@ void XMLDateTime::parseDay()
         int sign = findUTCSign(DAY_SIZE);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gDay_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -656,9 +708,10 @@ void XMLDateTime::parseMonth()
     if (fBuffer[0] != DATE_SEPARATOR ||
         fBuffer[1] != DATE_SEPARATOR  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gMth_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //set constants
@@ -683,9 +736,10 @@ void XMLDateTime::parseMonth()
         int sign = findUTCSign(fStart);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gMth_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -739,9 +793,10 @@ void XMLDateTime::parseMonthDay()
         fBuffer[1] != DATE_SEPARATOR ||
         fBuffer[4] != DATE_SEPARATOR )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_gMthDay_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
 
@@ -755,9 +810,10 @@ void XMLDateTime::parseMonthDay()
         int sign = findUTCSign(MONTHDAY_SIZE);
         if ( sign<0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_gMthDay_invalid
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
         else
         {
@@ -800,18 +856,20 @@ void XMLDateTime::parseDuration()
     if ( (c != DURATION_STARTER) &&
          (c != chDash)            )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_Start_dashP
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     // 'P' must ALWAYS be present in either case
     if ( (c == chDash) &&
          (fBuffer[fStart++]!= DURATION_STARTER ))
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_noP
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     // java code
@@ -828,9 +886,10 @@ void XMLDateTime::parseDuration()
     //
     if (indexOf(fStart, fEnd, chDash) != NOT_FOUND)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_DashNotFirst
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //at least one number and designator must be seen after P
@@ -873,9 +932,10 @@ void XMLDateTime::parseDuration()
     if ( (fEnd == endDate) &&   // 'T' absent
          (fStart != fEnd)   )   // something after Day
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_inv_b4T
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     if ( fEnd != endDate ) // 'T' present
@@ -921,9 +981,10 @@ void XMLDateTime::parseDuration()
                  */
                 if ( mlsec+1 == end )
                 {
-                    ThrowXML1(SchemaDateTimeException
+                    ThrowXMLwithMemMgr1(SchemaDateTimeException
                             , XMLExcepts::DateTime_dur_inv_seconds
-                            ,fBuffer);
+                            , fBuffer
+                            , fMemoryManager);
                 }
 
                 fValue[Second]     = negate * parseInt(fStart, mlsec);
@@ -943,17 +1004,19 @@ void XMLDateTime::parseDuration()
         if ( (fStart != fEnd) ||
               fBuffer[--fStart] == DATETIME_SEPARATOR )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_dur_NoTimeAfterT
-                    ,fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
         }
     }
 
     if ( !designator )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_dur_NoElementAtAll
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
 }
@@ -975,18 +1038,20 @@ void XMLDateTime::getDate()
 
     // Ensure enough chars in buffer
     if ( (fStart+YMD_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_date_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 
     getYearMonth();    // Scan YearMonth and
                        // fStart point to the next '-'
 
     if (fBuffer[fStart++] != DATE_SEPARATOR)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_date_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("CCYY-MM must be followed by '-' sign");
     }
 
@@ -1010,18 +1075,20 @@ void XMLDateTime::getTime()
 
     // Ensure enough chars in buffer
     if ( (fStart+TIME_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_time_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Imcomplete Time Format"
 
     // check (fixed) format first
     if ((fBuffer[fStart + 2] != TIME_SEPARATOR) ||
         (fBuffer[fStart + 5] != TIME_SEPARATOR)  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_time_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Error in parsing time" );
     }
 
@@ -1048,9 +1115,10 @@ void XMLDateTime::getTime()
         // make sure we have some thing between the '.' and fEnd
         if (fStart >= fEnd)
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_ms_noDigit
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //("ms shall be present once '.' is present" );
         }
 
@@ -1061,15 +1129,21 @@ void XMLDateTime::getTime()
         }
         else
         {
+            //to do: parseInt would eliminate any leading zeros
+            //       therefore for 12:01:01.0034
+            //       fValue[MiliSecond] would catch 34
+            //       rather than 0034
+            //
             fValue[MiliSecond] = parseInt(fStart, sign);  //get ms between UTC sign and fEnd
         }
 	}
     else if(sign == 0 || sign != fStart)
     {
         // seconds has more than 2 digits
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
     }
 
     //parse UTC time zone (hh:mm)
@@ -1090,9 +1164,10 @@ void XMLDateTime::getYearMonth()
 
     // Ensure enough chars in buffer
     if ( (fStart+YMONTH_MIN_SIZE) > fEnd)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_incomplete
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Imcomplete YearMonth Format";
 
     // skip the first leading '-'
@@ -1103,9 +1178,10 @@ void XMLDateTime::getYearMonth()
     //
     int yearSeparator = indexOf(start, fEnd, DATE_SEPARATOR);
     if ( yearSeparator == NOT_FOUND)
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Year separator is missing or misplaced");
 
     fValue[CentYear] = parseIntYear(yearSeparator);
@@ -1115,9 +1191,10 @@ void XMLDateTime::getYearMonth()
     //gonna check we have enough byte for month
     //
     if ((fStart + 2) > fEnd )
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_ym_noMonth
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"no month in buffer"
 
     fValue[Month] = parseInt(fStart, yearSeparator + 3);
@@ -1133,9 +1210,10 @@ void XMLDateTime::parseTimeZone()
         int sign = findUTCSign(fStart);
         if ( sign < 0 )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_tz_noUTCsign
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //("Error in month parsing");
         }
         else
@@ -1161,9 +1239,10 @@ void XMLDateTime::getTimeZone(const int sign)
     {
         if ((sign + 1) != fEnd )
         {
-            ThrowXML1(SchemaDateTimeException
+            ThrowXMLwithMemMgr1(SchemaDateTimeException
                     , XMLExcepts::DateTime_tz_stuffAfterZ
-                    , fBuffer);
+                    , fBuffer
+                    , fMemoryManager);
             //"Error in parsing time zone");
         }		
 
@@ -1179,9 +1258,10 @@ void XMLDateTime::getTimeZone(const int sign)
     if ( ( ( sign + TIMEZONE_SIZE + 1) != fEnd )      ||
          ( fBuffer[sign + 3] != TIMEZONE_SEPARATOR ) )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_tz_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Error in parsing time zone");
     }
 
@@ -1258,18 +1338,20 @@ void XMLDateTime::validateDateTime() const
     //          or reporting an error message should be sufficient?
     if ( fValue[CentYear] == 0 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_zero
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"The year \"0000\" is an illegal year value");
     }
 
     if ( fValue[Month] < 1  ||
          fValue[Month] > 12  )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_mth_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
 		//"The month must have values 1 to 12");
     }
 
@@ -1277,9 +1359,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Day] > maxDayInMonthFor( fValue[CentYear], fValue[Month]) ||
          fValue[Day] == 0 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_day_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"The day must have values 1 to 31");
     }
 
@@ -1290,9 +1373,10 @@ void XMLDateTime::validateDateTime() const
                                   (fValue[Second] !=0) ||
                                   (fValue[MiliSecond] !=0))))
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_hour_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Hour must have values 0-23");
     }
 
@@ -1300,9 +1384,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Minute] < 0 ||
          fValue[Minute] > 59 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Minute must have values 0-59");
     }
 
@@ -1310,9 +1395,10 @@ void XMLDateTime::validateDateTime() const
     if ( fValue[Second] < 0 ||
          fValue[Second] > 60 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_second_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Second must have values 0-60");
     }
 
@@ -1320,18 +1406,20 @@ void XMLDateTime::validateDateTime() const
     if ( (abs(fTimeZone[hh]) > 14) ||
          ((abs(fTimeZone[hh]) == 14) && (fTimeZone[mm] != 0)) )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_tz_hh_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Time zone should have range -14..+14");
     }
 
     //validate time-zone minutes
     if ( abs(fTimeZone[mm]) > 59 )
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_min_invalid
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //("Minute must have values 0-59");
     }
 	
@@ -1374,16 +1462,14 @@ int XMLDateTime::findUTCSign (const int start)
 //
 int XMLDateTime::parseInt(const int start, const int end) const
 {
+    unsigned int retVal = 0;
+    for (int i=start; i < end; i++) {
 
-    XMLCh* strToScan = (XMLCh*) fMemoryManager->allocate
-    (
-        (end - start + 1) * sizeof(XMLCh)
-    );//new XMLCh[end - start + 1];
-    ArrayJanitor<XMLCh>  jname(strToScan, fMemoryManager);
-    XMLString::subString(strToScan, fBuffer, start, end);
+        if (fBuffer[i] < chDigit_0 || fBuffer[i] > chDigit_9)
+            ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars, fMemoryManager);
 
-    unsigned int retVal;
-    XMLString::textToBin(strToScan, retVal);
+        retVal = (retVal * 10) + (unsigned int) (fBuffer[i] - chDigit_0);
+    }
 
     return (int) retVal;
 }
@@ -1403,17 +1489,19 @@ int XMLDateTime::parseIntYear(const int end) const
     int length = end - start;
     if (length < 4)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_tooShort
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Year must have 'CCYY' format");
     }
     else if (length > 4 &&
              fBuffer[start] == chDigit_0)
     {
-        ThrowXML1(SchemaDateTimeException
+        ThrowXMLwithMemMgr1(SchemaDateTimeException
                 , XMLExcepts::DateTime_year_leadingZero
-                , fBuffer);
+                , fBuffer
+                , fMemoryManager);
         //"Leading zeros are required if the year value would otherwise have fewer than four digits;
         // otherwise they are forbidden");
     }
@@ -1423,4 +1511,255 @@ int XMLDateTime::parseIntYear(const int end) const
     return ( negative ? (-1) * yearVal : yearVal );
 }
 
+/***
+ * E2-41
+ *
+ *  3.2.7.2 Canonical representation
+ * 
+ *  Except for trailing fractional zero digits in the seconds representation, 
+ *  '24:00:00' time representations, and timezone (for timezoned values), 
+ *  the mapping from literals to values is one-to-one. Where there is more 
+ *  than one possible representation, the canonical representation is as follows: 
+ *  redundant trailing zero digits in fractional-second literals are prohibited. 
+ *  An hour representation of '24' is prohibited. Timezoned values are canonically
+ *  represented by appending 'Z' to the nontimezoned representation. (All 
+ *  timezoned dateTime values are UTC.) 
+ *
+ *  .'24:00:00' -> '00:00:00'
+ *  .milisecond: trailing zeros removed
+ *  .'Z'
+ *
+ ***/
+XMLCh* XMLDateTime::getDateTimeCanonicalRepresentation(MemoryManager* const memMgr) const
+{
+    XMLCh *miliStartPtr, *miliEndPtr;
+    searchMiliSeconds(miliStartPtr, miliEndPtr);
+    int miliSecondsLen = miliEndPtr - miliStartPtr;
+
+    MemoryManager* toUse = memMgr? memMgr : fMemoryManager;
+    XMLCh* retBuf = (XMLCh*) toUse->allocate( (21 + miliSecondsLen + 2) * sizeof(XMLCh));
+    XMLCh* retPtr = retBuf;
+
+    // (-?) cc+yy-mm-dd'T'hh:mm:ss'Z'    ('.'s+)?
+    //      2+  8       1      8   1
+    //
+    int additionalLen = fillYearString(retPtr, CentYear);
+    if(additionalLen != 0)
+    {
+        // very bad luck; have to resize the buffer...
+        XMLCh *tmpBuf = (XMLCh*) toUse->allocate( (additionalLen+21+miliSecondsLen +2) * sizeof(XMLCh));
+        XMLString::moveChars(tmpBuf, retBuf, 4+additionalLen);
+        retPtr = tmpBuf+(retPtr-retBuf);
+        toUse->deallocate(retBuf);
+        retBuf = tmpBuf;
+    }
+    *retPtr++ = DATE_SEPARATOR;
+    fillString(retPtr, Month, 2);
+    *retPtr++ = DATE_SEPARATOR;
+    fillString(retPtr, Day, 2);
+    *retPtr++ = DATETIME_SEPARATOR;
+
+    fillString(retPtr, Hour, 2);
+    if (fValue[Hour] == 24)
+    {
+        *(retPtr - 2) = chDigit_0;
+        *(retPtr - 1) = chDigit_0;
+    }
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Minute, 2);
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Second, 2);
+
+    if (miliSecondsLen)
+    {
+        *retPtr++ = chPeriod;
+        XMLString::copyNString(retPtr, miliStartPtr, miliSecondsLen);
+        retPtr += miliSecondsLen;
+    }
+
+    *retPtr++ = UTC_STD_CHAR;
+    *retPtr = chNull;
+
+    return retBuf;
+}
+
+/***
+ * 3.2.8 time
+ *
+ *  . either the time zone must be omitted or, 
+ *    if present, the time zone must be Coordinated Universal Time (UTC) indicated by a "Z".   
+ *
+ *  . Additionally, the canonical representation for midnight is 00:00:00.
+ *
+***/
+XMLCh* XMLDateTime::getTimeCanonicalRepresentation(MemoryManager* const memMgr) const
+{
+    XMLCh *miliStartPtr, *miliEndPtr;
+    searchMiliSeconds(miliStartPtr, miliEndPtr);
+    int miliSecondsLen = miliEndPtr - miliStartPtr;
+
+    MemoryManager* toUse = memMgr? memMgr : fMemoryManager;
+    XMLCh* retBuf = (XMLCh*) toUse->allocate( (10 + miliSecondsLen + 2) * sizeof(XMLCh));
+    XMLCh* retPtr = retBuf;
+
+    // 'hh:mm:ss'Z'    ('.'s+)?
+    //      8    1
+    //
+
+    fillString(retPtr, Hour, 2);
+    if (fValue[Hour] == 24)
+    {
+        *(retPtr - 2) = chDigit_0;
+        *(retPtr - 1) = chDigit_0;
+    }
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Minute, 2);
+    *retPtr++ = TIME_SEPARATOR;
+    fillString(retPtr, Second, 2);
+
+    if (miliSecondsLen)
+    {
+        *retPtr++ = chPeriod;
+        XMLString::copyNString(retPtr, miliStartPtr, miliSecondsLen);
+        retPtr += miliSecondsLen;
+    }
+
+    *retPtr++ = UTC_STD_CHAR;
+    *retPtr = chNull;
+
+    return retBuf;
+}
+
+void XMLDateTime::fillString(XMLCh*& ptr, valueIndex ind, int expLen) const
+{
+    XMLCh strBuffer[16];
+    assert(expLen < 16);
+    XMLString::binToText(fValue[ind], strBuffer, expLen, 10, fMemoryManager);
+    int   actualLen = XMLString::stringLen(strBuffer);
+    int   i;
+    //append leading zeros
+    for (i = 0; i < expLen - actualLen; i++)
+    {
+        *ptr++ = chDigit_0;
+    }
+
+    for (i = 0; i < actualLen; i++)
+    {
+        *ptr++ = strBuffer[i];
+    }
+
+}
+
+int XMLDateTime::fillYearString(XMLCh*& ptr, valueIndex ind) const
+{
+    XMLCh strBuffer[16];
+    // let's hope we get no years of 15 digits...
+    XMLString::binToText(fValue[ind], strBuffer, 15, 10, fMemoryManager);
+    int   actualLen = XMLString::stringLen(strBuffer);
+    // don't forget that years can be negative...
+    int negativeYear = 0;
+    if(strBuffer[0] == chDash)
+    {
+        *ptr++ = strBuffer[0];
+        negativeYear = 1;
+    }
+    int   i;
+    //append leading zeros
+    for (i = 0; i < 4 - actualLen+negativeYear; i++)
+    {
+        *ptr++ = chDigit_0;
+    }
+
+    for (i = negativeYear; i < actualLen; i++)
+    {
+        *ptr++ = strBuffer[i];
+    }
+    if(actualLen > 4)
+        return actualLen-4;
+    return 0;
+}
+
+/***
+ *
+ *   .check if the rawData has the mili second component
+ *   .capture the substring
+ *
+ ***/
+void XMLDateTime::searchMiliSeconds(XMLCh*& miliStartPtr, XMLCh*& miliEndPtr) const
+{
+    miliStartPtr = miliEndPtr = 0;
+
+    int milisec = XMLString::indexOf(fBuffer, MILISECOND_SEPARATOR);
+    if (milisec == -1)
+        return;
+
+    miliStartPtr = fBuffer + milisec + 1;
+    miliEndPtr   = miliStartPtr;
+    while (*miliEndPtr)
+    {
+        if ((*miliEndPtr < chDigit_0) || (*miliEndPtr > chDigit_9))
+            break;
+
+        miliEndPtr++;
+    }
+
+    //remove trailing zeros
+    while( *(miliEndPtr - 1) == chDigit_0)
+        miliEndPtr--;
+
+    return;
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(XMLDateTime)
+
+void XMLDateTime::serialize(XSerializeEngine& serEng)
+{
+    //REVISIT: may not need to call base since it does nothing
+    XMLNumber::serialize(serEng);
+
+    int i = 0;
+
+    if (serEng.isStoring())
+    {
+        for (i = 0; i < TOTAL_SIZE; i++)
+        {
+            serEng<<fValue[i];
+        }
+
+        for (i = 0; i < TIMEZONE_ARRAYSIZE; i++)
+        {
+            serEng<<fTimeZone[i];
+        }
+
+        serEng<<fStart;
+        serEng<<fEnd;
+
+        serEng.writeString(fBuffer, fBufferMaxLen, XSerializeEngine::toWriteBufferLen);
+    }
+    else
+    {
+        for (i = 0; i < TOTAL_SIZE; i++)
+        {
+            serEng>>fValue[i];
+        }
+
+        for (i = 0; i < TIMEZONE_ARRAYSIZE; i++)
+        {
+            serEng>>fTimeZone[i];
+        }
+
+        serEng>>fStart;
+        serEng>>fEnd;
+
+        int dataLen = 0;
+        serEng.readString(fBuffer, fBufferMaxLen, dataLen ,XSerializeEngine::toReadBufferLen);
+
+    }
+
+}
+
 XERCES_CPP_NAMESPACE_END
