@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLString.cpp,v 1.27 2003/11/03 15:35:52 amassari Exp $
+ * $Id: XMLString.cpp,v 1.31 2004/01/13 17:12:34 cargilld Exp $
  */
 
 
@@ -105,7 +105,8 @@ MemoryManager* XMLString::fgMemoryManager = 0;
 void XMLString::binToText(  const   unsigned long   toFormat
                             ,       char* const     toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     static const char digitList[16] =
     {
@@ -114,7 +115,7 @@ void XMLString::binToText(  const   unsigned long   toFormat
     };
 
     if (!maxChars)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf, manager);
 
     // Handle special case
     if (!toFormat)
@@ -135,7 +136,7 @@ void XMLString::binToText(  const   unsigned long   toFormat
     //  having to check for overflow in the inner loops, and we have to flip
     //  the resulting XMLString anyway.
     //
-    char tmpBuf[128];
+    char   tmpBuf[128];
 
     //
     //  For each radix, do the optimal thing. For bin and hex, we can special
@@ -171,14 +172,16 @@ void XMLString::binToText(  const   unsigned long   toFormat
             tmpVal /= radix;
         }
     }
-     else
+    else
     {
-        ThrowXML(RuntimeException, XMLExcepts::Str_UnknownRadix);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Str_UnknownRadix, manager);
     }
 
     // See if have enough room in the caller's buffer
     if (tmpIndex > maxChars)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_TargetBufTooSmall);
+    {
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_TargetBufTooSmall, manager);
+    }
 
     // Reverse the tmp buffer into the caller's buffer
     unsigned int outIndex = 0;
@@ -192,16 +195,18 @@ void XMLString::binToText(  const   unsigned long   toFormat
 void XMLString::binToText(  const   unsigned int    toFormat
                             ,       char* const     toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     // Just call the unsigned long version
-    binToText((unsigned long)toFormat, toFill, maxChars, radix);
+    binToText((unsigned long)toFormat, toFill, maxChars, radix, manager);
 }
 
 void XMLString::binToText(  const   long            toFormat
                             ,       char* const     toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     //
     //  If its negative, then put a negative sign into the output and flip
@@ -221,13 +226,14 @@ void XMLString::binToText(  const   long            toFormat
     }
 
     // And now call the unsigned long version
-    binToText(actualVal, &toFill[startInd], maxChars, radix);
+    binToText(actualVal, &toFill[startInd], maxChars, radix, manager);
 }
 
 void XMLString::binToText(  const   int             toFormat
                             ,       char* const     toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     //
     //  If its negative, then put a negative sign into the output and flip
@@ -247,7 +253,7 @@ void XMLString::binToText(  const   int             toFormat
     }
 
     // And now call the unsigned long version
-    binToText(actualVal, &toFill[startInd], maxChars, radix);
+    binToText(actualVal, &toFill[startInd], maxChars, radix, manager);
 }
 
 
@@ -325,10 +331,11 @@ void XMLString::cut(        XMLCh* const    toCutFrom
 
 
 unsigned int XMLString::hash(   const   char* const     tohash
-                                , const unsigned int    hashModulus)
-{
+                                , const unsigned int    hashModulus
+                                , MemoryManager* const  manager)
+{    
     if (!hashModulus)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus, manager);
 
     unsigned int hashVal = 0;
     if (tohash) {
@@ -360,13 +367,14 @@ int XMLString::indexOf(const char* const toSearch, const char ch)
 
 int XMLString::indexOf( const   char* const     toSearch
                         , const char            ch
-                        , const unsigned int    fromIndex)
+                        , const unsigned int    fromIndex
+                        , MemoryManager* const  manager)
 {
     const unsigned int len = strlen(toSearch);
 
     // Make sure the start index is within the XMLString bounds
 	if ((int)fromIndex > len-1)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (unsigned int i = fromIndex; i < len; i++)
     {
@@ -389,13 +397,14 @@ int XMLString::lastIndexOf(const char* const toSearch, const char ch)
 
 int XMLString::lastIndexOf( const   char* const     toSearch
                             , const char            ch
-                            , const unsigned int    fromIndex)
+                            , const unsigned int    fromIndex
+                            , MemoryManager* const  manager)
 {
     const int len = strlen(toSearch);
 
     // Make sure the start index is within the XMLString bounds
 	if ((int)fromIndex > len-1)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (int i = (int)fromIndex; i >= 0; i--)
     {
@@ -411,7 +420,8 @@ unsigned int XMLString::replaceTokens(          XMLCh* const    errText
                                         , const XMLCh* const    text1
                                         , const XMLCh* const    text2
                                         , const XMLCh* const    text3
-                                        , const XMLCh* const    text4)
+                                        , const XMLCh* const    text4
+                                        , MemoryManager* const  manager)
 {
     //
     //  We have to build the string back into the source string, so allocate
@@ -419,8 +429,8 @@ unsigned int XMLString::replaceTokens(          XMLCh* const    errText
     //  incoming buffer as a target buffer. Put a janitor on it to make sure
     //  it gets cleaned up.
     //
-    XMLCh* orgText = replicate(errText);
-    ArrayJanitor<XMLCh> janText(orgText);
+    XMLCh* orgText = replicate(errText, manager);
+    ArrayJanitor<XMLCh> janText(orgText, manager);
 
     XMLCh* pszSrc = orgText;
     unsigned int curOutInd = 0;
@@ -574,9 +584,10 @@ char* XMLString::transcode(const XMLCh* const toTranscode,
 
 bool XMLString::transcode(  const   XMLCh* const    toTranscode
                             ,       char* const     toFill
-                            , const unsigned int    maxChars)
+                            , const unsigned int    maxChars
+                            , MemoryManager* const  manager)
 {
-    if (!gTranscoder->transcode(toTranscode, toFill, maxChars))
+    if (!gTranscoder->transcode(toTranscode, toFill, maxChars, manager))
         return false;
     return true;
 }
@@ -594,9 +605,10 @@ XMLCh* XMLString::transcode(const char* const toTranscode,
 
 bool XMLString::transcode(  const   char* const     toTranscode
                             ,       XMLCh* const    toFill
-                            , const unsigned int    maxChars)
+                            , const unsigned int    maxChars
+                            , MemoryManager* const  manager)
 {
-    if (!gTranscoder->transcode(toTranscode, toFill, maxChars))
+    if (!gTranscoder->transcode(toTranscode, toFill, maxChars, manager))
         return false;
     return true;
 }
@@ -636,17 +648,18 @@ void XMLString::trim(char* const toTrim)
 
 
 void XMLString::subString(char* const targetStr, const char* const srcStr
-                          , const int startIndex, const int endIndex)
+                          , const int startIndex, const int endIndex
+                          , MemoryManager* const manager)
 {
     if (targetStr == 0)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf, manager);
 
     const int srcLen = strlen(srcStr);
     const int copySize = endIndex - startIndex;
 
     // Make sure the start index is within the XMLString bounds
     if ( startIndex < 0 || startIndex > endIndex || endIndex > srcLen)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (int i= startIndex; i < endIndex; i++) {
         targetStr[i-startIndex] = srcStr[i];
@@ -756,7 +769,8 @@ bool XMLString::isAllWhiteSpace(const XMLCh* const toCheck)
 void XMLString::binToText(  const   unsigned long   toFormat
                             ,       XMLCh* const    toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     static const XMLCh digitList[16] =
     {
@@ -766,7 +780,7 @@ void XMLString::binToText(  const   unsigned long   toFormat
     };
 
     if (!maxChars)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf, manager);
 
     // Handle special case
     if (!toFormat)
@@ -787,7 +801,7 @@ void XMLString::binToText(  const   unsigned long   toFormat
     //  having to check for overflow in the inner loops, and we have to flip
     //  the resulting sring anyway.
     //
-    XMLCh tmpBuf[128];
+    XMLCh   tmpBuf[128];
 
     //
     //  For each radix, do the optimal thing. For bin and hex, we can special
@@ -825,12 +839,14 @@ void XMLString::binToText(  const   unsigned long   toFormat
     }
      else
     {
-        ThrowXML(RuntimeException, XMLExcepts::Str_UnknownRadix);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Str_UnknownRadix, manager);
     }
 
     // See if have enough room in the caller's buffer
     if (tmpIndex > maxChars)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_TargetBufTooSmall);
+    {
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_TargetBufTooSmall, manager);
+    }
 
     // Reverse the tmp buffer into the caller's buffer
     unsigned int outIndex = 0;
@@ -844,16 +860,18 @@ void XMLString::binToText(  const   unsigned long   toFormat
 void XMLString::binToText(  const   unsigned int    toFormat
                             ,       XMLCh* const    toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     // Just call the unsigned long version
-    binToText((unsigned long)toFormat, toFill, maxChars, radix);
+    binToText((unsigned long)toFormat, toFill, maxChars, radix, manager);
 }
 
 void XMLString::binToText(  const   long            toFormat
                             ,       XMLCh* const    toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     //
     //  If its negative, then put a negative sign into the output and flip
@@ -873,13 +891,14 @@ void XMLString::binToText(  const   long            toFormat
     }
 
     // And now call the unsigned long version
-    binToText(actualVal, &toFill[startInd], maxChars, radix);
+    binToText(actualVal, &toFill[startInd], maxChars, radix, manager);
 }
 
 void XMLString::binToText(  const   int             toFormat
                             ,       XMLCh* const    toFill
                             , const unsigned int    maxChars
-                            , const unsigned int    radix)
+                            , const unsigned int    radix
+                            , MemoryManager* const  manager)
 {
     //
     //  If its negative, then put a negative sign into the output and flip
@@ -899,7 +918,7 @@ void XMLString::binToText(  const   int             toFormat
     }
 
     // And now call the unsigned long version
-    binToText(actualVal, &toFill[startInd], maxChars, radix);
+    binToText(actualVal, &toFill[startInd], maxChars, radix, manager);
 }
 
 
@@ -1071,10 +1090,11 @@ bool XMLString::copyNString(        XMLCh* const    target
 
 
 unsigned int XMLString::hash(   const   XMLCh* const    tohash
-                                , const unsigned int    hashModulus)
-{
+                                , const unsigned int    hashModulus
+                                , MemoryManager* const  manager)
+{    
     if (!hashModulus)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus, manager);
 
     unsigned int hashVal = 0;
     if (tohash) {
@@ -1165,10 +1185,11 @@ int XMLString::patternMatch(  const XMLCh* const    toSearch
 
 unsigned int XMLString::hashN(  const   XMLCh* const    tohash
                                 , const unsigned int    n
-                                , const unsigned int    hashModulus)
+                                , const unsigned int    hashModulus
+                                , MemoryManager* const  manager)
 {
     if (!hashModulus)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Pool_ZeroModulus, manager);
 
     unsigned int hashVal = 0;
     if (tohash) {
@@ -1206,13 +1227,14 @@ int XMLString::indexOf(const XMLCh* const toSearch, const XMLCh ch)
 
 int XMLString::indexOf( const   XMLCh* const    toSearch
                         , const XMLCh           ch
-                        , const unsigned int    fromIndex)
+                        , const unsigned int    fromIndex
+                        , MemoryManager* const  manager)
 {
     const int len = stringLen(toSearch);
 
     // Make sure the start index is within the XMLString bounds
 	if ((int)fromIndex > len-1)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (int i = (int)fromIndex; i < len; i++)
     {
@@ -1236,11 +1258,12 @@ int XMLString::lastIndexOf(const XMLCh ch,
 
 int XMLString::lastIndexOf( const   XMLCh* const    toSearch
                             , const XMLCh           ch
-                            , const unsigned int    fromIndex)
+                            , const unsigned int    fromIndex
+                            , MemoryManager* const  manager)
 {
     const int len = stringLen(toSearch);
 	if ((int)fromIndex > len-1)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (int i = (int)fromIndex; i >= 0; i--)
     {
@@ -1282,7 +1305,8 @@ XMLString::makeUName(const XMLCh* const pszURI, const XMLCh* const pszName)
 }
 
 
-bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill)
+bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill
+                          , MemoryManager* const manager)
 {
     toFill = 0;
 
@@ -1290,8 +1314,8 @@ bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill)
     if ((!toConvert) || (!*toConvert))
         return false;
 
-	XMLCh* trimmedStr = XMLString::replicate(toConvert);
-	ArrayJanitor<XMLCh> jan1(trimmedStr);
+	XMLCh* trimmedStr = XMLString::replicate(toConvert, manager);
+	ArrayJanitor<XMLCh> jan1(trimmedStr, manager);
 	XMLString::trim(trimmedStr);
     unsigned int trimmedStrLen = XMLString::stringLen(trimmedStr);
 
@@ -1299,14 +1323,14 @@ bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill)
 		return false;
 
 	// we don't allow '-' sign
-	if (XMLString::indexOf(trimmedStr, chDash, 0) != -1)
+	if (XMLString::indexOf(trimmedStr, chDash, 0, manager) != -1)
 		return false;
 
 	//the errno set by previous run is NOT automatically cleared
 	errno = 0;
 
-	char *nptr = XMLString::transcode(trimmedStr);
-    ArrayJanitor<char> jan2(nptr);
+	char *nptr = XMLString::transcode(trimmedStr, manager);
+    ArrayJanitor<char> jan2(nptr, manager);
 
     char *endptr;
 	 //
@@ -1323,36 +1347,37 @@ bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill)
     return true;
 }
 
-int XMLString::parseInt(const XMLCh* const toConvert)
+int XMLString::parseInt(const XMLCh* const toConvert
+                     , MemoryManager* const manager)
 {
     // If no string, or empty string, then it is a failure
     if ((!toConvert) || (!*toConvert))
-        ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_null_ptr);
+        ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_null_ptr, manager);
 
-	XMLCh* trimmedStr = XMLString::replicate(toConvert);
-	ArrayJanitor<XMLCh> jan1(trimmedStr);
+	XMLCh* trimmedStr = XMLString::replicate(toConvert, manager);
+	ArrayJanitor<XMLCh> jan1(trimmedStr, manager);
 	XMLString::trim(trimmedStr);
     unsigned int trimmedStrLen = XMLString::stringLen(trimmedStr);
 
 	if ( !trimmedStrLen )
-        ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_null_ptr);
+        ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_null_ptr, manager);
 
 	//the errno set by previous run is NOT automatically cleared
 	errno = 0;
 
-	char *nptr = XMLString::transcode(trimmedStr);
-    ArrayJanitor<char> jan2(nptr);
+	char *nptr = XMLString::transcode(trimmedStr, manager);
+    ArrayJanitor<char> jan2(nptr, manager);
 
     char *endptr;
     long retVal = strtol(nptr, &endptr, 10);
 
 	// check if all chars are valid char
 	if ( (endptr - nptr) != (int) trimmedStrLen)
-		ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars);
+		ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars, manager);
 
 	// check if overflow/underflow occurs
     if (errno == ERANGE)
-        ThrowXML(NumberFormatException, XMLExcepts::Str_ConvertOverflow);
+        ThrowXMLwithMemMgr(NumberFormatException, XMLExcepts::Str_ConvertOverflow, manager);
 
 	 //
      // REVISIT: conversion of (long) to (int)
@@ -1409,20 +1434,21 @@ void XMLString::lowerCase(XMLCh* const toLowerCase)
 
 
 void XMLString::subString(XMLCh* const targetStr, const XMLCh* const srcStr
-                          , const int startIndex, const int endIndex)
+                          , const int startIndex, const int endIndex
+                          , MemoryManager* const manager)
 {
     //if (startIndex < 0 || endIndex < 0)
-    //    ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_NegativeIndex);
+    //    ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_NegativeIndex);
 
     if (targetStr == 0)
-        ThrowXML(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf);
+        ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Str_ZeroSizedTargetBuf, manager);
 
     const int srcLen = stringLen(srcStr);
     const int copySize = endIndex - startIndex;
 
     // Make sure the start index is within the XMLString bounds
     if ( startIndex < 0 || startIndex > endIndex || endIndex > srcLen)
-        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd);
+        ThrowXMLwithMemMgr(ArrayIndexOutOfBoundsException, XMLExcepts::Str_StartIndexPastEnd, manager);
 
     for (int i= startIndex; i < endIndex; i++) {
         targetStr[i-startIndex] = srcStr[i];
@@ -1431,13 +1457,14 @@ void XMLString::subString(XMLCh* const targetStr, const XMLCh* const srcStr
     targetStr[copySize] = 0;
 }
 
-BaseRefVectorOf<XMLCh>* XMLString::tokenizeString(const XMLCh* const tokenizeSrc)
+BaseRefVectorOf<XMLCh>* XMLString::tokenizeString(const XMLCh*      const   tokenizeSrc
+                                               ,  MemoryManager*    const   manager)
 {
-    XMLCh* orgText = replicate(tokenizeSrc, fgMemoryManager);
-    ArrayJanitor<XMLCh> janText(orgText, fgMemoryManager);
+    XMLCh* orgText = replicate(tokenizeSrc, manager);
+    ArrayJanitor<XMLCh> janText(orgText, manager);
     XMLCh* tokenizeStr = orgText;
 
-    RefArrayVectorOf<XMLCh>* tokenStack = new RefArrayVectorOf<XMLCh>(16, true);
+    RefArrayVectorOf<XMLCh>* tokenStack = new (manager) RefArrayVectorOf<XMLCh>(16, true, manager);
 
     unsigned int len = stringLen(tokenizeStr);
     unsigned int skip;
@@ -1465,12 +1492,12 @@ BaseRefVectorOf<XMLCh>* XMLString::tokenizeString(const XMLCh* const tokenizeSrc
 
         // these tokens are adopted in the RefVector and will be deleted
         // when the vector is deleted by the caller
-        XMLCh* token = (XMLCh*) fgMemoryManager->allocate
+        XMLCh* token = (XMLCh*) manager->allocate
         (
             (skip+1-index) * sizeof(XMLCh)
         );//new XMLCh[skip+1-index];
 
-        XMLString::subString(token, tokenizeStr, index, skip);
+        XMLString::subString(token, tokenizeStr, index, skip, manager);
         tokenStack->addElement(token);
         index = skip;
     }
@@ -1563,13 +1590,14 @@ bool XMLString::isWSReplaced(const XMLCh* const toCheck)
 //    #xA  Line Feed
 //    #x9  TAB
 //
-void XMLString::replaceWS(XMLCh* const toConvert)
+void XMLString::replaceWS(XMLCh* const toConvert
+                          , MemoryManager* const  manager)
 {
     int strLen = XMLString::stringLen(toConvert);
     if (strLen == 0)
         return;
 
-    XMLCh* retBuf = (XMLCh*) fgMemoryManager->allocate
+    XMLCh* retBuf = (XMLCh*) manager->allocate
     (
         (strLen+1) * sizeof(XMLCh)
     );//new XMLCh[strLen+1];
@@ -1592,7 +1620,7 @@ void XMLString::replaceWS(XMLCh* const toConvert)
     retBuf[strLen] = chNull;
 
     XMLString::moveChars(toConvert, retBuf, strLen);
-    fgMemoryManager->deallocate(retBuf);//delete[] retBuf;
+    manager->deallocate(retBuf);//delete[] retBuf;
     return;
 }
 
@@ -1643,14 +1671,15 @@ bool XMLString::isWSCollapsed(const XMLCh* const toCheck)
 // no leading and/or trailing spaces
 // no continuous sequences of spaces
 //
-void XMLString::collapseWS(XMLCh* const toConvert)
+void XMLString::collapseWS(XMLCh* const toConvert
+                           , MemoryManager* const  manager)
 {
     // If no string, then its a failure
     if (( !toConvert ) || ( !*toConvert ))
         return;
 
     // replace whitespace first
-    replaceWS(toConvert);
+    replaceWS(toConvert, manager);
 
     // remove leading spaces
     const XMLCh* startPtr = toConvert;
@@ -1668,7 +1697,7 @@ void XMLString::collapseWS(XMLCh* const toConvert)
     //
     //  Work through what remains and chop continuous spaces
     //
-    XMLCh* retBuf = (XMLCh*) fgMemoryManager->allocate
+    XMLCh* retBuf = (XMLCh*) manager->allocate
     (
         (endPtr - startPtr + 1) * sizeof(XMLCh)
     );//new XMLCh[endPtr - startPtr + 1];
@@ -1701,20 +1730,21 @@ void XMLString::collapseWS(XMLCh* const toConvert)
 
     *retPtr = chNull;
     XMLString::moveChars(toConvert, retBuf, stringLen(retBuf)+1); //copy the last chNull as well
-    fgMemoryManager->deallocate(retBuf);//delete[] retBuf;
+    manager->deallocate(retBuf);//delete[] retBuf;
     return;
 }
 
 //
 // remove whitespace
 //
-void XMLString::removeWS(XMLCh* const toConvert)
+void XMLString::removeWS(XMLCh* const toConvert
+                         , MemoryManager* const manager)
 {
     // If no string, then its a failure
     if (( !toConvert ) || ( !*toConvert ))
         return;
 
-    XMLCh* retBuf = (XMLCh*) fgMemoryManager->allocate
+    XMLCh* retBuf = (XMLCh*) manager->allocate
     (
         (XMLString::stringLen(toConvert) + 1) * sizeof(XMLCh)
     );//new XMLCh[ XMLString::stringLen(toConvert) + 1];
@@ -1737,7 +1767,7 @@ void XMLString::removeWS(XMLCh* const toConvert)
 
     *retPtr = chNull;
     XMLString::moveChars(toConvert, retBuf, stringLen(retBuf)+1); //copy the last chNull as well
-    fgMemoryManager->deallocate(retBuf);//delete[] retBuf;
+    manager->deallocate(retBuf);//delete[] retBuf;
     return;
 }
 
