@@ -56,6 +56,27 @@
 
 /*
  * $Log: RegularExpression.cpp,v $
+ * Revision 1.22  2004/01/13 20:05:00  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.19  2003/12/24 15:24:15  cargilld
+ * More updates to memory management so that the static memory manager.
+ *
+ * Revision 1.18  2003/12/17 05:16:59  neilg
+ * ensure all uses of ArrayJanitor use a memory manager
+ *
+ * Revision 1.17  2003/12/17 00:18:37  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.16  2003/12/16 12:25:48  cargilld
+ * Change a conditional expression to an if-else to avoid a compiler problem.
+ *
+ * Revision 1.15  2003/10/01 16:32:40  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.14  2003/08/14 02:57:27  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.13  2003/05/25 21:42:41  knoaman
  * Allocate/Deallocate Context::xxx only when necessary.
  *
@@ -139,6 +160,7 @@
 #include <xercesc/util/ParseException.hpp>
 #include <xercesc/util/IllegalArgumentException.hpp>
 #include <xercesc/framework/XMLBuffer.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -164,13 +186,13 @@ RangeToken*          RegularExpression::fWordRange = 0;
 // ---------------------------------------------------------------------------
 //  RegularExpression::Context: Constructors and Destructor
 // ---------------------------------------------------------------------------
-RegularExpression::Context::Context(MemoryManager* const manager) :
-    fInUse(false)
-	, fAdoptMatch(false)
+RegularExpression::Context::Context(MemoryManager* const manager) :    
+	fAdoptMatch(false)
     , fStart(0)
 	, fLimit(0)
 	, fLength(0)
 	, fSize(0)
+    , fStringMaxLen(0)
 	, fOffsets(0)
 	, fMatch(0)
 	, fString(0)
@@ -193,22 +215,31 @@ RegularExpression::Context::~Context()
 //  RegularExpression::Context: Public methods
 // ---------------------------------------------------------------------------
 void RegularExpression::Context::reset(const XMLCh* const string
-                                       , const int start, const int limit
+                                       , const int stringLen
+                                       , const int start
+                                       , const int limit
                                        , const int noClosures)
 {
-    if (fString)
-        fMemoryManager->deallocate(fString);//delete [] fString;
+    if (stringLen > fStringMaxLen || !fString) {
+
+        fStringMaxLen = stringLen;
+
+        if (fString)
+            fMemoryManager->deallocate(fString);
+
+        fString = XMLString::replicate(string, fMemoryManager);
+    }
+    else {
+        memcpy(fString, string, (stringLen + 1) * sizeof(XMLCh));
+    }
 
-    fString = XMLString::replicate(string, fMemoryManager);
 	fStart = start;
 	fLimit = limit;
-	fLength = fLimit - fStart;
-	fInUse = true;
+	fLength = fLimit - fStart;	
 	if (fAdoptMatch)
 		delete fMatch;
 	fMatch = 0;
 
-
 	if (fSize != noClosures) {
 
 		if (fOffsets)
@@ -261,7 +292,6 @@ RegularExpression::RegularExpression(const char* const pattern,
 	 fMinLength(0),
 	 fNoClosures(0),
 	 fOptions(0),
-	 fContext(0),
 	 fBMPattern(0),
 	 fPattern(0),
 	 fFixedString(0),
@@ -278,7 +308,11 @@ RegularExpression::RegularExpression(const char* const pattern,
         ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 		setPattern(tmpBuf);
 	}
-	catch (...) {
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
+    catch (...) {
 
 		cleanUp();
 		throw;
@@ -294,7 +328,6 @@ RegularExpression::RegularExpression(const char* const pattern,
 	 fMinLength(0),
 	 fNoClosures(0),
 	 fOptions(0),
-	 fContext(0),
 	 fBMPattern(0),
 	 fPattern(0),
 	 fFixedString(0),
@@ -313,7 +346,11 @@ RegularExpression::RegularExpression(const char* const pattern,
 		ArrayJanitor<XMLCh> janOps(tmpOptions, fMemoryManager);
 		setPattern(tmpBuf, tmpOptions);
 	}
-	catch (...) {
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
+    catch (...) {
 
 		cleanUp();
 		throw;
@@ -329,7 +366,6 @@ RegularExpression::RegularExpression(const XMLCh* const pattern,
 	 fMinLength(0),
 	 fNoClosures(0),
 	 fOptions(0),
-	 fContext(0),
 	 fBMPattern(0),
 	 fPattern(0),
 	 fFixedString(0),
@@ -344,7 +380,11 @@ RegularExpression::RegularExpression(const XMLCh* const pattern,
 
 		setPattern(pattern);
 	}
-	catch (...) {
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
+    catch (...) {
 
 		cleanUp();
 		throw;
@@ -360,7 +400,6 @@ RegularExpression::RegularExpression(const XMLCh* const pattern,
 	 fMinLength(0),
 	 fNoClosures(0),
 	 fOptions(0),
-	 fContext(0),
 	 fBMPattern(0),
 	 fPattern(0),
 	 fFixedString(0),
@@ -375,7 +414,11 @@ RegularExpression::RegularExpression(const XMLCh* const pattern,
 
 		setPattern(pattern, options);
 	}
-	catch (...) {
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
+    catch (...) {
 
 		cleanUp();
 		throw;
@@ -397,9 +440,19 @@ void RegularExpression::setPattern(const XMLCh* const pattern,
 	fOptions = parseOptions(options);
 	fPattern = XMLString::replicate(pattern, fMemoryManager);
 
-	RegxParser* regxParser = isSet(fOptions, XMLSCHEMA_MODE)
-		? new (fMemoryManager) ParserForXMLSchema(fMemoryManager) 
-        : new (fMemoryManager) RegxParser(fMemoryManager);
+    // the following construct causes an error in an Intel 7.1 32 bit compiler for 
+    // red hat linux 7.2
+    // (when an exception is thrown the wrong object is deleted)
+    //RegxParser* regxParser = isSet(fOptions, XMLSCHEMA_MODE)
+    //	? new (fMemoryManager) ParserForXMLSchema(fMemoryManager) 
+    //    : new (fMemoryManager) RegxParser(fMemoryManager);
+    RegxParser* regxParser;
+    if (isSet(fOptions, XMLSCHEMA_MODE)) {
+	    regxParser = new (fMemoryManager) ParserForXMLSchema(fMemoryManager);
+    }
+    else {
+        regxParser = new (fMemoryManager) RegxParser(fMemoryManager);
+    }
 
     if (regxParser) {
         regxParser->setTokenFactory(fTokenFactory);
@@ -414,86 +467,75 @@ void RegularExpression::setPattern(const XMLCh* const pattern,
 // ---------------------------------------------------------------------------
 //  RegularExpression: Matching methods
 // ---------------------------------------------------------------------------
-bool RegularExpression::matches(const char* const expression) {
+bool RegularExpression::matches(const char* const expression
+                                , MemoryManager* const manager) {
 
-    XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
-	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), 0);
+    XMLCh* tmpBuf = XMLString::transcode(expression, manager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, manager);
+	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), 0, manager);
 }
 
 bool RegularExpression::matches(const char* const expression,
-								const int start, const int end) {
+								const int start, const int end
+                                , MemoryManager* const manager) {
 
-	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
-	return matches(tmpBuf, start, end, 0);
+	XMLCh* tmpBuf = XMLString::transcode(expression, manager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, manager);
+	return matches(tmpBuf, start, end, 0, manager);
 }
 
 bool RegularExpression::matches(const char* const expression,
-								Match* const match)				{
+								Match* const match
+                                , MemoryManager* const manager)				{
 
-	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
-	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), match);
+	XMLCh* tmpBuf = XMLString::transcode(expression, manager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, manager);
+	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), match, manager);
 }
 
 bool RegularExpression::matches(const char* const expression, const int start,
-                                const int end, Match* const pMatch)				{
+                                const int end, Match* const pMatch
+                                , MemoryManager* const manager)				{
 
-	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
-	return matches(tmpBuf, start, end, pMatch);
+	XMLCh* tmpBuf = XMLString::transcode(expression, manager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, manager);
+	return matches(tmpBuf, start, end, pMatch, manager);
 }
 
 
 // ---------------------------------------------------------------------------
 //  RegularExpression: Matching methods - Wide char version
 // ---------------------------------------------------------------------------
-bool RegularExpression::matches(const XMLCh* const expression) {
+bool RegularExpression::matches(const XMLCh* const expression, MemoryManager* const manager) {
 
-	return matches(expression, 0, XMLString::stringLen(expression), 0);
+	return matches(expression, 0, XMLString::stringLen(expression), 0, manager);
 }
 
 bool RegularExpression::matches(const XMLCh* const expression,
-								const int start, const int end) {
+								const int start, const int end
+                                , MemoryManager* const manager) {
 
-	return matches(expression, start, end, 0);
+	return matches(expression, start, end, 0, manager);
 }
 
 bool RegularExpression::matches(const XMLCh* const expression,
-								Match* const match)				{
+								Match* const match
+                                , MemoryManager* const manager)				{
 
-	return matches(expression, 0, XMLString::stringLen(expression), match);
+	return matches(expression, 0, XMLString::stringLen(expression), match, manager);
 }
 
 bool RegularExpression::matches(const XMLCh* const expression, const int start,
-                                const int end, Match* const pMatch)				{
+                                const int end, Match* const pMatch
+                                , MemoryManager* const manager)	{
 		
 	if (fOperations == 0)
 		prepare();
 
-	Context* context = 0;
-	Context* tmpContext = 0;
+	Context context(manager);
 	int		 strLength = XMLString::stringLen(expression);
 
-	{
-		XMLMutexLock lockInit(&fMutex);
-
-		if (fContext == 0)
-			fContext = new (fMemoryManager) Context(fMemoryManager);
-
-		if (fContext->fInUse) {
-			context = new (fMemoryManager) Context(fMemoryManager);
-			tmpContext = context;
-		}
-		else {
-			context = fContext;
-		}
-
-		context->reset(expression, start, end, fNoClosures);
-	}
-
-	Janitor<Context> janContext(tmpContext);
+    context.reset(expression, strLength, start, end, fNoClosures);
 
 	bool adoptMatch = false;
 	Match* lMatch = pMatch;
@@ -508,24 +550,22 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 		adoptMatch = true;
 	}
 
-	if (context->fAdoptMatch)
-		delete context->fMatch;
-    context->fMatch = lMatch;
-	context->fAdoptMatch = adoptMatch;
+	if (context.fAdoptMatch)
+		delete context.fMatch;
+    context.fMatch = lMatch;
+	context.fAdoptMatch = adoptMatch;
 
 	if (isSet(fOptions, XMLSCHEMA_MODE)) {
 
-		int matchEnd = match(context, fOperations, context->fStart, 1);
+		int matchEnd = match(&context, fOperations, context.fStart, 1);
 
-		if (matchEnd == context->fLimit) {
+		if (matchEnd == context.fLimit) {
 
-			if (context->fMatch != 0) {
+			if (context.fMatch != 0) {
 
-				context->fMatch->setStartPos(0, context->fStart);
-				context->fMatch->setEndPos(0, matchEnd);
-			}
-
-			context->fInUse = false;
+				context.fMatch->setStartPos(0, context.fStart);
+				context.fMatch->setEndPos(0, matchEnd);
+			}		
 			return true;
 		}
 
@@ -537,19 +577,16 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	 */
 	if (fFixedStringOnly) {
 
-		int ret = fBMPattern->matches(expression, context->fStart,
-			                          context->fLimit);
+		int ret = fBMPattern->matches(expression, context.fStart,
+			                          context.fLimit);
 		if (ret >= 0) {
 
-			if (context->fMatch != 0) {
-				context->fMatch->setStartPos(0, ret);
-				context->fMatch->setEndPos(0, ret + strLength);
-			}
-
-			context->fInUse = false;
+			if (context.fMatch != 0) {
+				context.fMatch->setStartPos(0, ret);
+				context.fMatch->setEndPos(0, ret + strLength);
+			}		
 			return true;
-		}
-		context->fInUse = false;
+		}		
 		return false;
 	}
 
@@ -560,17 +597,15 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	 */
 	if (fFixedString != 0) {
 
-		int ret = fBMPattern->matches(expression, context->fStart,
-                                      context->fLimit);
+		int ret = fBMPattern->matches(expression, context.fStart,
+                                      context.fLimit);
 
 		if (ret < 0) { // No match
-
-			context->fInUse = false;
 			return false;
 		}
 	}
 
-	int limit = context->fLimit - fMinLength;
+	int limit = context.fLimit - fMinLength;
 	int matchStart;
 	int matchEnd = -1;
 
@@ -581,13 +616,13 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
         && fOperations->getChild()->getOpType() == Op::O_DOT) {
 
 		if (isSet(fOptions, SINGLE_LINE)) {
-			matchStart = context->fStart;
-			matchEnd = match(context, fOperations, matchStart, 1);
+			matchStart = context.fStart;
+			matchEnd = match(&context, fOperations, matchStart, 1);
 		}
 		else {
 			bool previousIsEOL = true;
 
-			for (matchStart=context->fStart; matchStart<=limit; matchStart++) {
+			for (matchStart=context.fStart; matchStart<=limit; matchStart++) {
 
 				XMLCh ch = expression[matchStart];
 				if (RegxUtil::isEOLChar(ch)) {
@@ -596,7 +631,7 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 				else {
 
 					if (previousIsEOL) {
-						if (0 <= (matchEnd = match(context, fOperations,
+						if (0 <= (matchEnd = match(&context, fOperations,
                                                    matchStart, 1)))
                             break;
 					}
@@ -617,11 +652,11 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 			if (ignoreCase)
 				range = fFirstChar->getCaseInsensitiveToken(fTokenFactory);
 
-			for (matchStart=context->fStart; matchStart<=limit; matchStart++) {
+			for (matchStart=context.fStart; matchStart<=limit; matchStart++) {
 
                 XMLInt32 ch;
 
-				if (!context->nextCh(ch, matchStart, 1))
+				if (!context.nextCh(ch, matchStart, 1))
 					break;
 
 				if (!range->match(ch)) {
@@ -634,7 +669,7 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 					continue;
 				}
 
-				if (0 <= (matchEnd = match(context,fOperations,matchStart,1)))
+				if (0 <= (matchEnd = match(&context,fOperations,matchStart,1)))
 					break;
             }
 		}
@@ -643,9 +678,9 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
             /*
              *	Straightforward matching
              */
-			for (matchStart=context->fStart; matchStart<=limit; matchStart++) {
+			for (matchStart=context.fStart; matchStart<=limit; matchStart++) {
 
-				if (0 <= (matchEnd = match(context,fOperations,matchStart,1)))
+				if (0 <= (matchEnd = match(&context,fOperations,matchStart,1)))
 					break;
 			}
 		}
@@ -653,17 +688,13 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 
 	if (matchEnd >= 0) {
 
-		if (context->fMatch != 0) {
+		if (context.fMatch != 0) {
 
-			context->fMatch->setStartPos(0, matchStart);
-			context->fMatch->setEndPos(0, matchEnd);
-		}
-
-		context->fInUse = false;
+			context.fMatch->setStartPos(0, matchStart);
+			context.fMatch->setEndPos(0, matchEnd);
+		}		
 		return true;
 	}
-
-	context->fInUse = false;
 	return false;
 }
 
@@ -672,16 +703,16 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 // ---------------------------------------------------------------------------
 RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression) {
 
-  XMLCh* tmpBuf = XMLString::transcode(expression);
-  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
   return tokenize(tmpBuf, 0, XMLString::stringLen(tmpBuf));
 }
 
 RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression,
 								const int start, const int end) {
 
-  XMLCh* tmpBuf = XMLString::transcode(expression);
-  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
   return tokenize(tmpBuf, start, end);
 }
 
@@ -709,29 +740,12 @@ RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expressi
 
   RefArrayVectorOf<XMLCh>* tokenStack = new (fMemoryManager) RefArrayVectorOf<XMLCh>(16, true, fMemoryManager);
 
-  Context* context = 0;
-  Context* tmpContext = 0;
+  Context context(fMemoryManager);
 
   int		 strLength = XMLString::stringLen(expression);
  
-  {
- 	   XMLMutexLock lockInit(&fMutex);
-
- 	   if (fContext == 0)
- 	     fContext = new (fMemoryManager) Context(fMemoryManager);
-
- 	   if (fContext->fInUse) {
- 	     context = new (fMemoryManager) Context(fMemoryManager);
- 	     tmpContext = context;
- 	   }
- 	   else {
- 	     context = fContext;
- 	   }
-
- 	   context->reset(expression, start, end, fNoClosures);
-  }
-
-  Janitor<Context> janContext(tmpContext);
+  context.reset(expression, strLength, start, end, fNoClosures);
+ 
 
   Match* lMatch = 0;
   bool adoptMatch = false;
@@ -742,36 +756,34 @@ RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expressi
     lMatch->setNoGroups(fNoGroups);
   }
 
-  if (context->fAdoptMatch)
- 	  delete context->fMatch;
+  if (context.fAdoptMatch)
+ 	  delete context.fMatch;
   
-  context->fMatch = lMatch;
-  context->fAdoptMatch = adoptMatch;
+  context.fMatch = lMatch;
+  context.fAdoptMatch = adoptMatch;
 
   int tokStart = start;
   int matchStart = start;
 
   for (; matchStart <= end; matchStart++) { 
   
- 	  int matchEnd = match(context, fOperations, matchStart, 1);
+ 	  int matchEnd = match(&context, fOperations, matchStart, 1);
   
  	  if (matchEnd != -1) {
 
- 	    if (context->fMatch != 0) {
- 	      context->fMatch->setStartPos(0, context->fStart);
- 	      context->fMatch->setEndPos(0, matchEnd);
+ 	    if (context.fMatch != 0) {
+ 	      context.fMatch->setStartPos(0, context.fStart);
+ 	      context.fMatch->setEndPos(0, matchEnd);
  	    }
 
       if (subEx){
         subEx->addElement(lMatch);
-        lMatch = new (fMemoryManager) Match(*(context->fMatch));
+        lMatch = new (fMemoryManager) Match(*(context.fMatch));
         adoptMatch = true;
         
-        context->fAdoptMatch = adoptMatch;
-        context->fMatch = lMatch;
+        context.fAdoptMatch = adoptMatch;
+        context.fMatch = lMatch;
       }
-  
- 	    context->fInUse = false;
 
       XMLCh* token;
       if (tokStart == matchStart){
@@ -798,7 +810,7 @@ RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expressi
         (
             (matchStart + 1 - tokStart) * sizeof(XMLCh)
         );//new XMLCh[matchStart + 1 - tokStart];
-        XMLString::subString(token, expression, tokStart, matchStart);
+        XMLString::subString(token, expression, tokStart, matchStart, fMemoryManager);
         tokenStack->addElement(token);
       } 
 
@@ -821,7 +833,7 @@ RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expressi
     (
         (strLength + 1 - tokStart) * sizeof(XMLCh)
     );//new XMLCh[strLength + 1 - tokStart];
-    XMLString::subString(token, expression, tokStart, strLength);
+    XMLString::subString(token, expression, tokStart, strLength, fMemoryManager);
   }  
 
   if (!XMLString::equals(fPattern, &chNull)) 
@@ -840,10 +852,10 @@ RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expressi
 XMLCh* RegularExpression::replace(const char* const matchString, 
                                   const char* const replaceString){
 
-	XMLCh* tmpBuf = XMLString::transcode(matchString);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
-    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+	XMLCh* tmpBuf = XMLString::transcode(matchString, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
+	XMLCh* tmpBuf2 = XMLString::transcode(replaceString, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2, fMemoryManager);
 
 	return replace(tmpBuf, tmpBuf2, 0, XMLString::stringLen(tmpBuf));
 }
@@ -852,10 +864,10 @@ XMLCh* RegularExpression::replace(const char* const matchString,
                                   const char* const replaceString,
                                   const int start, const int end){
 
- 	XMLCh* tmpBuf = XMLString::transcode(matchString);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
- 	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
-    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+ 	XMLCh* tmpBuf = XMLString::transcode(matchString, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
+ 	XMLCh* tmpBuf2 = XMLString::transcode(replaceString, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2, fMemoryManager);
   
   return replace(tmpBuf, tmpBuf2, start, end);
 }
@@ -875,9 +887,10 @@ XMLCh* RegularExpression::replace(const XMLCh* const matchString,
                                   const XMLCh* const replaceString,
                                   const int start, const int end)
 {
+
   //check if matches zero length string - throw error if so
-  if (matches(XMLUni::fgZeroLenString)){
-		ThrowXML(RuntimeException, XMLExcepts::Regex_RepPatMatchesZeroString);
+  if (matches(XMLUni::fgZeroLenString, fMemoryManager)){
+		ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Regex_RepPatMatchesZeroString, fMemoryManager);
   }
       
   RefVectorOf<Match> *subEx = new (fMemoryManager) RefVectorOf<Match>(10, true, fMemoryManager);
@@ -896,7 +909,7 @@ XMLCh* RegularExpression::replace(const XMLCh* const matchString,
     numSubEx = subEx->elementAt(0)->getNoGroups() - 1;
   
   int tokStackSize = tokenStack->size();
-  const XMLCh* curRepString = XMLString::replicate(replaceString);
+  const XMLCh* curRepString = XMLString::replicate(replaceString, fMemoryManager);
     
   for (int i = 0; i < tokStackSize; i++){
       
@@ -907,15 +920,15 @@ XMLCh* RegularExpression::replace(const XMLCh* const matchString,
       //if there are subExpressions, then determine the string we want to 
       //substitute in.
         if (numSubEx != 0) {
-            delete[] (XMLCh*)curRepString;
+            fMemoryManager->deallocate((XMLCh*)curRepString);
             curRepString = subInExp(replaceString, matchString, subEx->elementAt(i));     
         }
       result.append(curRepString);
     }
   }  
     
-  delete[] (XMLCh*)curRepString;
-  return XMLString::replicate(result.getRawBuffer()); 
+  fMemoryManager->deallocate((XMLCh*)curRepString);
+  return XMLString::replicate(result.getRawBuffer(), fMemoryManager); 
     
 }
 
@@ -1309,7 +1322,7 @@ bool RegularExpression::matchBackReference(Context* const context,
 										   const bool ignoreCase)
 {
 	if (refNo <=0 || refNo >= fNoGroups)
-		ThrowXML(IllegalArgumentException, XMLExcepts::Regex_BadRefNo);
+		ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::Regex_BadRefNo, fMemoryManager);
 
 	if (context->fMatch->getStartPos(refNo) < 0
 		|| context->fMatch->getEndPos(refNo) < 0)
@@ -1410,7 +1423,7 @@ int RegularExpression::parseOptions(const XMLCh* const options)
 		int v = getOptionValue(options[i]);
 
 		if (v == 0)
-			ThrowXML1(ParseException, XMLExcepts::Regex_UnknownOption, options);
+			ThrowXMLwithMemMgr1(ParseException, XMLExcepts::Regex_UnknownOption, options, fMemoryManager);
 
 		opts |= v;
 	}
@@ -1474,7 +1487,7 @@ Op* RegularExpression::compile(const Token* const token, Op* const next,
 		ret = compileCondition(token, next, reverse);
 		break;
 	default:
-		ThrowXML(RuntimeException, XMLExcepts::Regex_UnknownTokenType);
+		ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Regex_UnknownTokenType, fMemoryManager);
 		break; // this line to be deleted
 	}
 
@@ -1497,7 +1510,7 @@ const XMLCh* RegularExpression::subInExp(const XMLCh* const repString,
   int numSubExp = subEx->getNoGroups() - 1;
 
   if (numSubExp == 0)
-    return XMLString::replicate(repString);
+    return XMLString::replicate(repString, fMemoryManager);
   
   bool notEscaped = true;                 
   
@@ -1518,15 +1531,15 @@ const XMLCh* RegularExpression::subInExp(const XMLCh* const repString,
       if (!XMLString::isDigit(*ptr)){
        
         //invalid replace string - $ must be followed by a digit
-				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+				ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Regex_InvalidRepPattern, fMemoryManager);
       }
         
       indexStr[0] = *ptr;                     //get the digit 
-      index = XMLString::parseInt(indexStr);  //convert it to an int
+      index = XMLString::parseInt(indexStr, fMemoryManager);  //convert it to an int
 
       //now check that the index is legal
       if (index > numSubExp){
-				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+				ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Regex_InvalidRepPattern, fMemoryManager);
       }
         
       int start = subEx->getStartPos(index);
@@ -1542,7 +1555,7 @@ const XMLCh* RegularExpression::subInExp(const XMLCh* const repString,
       //if you have a slash and then a character that's not a $ or /, 
       //then it's an invalid replace string  
       if (!notEscaped && (*ptr != chDollarSign && *ptr != chBackSlash)){
-				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+				ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Regex_InvalidRepPattern, fMemoryManager);
       }
       
       if (*ptr == chBackSlash){
@@ -1556,7 +1569,7 @@ const XMLCh* RegularExpression::subInExp(const XMLCh* const repString,
     }
   }
 
-  return XMLString::replicate(newString.getRawBuffer());
+  return XMLString::replicate(newString.getRawBuffer(), fMemoryManager);
        
 }
 
@@ -1615,7 +1628,7 @@ void RegularExpression::prepare() {
 		}
 
 		fBMPattern = new (fMemoryManager) BMPattern(fFixedString, 256,
-								  isSet(fOptions, IGNORE_CASE));
+								  isSet(fOptions, IGNORE_CASE), fMemoryManager);
 	}
 	else if (!isSet(fOptions, XMLSCHEMA_MODE) &&
 			 !isSet(fOptions, PROHIBIT_FIXED_STRING_OPTIMIZATION)) {
@@ -1652,7 +1665,7 @@ unsigned short RegularExpression::getCharType(const XMLCh ch) {
 
 				fWordRange = fTokenFactory->getRange(fgUniIsWord);
 				if (fWordRange == 0)
-					ThrowXML1(RuntimeException, XMLExcepts::Regex_RangeTokenGetError, fgUniIsWord);
+					ThrowXMLwithMemMgr1(RuntimeException, XMLExcepts::Regex_RangeTokenGetError, fgUniIsWord, fMemoryManager);
 			}
 
 			return fWordRange->match(ch) ? WT_LETTER : WT_OTHER;
