@@ -56,6 +56,12 @@
 
 /*
  * $Log: StringToken.cpp,v $
+ * Revision 1.5  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.4  2003/12/17 00:18:37  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
  * Revision 1.3  2003/05/16 00:03:10  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -87,9 +93,9 @@ StringToken::StringToken(const unsigned short tokType,
                          const XMLCh* const literal,
                          const int refNo,
                          MemoryManager* const manager)
-    : Token(tokType)
-    , fString(XMLString::replicate(literal, manager))
+    : Token(tokType, manager)
     , fRefNo(refNo)
+    , fString(XMLString::replicate(literal, manager))    
     , fMemoryManager(manager)
 {
 
