@@ -56,6 +56,18 @@
 
 /*
  * $Log: TokenFactory.cpp,v $
+ * Revision 1.12  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.11  2004/01/09 22:41:58  knoaman
+ * Use a global static mutex for locking when creating local static mutexes instead of compareAndSwap
+ *
+ * Revision 1.10  2003/12/17 00:18:37  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.9  2003/10/17 16:44:34  knoaman
+ * Fix multithreading problem.
+ *
  * Revision 1.8  2003/05/18 14:02:06  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -120,11 +132,51 @@
 #include <xercesc/util/regx/BlockRangeFactory.hpp>
 #include <xercesc/util/regx/RangeTokenMap.hpp>
 #include <xercesc/util/regx/RegxDefs.hpp>
+#include <xercesc/util/XMLRegisterCleanup.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+// ---------------------------------------------------------------------------
+//  Static member data initialization
+// ---------------------------------------------------------------------------
 bool TokenFactory::fRangeInitialized = false;
 
+// ---------------------------------------------------------------------------
+//  Local static data
+// ---------------------------------------------------------------------------
+static bool               sTokFactoryMutexRegistered = false;
+static XMLMutex*          sTokFactoryMutex = 0;
+static XMLRegisterCleanup tokenFactoryMutexCleanup;
+
+// ---------------------------------------------------------------------------
+//  Local, static functions
+// ---------------------------------------------------------------------------
+//  Cleanup for the TokenFactory mutex
+void TokenFactory::reinitTokenFactoryMutex()
+{
+    delete sTokFactoryMutex;
+    sTokFactoryMutex = 0;
+    sTokFactoryMutexRegistered = false;
+}
+
+//  We need to fault in this mutex. But, since its used for synchronization
+//  itself, we have to do this the low level way using a compare and swap.
+static XMLMutex& gTokenFactoryMutex()
+{
+    if (!sTokFactoryMutexRegistered)
+    {
+        XMLMutexLock lock(XMLPlatformUtils::fgAtomicMutex);
+
+        if (!sTokFactoryMutexRegistered)
+        {
+            sTokFactoryMutex = new XMLMutex;
+            tokenFactoryMutexCleanup.registerCleanup(TokenFactory::reinitTokenFactoryMutex);
+            sTokFactoryMutexRegistered = true;
+        }
+    }
+    return *sTokFactoryMutex;
+}
+
 // ---------------------------------------------------------------------------
 //  TokenFactory: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -163,7 +215,7 @@ Token* TokenFactory::createToken(const unsigned short tokType) {
 	if (tokType == Token::T_EMPTY && fEmpty != 0)
 		return fEmpty;
 
-	Token* tmpTok = new (fMemoryManager) Token(tokType);
+	Token* tmpTok = new (fMemoryManager) Token(tokType, fMemoryManager);
 
 	if (tokType == Token::T_EMPTY) {
 		fEmpty = tmpTok;
@@ -178,7 +230,7 @@ Token* TokenFactory::createToken(const unsigned short tokType) {
 ParenToken* TokenFactory::createLook(const unsigned short tokType,
 									 Token* const token) {
 
-	ParenToken* tmpTok = new (fMemoryManager) ParenToken(tokType, token, 0);
+	ParenToken* tmpTok = new (fMemoryManager) ParenToken(tokType, token, 0, fMemoryManager);
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -187,7 +239,7 @@ ParenToken* TokenFactory::createLook(const unsigned short tokType,
 ParenToken* TokenFactory::createParenthesis(Token* const token,
 											const int noGroups) {
 
-	ParenToken* tmpTok = new (fMemoryManager) ParenToken(Token::T_PAREN, token, noGroups);
+	ParenToken* tmpTok = new (fMemoryManager) ParenToken(Token::T_PAREN, token, noGroups, fMemoryManager);
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -196,8 +248,8 @@ ParenToken* TokenFactory::createParenthesis(Token* const token,
 ClosureToken* TokenFactory::createClosure(Token* const token,
 										  bool isNonGreedy) {
 
-	ClosureToken* tmpTok = isNonGreedy ? new (fMemoryManager) ClosureToken(Token::T_NONGREEDYCLOSURE, token)
-									   : new (fMemoryManager) ClosureToken(Token::T_CLOSURE, token);
+	ClosureToken* tmpTok = isNonGreedy ? new (fMemoryManager) ClosureToken(Token::T_NONGREEDYCLOSURE, token, fMemoryManager)
+									   : new (fMemoryManager) ClosureToken(Token::T_CLOSURE, token, fMemoryManager);
 	
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -206,7 +258,7 @@ ClosureToken* TokenFactory::createClosure(Token* const token,
 ConcatToken* TokenFactory::createConcat(Token* const token1,
                                         Token* const token2) {
 
-    ConcatToken* tmpTok = new (fMemoryManager) ConcatToken(token1, token2);
+    ConcatToken* tmpTok = new (fMemoryManager) ConcatToken(token1, token2, fMemoryManager);
 	
     fTokens->addElement(tmpTok);
     return tmpTok;
@@ -214,8 +266,8 @@ ConcatToken* TokenFactory::createConcat(Token* const token1,
 
 UnionToken* TokenFactory::createUnion(const bool isConcat) {
 
-	UnionToken* tmpTok = isConcat ? new (fMemoryManager) UnionToken(Token::T_CONCAT)
-								  : new (fMemoryManager) UnionToken(Token::T_UNION);
+	UnionToken* tmpTok = isConcat ? new (fMemoryManager) UnionToken(Token::T_CONCAT, fMemoryManager)
+								  : new (fMemoryManager) UnionToken(Token::T_UNION, fMemoryManager);
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -229,14 +281,12 @@ RangeToken* TokenFactory::createRange(const bool isNegRange){
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
-
-	return 0;
 }
 
 CharToken* TokenFactory::createChar(const XMLUInt32 ch, const bool isAnchor) {
 
-	CharToken* tmpTok = isAnchor ? new (fMemoryManager) CharToken(Token::T_ANCHOR, ch)
-								: new (fMemoryManager) CharToken(Token::T_CHAR, ch);
+	CharToken* tmpTok = isAnchor ? new (fMemoryManager) CharToken(Token::T_ANCHOR, ch, fMemoryManager)
+								: new (fMemoryManager) CharToken(Token::T_CHAR, ch, fMemoryManager);
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -262,7 +312,7 @@ ModifierToken* TokenFactory::createModifierGroup(Token* const child,
                                                  const int add,
                                                  const int mask) {
 
-	ModifierToken* tmpTok = new (fMemoryManager) ModifierToken(child, add, mask);
+	ModifierToken* tmpTok = new (fMemoryManager) ModifierToken(child, add, mask, fMemoryManager);
 
 	fTokens->addElement(tmpTok);
 	return tmpTok;
@@ -274,7 +324,7 @@ ConditionToken* TokenFactory::createCondition(const int refNo,
                                               Token* const noFlow) {
 
 	ConditionToken* tmpTok = new (fMemoryManager) ConditionToken(refNo, condition, yesFlow,
-                                                noFlow);
+                                                noFlow, fMemoryManager);
 	fTokens->addElement(tmpTok);
 	return tmpTok;
 }
@@ -446,12 +496,9 @@ Token* TokenFactory::getGraphemePattern() {
 // ---------------------------------------------------------------------------
 void TokenFactory::initializeRegistry() {
 
-    if (fRangeInitialized)
-        return;
-
-    // Use a faux scope to synchronize while we do this
+    if (!fRangeInitialized)
     {
-        XMLMutexLock lockInit(&fMutex);
+        XMLMutexLock lockInit(&gTokenFactoryMutex());
 
         if (!fRangeInitialized) {
 
