@@ -56,6 +56,12 @@
 
 /*
  * $Log: DTDElementDecl.hpp,v $
+ * Revision 1.9  2004/01/29 11:52:30  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.8  2003/12/12 18:35:44  peiyongz
+ * getObjectType()
+ *
  * Revision 1.7  2003/10/10 16:24:51  peiyongz
  * Implementation of Serialization/Deserialization
  *
@@ -238,6 +244,8 @@ public :
      ***/
     DECL_XSERIALIZABLE(DTDElementDecl)
 
+    virtual XMLElementDecl::objectType  getObjectType() const;
+
 private :
     // -----------------------------------------------------------------------
     //  Private helper methods
@@ -247,6 +255,11 @@ private :
     XMLContentModel* makeContentModel() ;
     XMLCh* formatContentModel () const ;
 
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DTDElementDecl(const DTDElementDecl &);
+    DTDElementDecl& operator = (const  DTDElementDecl&);
 
     // -----------------------------------------------------------------------
     //  Private data members
