@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DTDValidator.cpp,v 1.14 2003/05/16 21:43:19 knoaman Exp $
+ * $Id: DTDValidator.cpp,v 1.20 2004/02/03 16:41:25 peiyongz Exp $
  */
 
 
@@ -99,7 +99,7 @@ int DTDValidator::checkContent(XMLElementDecl* const elemDecl
     //  the element decl in our own way of looking at them.
     //
     if (!elemDecl)
-        ThrowXML(RuntimeException, XMLExcepts::Val_InvalidElemId);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::Val_InvalidElemId, getScanner()->getMemoryManager());
 
     //
     //  Get the content spec type of this element. This will tell us what
@@ -131,7 +131,7 @@ int DTDValidator::checkContent(XMLElementDecl* const elemDecl
     }
      else
     {
-        ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMType);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMType, getScanner()->getMemoryManager());
     }
 
     // Went ok, so return success
@@ -166,8 +166,8 @@ void DTDValidator::faultInAttr(XMLAttr& toFill, const XMLAttDef& attDef) const
         //  There is a colon, so we have to split apart the name and prefix
         //  part.
         //
-        XMLCh* tmpNameBuf = XMLString::replicate(fullName);
-        ArrayJanitor<XMLCh> janNameBuf(tmpNameBuf);
+        XMLCh* tmpNameBuf = XMLString::replicate(fullName, getScanner()->getMemoryManager());
+        ArrayJanitor<XMLCh> janNameBuf(tmpNameBuf, getScanner()->getMemoryManager());
 
         // Put a null where the colon is, to split it into two strings
         tmpNameBuf[colonInd] = chNull;
@@ -203,7 +203,7 @@ void
 DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                                 , const XMLCh* const    attrValue
                                 , bool                  preValidation
-                                , const XMLElementDecl* elemDecl)
+                                , const XMLElementDecl*)
 {
     //
     //  Get quick refs to lost of of the stuff in the passed objects in
@@ -274,8 +274,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
         || (type == XMLAttDef::IDRefs)
     );
 
-    // Some trigger flags to avoid issuing redundant errors and whatnot
-    bool sawOneValue;
+    // Some trigger flags to avoid issuing redundant errors and whatnot    
     bool alreadyCapped = false;
 
     //
@@ -297,7 +296,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
     }
     else
     {
-        janTmpVal.reset(XMLString::replicate(attrValue));
+        janTmpVal.reset(XMLString::replicate(attrValue, getScanner()->getMemoryManager()), getScanner()->getMemoryManager());
         pszTmpVal = janTmpVal.get();
     }
 
@@ -307,9 +306,6 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
 
     while (true)
     {
-        // Reset the trigger flags
-        sawOneValue = false;
-
         //
         //  Make sure the first character is a valid first name char, i.e.
         //  if its a Name value. For NmToken values we don't treat the first
@@ -512,7 +508,7 @@ void DTDValidator::preContentValidation(bool reuseGrammar,
                   if(reuseGrammar && reason == XMLElementDecl::JustFaultIn){
                   }
                   else
-                      ThrowXML(RuntimeException, XMLExcepts::DTD_UnknownCreateReason);
+                      ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::DTD_UnknownCreateReason, getScanner()->getMemoryManager());
                 #endif
             }
         }
@@ -529,9 +525,9 @@ void DTDValidator::preContentValidation(bool reuseGrammar,
         //
         XMLAttDefList& attDefList = curElem.getAttDefList();
         bool seenId = false;
-        while (attDefList.hasMoreElements())
+        for(unsigned int i=0; i<attDefList.getAttDefCount(); i++)
         {
-            const XMLAttDef& curAttDef = attDefList.nextElement();
+            const XMLAttDef& curAttDef = attDefList.getAttDef(i);
 
             if (curAttDef.getType() == XMLAttDef::ID)
             {
@@ -553,8 +549,8 @@ void DTDValidator::preContentValidation(bool reuseGrammar,
                 //  We need to verify that all of its possible values
                 //  (in the enum list) refer to valid notations.
                 //
-                XMLCh* list = XMLString::replicate(curAttDef.getEnumeration());
-                ArrayJanitor<XMLCh> janList(list);
+                XMLCh* list = XMLString::replicate(curAttDef.getEnumeration(), getScanner()->getMemoryManager());
+                ArrayJanitor<XMLCh> janList(list, getScanner()->getMemoryManager());
 
                 //
                 //  Search forward for a space or a null. If a null,
