@@ -56,6 +56,18 @@
 
 /*
  * $Log: MixedContentModel.cpp,v $
+ * Revision 1.10  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.9  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.8  2003/11/20 18:12:20  knoaman
+ * Use a bitwise operation to check the node type.
+ *
+ * Revision 1.7  2003/11/07 17:08:11  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
  * Revision 1.6  2003/05/18 14:02:06  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -182,7 +194,7 @@ MixedContentModel::MixedContentModel(const bool             dtd
     //
     ContentSpecNode* curNode = parentContentSpec;
     if (!curNode)
-        ThrowXML(RuntimeException, XMLExcepts::CM_NoParentCSN);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_NoParentCSN, fMemoryManager);
 
     // And now call the private recursive method that iterates the tree
     buildChildList(curNode, children, childTypes);
@@ -256,7 +268,7 @@ bool MixedContentModel::hasDups() const
 int
 MixedContentModel::validateContent( QName** const         children
                                   , const unsigned int    childCount
-                                  , const unsigned int    emptyNamespaceId) const
+                                  , const unsigned int) const
 {
     // must match order
     if (fOrdered) {
@@ -363,7 +375,7 @@ MixedContentModel::validateContent( QName** const         children
 
 int MixedContentModel::validateContentSpecial(QName** const           children
                                             , const unsigned int      childCount
-                                            , const unsigned int      emptyNamespaceId
+                                            , const unsigned int
                                             , GrammarResolver*  const pGrammarResolver
                                             , XMLStringPool*    const pStringPool) const
 {
@@ -481,8 +493,8 @@ MixedContentModel::buildChildList(  ContentSpecNode* const       curNode
     ContentSpecNode* rightNode = curNode->getSecond();
 
     // And recurse according to the type of node
-    if ((curType == ContentSpecNode::Choice)
-    ||  (curType == ContentSpecNode::Sequence))
+    if (((curType & 0x0f) == ContentSpecNode::Choice)
+    ||  ((curType & 0x0f) == ContentSpecNode::Sequence))
     {
         // Recurse on the left and right nodes
         buildChildList(leftNode, toFill, toType);
