@@ -56,6 +56,18 @@
 
 /*
  * $Log: SimpleContentModel.cpp,v $
+ * Revision 1.8  2004/01/29 11:51:21  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.7  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.6  2003/11/20 18:12:20  knoaman
+ * Use a bitwise operation to check the node type.
+ *
+ * Revision 1.5  2003/11/07 17:08:11  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
  * Revision 1.4  2002/11/04 14:54:58  tng
  * C++ Namespace Support.
  *
@@ -146,14 +158,14 @@ XERCES_CPP_NAMESPACE_BEGIN
 int
 SimpleContentModel::validateContent(QName** const       children
                                   , const unsigned int  childCount
-                                  , const unsigned int  emptyNamespaceId) const
+                                  , const unsigned int) const
 {
     //
     //  According to the type of operation, we do the correct type of
     //  content check.
     //
     unsigned int index;
-    switch(fOp)
+    switch(fOp & 0x0f)
     {
         case ContentSpecNode::Leaf :
             //
@@ -325,7 +337,7 @@ SimpleContentModel::validateContent(QName** const       children
             break;
 
         default :
-            ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
             break;
     }
     return -1;
@@ -333,7 +345,7 @@ SimpleContentModel::validateContent(QName** const       children
 
 int SimpleContentModel::validateContentSpecial(QName** const          children
                                             , const unsigned int      childCount
-                                            , const unsigned int      emptyNamespaceId
+                                            , const unsigned int
                                             , GrammarResolver*  const pGrammarResolver
                                             , XMLStringPool*    const pStringPool) const
 {
@@ -345,7 +357,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
     //  content check.
     //
     unsigned int index;
-    switch(fOp)
+    switch(fOp & 0x0f)
     {
         case ContentSpecNode::Leaf :
             //
@@ -485,7 +497,7 @@ int SimpleContentModel::validateContentSpecial(QName** const          children
             break;
 
         default :
-            ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
             break;
     }
     return -1;
@@ -521,7 +533,7 @@ void SimpleContentModel::checkUniqueParticleAttribution
         fSecondChild->setURI(pContentSpecOrgURI[orgURIIndex]);
 
     // only possible violation is when it's a choice
-    if (fOp == ContentSpecNode::Choice) {
+    if ((fOp & 0x0f) == ContentSpecNode::Choice) {
 
         SubstitutionGroupComparator comparator(pGrammarResolver, pStringPool);
 
