@@ -56,6 +56,36 @@
 
 /*
  * $Log: AbstractStringValidator.cpp,v $
+ * Revision 1.22  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.21  2004/01/13 21:18:18  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.20  2004/01/12 16:25:09  neilg
+ * remove use of static buffers
+ *
+ * Revision 1.19  2004/01/06 18:13:59  peiyongz
+ * using the no-exception-thrown ctor
+ *
+ * Revision 1.18  2003/12/31 10:38:00  amassari
+ * Made virtual function checkAdditionalFacet 'const', so that it matches the declaration in a derived class
+ *
+ * Revision 1.17  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.16  2003/11/12 20:32:03  peiyongz
+ * Statless Grammar: ValidationContext
+ *
+ * Revision 1.15  2003/10/17 21:13:43  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.14  2003/10/07 19:39:37  peiyongz
+ * Use of Template_Class Object Serialization/Deserialization API
+ *
+ * Revision 1.13  2003/09/29 21:47:35  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
  * Revision 1.12  2003/05/16 06:01:57  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -132,28 +162,34 @@
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+#include <xercesc/internal/XTemplateSerializer.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 static const int BUF_LEN = 64;
-static XMLCh value1[BUF_LEN+1];
-static XMLCh value2[BUF_LEN+1];
 
-#define  REPORT_FACET_ERROR(val1, val2, except_code)    \
-   XMLString::binToText(val1, value1, BUF_LEN, 10);     \
-   XMLString::binToText(val2, value2, BUF_LEN, 10);     \
-   ThrowXML2(InvalidDatatypeFacetException              \
+#define  REPORT_FACET_ERROR(val1, val2, except_code, manager)    \
+    XMLCh value1[BUF_LEN+1]; \
+    XMLCh value2[BUF_LEN+1]; \
+   XMLString::binToText(val1, value1, BUF_LEN, 10, manager);     \
+   XMLString::binToText(val2, value2, BUF_LEN, 10, manager);     \
+   ThrowXMLwithMemMgr2(InvalidDatatypeFacetException             \
            , except_code                                \
            , value1                                     \
-           , value2);
+           , value2                                     \
+           , manager);
 
-#define  REPORT_VALUE_ERROR(data, val1, val2, except_code)      \
-   XMLString::binToText(val1, value1, BUF_LEN, 10);             \
-   XMLString::binToText(val2, value2, BUF_LEN, 10);             \
-   ThrowXML3(InvalidDatatypeValueException                      \
+#define  REPORT_VALUE_ERROR(data, val1, val2, except_code, manager)      \
+    XMLCh value1[BUF_LEN+1]; \
+    XMLCh value2[BUF_LEN+1]; \
+   XMLString::binToText(val1, value1, BUF_LEN, 10, manager);             \
+   XMLString::binToText(val2, value2, BUF_LEN, 10, manager);             \
+   ThrowXMLwithMemMgr3(InvalidDatatypeValueException                     \
            , except_code                                        \
            , data                                               \
            , value1                                             \
-           , value2);
+           , value2                                             \
+           , manager);
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
@@ -186,18 +222,19 @@ AbstractStringValidator::AbstractStringValidator(
     // assigneAdditionalFacet(), inheritAdditionalFacet().
 }
 
-void AbstractStringValidator::init(RefArrayVectorOf<XMLCh>*           const enums)
+void AbstractStringValidator::init(RefArrayVectorOf<XMLCh>*           const enums
+                                   ,MemoryManager*                    const manager)
 {
 
     if (enums)
     {
         setEnumeration(enums, false);
-        normalizeEnumeration();
+        normalizeEnumeration(manager);
     }
 
-    assignFacet();
-    inspectFacet();
-    inspectFacetBase();
+    assignFacet(manager);
+    inspectFacet(manager);
+    inspectFacetBase(manager);
     inheritFacet();
 
 }
@@ -207,7 +244,7 @@ void AbstractStringValidator::init(RefArrayVectorOf<XMLCh>*           const enum
 //        assign common facets
 //        assign additional facet
 //
-void AbstractStringValidator::assignFacet()
+void AbstractStringValidator::assignFacet(MemoryManager* const manager)
 {
 
     RefHashTableOf<KVStringPair>* facets = getFacets();
@@ -217,7 +254,7 @@ void AbstractStringValidator::assignFacet()
 
     XMLCh* key;
     XMLCh* value;
-    RefHashTableOfEnumerator<KVStringPair> e(facets);
+    RefHashTableOfEnumerator<KVStringPair> e(facets, false, manager);
 
     while (e.hasMoreElements())
     {
@@ -230,33 +267,33 @@ void AbstractStringValidator::assignFacet()
             int val;
             try
             {
-                val = XMLString::parseInt(value);
+                val = XMLString::parseInt(value, manager);
             }
             catch (NumberFormatException&)
             {
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_Len, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_Len, value, manager);
             }
 
             if ( val < 0 )
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_Len, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_Len, value, manager);
 
-                setLength(val);
-                setFacetsDefined(DatatypeValidator::FACET_LENGTH);
+            setLength(val);
+            setFacetsDefined(DatatypeValidator::FACET_LENGTH);
         }
         else if (XMLString::equals(key, SchemaSymbols::fgELT_MINLENGTH))
         {
             int val;
             try
             {
-                val = XMLString::parseInt(value);
+                val = XMLString::parseInt(value, manager);
             }
             catch (NumberFormatException&)
             {
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_minLen, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_minLen, value, manager);
             }
 
             if ( val < 0 )
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_minLen, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_minLen, value, manager);
 
             setMinLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MINLENGTH);
@@ -266,15 +303,15 @@ void AbstractStringValidator::assignFacet()
             int val;
             try
             {
-                val = XMLString::parseInt(value);
+                val = XMLString::parseInt(value, manager);
             }
             catch (NumberFormatException&)
             {
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_maxLen, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_maxLen, value, manager);
             }
 
             if ( val < 0 )
-                ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_maxLen, value);
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_maxLen, value, manager);
 
             setMaxLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MAXLENGTH);
@@ -292,16 +329,16 @@ void AbstractStringValidator::assignFacet()
             bool         retStatus;
             try
             {
-                retStatus = XMLString::textToBin(value, val);
+                retStatus = XMLString::textToBin(value, val, fMemoryManager);
             }
             catch (RuntimeException&)
             {
-                ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed);
+                ThrowXMLwithMemMgr(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed, manager);
             }
 
             if (!retStatus)
             {
-                ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed);
+                ThrowXMLwithMemMgr(InvalidDatatypeFacetException, XMLExcepts::FACET_internalError_fixed, manager);
             }
 
             setFixed(val);
@@ -315,7 +352,7 @@ void AbstractStringValidator::assignFacet()
         //
         else
         {
-            assignAdditionalFacet(key, value);
+            assignAdditionalFacet(key, value, manager);
         }
     }//while
 }//end of assigneFacet()
@@ -325,7 +362,7 @@ void AbstractStringValidator::assignFacet()
 //         check common facets
 //         check Additional Facet Constraint
 //
-void AbstractStringValidator::inspectFacet()
+void AbstractStringValidator::inspectFacet(MemoryManager* const manager)
 {
 
     int thisFacetsDefined = getFacetsDefined();
@@ -337,9 +374,9 @@ void AbstractStringValidator::inspectFacet()
     if ((thisFacetsDefined & DatatypeValidator::FACET_LENGTH) != 0)
     {
         if ((thisFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) != 0)
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_maxLen);
+            ThrowXMLwithMemMgr(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_maxLen, manager);
         else if (((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) != 0))
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_minLen);
+            ThrowXMLwithMemMgr(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_minLen, manager);
     }
 
     // check 4.3.2.c1 must: minLength <= maxLength
@@ -352,7 +389,8 @@ void AbstractStringValidator::inspectFacet()
         {
             REPORT_FACET_ERROR(thisMaxLength
                              , thisMinLength
-                             , XMLExcepts::FACET_maxLen_minLen)
+                             , XMLExcepts::FACET_maxLen_minLen
+                             , manager)
         }
     }
 
@@ -364,7 +402,7 @@ void AbstractStringValidator::inspectFacet()
 //         check enumeration
 //         check Additional Facet Constraint
 //
-void AbstractStringValidator::inspectFacetBase()
+void AbstractStringValidator::inspectFacetBase(MemoryManager* const manager)
 {
 
     AbstractStringValidator *pBaseValidator = (AbstractStringValidator*) getBaseValidator();
@@ -410,7 +448,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisLength
                              , baseMaxLength
-                             , XMLExcepts::FACET_Len_baseMaxLen)
+                             , XMLExcepts::FACET_Len_baseMaxLen
+                             , manager)
         }
         
         if (((baseFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0) &&
@@ -418,7 +457,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisLength
                              , baseMinLength
-                             , XMLExcepts::FACET_Len_baseMinLen)
+                             , XMLExcepts::FACET_Len_baseMinLen
+                             , manager)
         }
     }
 
@@ -431,7 +471,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisMaxLength
                              , baseLength
-                             , XMLExcepts::FACET_maxLen_baseLen)
+                             , XMLExcepts::FACET_maxLen_baseLen
+                             , manager)
         }
         
         if (((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0) &&
@@ -439,7 +480,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisMinLength
                              , baseLength
-                             , XMLExcepts::FACET_minLen_baseLen)
+                             , XMLExcepts::FACET_minLen_baseLen
+                             , manager)
         }
     }
 
@@ -451,7 +493,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisLength
                              , baseLength
-                             , XMLExcepts::FACET_Len_baseLen)
+                             , XMLExcepts::FACET_Len_baseLen
+                             , manager)
         }
     }
 
@@ -469,7 +512,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisMinLength
                              , baseMaxLength
-                             , XMLExcepts::FACET_minLen_basemaxLen)
+                             , XMLExcepts::FACET_minLen_basemaxLen
+                             , manager)
         }
     }
 
@@ -483,7 +527,8 @@ void AbstractStringValidator::inspectFacetBase()
             {
                 REPORT_FACET_ERROR(thisMinLength
                                  , baseMinLength
-                                 , XMLExcepts::FACET_minLen_base_fixed)
+                                 , XMLExcepts::FACET_minLen_base_fixed
+                                 , manager)
             }
 
         }
@@ -493,7 +538,8 @@ void AbstractStringValidator::inspectFacetBase()
             {
                 REPORT_FACET_ERROR(thisMinLength
                                  , baseMinLength
-                                 , XMLExcepts::FACET_minLen_baseminLen)
+                                 , XMLExcepts::FACET_minLen_baseminLen
+                                 , manager)
             }
         }
     }
@@ -506,7 +552,8 @@ void AbstractStringValidator::inspectFacetBase()
         {
             REPORT_FACET_ERROR(thisMaxLength
                              , baseMinLength
-                             , XMLExcepts::FACET_maxLen_baseminLen)
+                             , XMLExcepts::FACET_maxLen_baseminLen
+                             , manager)
         }
     }
 
@@ -520,7 +567,8 @@ void AbstractStringValidator::inspectFacetBase()
             {
                 REPORT_FACET_ERROR(thisMaxLength
                                  , baseMaxLength
-                                 , XMLExcepts::FACET_maxLen_base_fixed)
+                                 , XMLExcepts::FACET_maxLen_base_fixed
+                                 , manager)
             }
         }
         else
@@ -529,7 +577,8 @@ void AbstractStringValidator::inspectFacetBase()
             {
                 REPORT_FACET_ERROR(thisMaxLength
                                  , baseMaxLength
-                                 , XMLExcepts::FACET_maxLen_basemaxLen)
+                                 , XMLExcepts::FACET_maxLen_basemaxLen
+                                 , manager)
             }
         }
     }
@@ -543,13 +592,13 @@ void AbstractStringValidator::inspectFacetBase()
         for ( ; i < enumLength; i++)
         {
             // ask parent do a complete check
-            pBaseValidator->checkContent(getEnumeration()->elementAt(i), false);
+            pBaseValidator->checkContent(getEnumeration()->elementAt(i), (ValidationContext*)0, false, manager);
             // enum shall pass this->checkContent() as well.
-            checkContent(getEnumeration()->elementAt(i), false);
+            checkContent(getEnumeration()->elementAt(i), (ValidationContext*)0, false, manager);
         }
     }
 
-    checkAdditionalFacetConstraints();
+    checkAdditionalFacetConstraints(manager);
 
 } //end of inspectFacetBase
 
@@ -623,23 +672,30 @@ void AbstractStringValidator::inheritFacet()
 // Compare methods
 // -----------------------------------------------------------------------
 int AbstractStringValidator::compare(const XMLCh* const lValue
-                                   , const XMLCh* const rValue)
+                                   , const XMLCh* const rValue
+                                   , MemoryManager*     const)
 {
     return XMLString::compareString(lValue, rValue);
 }
 
-void AbstractStringValidator::validate( const XMLCh* const content)
+void AbstractStringValidator::validate( const XMLCh*             const content
+                                      ,       ValidationContext* const context
+                                      ,       MemoryManager*     const manager)
 {
-    checkContent(content, false);
+    checkContent(content, context, false, manager);
 }
 
-void AbstractStringValidator::checkContent( const XMLCh* const content, bool asBase)
+void AbstractStringValidator::checkContent( const XMLCh*             const content
+                                          ,       ValidationContext* const context
+                                          ,       bool                     asBase
+                                          ,       MemoryManager*     const manager
+                                          )
 {
 
     //validate against base validator if any
     AbstractStringValidator *pBaseValidator = (AbstractStringValidator*) this->getBaseValidator();
     if (pBaseValidator)
-        pBaseValidator->checkContent(content, true);
+        pBaseValidator->checkContent(content, context, true, manager);
 
     int thisFacetsDefined = getFacetsDefined();
 
@@ -649,20 +705,22 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         // lazy construction
         if (getRegex() ==0) {
             try {
-                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
+                // REVISIT: cargillmem fMemoryManager or manager?
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));                
             }
             catch (XMLException &e)
             {
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage());
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage(), fMemoryManager);
             }
         }
 
-        if (getRegex()->matches(content) ==false)
+        if (getRegex()->matches(content, manager) ==false)
         {
-            ThrowXML2(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr2(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_NotMatch_Pattern
                     , content
-                    , getPattern());
+                    , getPattern()
+                    , manager);
         }
     }
 
@@ -671,8 +729,8 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
     if (asBase)
         return;
 
-    checkValueSpace(content);
-    unsigned int length = getLength(content);
+    checkValueSpace(content, manager);
+    unsigned int length = getLength(content, manager);
 
     if (((thisFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) != 0) &&
         (length > getMaxLength()))
@@ -680,7 +738,8 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         REPORT_VALUE_ERROR(content
                          , length
                          , getMaxLength()
-                         , XMLExcepts::VALUE_GT_maxLen)
+                         , XMLExcepts::VALUE_GT_maxLen
+                         , manager)
     }
 
     if (((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) != 0) &&
@@ -689,7 +748,8 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         REPORT_VALUE_ERROR(content
                          , length
                          , getMinLength()
-                         , XMLExcepts::VALUE_LT_minLen)
+                         , XMLExcepts::VALUE_LT_minLen
+                         , manager)
     }
 
     if (((thisFacetsDefined & DatatypeValidator::FACET_LENGTH) != 0) &&
@@ -698,15 +758,16 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         REPORT_VALUE_ERROR(content
                          , length
                          , getLength()
-                         , XMLExcepts::VALUE_NE_Len)
+                         , XMLExcepts::VALUE_NE_Len
+                         , manager)
     }
 
     if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
         (getEnumeration() != 0))
     {
-        XMLCh* normContent = XMLString::replicate(content, fMemoryManager);
-        ArrayJanitor<XMLCh>  jan(normContent, fMemoryManager);
-        normalizeContent(normContent);
+        XMLCh* normContent = XMLString::replicate(content, manager);
+        ArrayJanitor<XMLCh>  jan(normContent, manager);
+        normalizeContent(normContent, manager);
 
         int i=0;
         int enumLength = getEnumeration()->size();
@@ -717,10 +778,10 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         }
 
         if (i == enumLength)
-            ThrowXML1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content);
+            ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content, manager);
     }
 
-    checkAdditionalFacet(content);
+    checkAdditionalFacet(content, manager);
 
 }
 
@@ -729,18 +790,95 @@ const RefArrayVectorOf<XMLCh>* AbstractStringValidator::getEnumString() const
 	return getEnumeration();
 }
 
-void AbstractStringValidator::normalizeEnumeration()
+void AbstractStringValidator::normalizeEnumeration(MemoryManager* const)
 {
     // default implementation: do nothing
     return;
 }
 
-void AbstractStringValidator::normalizeContent(XMLCh* const) const
+void AbstractStringValidator::normalizeContent(XMLCh* const, MemoryManager* const) const
 {
     // default implementation: do nothing
     return;
 }
 
+
+void AbstractStringValidator::checkAdditionalFacetConstraints(MemoryManager* const) const
+{
+    return;
+}
+
+void AbstractStringValidator::checkAdditionalFacet(const XMLCh* const
+                                    , MemoryManager* const) const
+{
+    return;
+}
+
+void AbstractStringValidator::inheritAdditionalFacet()
+{
+    return;
+}
+
+void AbstractStringValidator::assignAdditionalFacet( const XMLCh* const key
+                                                   , const XMLCh* const
+                                                   , MemoryManager* const manager)
+{
+    ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
+            , XMLExcepts::FACET_Invalid_Tag
+            , key
+            , manager);
+}
+
+int AbstractStringValidator::getLength(const XMLCh* const content
+                                   , MemoryManager* const) const
+{
+    return XMLString::stringLen(content);
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_NOCREATE(AbstractStringValidator)
+
+void AbstractStringValidator::serialize(XSerializeEngine& serEng)
+{
+
+    DatatypeValidator::serialize(serEng);
+
+    if (serEng.isStoring())
+    {
+        serEng<<fLength;
+        serEng<<fMaxLength;
+        serEng<<fMinLength;
+        serEng<<fEnumerationInherited;
+
+        /***
+         *
+         * Serialize RefArrayVectorOf<XMLCh>
+         *
+         ***/
+        XTemplateSerializer::storeObject(fEnumeration, serEng);
+
+    }
+    else
+    {
+        serEng>>fLength;
+        serEng>>fMaxLength;
+        serEng>>fMinLength;
+        serEng>>fEnumerationInherited;
+
+        /***
+         *
+         *  Deserialize RefArrayVectorOf<XMLCh>         
+         *
+        ***/
+        XTemplateSerializer::loadObject(&fEnumeration, 8, true, serEng);
+
+    }
+
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
