@@ -56,6 +56,12 @@
 
 /*
  * $Log: AnySimpleTypeDatatypeValidator.cpp,v $
+ * Revision 1.11  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.10  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
  * Revision 1.9  2003/11/06 15:30:07  neilg
  * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
  *
@@ -120,7 +126,7 @@ DatatypeValidator* AnySimpleTypeDatatypeValidator::newInstance
 (
       RefHashTableOf<KVStringPair>* const facets
     , RefArrayVectorOf<XMLCh>* const      enums
-    , const int                           finalSet
+    , const int
     , MemoryManager* const                manager
 )
 {
@@ -128,7 +134,7 @@ DatatypeValidator* AnySimpleTypeDatatypeValidator::newInstance
     delete facets;
     delete enums;
 
-    ThrowXML(RuntimeException, XMLExcepts::DV_InvalidOperation);
+    ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::DV_InvalidOperation, manager);
 
     // to satisfy some compilers
     return 0;
