@@ -55,8 +55,38 @@
  */
 
 /*
- * $Id: DateTimeValidator.cpp,v 1.7 2003/05/18 14:02:07 knoaman Exp $
+ * $Id: DateTimeValidator.cpp,v 1.17 2004/01/29 11:51:22 cargilld Exp $
  * $Log: DateTimeValidator.cpp,v $
+ * Revision 1.17  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.16  2004/01/13 21:18:18  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.15  2004/01/12 16:25:49  neilg
+ * remove use of static buffers
+ *
+ * Revision 1.14  2004/01/06 18:13:59  peiyongz
+ * using the no-exception-thrown ctor
+ *
+ * Revision 1.13  2003/12/19 23:02:25  cargilld
+ * More memory management updates.
+ *
+ * Revision 1.12  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.11  2003/11/12 20:32:03  peiyongz
+ * Statless Grammar: ValidationContext
+ *
+ * Revision 1.10  2003/10/02 19:21:06  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.9  2003/10/01 16:32:41  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.8  2003/08/14 03:00:11  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.7  2003/05/18 14:02:07  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -94,27 +124,26 @@
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-static const int BUF_LEN = 64;
-static XMLCh value1[BUF_LEN+1];
-static XMLCh value2[BUF_LEN+1];
-
 // ---------------------------------------------------------------------------
 //  Macro
 // ---------------------------------------------------------------------------
-#define  REPORT_VALUE_ERROR(val1, val2, except_code)    \
-  ThrowXML2(InvalidDatatypeValueException               \
+#define  REPORT_VALUE_ERROR(val1, val2, except_code, manager)    \
+  ThrowXMLwithMemMgr2(InvalidDatatypeValueException               \
           , except_code                                 \
           , val1->getRawData()                          \
-          , val2->getRawData());
+          , val2->getRawData()                          \
+          , manager);
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
 // ---------------------------------------------------------------------------
 DateTimeValidator::~DateTimeValidator()
-{}
+{
+}
 
 DateTimeValidator::DateTimeValidator(
                           DatatypeValidator*            const baseValidator
@@ -127,23 +156,30 @@ DateTimeValidator::DateTimeValidator(
     //do not invoke init() here !!!
 }
 
-void DateTimeValidator::validate(const XMLCh* const content)
+void DateTimeValidator::validate(const XMLCh*             const content
+                               ,       ValidationContext* const context
+                               ,       MemoryManager*     const manager)
 {
-    checkContent(content, false);
+    checkContent(content, context, false, manager);
 }
 
 int DateTimeValidator::compare(const XMLCh* const value1
-                             , const XMLCh* const value2)
+                             , const XMLCh* const value2
+                             , MemoryManager* const manager)
 {
     try
     {
-        XMLDateTime *pDate1 = parse(value1);
+        XMLDateTime *pDate1 = parse(value1, manager);
         Janitor<XMLDateTime> jName1(pDate1);
-        XMLDateTime *pDate2 = parse(value2);
+        XMLDateTime *pDate2 = parse(value2, manager);
         Janitor<XMLDateTime> jName2(pDate2);
         int result = compareDates(pDate1, pDate2, true);
         return (result==INDETERMINATE)? -1 : result;
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...) // RuntimeException e
     {
         return -1; // revisit after implement compareDates()
@@ -151,31 +187,16 @@ int DateTimeValidator::compare(const XMLCh* const value1
 
 }
 
-void DateTimeValidator::assignAdditionalFacet( const XMLCh* const key
-                                             , const XMLCh* const)
+
+void DateTimeValidator::checkContent(const XMLCh*             const content
+                                   ,       ValidationContext* const context
+                                   ,       bool                     asBase
+                                   ,       MemoryManager*     const manager)
 {
-    ThrowXML1(InvalidDatatypeFacetException
-            , XMLExcepts::FACET_Invalid_Tag
-            , key);
-}
-
-void DateTimeValidator::inheritAdditionalFacet()
-{}
-
-void DateTimeValidator::checkAdditionalFacetConstraints() const
-{}
-
-void DateTimeValidator::checkAdditionalFacetConstraintsBase() const
-{}
-
-void DateTimeValidator::checkContent(const XMLCh* const content
-                                   , bool               asBase)
-{
-
     //validate against base validator if any
     DateTimeValidator *pBaseValidator = (DateTimeValidator*) this->getBaseValidator();
     if (pBaseValidator)
-        pBaseValidator->checkContent(content, true);
+        pBaseValidator->checkContent(content, context, true, manager);
 
     int thisFacetsDefined = getFacetsDefined();
 
@@ -184,21 +205,22 @@ void DateTimeValidator::checkContent(const XMLCh* const content
     {
         // lazy construction
         if (getRegex() ==0) {
-            try {
-                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
+            try {           	
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));                
             }
             catch (XMLException &e)
             {
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage());
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage(), fMemoryManager);
             }
         }
 
-        if (getRegex()->matches(content) ==false)
+        if (getRegex()->matches(content, manager) ==false)
         {
-            ThrowXML2(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr2(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_NotMatch_Pattern
                     , content
-                    , getPattern());
+                    , getPattern()
+                    , manager);
         }
     }
 
@@ -207,78 +229,77 @@ void DateTimeValidator::checkContent(const XMLCh* const content
     if (asBase)
         return;
 
+    // the derived classes' parse() method constructs an
+    // XMLDateTime object anc invokes appropriate XMLDateTime's
+    // parser to parse the content.
+    XMLDateTime dateTimeValue(content, manager);
+    XMLDateTime* dateTime = &dateTimeValue;
+    
+    parse(dateTime);
 
+    // must be < MaxExclusive
+    if ((thisFacetsDefined & DatatypeValidator::FACET_MAXEXCLUSIVE) != 0)
     {
-        // the derived classes' parse() method constructs an
-        // XMLDateTime object anc invokes appropriate XMLDateTime's
-        // parser to parse the content.
-        XMLDateTime *theDate = parse(content);
-        Janitor<XMLDateTime> jname(theDate);
-        int result;
-
-        // must be < MaxExclusive
-        if ( (thisFacetsDefined & DatatypeValidator::FACET_MAXEXCLUSIVE) != 0 )
+        if (compareValues(dateTime, getMaxExclusive()) != XMLDateTime::LESS_THAN)
         {
-            result = compareValues(theDate, getMaxExclusive());
-            if ( result != XMLDateTime::LESS_THAN )
-            {
-                REPORT_VALUE_ERROR(theDate
-                    , getMaxExclusive()
-                    , XMLExcepts::VALUE_exceed_maxExcl)
-            }
-        } 	
+            REPORT_VALUE_ERROR( dateTime
+                              , getMaxExclusive()
+                              , XMLExcepts::VALUE_exceed_maxExcl
+                              , manager)
+        }
+    } 	
 
-        // must be <= MaxInclusive
-        if ( (thisFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0 )
+    // must be <= MaxInclusive
+    if ((thisFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0)
+    {
+        int result = compareValues(dateTime, getMaxInclusive());
+        if ( result == XMLDateTime::GREATER_THAN || result == XMLDateTime::INDETERMINATE )
         {
-            result = compareValues(theDate, getMaxInclusive());
-            if ( result == XMLDateTime::GREATER_THAN || result == XMLDateTime::INDETERMINATE )
-            {
-                REPORT_VALUE_ERROR(theDate
-                    , getMaxInclusive()
-                    , XMLExcepts::VALUE_exceed_maxIncl)
-            }
+            REPORT_VALUE_ERROR( dateTime
+                              , getMaxInclusive()
+                              , XMLExcepts::VALUE_exceed_maxIncl
+                              , manager)
         }
+    }
 
-        // must be >= MinInclusive
-        if ( (thisFacetsDefined & DatatypeValidator::FACET_MININCLUSIVE) != 0 )
+    // must be >= MinInclusive
+    if ((thisFacetsDefined & DatatypeValidator::FACET_MININCLUSIVE) != 0)
+    {
+        int result = compareValues(dateTime, getMinInclusive());
+        if (result == XMLDateTime::LESS_THAN || result == XMLDateTime::INDETERMINATE)
         {
-            result = compareValues(theDate, getMinInclusive());
-            if (result == XMLDateTime::LESS_THAN || result == XMLDateTime::INDETERMINATE )
-            {
-                REPORT_VALUE_ERROR(theDate
-                    , getMinInclusive()
-                    , XMLExcepts::VALUE_exceed_minIncl)
-            }
+            REPORT_VALUE_ERROR( dateTime
+                              , getMinInclusive()
+                              , XMLExcepts::VALUE_exceed_minIncl
+                              , manager)
         }
+    }
 
-        // must be > MinExclusive
-        if ( (thisFacetsDefined & DatatypeValidator::FACET_MINEXCLUSIVE) != 0 )
+    // must be > MinExclusive
+    if ( (thisFacetsDefined & DatatypeValidator::FACET_MINEXCLUSIVE) != 0 )
+    {
+        if (compareValues(dateTime, getMinExclusive()) != XMLDateTime::GREATER_THAN)
         {
-            result = compareValues(theDate, getMinExclusive());
-            if (result != XMLDateTime::GREATER_THAN)
-            {
-                REPORT_VALUE_ERROR(theDate
-                    , getMinExclusive()
-                    , XMLExcepts::VALUE_exceed_minExcl)
-            }
+            REPORT_VALUE_ERROR( dateTime
+                              , getMinExclusive()
+                              , XMLExcepts::VALUE_exceed_minExcl
+                              , manager)
         }
+    }
 
-        if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
-            (getEnumeration() != 0))
+    if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
+        (getEnumeration() != 0))
+    {
+        int i=0;
+        int enumLength = getEnumeration()->size();
+        for ( ; i < enumLength; i++)
         {
-            int i=0;
-            int enumLength = getEnumeration()->size();
-            for ( ; i < enumLength; i++)
-            {
-                if (compareValues(theDate, getEnumeration()->elementAt(i)) == XMLDateTime::EQUAL)
-                    break;
-            }
-
-            if (i == enumLength)
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content);
+            if (compareValues(dateTime, getEnumeration()->elementAt(i)) == XMLDateTime::EQUAL)
+                break;
         }
 
+        if (i == enumLength)
+            ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content, manager);
     }
 }
 
@@ -302,7 +323,7 @@ int DateTimeValidator::compareValues(const XMLNumber* const lValue
  */
 int DateTimeValidator::compareDates(const XMLDateTime* const date1
                                   , const XMLDateTime* const date2
-                                  , bool                     strict)
+                                  , bool)
 {
     return XMLDateTime::compare(date1, date2);
 }
@@ -325,25 +346,25 @@ int DateTimeValidator::compareDates(const XMLDateTime* const date1
 
 void DateTimeValidator::setMaxInclusive(const XMLCh* const value)
 {
-    fMaxInclusive = parse(value);
+    fMaxInclusive = parse(value, fMemoryManager);
 }
 
 void DateTimeValidator::setMaxExclusive(const XMLCh* const value)
 {
-    fMaxExclusive = parse(value);
+    fMaxExclusive = parse(value, fMemoryManager);
 }
 
 void DateTimeValidator::setMinInclusive(const XMLCh* const value)
 {
-    fMinInclusive = parse(value);
+    fMinInclusive = parse(value, fMemoryManager);
 }
 
 void DateTimeValidator::setMinExclusive(const XMLCh* const value)
 {
-    fMinExclusive = parse(value);
+    fMinExclusive = parse(value, fMemoryManager);
 }
 
-void DateTimeValidator::setEnumeration()
+void DateTimeValidator::setEnumeration(MemoryManager* const)
 {
 // to do: do we need to check against base value space???
 
@@ -355,10 +376,34 @@ void DateTimeValidator::setEnumeration()
     fEnumerationInherited = false;
 
     for ( int i = 0; i < enumLength; i++)
-        fEnumeration->insertElementAt(parse(fStrEnumeration->elementAt(i)), i);
+        fEnumeration->insertElementAt(parse(fStrEnumeration->elementAt(i), fMemoryManager), i);
 
 }
 
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_NOCREATE(DateTimeValidator)
+
+void DateTimeValidator::serialize(XSerializeEngine& serEng)
+{
+    /***
+     *
+     * Note: All its derivatives share the same number type, that is
+     *       XMLNumber::DateTime, so this class would write it.
+     ***/
+
+    if (serEng.isStoring())
+    {
+        serEng<<(int) XMLNumber::DateTime;
+    }
+
+    AbstractNumericFacetValidator::serialize(serEng);
+
+    //dateTime can be instantiated during checkContent(), so don't serialize it.
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
