@@ -56,6 +56,52 @@
 
 /*
  * $Log: DecimalDatatypeValidator.cpp,v $
+ * Revision 1.26  2004/01/13 21:18:18  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.25  2004/01/12 16:25:09  neilg
+ * remove use of static buffers
+ *
+ * Revision 1.24  2004/01/06 18:13:59  peiyongz
+ * using the no-exception-thrown ctor
+ *
+ * Revision 1.23  2004/01/06 04:42:53  neilg
+ * On some platforms, it is problematic to throw a different exception from inside the catch block of another exception
+ *
+ * Revision 1.22  2004/01/03 00:04:36  peiyongz
+ * using ctor/parseContent to avoid exception thrown from ctor
+ *
+ * Revision 1.21  2003/12/23 21:50:36  peiyongz
+ * Absorb exception thrown in getCanonicalRepresentation and return 0,
+ * only validate when required
+ *
+ * Revision 1.20  2003/12/19 23:02:25  cargilld
+ * More memory management updates.
+ *
+ * Revision 1.19  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.18  2003/12/11 21:40:24  peiyongz
+ * support for Canonical Representation for Datatype
+ *
+ * Revision 1.17  2003/11/28 18:53:07  peiyongz
+ * Support for getCanonicalRepresentation
+ *
+ * Revision 1.16  2003/11/12 20:32:03  peiyongz
+ * Statless Grammar: ValidationContext
+ *
+ * Revision 1.15  2003/11/06 15:30:07  neilg
+ * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
+ *
+ * Revision 1.14  2003/10/31 22:16:58  peiyongz
+ * to serialize/deserialize data member
+ *
+ * Revision 1.13  2003/10/02 19:21:06  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.12  2003/08/14 03:00:11  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.11  2003/05/18 14:02:07  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -150,17 +196,19 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/datatype/DecimalDatatypeValidator.hpp>
+#include <xercesc/validators/datatype/XMLCanRepGroup.hpp>
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
+
+#include <xercesc/validators/datatype/DatatypeValidatorFactory.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 #include <xercesc/util/XMLBigDecimal.hpp>
+#include <xercesc/util/XMLBigInteger.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
 static const int BUF_LEN = 64;
-static XMLCh value1[BUF_LEN+1];
-static XMLCh value2[BUF_LEN+1];
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
@@ -169,32 +217,37 @@ DecimalDatatypeValidator::DecimalDatatypeValidator(MemoryManager* const manager)
 :AbstractNumericValidator(0, 0, 0, DatatypeValidator::Decimal, manager)
 , fTotalDigits(0)
 , fFractionDigits(0)
-{}
+{
+    setOrdered(XSSimpleTypeDefinition::ORDERED_TOTAL);
+    setNumeric(true);
+}
 
 DecimalDatatypeValidator::DecimalDatatypeValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
-                        , RefArrayVectorOf<XMLCh>*           const enums
+                        , RefArrayVectorOf<XMLCh>*      const enums
                         , const int                           finalSet
-                        , MemoryManager* const manager)
+                        , MemoryManager*                const manager)
 :AbstractNumericValidator(baseValidator, facets, finalSet, DatatypeValidator::Decimal, manager)
 , fTotalDigits(0)
 , fFractionDigits(0)
 {
-    init(enums);
+    init(enums, manager);
 }
 
 DecimalDatatypeValidator::~DecimalDatatypeValidator()
-{}
+{
+}
 
 // -----------------------------------------------------------------------
 // Compare methods
 // -----------------------------------------------------------------------
 int DecimalDatatypeValidator::compare(const XMLCh* const lValue
-                                    , const XMLCh* const rValue)
+                                    , const XMLCh* const rValue
+                                    , MemoryManager* const manager)
 {
-    XMLBigDecimal lObj(lValue, fMemoryManager);
-    XMLBigDecimal rObj(rValue, fMemoryManager);
+    XMLBigDecimal lObj(lValue, manager);
+    XMLBigDecimal rObj(rValue, manager);
 
     return compareValues(&lObj, &rObj);
 }
@@ -226,23 +279,24 @@ DecimalDatatypeValidator::DecimalDatatypeValidator(DatatypeValidator*
 }
 
 void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
-                                                   , const XMLCh* const value)
+                                                   , const XMLCh* const value
+                                                   , MemoryManager* const manager)
 {
     if (XMLString::equals(key, SchemaSymbols::fgELT_TOTALDIGITS))
     {
         int val;
         try
         {
-            val = XMLString::parseInt(value);
+            val = XMLString::parseInt(value, manager);
         }
         catch (NumberFormatException&)
         {
-            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_TotalDigit, value);
+            ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_TotalDigit, value, manager);
         }
 
         // check 4.3.11.c0 must: totalDigits > 0
         if ( val <= 0 )
-            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_PosInt_TotalDigit, value);
+            ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_PosInt_TotalDigit, value, manager);
 
         setTotalDigits(val);
         setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);
@@ -252,25 +306,26 @@ void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
         int val;
         try
         {
-            val = XMLString::parseInt(value);
+            val = XMLString::parseInt(value, manager);
         }
         catch (NumberFormatException&)
         {
-            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_FractDigit, value);
+            ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_Invalid_FractDigit, value, manager);
         }
 
         // check 4.3.12.c0 must: fractionDigits > 0
         if ( val < 0 )
-            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_FractDigit, value);
+            ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_FractDigit, value, manager);
 
         setFractionDigits(val);
         setFacetsDefined(DatatypeValidator::FACET_FRACTIONDIGITS);
     }
     else
     {
-        ThrowXML1(InvalidDatatypeFacetException
+        ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                 , XMLExcepts::FACET_Invalid_Tag
-                , key);
+                , key
+                , manager);
     }
 }
 
@@ -302,7 +357,7 @@ void DecimalDatatypeValidator::inheritAdditionalFacet()
     }
 }
 
-void DecimalDatatypeValidator::checkAdditionalFacetConstraints() const
+void DecimalDatatypeValidator::checkAdditionalFacetConstraints(MemoryManager* const manager) const
 {
     int thisFacetsDefined = getFacetsDefined();
 
@@ -312,18 +367,21 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraints() const
     {
         if ( fFractionDigits > fTotalDigits )
         {
-            XMLString::binToText(getFractionDigits(), value1, BUF_LEN, 10);
-            XMLString::binToText(getTotalDigits(), value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(getFractionDigits(), value1, BUF_LEN, 10, manager);
+            XMLString::binToText(getTotalDigits(), value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_TotDigit_FractDigit
                                  , value2
-                                 , value1);
+                                 , value1
+                                 , manager);
         }
     }
 
 }
 
-void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
+void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase(MemoryManager* const manager) const
 {
 
     DecimalDatatypeValidator *numBase = (DecimalDatatypeValidator*) getBaseValidator();
@@ -341,24 +399,30 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
         if ( (( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
             ( fTotalDigits > numBase->fTotalDigits ))
         {
-            XMLString::binToText(fTotalDigits, value1, BUF_LEN, 10);
-            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(fTotalDigits, value1, BUF_LEN, 10, manager);
+            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_totalDigit_base_totalDigit
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
         }
 
         if ( (( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
             (( numBase->getFixed() & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
             ( fTotalDigits != numBase->fTotalDigits ))
         {
-            XMLString::binToText(fTotalDigits, value1, BUF_LEN, 10);
-            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(fTotalDigits, value1, BUF_LEN, 10, manager);
+            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_totalDigit_base_fixed
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
         }
     }
 
@@ -368,24 +432,30 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
         if ( (( baseFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&
             ( fFractionDigits > numBase->fFractionDigits ))
         {
-            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10);
-            XMLString::binToText(numBase->fFractionDigits, value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10, manager);
+            XMLString::binToText(numBase->fFractionDigits, value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_fractDigit_base_fractDigit
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
                         }
 
         // check question error: fractionDigits > base.totalDigits ???
         if ( (( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
             ( fFractionDigits > numBase->fTotalDigits ))
         {
-            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10);
-            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10, manager);
+            XMLString::binToText(numBase->fTotalDigits, value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_fractDigit_base_totalDigit
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
         }
 
         // fractionDigits != base.fractionDigits if (base.fixed)
@@ -393,12 +463,15 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
             (( numBase->getFixed() & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&
             ( fFractionDigits != numBase->fFractionDigits ))
         {
-            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10);
-            XMLString::binToText(numBase->fFractionDigits, value2, BUF_LEN, 10);
-            ThrowXML2(InvalidDatatypeFacetException
+            XMLCh value1[BUF_LEN+1];
+            XMLCh value2[BUF_LEN+1];
+            XMLString::binToText(fFractionDigits, value1, BUF_LEN, 10, manager);
+            XMLString::binToText(numBase->fFractionDigits, value2, BUF_LEN, 10, manager);
+            ThrowXMLwithMemMgr2(InvalidDatatypeFacetException
                                  , XMLExcepts::FACET_fractDigit_base_fixed
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
         }
     }
 
@@ -407,7 +480,8 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
 int  DecimalDatatypeValidator::compareValues(const XMLNumber* const lValue
                                            , const XMLNumber* const rValue)
 {
-    return XMLBigDecimal::compareValues((XMLBigDecimal*) lValue, (XMLBigDecimal*) rValue);
+    return XMLBigDecimal::compareValues((XMLBigDecimal*) lValue, (XMLBigDecimal*) rValue,
+                                        ((XMLBigDecimal*)lValue)->getMemoryManager());
 }
 
 void  DecimalDatatypeValidator::setMaxInclusive(const XMLCh* const value)
@@ -430,7 +504,7 @@ void  DecimalDatatypeValidator::setMinExclusive(const XMLCh* const value)
     fMinExclusive = new (fMemoryManager) XMLBigDecimal(value, fMemoryManager);
 }
 
-void DecimalDatatypeValidator::setEnumeration()
+void DecimalDatatypeValidator::setEnumeration(MemoryManager* const manager)
 {
     // check 4.3.5.c0 must: enumeration values from the value space of base
     //
@@ -450,14 +524,15 @@ void DecimalDatatypeValidator::setEnumeration()
         {
             for ( i = 0; i < enumLength; i++)
             {
-                numBase->checkContent(fStrEnumeration->elementAt(i), false);
+                numBase->checkContent(fStrEnumeration->elementAt(i), (ValidationContext*)0, false, manager);
             }
         }
         catch (XMLException&)
         {
-            ThrowXML1(InvalidDatatypeFacetException
+            ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                     , XMLExcepts::FACET_enum_base
-                    , fStrEnumeration->elementAt(i));
+                    , fStrEnumeration->elementAt(i)
+                    , manager);
         }
     }
 
@@ -466,15 +541,15 @@ void DecimalDatatypeValidator::setEnumeration()
     //
     for ( i = 0; i < enumLength; i++)
     {
-        checkContent(fStrEnumeration->elementAt(i), false);
+        checkContent(fStrEnumeration->elementAt(i), (ValidationContext*)0, false, manager);
     }
 
-    fEnumeration = new (fMemoryManager) RefVectorOf<XMLNumber>(enumLength, true, fMemoryManager);
+    fEnumeration = new (manager) RefVectorOf<XMLNumber>(enumLength, true, manager);
     fEnumerationInherited = false;
 
     for ( i = 0; i < enumLength; i++)
     {
-        fEnumeration->insertElementAt(new (fMemoryManager) XMLBigDecimal(fStrEnumeration->elementAt(i), fMemoryManager), i);
+        fEnumeration->insertElementAt(new (manager) XMLBigDecimal(fStrEnumeration->elementAt(i), manager), i);        
     }
 
 }
@@ -482,13 +557,15 @@ void DecimalDatatypeValidator::setEnumeration()
 // -----------------------------------------------------------------------
 // Abstract interface from AbstractNumericValidator
 // -----------------------------------------------------------------------
-void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool asBase)
+void DecimalDatatypeValidator::checkContent(const XMLCh*             const content
+                                           ,      ValidationContext* const context
+                                           ,      bool                     asBase
+                                           ,      MemoryManager*     const manager)
 {
-
     //validate against base validator if any
     DecimalDatatypeValidator *pBase = (DecimalDatatypeValidator*) this->getBaseValidator();
     if (pBase)
-        pBase->checkContent(content, true);
+        pBase->checkContent(content, context, true, manager);
 
     int thisFacetsDefined = getFacetsDefined();
 
@@ -498,20 +575,22 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         // lazy construction
         if (getRegex() ==0) {
             try {
-                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
+                // REVISIT: cargillmem fMemoryManager vs manager
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));                
             }
             catch (XMLException &e)
             {
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage());
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage(), manager);
             }
         }
 
-        if (getRegex()->matches(content) ==false)
+        if (getRegex()->matches(content, manager) ==false)
         {
-            ThrowXML2(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr2(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_NotMatch_Pattern
                     , content
-                    , getPattern());
+                    , getPattern()
+                    , manager);
         }
     }
 
@@ -519,52 +598,58 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
     // all other facet were inherited by the derived type
     if (asBase)
         return;
-
+    XMLCh *errorMsg = 0;
     try {
-        XMLBigDecimal theValue(content, fMemoryManager);
-        XMLBigDecimal *theData = &theValue;
-
+        XMLBigDecimal  compareDataValue(content, manager);
+        XMLBigDecimal* compareData = &compareDataValue;        
+        
         if (getEnumeration())
         {
             int i=0;
             int enumLength = getEnumeration()->size();
             for ( ; i < enumLength; i++)
             {
-                if (compareValues(theData, (XMLBigDecimal*) getEnumeration()->elementAt(i)) ==0 )
+                if (compareValues(compareData, (XMLBigDecimal*) getEnumeration()->elementAt(i)) ==0 )
                     break;
             }
 
             if (i == enumLength)
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content);
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content, manager);
         }
 
-        boundsCheck(theData);
+        boundsCheck(compareData, manager);
 
         if ( (thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0 )
         {
-            if ( theData->getScale() > fFractionDigits )
+            if ( compareData->getScale() > fFractionDigits )
             {                
-                XMLString::binToText(theData->getScale(), value1, BUF_LEN, 10);
-                XMLString::binToText(fFractionDigits, value2, BUF_LEN, 10);
-                ThrowXML3(InvalidDatatypeFacetException
+                XMLCh value1[BUF_LEN+1];
+                XMLCh value2[BUF_LEN+1];
+                XMLString::binToText(compareData->getScale(), value1, BUF_LEN, 10, manager);
+                XMLString::binToText(fFractionDigits, value2, BUF_LEN, 10, manager);
+                ThrowXMLwithMemMgr3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_fractDigit
-                                 , theData->getRawData()
+                                 , compareData->getRawData()
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
             }
         }
 
         if ( (thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0 )
         {
-            if ( theData->getTotalDigit() > fTotalDigits )
+            if ( compareData->getTotalDigit() > fTotalDigits )
             {                
-                XMLString::binToText(theData->getTotalDigit(), value1, BUF_LEN, 10);
-                XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10);
-                ThrowXML3(InvalidDatatypeFacetException
+                XMLCh value1[BUF_LEN+1];
+                XMLCh value2[BUF_LEN+1];
+                XMLString::binToText(compareData->getTotalDigit(), value1, BUF_LEN, 10, manager);
+                XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10, manager);
+                ThrowXMLwithMemMgr3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_totalDigit
-                                 , theData->getRawData()
+                                 , compareData->getRawData()
                                  , value1
-                                 , value2);
+                                 , value2
+                                 , manager);
             }
 
             /***
@@ -574,23 +659,147 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
              where i and n are integers such that |i| < 10^totalDigits and 0 <= n <= totalDigits.
             ***/
 
-            if ( theData->getScale() > fTotalDigits )  
+            if ( compareData->getScale() > fTotalDigits )  
             {                
-                XMLString::binToText(theData->getScale(), value1, BUF_LEN, 10);
-                XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10);
-                ThrowXML3(InvalidDatatypeFacetException
+                XMLCh value1[BUF_LEN+1];
+                XMLCh value2[BUF_LEN+1];
+                XMLString::binToText(compareData->getScale(), value1, BUF_LEN, 10, manager);
+                XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10, manager);
+                ThrowXMLwithMemMgr3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_totalDigit
-                                 , theData->getRawData()
+                                 , compareData->getRawData()
                                  , value1
-                                 , value2);
-            }
-        
+                                 , value2
+                                 , manager);
+            }        
         }
-
     }
     catch (XMLException &e)
     {
-       ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::RethrowError, e.getMessage());
+       errorMsg = XMLString::replicate(e.getMessage(), manager);
+    }
+    if(errorMsg)
+    {
+       ArrayJanitor<XMLCh> jan(errorMsg, manager);
+       ThrowXMLwithMemMgr1(InvalidDatatypeFacetException, XMLExcepts::RethrowError, errorMsg, manager);
+    }
+}
+
+/***
+ * 3.2.3 decimal  
+ *
+ * . the preceding optional "+" sign is prohibited. 
+ * . The decimal point is required. 
+ * . Leading and trailing zeroes are prohibited subject to the following: 
+ *   there must be at least one digit to the right and to the left of the decimal point which may be a zero.
+ *
+ *
+ *  3.3.13 integer
+ *  3.3.16 long
+ *  3.3.17 int
+ *  3.3.18 short 
+ *  3.3.19 byte
+ *  3.3.20 nonNegativeInteger
+ *  3.3.25 positiveInteger
+ *
+ *   . the preceding optional "+" sign is prohibited and
+ *   . leading zeroes are prohibited.
+ *   
+ *
+ *  E2-27
+ *  3.3.14 nonPositiveInteger
+ *
+ *   . In the canonical form for zero, the sign must be omitted. 
+ *   . leading zeroes are prohibited.
+ *  
+ *  3.3.15 negativeInteger
+ *  3.3.21 unsignedLong
+ *  3.3.22 unsignedInt
+ *  3.3.23 unsignedShort
+ *  3.3.24 unsignedByte
+ *
+ *  . leading zeroes are prohibited.
+ *  
+ *  Summary:
+ *  . leading zeros are prohibited for all three groups
+ *  . '-' is required for nonPositiveInteger if it is zero
+ *
+ ***/
+
+const XMLCh* DecimalDatatypeValidator::getCanonicalRepresentation(const XMLCh*         const rawData
+                                                                 ,      MemoryManager* const memMgr
+                                                                 ,      bool                 toValidate) const
+{
+    MemoryManager* toUse = memMgr? memMgr : fMemoryManager;
+    DecimalDatatypeValidator* temp = (DecimalDatatypeValidator*) this;
+
+    if (toValidate)
+    {
+        try
+        {
+            temp->checkContent(rawData, 0, false, toUse);   
+        }
+        catch (...)
+        {
+            return 0;
+        }
+    }
+
+    // XMLBigInteger::getCanonicalRepresentation and
+    // XMLBigDecimal::getCanonicalRepresentation will handle exceptional cases
+    XMLCanRepGroup::CanRepGroup dvType = DatatypeValidatorFactory::getCanRepGroup(temp);
+
+    if ((dvType == XMLCanRepGroup::Decimal_Derivated_signed)   ||
+        (dvType == XMLCanRepGroup::Decimal_Derivated_unsigned) ||
+        (dvType == XMLCanRepGroup::Decimal_Derivated_npi)        )
+    {          
+        return XMLBigInteger::getCanonicalRepresentation(rawData, toUse);
+    }
+    else if (dvType == XMLCanRepGroup::Decimal) 
+    {
+        return XMLBigDecimal::getCanonicalRepresentation(rawData, toUse);
+    }
+    else //in case?
+    {
+        return XMLString::replicate(rawData, toUse);
+    }
+
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(DecimalDatatypeValidator)
+
+void DecimalDatatypeValidator::serialize(XSerializeEngine& serEng)
+{
+    /***
+     * Note: 
+     *
+     *     During storing, we need write the specific number
+     *     type info before calling base::serialize().
+     *
+     *     While loading, we do nothing here
+     ***/
+
+    if (serEng.isStoring())
+    {
+        serEng<<(int) (XMLNumber::BigDecimal);
+    }
+
+    AbstractNumericValidator::serialize(serEng);
+
+    //don't serialize XMLBigDecimal*       
+    if (serEng.isStoring())
+    {
+        serEng<<fTotalDigits;
+        serEng<<fFractionDigits;
+    }
+    else
+    {
+        serEng>>fTotalDigits;
+        serEng>>fFractionDigits;
     }
 
 }
