@@ -55,8 +55,30 @@
  */
 
 /*
- * $Id: DecimalDatatypeValidator.hpp,v 1.6 2003/05/22 02:10:52 knoaman Exp $
+ * $Id: DecimalDatatypeValidator.hpp,v 1.14 2004/01/29 11:51:22 cargilld Exp $
  * $Log: DecimalDatatypeValidator.hpp,v $
+ * Revision 1.14  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.13  2003/12/23 21:50:36  peiyongz
+ * Absorb exception thrown in getCanonicalRepresentation and return 0,
+ * only validate when required
+ *
+ * Revision 1.11  2003/12/17 00:18:38  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.10  2003/11/28 18:53:07  peiyongz
+ * Support for getCanonicalRepresentation
+ *
+ * Revision 1.9  2003/11/12 20:32:03  peiyongz
+ * Statless Grammar: ValidationContext
+ *
+ * Revision 1.8  2003/10/02 19:21:06  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.7  2003/08/14 03:00:11  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.6  2003/05/22 02:10:52  knoaman
  * Default the memory manager.
  *
@@ -97,6 +119,8 @@
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+class XMLBigDecimal;
+
 class VALIDATORS_EXPORT DecimalDatatypeValidator : public AbstractNumericValidator
 {
 public:
@@ -136,7 +160,9 @@ public:
      * @param content2
      * @return
      */
-    virtual int compare(const XMLCh* const, const XMLCh* const);
+    virtual int compare(const XMLCh* const, const XMLCh* const
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+        );
 
     //@}
 
@@ -152,6 +178,18 @@ public:
         , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
     );
 
+    virtual const XMLCh* getCanonicalRepresentation
+                        (
+                          const XMLCh*         const rawData
+                        ,       MemoryManager* const memMgr = 0
+                        ,       bool                 toValidate = false
+                        ) const;
+
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(DecimalDatatypeValidator)
+
 protected:
 
 // -----------------------------------------------------------------------
@@ -171,13 +209,14 @@ protected:
 // -----------------------------------------------------------------------
 
     virtual void assignAdditionalFacet(const XMLCh* const key
-                                     , const XMLCh* const value);
+                                     , const XMLCh* const value
+                                     , MemoryManager* const manager);
 
     virtual void inheritAdditionalFacet();
 
-    virtual void checkAdditionalFacetConstraints() const;
+    virtual void checkAdditionalFacetConstraints(MemoryManager* const manager) const;
 
-    virtual void checkAdditionalFacetConstraintsBase() const;
+    virtual void checkAdditionalFacetConstraintsBase(MemoryManager* const manager) const;
 
     virtual int  compareValues(const XMLNumber* const lValue
                              , const XMLNumber* const rValue);
@@ -190,14 +229,16 @@ protected:
 
     virtual void  setMinExclusive(const XMLCh* const);
 
-    virtual void  setEnumeration();
+    virtual void  setEnumeration(MemoryManager* const manager);
 
 // -----------------------------------------------------------------------
 // Abstract interface from AbstractNumericValidator
 // -----------------------------------------------------------------------
 
-    virtual void checkContent( const XMLCh* const content, bool asBase);
-
+    virtual void checkContent(const XMLCh*             const content
+                            ,       ValidationContext* const context
+                            , bool                           asBase
+                            ,       MemoryManager*     const manager);
 public:
 
 // -----------------------------------------------------------------------
@@ -221,9 +262,15 @@ private:
     //  Private data members
     //
     // -----------------------------------------------------------------------
-	 unsigned int         fTotalDigits;
-	 unsigned int         fFractionDigits;
+    unsigned int         fTotalDigits;
+    unsigned int         fFractionDigits;
 
+
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DecimalDatatypeValidator(const DecimalDatatypeValidator&);
+    DecimalDatatypeValidator& operator=(const DecimalDatatypeValidator&);
 };
 
 // -----------------------------------------------------------------------
