@@ -55,8 +55,36 @@
  */
 
 /*
- * $Id: ListDatatypeValidator.cpp,v 1.13 2003/11/28 18:53:07 peiyongz Exp $
+ * $Id: ListDatatypeValidator.cpp,v 1.22 2004/01/29 11:51:22 cargilld Exp $
  * $Log: ListDatatypeValidator.cpp,v $
+ * Revision 1.22  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.21  2004/01/13 21:18:18  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.20  2004/01/12 16:25:09  neilg
+ * remove use of static buffers
+ *
+ * Revision 1.19  2004/01/06 18:13:59  peiyongz
+ * using the no-exception-thrown ctor
+ *
+ * Revision 1.18  2003/12/31 02:34:41  neilg
+ * fix one more buffer overrun, affecting boolean lists
+ *
+ * Revision 1.17  2003/12/23 21:50:36  peiyongz
+ * Absorb exception thrown in getCanonicalRepresentation and return 0,
+ * only validate when required
+ *
+ * Revision 1.16  2003/12/17 20:41:47  neilg
+ * fix a segfault and a possible buffer overflow condition
+ *
+ * Revision 1.15  2003/12/17 00:18:39  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.14  2003/12/10 20:52:27  neilg
+ * fixes for canonical value production; memory management was not implemented correctly
+ *
  * Revision 1.13  2003/11/28 18:53:07  peiyongz
  * Support for getCanonicalRepresentation
  *
@@ -136,8 +164,6 @@
 XERCES_CPP_NAMESPACE_BEGIN
 
 static const int BUF_LEN = 64;
-static XMLCh value1[BUF_LEN+1];
-static XMLCh value2[BUF_LEN+1];
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
@@ -164,9 +190,9 @@ ListDatatypeValidator::ListDatatypeValidator(
     // In either case, it shall be not null
     //
     if (!baseValidator)
-        ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_List_Null_baseValidator);
+        ThrowXMLwithMemMgr(InvalidDatatypeFacetException, XMLExcepts::FACET_List_Null_baseValidator, manager);
 
-    init(enums);
+    init(enums, manager);
 }
 
 ListDatatypeValidator::~ListDatatypeValidator()
@@ -184,13 +210,14 @@ DatatypeValidator* ListDatatypeValidator::newInstance
 }
 
 
-int ListDatatypeValidator::compare(const XMLCh* const lValue
-                                 , const XMLCh* const rValue)
+int ListDatatypeValidator::compare(const XMLCh*     const lValue
+                                 , const XMLCh*     const rValue
+                                 , MemoryManager*   const manager)
 {
     DatatypeValidator* theItemTypeDTV = getItemTypeDTV();
-    BaseRefVectorOf<XMLCh>* lVector = XMLString::tokenizeString(lValue);
+    BaseRefVectorOf<XMLCh>* lVector = XMLString::tokenizeString(lValue, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janl(lVector);
-    BaseRefVectorOf<XMLCh>* rVector = XMLString::tokenizeString(rValue);
+    BaseRefVectorOf<XMLCh>* rVector = XMLString::tokenizeString(rValue, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janr(rVector);
 
     int lNumberOfTokens = lVector->size();
@@ -204,7 +231,7 @@ int ListDatatypeValidator::compare(const XMLCh* const lValue
     { //compare each token
         for ( int i = 0; i < lNumberOfTokens; i++)
         {
-            int returnValue = theItemTypeDTV->compare(lVector->elementAt(i), rVector->elementAt(i));
+            int returnValue = theItemTypeDTV->compare(lVector->elementAt(i), rVector->elementAt(i), manager);
             if (returnValue != 0)
                 return returnValue; //REVISIT: does it make sense to return -1 or +1..?
         }
@@ -214,22 +241,24 @@ int ListDatatypeValidator::compare(const XMLCh* const lValue
 }
 
 void ListDatatypeValidator::validate( const XMLCh*             const content
-                                    ,       ValidationContext* const context)
+                                    ,       ValidationContext* const context
+                                    ,       MemoryManager*     const manager)
 {
     setContent(content);
-    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content);
+    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janName(tokenVector);
-    checkContent(tokenVector, content, context, false);
+    checkContent(tokenVector, content, context, false, manager);
 }
 
 void ListDatatypeValidator::checkContent( const XMLCh*             const content
                                          ,      ValidationContext* const context
-                                         ,      bool                     asBase)
+                                         ,      bool                     asBase
+                                         ,      MemoryManager*     const manager)
 {
     setContent(content);
-    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content);
+    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janName(tokenVector);
-    checkContent(tokenVector, content, context, asBase);
+    checkContent(tokenVector, content, context, asBase, manager);
 }
 
 //
@@ -238,16 +267,17 @@ void ListDatatypeValidator::checkContent( const XMLCh*             const content
 void ListDatatypeValidator::checkContent(       BaseRefVectorOf<XMLCh>*       tokenVector
                                         , const XMLCh*                  const content
                                         ,       ValidationContext*      const context
-                                        ,       bool                          asBase)
+                                        ,       bool                          asBase
+                                        ,       MemoryManager*          const manager)
 {
     DatatypeValidator* bv = getBaseValidator();
 
     if (bv->getType() == DatatypeValidator::List)
-        ((ListDatatypeValidator*)bv)->checkContent(tokenVector, content, context, true);
+        ((ListDatatypeValidator*)bv)->checkContent(tokenVector, content, context, true, manager);
     else
     {   // the ultimate itemType DTV
         for (unsigned int i = 0; i < tokenVector->size(); i++)
-            bv->validate(tokenVector->elementAt(i), context);
+            bv->validate(tokenVector->elementAt(i), context, manager);
     }
 
     int thisFacetsDefined = getFacetsDefined();
@@ -259,21 +289,22 @@ void ListDatatypeValidator::checkContent(       BaseRefVectorOf<XMLCh>*       to
         if (getRegex() == 0)
         {
             try {
-                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));            	
             }
             catch (XMLException &e)
             {
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage());
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage(), manager);
             }
         }
 
         //check every item in the list as a whole
-        if (getRegex()->matches(content) == false)
+        if (getRegex()->matches(content, manager) == false)
         {
-            ThrowXML2(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr2(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_NotMatch_Pattern
                     , content
-                    , getPattern());
+                    , getPattern()
+                    , manager);
         }
 
     }
@@ -288,40 +319,49 @@ void ListDatatypeValidator::checkContent(       BaseRefVectorOf<XMLCh>*       to
     if (((thisFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) != 0) &&
         (tokenNumber > getMaxLength()))
     {
-        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10);
-        XMLString::binToText(getMaxLength(), value2, BUF_LEN, 10);
+        XMLCh value1[BUF_LEN+1];
+        XMLCh value2[BUF_LEN+1];
+        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10, manager);
+        XMLString::binToText(getMaxLength(), value2, BUF_LEN, 10, manager);
 
-        ThrowXML3(InvalidDatatypeValueException
+        ThrowXMLwithMemMgr3(InvalidDatatypeValueException
                 , XMLExcepts::VALUE_GT_maxLen
                 , getContent()
                 , value1
-                , value2);
+                , value2
+                , manager);
     }
 
     if (((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) != 0) &&
         (tokenNumber < getMinLength()))
     {
-        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10);
-        XMLString::binToText(getMinLength(), value2, BUF_LEN, 10);
+        XMLCh value1[BUF_LEN+1];
+        XMLCh value2[BUF_LEN+1];
+        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10, manager);
+        XMLString::binToText(getMinLength(), value2, BUF_LEN, 10, manager);
 
-        ThrowXML3(InvalidDatatypeValueException
+        ThrowXMLwithMemMgr3(InvalidDatatypeValueException
                 , XMLExcepts::VALUE_LT_minLen
                 , getContent()
                 , value1
-                , value2);
+                , value2
+                , manager);
     }
 
     if (((thisFacetsDefined & DatatypeValidator::FACET_LENGTH) != 0) &&
         (tokenNumber != AbstractStringValidator::getLength()))
     {
-        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10);
-        XMLString::binToText(AbstractStringValidator::getLength(), value2, BUF_LEN, 10);
+        XMLCh value1[BUF_LEN+1];
+        XMLCh value2[BUF_LEN+1];
+        XMLString::binToText(tokenNumber, value1, BUF_LEN, 10, manager);
+        XMLString::binToText(AbstractStringValidator::getLength(), value2, BUF_LEN, 10, manager);
 
-        ThrowXML3(InvalidDatatypeValueException
+        ThrowXMLwithMemMgr3(InvalidDatatypeValueException
                 , XMLExcepts::VALUE_NE_Len
                 , getContent()
                 , value1
-                , value2);
+                , value2
+                , manager);
     }
 
     if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
@@ -343,22 +383,23 @@ void ListDatatypeValidator::checkContent(       BaseRefVectorOf<XMLCh>*       to
             // eg.
             // tokenVector = "1 2 3.0 4" vs enumeration = "1 2 3 4.0"
             //
-            if (valueSpaceCheck(tokenVector, getEnumeration()->elementAt(i)))
+            if (valueSpaceCheck(tokenVector, getEnumeration()->elementAt(i), manager))
                 break;
         }
 
         if (i == enumLength)
-            ThrowXML1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, getContent());
+            ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, getContent(), manager);
 
     } // enumeration
 
 }
 
 bool ListDatatypeValidator::valueSpaceCheck(BaseRefVectorOf<XMLCh>* tokenVector
-                                          , const XMLCh* const  enumStr) const
+                                          , const XMLCh*    const  enumStr
+                                          , MemoryManager*  const  manager) const
 {
     DatatypeValidator* theItemTypeDTV = getItemTypeDTV();
-    BaseRefVectorOf<XMLCh>* enumVector = XMLString::tokenizeString(enumStr);
+    BaseRefVectorOf<XMLCh>* enumVector = XMLString::tokenizeString(enumStr, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janName(enumVector);
 
     if (tokenVector->size() != enumVector->size())
@@ -366,7 +407,7 @@ bool ListDatatypeValidator::valueSpaceCheck(BaseRefVectorOf<XMLCh>* tokenVector
 
     for ( unsigned int j = 0; j < tokenVector->size(); j++ )
     {
-        if (theItemTypeDTV->compare(tokenVector->elementAt(j), enumVector->elementAt(j)) != 0)
+        if (theItemTypeDTV->compare(tokenVector->elementAt(j), enumVector->elementAt(j), manager) != 0)
             return false;
     }
 
@@ -386,35 +427,21 @@ DatatypeValidator* ListDatatypeValidator::getItemTypeDTV() const
 // ---------------------------------------------------------------------------
 //  Utilities
 // ---------------------------------------------------------------------------
-void ListDatatypeValidator::assignAdditionalFacet( const XMLCh* const key
-                                                 , const XMLCh* const)
+
+void ListDatatypeValidator::checkValueSpace(const XMLCh* const
+                                            , MemoryManager* const)
+{}
+
+int ListDatatypeValidator::getLength(const XMLCh* const content
+                                     , MemoryManager* const manager) const
 {
-    ThrowXML1(InvalidDatatypeFacetException
-            , XMLExcepts::FACET_Invalid_Tag
-            , key);
-}
-
-void ListDatatypeValidator::inheritAdditionalFacet()
-{}
-
-void ListDatatypeValidator::checkAdditionalFacetConstraints() const
-{}
-
-void ListDatatypeValidator::checkAdditionalFacet(const XMLCh* const) const
-{}
-
-void ListDatatypeValidator::checkValueSpace(const XMLCh* const content)
-{}
-
-int ListDatatypeValidator::getLength(const XMLCh* const content) const
-{
-    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content);
+    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(content, manager);
     Janitor<BaseRefVectorOf<XMLCh> > janName(tokenVector);
 
     return tokenVector->size();
 }
 
-void ListDatatypeValidator::inspectFacetBase()
+void ListDatatypeValidator::inspectFacetBase(MemoryManager* const manager)
 {
 
     //
@@ -423,7 +450,7 @@ void ListDatatypeValidator::inspectFacetBase()
 
     if (getBaseValidator()->getType() == DatatypeValidator::List)
     {
-        AbstractStringValidator::inspectFacetBase();
+        AbstractStringValidator::inspectFacetBase(manager);
     }
     else
     {
@@ -439,13 +466,13 @@ void ListDatatypeValidator::inspectFacetBase()
                 for ( i = 0; i < enumLength; i++)
                 {
                     // ask the itemType for a complete check
-                    BaseRefVectorOf<XMLCh>* tempList = XMLString::tokenizeString(getEnumeration()->elementAt(i));
+                    BaseRefVectorOf<XMLCh>* tempList = XMLString::tokenizeString(getEnumeration()->elementAt(i), manager);
                     int tokenNumber = tempList->size();
 
                     try
                     {
                         for ( int j = 0; j < tokenNumber; j++)
-                            getBaseValidator()->validate(tempList->elementAt(j), (ValidationContext*)0);
+                            getBaseValidator()->validate(tempList->elementAt(j), (ValidationContext*)0, manager);
                     }
                     catch(const OutOfMemoryException&)
                     {
@@ -460,15 +487,16 @@ void ListDatatypeValidator::inspectFacetBase()
                     delete tempList;
 
                     // enum shall pass this->checkContent() as well.
-                    checkContent(getEnumeration()->elementAt(i), (ValidationContext*)0, false);
+                    checkContent(getEnumeration()->elementAt(i), (ValidationContext*)0, false, manager);
                 }
             }
 
             catch ( XMLException& )
             {
-                ThrowXML1(InvalidDatatypeFacetException
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                         , XMLExcepts::FACET_enum_base
-                        , getEnumeration()->elementAt(i));
+                        , getEnumeration()->elementAt(i)
+                        , manager);
             }
 
         }
@@ -497,32 +525,65 @@ void ListDatatypeValidator::inheritFacet()
  * lexical representation of its itemType.
  ***/
 const XMLCh* ListDatatypeValidator::getCanonicalRepresentation(const XMLCh*         const rawData
-                                                             ,       MemoryManager* const memMgr) const
+                                                             ,       MemoryManager* const memMgr
+                                                             ,       bool                 toValidate) const
 {
-    ListDatatypeValidator* temp = (ListDatatypeValidator*) this;
-
-    temp->setContent(rawData);
-    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(rawData);
-    Janitor<BaseRefVectorOf<XMLCh> > janName(tokenVector);
-    temp->checkContent(tokenVector, rawData, 0, false);
-
     MemoryManager* toUse = memMgr? memMgr : getMemoryManager();
-    int  retBufSize = 2 * XMLString::stringLen(rawData);
-    XMLCh* retBuf = (XMLCh*) toUse->allocate(retBufSize * sizeof(XMLCh));
-    XMLCh* retBufPtr = retBuf;
+    ListDatatypeValidator* temp = (ListDatatypeValidator*) this;
+    temp->setContent(rawData);
+    BaseRefVectorOf<XMLCh>* tokenVector = XMLString::tokenizeString(rawData, toUse);
+    Janitor<BaseRefVectorOf<XMLCh> > janName(tokenVector);    
 
-    DatatypeValidator* itemDv = this->getItemTypeDTV();
-    for (unsigned int i = 0; i < tokenVector->size(); i++)
+    if (toValidate)
     {
-        XMLCh* itemCanRep = (XMLCh*) itemDv->getCanonicalRepresentation(tokenVector->elementAt(i), memMgr);
-        XMLString::catString(retBufPtr, itemCanRep);
-        retBufPtr = retBufPtr + XMLString::stringLen(itemCanRep) + 1;
-        *(retBufPtr++) = chSpace;
-        *(retBufPtr) = chNull;
-        toUse->deallocate(itemCanRep);
+        try
+        {
+            temp->checkContent(tokenVector, rawData, 0, false, toUse);
+        }
+        catch (...)
+        {
+            return 0;
+        }
     }
+   
+    unsigned int  retBufSize = 2 * XMLString::stringLen(rawData);
+    XMLCh* retBuf = (XMLCh*) toUse->allocate(retBufSize * sizeof(XMLCh));
+    retBuf[0] = 0;
+    XMLCh* retBufPtr = retBuf;
+    DatatypeValidator* itemDv = this->getItemTypeDTV();
 
-    return 0;
+    try 
+    {
+        for (unsigned int i = 0; i < tokenVector->size(); i++)
+        {
+            XMLCh* itemCanRep = (XMLCh*) itemDv->getCanonicalRepresentation(tokenVector->elementAt(i), toUse, false);
+            unsigned int itemLen = XMLString::stringLen(itemCanRep); 
+
+            if(retBufPtr+itemLen+2 >= retBuf+retBufSize)
+            {
+                // need to resize
+                XMLCh * oldBuf = retBuf;
+                retBuf = (XMLCh*) toUse->allocate(retBufSize * sizeof(XMLCh) * 4);
+                memcpy(retBuf, oldBuf, retBufSize * sizeof(XMLCh ));
+                retBufPtr = (retBufPtr - oldBuf) + retBuf;
+                toUse->deallocate(oldBuf);
+                retBufSize <<= 2;
+            }
+
+            XMLString::catString(retBufPtr, itemCanRep);
+            retBufPtr = retBufPtr + itemLen + 1;
+            *(retBufPtr++) = chSpace;
+            *(retBufPtr) = chNull;
+            toUse->deallocate(itemCanRep);
+        }
+
+        return retBuf;
+
+    }
+    catch (...)
+    {
+        return 0;
+    }
 }
 
 /***
