@@ -55,8 +55,57 @@
  */
 
 /*
- * $Id: UnionDatatypeValidator.cpp,v 1.10 2003/05/16 06:01:57 knoaman Exp $
+ * $Id: UnionDatatypeValidator.cpp,v 1.26 2004/01/29 11:51:22 cargilld Exp $
  * $Log: UnionDatatypeValidator.cpp,v $
+ * Revision 1.26  2004/01/29 11:51:22  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.25  2004/01/13 21:18:18  peiyongz
+ * revert code back to previous version
+ *
+ * Revision 1.24  2004/01/12 16:25:09  neilg
+ * remove use of static buffers
+ *
+ * Revision 1.23  2004/01/06 18:13:59  peiyongz
+ * using the no-exception-thrown ctor
+ *
+ * Revision 1.22  2003/12/23 21:50:36  peiyongz
+ * Absorb exception thrown in getCanonicalRepresentation and return 0,
+ * only validate when required
+ *
+ * Revision 1.21  2003/12/17 00:18:39  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.20  2003/12/10 20:55:18  neilg
+ * fixes for canonical value production; memory management was not implemented correctly
+ *
+ * Revision 1.19  2003/11/28 18:53:07  peiyongz
+ * Support for getCanonicalRepresentation
+ *
+ * Revision 1.18  2003/11/24 05:10:26  neilg
+ * implement method for determining member type of union that validated some value
+ *
+ * Revision 1.17  2003/11/13 23:19:18  peiyongz
+ * initSize
+ *
+ * Revision 1.16  2003/11/12 20:32:03  peiyongz
+ * Statless Grammar: ValidationContext
+ *
+ * Revision 1.15  2003/10/31 22:17:11  peiyongz
+ * solve ownership
+ *
+ * Revision 1.14  2003/10/17 21:13:44  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.13  2003/10/07 19:39:03  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.12  2003/10/01 16:32:41  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.11  2003/08/16 18:42:49  neilg
+ * fix for bug 22457.  Union types that are restrictions of other union types were previously considered not to inherit their parents member types.  This is at variance with the behaviour of the Java parser and apparently with the spec, so I have changed this.
+ *
  * Revision 1.10  2003/05/16 06:01:57  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -111,12 +160,13 @@
 #include <xercesc/validators/datatype/UnionDatatypeValidator.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+
+#include <xercesc/internal/XTemplateSerializer.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-static const int BUF_LEN = 64;
-static XMLCh value1[BUF_LEN+1];
-static XMLCh value2[BUF_LEN+1];
+static const unsigned int BUF_LEN = 64;
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
@@ -124,9 +174,11 @@ static XMLCh value2[BUF_LEN+1];
 UnionDatatypeValidator::UnionDatatypeValidator(MemoryManager* const manager)
 :DatatypeValidator(0, 0, 0, DatatypeValidator::Union, manager)
 ,fEnumerationInherited(false)
+,fMemberTypesInherited(false)
 ,fEnumeration(0)
 ,fMemberTypeValidators(0)
 ,fValidatedDatatype(0)
+
 {}
 
 UnionDatatypeValidator::~UnionDatatypeValidator()
@@ -140,14 +192,15 @@ UnionDatatypeValidator::UnionDatatypeValidator(
                       , MemoryManager* const                  manager)
 :DatatypeValidator(0, 0, finalSet, DatatypeValidator::Union, manager)
 ,fEnumerationInherited(false)
+,fMemberTypesInherited(false)
 ,fEnumeration(0)
 ,fMemberTypeValidators(0)
 ,fValidatedDatatype(0)
 {
     if ( !memberTypeValidators )
     {
-        ThrowXML(InvalidDatatypeFacetException
-               , XMLExcepts::FACET_Union_Null_memberTypeValidators);
+        ThrowXMLwithMemMgr(InvalidDatatypeFacetException
+               , XMLExcepts::FACET_Union_Null_memberTypeValidators, manager);
     }
 
     // no pattern, no enumeration
@@ -159,11 +212,15 @@ UnionDatatypeValidator::UnionDatatypeValidator(
                         , RefHashTableOf<KVStringPair>* const facets
                         , RefArrayVectorOf<XMLCh>*      const enums
                         , const int                           finalSet
-                        , MemoryManager* const                manager)
+                        , MemoryManager* const                manager
+                        , RefVectorOf<DatatypeValidator>* const memberTypeValidators 
+                        , const bool memberTypesInherited
+                        )
 :DatatypeValidator(baseValidator, facets, finalSet, DatatypeValidator::Union, manager)
 ,fEnumerationInherited(false)
+,fMemberTypesInherited(memberTypesInherited)
 ,fEnumeration(0)
-,fMemberTypeValidators(0)
+,fMemberTypeValidators(memberTypeValidators)
 ,fValidatedDatatype(0)
 {
     //
@@ -173,21 +230,27 @@ UnionDatatypeValidator::UnionDatatypeValidator(
     //
     if (!baseValidator)
     {
-        ThrowXML(InvalidDatatypeFacetException
-               , XMLExcepts::FACET_Union_Null_baseValidator);
+        ThrowXMLwithMemMgr(InvalidDatatypeFacetException
+               , XMLExcepts::FACET_Union_Null_baseValidator, manager);
     }
 
     if (baseValidator->getType() != DatatypeValidator::Union)
     {
-        XMLString::binToText(baseValidator->getType(), value1, BUF_LEN, 10);
-        ThrowXML1(InvalidDatatypeFacetException
+        XMLCh value1[BUF_LEN+1];
+        XMLString::binToText(baseValidator->getType(), value1, BUF_LEN, 10, manager);
+        ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                 , XMLExcepts::FACET_Union_invalid_baseValidatorType
-                , value1);
+                , value1
+                , manager);
     }
 
     try
     {
-        init(baseValidator, facets, enums);
+        init(baseValidator, facets, enums, manager);
+    }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
     }
     catch (...)
     {
@@ -198,7 +261,8 @@ UnionDatatypeValidator::UnionDatatypeValidator(
 
 void UnionDatatypeValidator::init(DatatypeValidator*            const baseValidator
                                 , RefHashTableOf<KVStringPair>* const facets
-                                , RefArrayVectorOf<XMLCh>*           const enums)
+                                , RefArrayVectorOf<XMLCh>*      const enums
+                                , MemoryManager*                const manager)
 {
     if (enums)
         setEnumeration(enums, false);
@@ -208,7 +272,7 @@ void UnionDatatypeValidator::init(DatatypeValidator*            const baseValida
     {
         XMLCh* key;
         XMLCh* value;
-        RefHashTableOfEnumerator<KVStringPair> e(facets);
+        RefHashTableOfEnumerator<KVStringPair> e(facets, false, manager);
 
         while (e.hasMoreElements())
         {
@@ -225,9 +289,10 @@ void UnionDatatypeValidator::init(DatatypeValidator*            const baseValida
             }
             else
             {
-                 ThrowXML1(InvalidDatatypeFacetException
+                 ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                          , XMLExcepts::FACET_Invalid_Tag
-                         , key);
+                         , key
+                         , manager);
             }
         }//while
 
@@ -255,16 +320,17 @@ void UnionDatatypeValidator::init(DatatypeValidator*            const baseValida
                     // since there are no other facets for Union, parent
                     // checking is good enough.
                     //
-                    baseValidator->validate(getEnumeration()->elementAt(i));
+                    baseValidator->validate(getEnumeration()->elementAt(i), (ValidationContext*)0, manager);
 
                 }
             }
 
             catch ( XMLException& )
             {
-                ThrowXML1(InvalidDatatypeFacetException
+                ThrowXMLwithMemMgr1(InvalidDatatypeFacetException
                             , XMLExcepts::FACET_enum_base
-                            , getEnumeration()->elementAt(i));
+                            , getEnumeration()->elementAt(i)
+                            , manager);
             }
         }
 
@@ -299,12 +365,15 @@ void UnionDatatypeValidator::init(DatatypeValidator*            const baseValida
 // 3) the natvie Union DTV (the top level DTV) would invoke
 //        memberTypeValidator to validate
 //
-void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBase)
+void UnionDatatypeValidator::checkContent(const XMLCh*             const content
+                                        ,       ValidationContext* const context
+                                        ,       bool                     asBase
+                                        ,       MemoryManager*     const manager)
 {
 
     DatatypeValidator* bv = getBaseValidator();
     if (bv)
-        ((UnionDatatypeValidator*)bv)->checkContent(content, true);
+        ((UnionDatatypeValidator*)bv)->checkContent(content, context, true, manager);
     else
     {   // 3) native union type
         // check content against each member type validator in Union
@@ -318,12 +387,15 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
 
             try
             {
-                fMemberTypeValidators->elementAt(i)->validate(content);
+                fMemberTypeValidators->elementAt(i)->validate(content, context, manager);
                 memTypeValid = true;
                 
                 //set the validator of the type actually used to validate the content
                 DatatypeValidator *dtv = fMemberTypeValidators->elementAt(i);
                 fValidatedDatatype = dtv;
+                // context will be null during schema construction
+                if(context)
+                    context->setValidatingMemberType(dtv);
             }
             catch (XMLException&)
             {
@@ -333,9 +405,10 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
 
         if ( !memTypeValid )
         {
-            ThrowXML1(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr1(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_no_match_memberType
-                    , content);
+                    , content
+                    , manager);
             //( "Content '"+content+"' does not match any union types" );
         }
     }
@@ -347,20 +420,21 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
         if (getRegex() == 0)
         {
             try {
-                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));            	
             }
             catch (XMLException &e)
             {
-                ThrowXML1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage());
+                ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::RethrowError, e.getMessage(), manager);
             }
         }
 
-        if (getRegex()->matches(content) == false)
+        if (getRegex()->matches(content, manager) == false)
         {
-            ThrowXML2(InvalidDatatypeValueException
+            ThrowXMLwithMemMgr2(InvalidDatatypeValueException
                     , XMLExcepts::VALUE_NotMatch_Pattern
                     , content
-                    , getPattern());
+                    , getPattern()
+                    , manager);
         }
     }
 
@@ -387,7 +461,7 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
             {
                 try
                 {
-                    if (memberDTV->elementAt(memberIndex)->compare(content, tmpEnum->elementAt(enumIndex)) == 0)
+                    if (memberDTV->elementAt(memberIndex)->compare(content, tmpEnum->elementAt(enumIndex), manager) == 0)
                         return;
                 }
                 catch (XMLException&)
@@ -397,7 +471,7 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
             } // for enumIndex
         } // for memberIndex
 
-        ThrowXML1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content);
+        ThrowXMLwithMemMgr1(InvalidDatatypeValueException, XMLExcepts::VALUE_NotIn_Enumeration, content, manager);
 
     } // enumeration
 
@@ -407,14 +481,15 @@ void UnionDatatypeValidator::checkContent(const XMLCh* const content, bool asBas
 //
 //
 int UnionDatatypeValidator::compare(const XMLCh* const lValue
-                                  , const XMLCh* const rValue)
+                                  , const XMLCh* const rValue
+                                  , MemoryManager* const manager)
 {
     RefVectorOf<DatatypeValidator>* memberDTV = getMemberTypeValidators();
     unsigned int memberTypeNumber = memberDTV->size();
 
     for ( unsigned int memberIndex = 0; memberIndex < memberTypeNumber; ++memberIndex)
     {
-        if (memberDTV->elementAt(memberIndex)->compare(lValue, rValue) ==0)
+        if (memberDTV->elementAt(memberIndex)->compare(lValue, rValue, manager) ==0)
             return  0;
     }
 
@@ -428,6 +503,105 @@ const RefArrayVectorOf<XMLCh>* UnionDatatypeValidator::getEnumString() const
 	return getEnumeration();
 }
 
+/***
+ * 2.5.1.3 Union datatypes
+ *
+ * The canonical-lexical-representation for a union datatype is defined as the lexical form 
+ * in which the values have the canonical lexical representation of the appropriate memberTypes.       
+ ***/
+const XMLCh* UnionDatatypeValidator::getCanonicalRepresentation(const XMLCh*         const rawData
+                                                              ,       MemoryManager* const memMgr
+                                                              ,       bool                 toValidate) const
+{
+    MemoryManager* toUse = memMgr? memMgr : getMemoryManager();
+    UnionDatatypeValidator* temp = (UnionDatatypeValidator*) this;
+
+    if (toValidate)
+    {
+        try
+        {
+            temp->checkContent(rawData, 0, false, toUse);
+        }
+        catch (...)
+        {
+            return 0;
+        }
+    }    
+
+    //get the native unionDv
+    UnionDatatypeValidator* bdv = (UnionDatatypeValidator*) temp->getBaseValidator();
+    while (bdv)
+    {
+        temp = bdv;
+        bdv = (UnionDatatypeValidator*) temp->getBaseValidator();
+    }
+
+    //let the member dv which recognize the rawData, to return
+    //us the canonical form
+    for ( unsigned int i = 0; i < temp->fMemberTypeValidators->size(); ++i )
+    {
+        try
+        {
+            temp->fMemberTypeValidators->elementAt(i)->validate(rawData, 0, toUse);                       
+            return temp->fMemberTypeValidators->elementAt(i)->getCanonicalRepresentation(rawData, toUse, false);
+        }
+        catch (XMLException&)
+        {
+            //absorbed
+        }
+    }
+
+    //if no member dv recognize it
+    return 0;
+}
+
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(UnionDatatypeValidator)
+
+void UnionDatatypeValidator::serialize(XSerializeEngine& serEng)
+{
+
+    DatatypeValidator::serialize(serEng);
+
+    if (serEng.isStoring())
+    {
+        serEng<<fEnumerationInherited;
+        serEng<<fMemberTypesInherited;
+
+        /***
+         * Serialize RefArrayVectorOf<XMLCh>
+         * Serialize RefVectorOf<DatatypeValidator>
+         ***/
+        XTemplateSerializer::storeObject(fEnumeration, serEng);
+        XTemplateSerializer::storeObject(fMemberTypeValidators, serEng);
+
+        DatatypeValidator::storeDV(serEng, fValidatedDatatype);
+
+    }
+    else
+    {
+        serEng>>fEnumerationInherited;
+        serEng>>fMemberTypesInherited;
+
+        /***
+         * Deserialize RefArrayVectorOf<XMLCh>
+         * Deserialize RefVectorOf<DatatypeValidator>
+         ***/
+        bool owned = fEnumerationInherited ? true : false;
+        XTemplateSerializer::loadObject(&fEnumeration, 8, owned, serEng);
+
+        owned = fMemberTypesInherited ? true : false;
+        XTemplateSerializer::loadObject(&fMemberTypeValidators, 4, owned, serEng);
+
+        fValidatedDatatype = DatatypeValidator::loadDV(serEng);
+
+    }
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
