@@ -55,8 +55,23 @@
  */
 
 /*
- * $Id: YearDatatypeValidator.cpp,v 1.5 2003/05/18 14:02:07 knoaman Exp $
+ * $Id: YearDatatypeValidator.cpp,v 1.10 2003/12/17 00:18:39 cargilld Exp $
  * $Log: YearDatatypeValidator.cpp,v $
+ * Revision 1.10  2003/12/17 00:18:39  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.9  2003/11/06 15:30:07  neilg
+ * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
+ *
+ * Revision 1.8  2003/10/02 19:21:06  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.7  2003/10/01 16:32:41  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.6  2003/08/14 03:00:11  knoaman
+ * Code refactoring to improve performance of validation.
+ *
  * Revision 1.5  2003/05/18 14:02:07  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -87,6 +102,7 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/datatype/YearDatatypeValidator.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -95,7 +111,9 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 YearDatatypeValidator::YearDatatypeValidator(MemoryManager* const manager)
 :DateTimeValidator(0, 0, 0, DatatypeValidator::Year, manager)
-{}
+{
+    setOrdered(XSSimpleTypeDefinition::ORDERED_PARTIAL);
+}
 
 YearDatatypeValidator::YearDatatypeValidator(
                           DatatypeValidator*            const baseValidator
@@ -105,7 +123,7 @@ YearDatatypeValidator::YearDatatypeValidator(
                         , MemoryManager* const                manager)
 :DateTimeValidator(baseValidator, facets, finalSet, DatatypeValidator::Year, manager)
 {
-    init(enums);
+    init(enums, manager);
 }
 
 YearDatatypeValidator::~YearDatatypeValidator()
@@ -125,14 +143,18 @@ DatatypeValidator* YearDatatypeValidator::newInstance
 //
 // caller need to release the date created here
 //
-XMLDateTime* YearDatatypeValidator::parse(const XMLCh* const content)
+XMLDateTime* YearDatatypeValidator::parse(const XMLCh* const content, MemoryManager* const manager)
 {
-    XMLDateTime *pRetDate = new (fMemoryManager) XMLDateTime(content, fMemoryManager);
+    XMLDateTime *pRetDate = new (manager) XMLDateTime(content, manager);
 
     try
     {
         pRetDate->parseYear();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         delete pRetDate;
@@ -142,6 +164,22 @@ XMLDateTime* YearDatatypeValidator::parse(const XMLCh* const content)
     return pRetDate;
 }
 
+void YearDatatypeValidator::parse(XMLDateTime* const pDate)
+{
+    pDate->parseYear();
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(YearDatatypeValidator)
+
+void YearDatatypeValidator::serialize(XSerializeEngine& serEng)
+{
+    DateTimeValidator::serialize(serEng);
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
