@@ -56,6 +56,57 @@
 
 /*
  * $Log: ComplexTypeInfo.cpp,v $
+ * Revision 1.26  2004/01/29 11:52:31  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.25  2004/01/13 16:17:09  knoaman
+ * Fo sanity, use class name to qualify method
+ *
+ * Revision 1.24  2004/01/09 22:41:58  knoaman
+ * Use a global static mutex for locking when creating local static mutexes instead of compareAndSwap
+ *
+ * Revision 1.23  2003/12/24 17:42:03  knoaman
+ * Misc. PSVI updates
+ *
+ * Revision 1.22  2003/12/17 00:18:40  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.21  2003/12/16 17:18:50  peiyongz
+ * don't expand ContextSpecNode when deserilized
+ *
+ * Revision 1.20  2003/11/20 18:05:16  knoaman
+ * PSVI: Use a copy of the content spec node when creating the content model.
+ *
+ * Revision 1.19  2003/11/13 23:20:47  peiyongz
+ * initSize
+ *
+ * Revision 1.18  2003/11/10 21:54:51  neilg
+ * implementation for new stateless means of traversing attribute definition lists
+ *
+ * Revision 1.17  2003/11/07 17:08:12  knoaman
+ * For PSVI support, distinguish wildcard elements with namespace lists.
+ *
+ * Revision 1.16  2003/10/29 16:22:58  peiyongz
+ * allow proper recreation of ContentModel from deserialization
+ *
+ * Revision 1.15  2003/10/21 10:23:53  amassari
+ * When creating a new list of attributes, use the assigned memory manager
+ *
+ * Revision 1.14  2003/10/17 21:17:12  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.13  2003/10/14 15:22:28  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.12  2003/10/10 16:25:40  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.11  2003/10/08 21:33:48  peiyongz
+ * Synchronize ContentSpec/ContentModel/FormattedModel
+ *
+ * Revision 1.10  2003/09/26 18:31:05  peiyongz
+ * Synchronize ContentSpecNode and formattedModel
+ *
  * Revision 1.9  2003/05/18 14:02:07  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -179,9 +230,119 @@
 #include <xercesc/validators/common/MixedContentModel.hpp>
 #include <xercesc/validators/common/SimpleContentModel.hpp>
 #include <xercesc/validators/schema/XSDLocator.hpp>
+#include <xercesc/internal/XTemplateSerializer.hpp>
+#include <xercesc/util/XMLRegisterCleanup.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+// ---------------------------------------------------------------------------
+//  Local static data
+// ---------------------------------------------------------------------------
+static bool               sAnyTypeMutexRegistered = false;
+static XMLMutex*          sAnyTypeMutex = 0;
+static XMLRegisterCleanup anyTypeCleanup;
+
+
+// ---------------------------------------------------------------------------
+//  ComplexTypeInfo: Static member data
+// ---------------------------------------------------------------------------
+ComplexTypeInfo* ComplexTypeInfo::fAnyType = 0;
+
+
+// ---------------------------------------------------------------------------
+//  ComplexTypeInfo: Static meber methods
+// ---------------------------------------------------------------------------
+void ComplexTypeInfo::reinitAnyType() {
+
+    delete fAnyType;
+    fAnyType = 0;
+
+    // delete local static data
+    delete sAnyTypeMutex;
+    sAnyTypeMutex = 0;
+    sAnyTypeMutexRegistered = false;
+}
+
+ComplexTypeInfo* ComplexTypeInfo::getAnyType(unsigned int emptyNSId)
+{
+    if (!sAnyTypeMutexRegistered)
+    {
+        if (!sAnyTypeMutex)
+        {
+            XMLMutexLock lock(XMLPlatformUtils::fgAtomicMutex);
+            if (!sAnyTypeMutex)
+                sAnyTypeMutex = new XMLMutex;
+        }
+
+        // Use a faux scope to synchronize while we do this
+        {
+            XMLMutexLock lock(sAnyTypeMutex);
+
+            // If we got here first, then register it and set the registered flag
+            if (!sAnyTypeMutexRegistered)
+            {
+                // create type name
+                XMLCh typeName[128];
+                unsigned int nsLen = XMLString::stringLen(
+                    SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+
+			    XMLString::copyString(
+                    typeName, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                typeName[nsLen] = chComma;
+                XMLString::copyString(
+                    typeName + nsLen + 1, SchemaSymbols::fgATTVAL_ANYTYPE);
+
+                // Create and initialize 'anyType'
+                fAnyType = new ComplexTypeInfo();
+
+                ContentSpecNode* term = new ContentSpecNode
+                (
+                    new QName
+                    (
+                        XMLUni::fgZeroLenString
+                        , XMLUni::fgZeroLenString
+                        , emptyNSId
+                    )
+                    , false
+                );
+                term->setType(ContentSpecNode::Any_Lax);
+                term->setMinOccurs(0);
+                term->setMaxOccurs(SchemaSymbols::XSD_UNBOUNDED);
+
+                ContentSpecNode* particle = new ContentSpecNode
+                (
+                    ContentSpecNode::ModelGroupSequence
+                    , term
+                    , 0
+                );
+
+                SchemaAttDef* attWildCard = new SchemaAttDef
+                (
+                    XMLUni::fgZeroLenString
+                    , XMLUni::fgZeroLenString
+                    , emptyNSId
+                    , XMLAttDef::Any_Any
+                    , XMLAttDef::ProcessContents_Lax
+                );
+
+                fAnyType->setTypeName(typeName);
+                fAnyType->setBaseComplexTypeInfo(fAnyType);
+                fAnyType->setDerivedBy(SchemaSymbols::XSD_RESTRICTION);
+                fAnyType->setContentType(SchemaElementDecl::Mixed_Complex);
+                fAnyType->setContentSpec(particle);
+                fAnyType->setAttWildCard(attWildCard);
+
+                // register cleanup method
+                anyTypeCleanup.registerCleanup(ComplexTypeInfo::reinitAnyType);
+                sAnyTypeMutexRegistered = true;
+            }
+        }
+    }
+
+    return fAnyType;
+}
+
+
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -204,16 +365,16 @@ ComplexTypeInfo::ComplexTypeInfo(MemoryManager* const manager)
     , fDatatypeValidator(0)
     , fBaseComplexTypeInfo(0)
     , fContentSpec(0)
-    , fAttWildCard(0)
-    , fAttDefs(0)
+    , fAttWildCard(0)    
     , fAttList(0)
     , fElements(0)
+    , fSpecNodesToDelete(0)
+    , fAttDefs(0)
     , fContentModel(0)
     , fFormattedModel(0)
     , fContentSpecOrgURI(0)
     , fUniqueURI(0)
-    , fContentSpecOrgURISize(16)
-    , fSpecNodesToDelete(0)
+    , fContentSpecOrgURISize(16)    
     , fLocator(0)
     , fMemoryManager(manager)
 {
@@ -241,6 +402,7 @@ ComplexTypeInfo::~ComplexTypeInfo()
     delete fContentModel;
     fMemoryManager->deallocate(fFormattedModel); //delete [] fFormattedModel;
     fMemoryManager->deallocate(fContentSpecOrgURI); //delete [] fContentSpecOrgURI;
+
 }
 
 // ---------------------------------------------------------------------------
@@ -257,6 +419,10 @@ void ComplexTypeInfo::addAttDef(SchemaAttDef* const toAdd) {
 
     fAttDefs->put((void*)(toAdd->getAttName()->getLocalPart()),
                           toAdd->getAttName()->getURI(), toAdd);
+    // update and/or create fAttList
+    if(!fAttList)
+        ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs,fMemoryManager);
+    fAttList->addAttDef(toAdd);
 }
 
 void ComplexTypeInfo::setContentSpec(ContentSpecNode* const toAdopt) {
@@ -266,6 +432,9 @@ void ComplexTypeInfo::setContentSpec(ContentSpecNode* const toAdopt) {
     }
 
     fContentSpec = toAdopt;
+
+    //reset Content Model
+    setContentModel(0);
 }
 
 void ComplexTypeInfo::setLocator(XSDLocator* const aLocator) {
@@ -287,7 +456,7 @@ XMLAttDefList& ComplexTypeInfo::getAttDefList() const
         if (!fAttDefs)
             faultInAttDefList();
 
-        ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs);
+        ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs, fMemoryManager);
     }
 
     // Reset it before we return it
@@ -315,7 +484,7 @@ ComplexTypeInfo::getFormattedContentModel() const
 // ---------------------------------------------------------------------------
 //  ComplexTypeInfo: Helper methods
 // ---------------------------------------------------------------------------
-XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const qName
+XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const
                                      , const unsigned int uriId
                                      , const XMLCh* const baseName
                                      , const XMLCh* const prefix
@@ -348,9 +517,13 @@ XMLAttDef* ComplexTypeInfo::findAttr(const XMLCh* const qName
         retVal->setElemId(getElementId());
         fAttDefs->put((void*)retVal->getAttName()->getLocalPart(), uriId, retVal);
 
+        // update and/or create fAttList
+        if(!fAttList)
+            ((ComplexTypeInfo*)this)->fAttList = new (fMemoryManager) SchemaAttDefList(fAttDefs,fMemoryManager);
+        fAttList->addAttDef(retVal);
         wasAdded = true;
     }
-     else
+    else
     {
         wasAdded = false;
     }
@@ -368,7 +541,7 @@ bool ComplexTypeInfo::resetDefs() {
     //  This lets the scanner use them to track which has been provided and
     //  which have not.
     //
-    RefHash2KeysTableOfEnumerator<SchemaAttDef> enumDefs(fAttDefs);
+    RefHash2KeysTableOfEnumerator<SchemaAttDef> enumDefs(fAttDefs, false, fMemoryManager);
     while (enumDefs.hasMoreElements())
         enumDefs.nextElement().setProvided(false);
 
@@ -446,6 +619,7 @@ XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentS
 
     // expand the content spec first
     ContentSpecNode* aSpecNode = specNode;
+    XMLContentModel* retModel = 0;
     if (aSpecNode) {
 
         fContentSpecOrgURI = (unsigned int*) fMemoryManager->allocate
@@ -453,21 +627,22 @@ XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentS
             fContentSpecOrgURISize * sizeof(unsigned int)
         ); //new unsigned int[fContentSpecOrgURISize];
         aSpecNode = convertContentSpecTree(aSpecNode, checkUPA);
+        retModel = buildContentModel(aSpecNode);
         fSpecNodesToDelete->addElement(aSpecNode);
     }
     else {
-        aSpecNode = convertContentSpecTree(fContentSpec, checkUPA);
-        if (aSpecNode != fContentSpec) {
-            if (!fAdoptContentSpec && (aSpecNode == fContentSpec->getFirst()))
-                fAdoptContentSpec = false;
-            else
-                fAdoptContentSpec = true;
-
-            fContentSpec = aSpecNode;
-        }
+        // building content model for the complex type
+        aSpecNode = new (fMemoryManager) ContentSpecNode(*fContentSpec);
+        aSpecNode = convertContentSpecTree(aSpecNode, checkUPA);
+        retModel = buildContentModel(aSpecNode);
+        delete aSpecNode;
     }
 
+    return retModel;
+}
 
+XMLContentModel* ComplexTypeInfo::buildContentModel(ContentSpecNode* const aSpecNode)
+{
     XMLContentModel* cmRet = 0;
     if (fContentType == SchemaElementDecl::Simple) {
        // just return nothing
@@ -497,21 +672,19 @@ XMLContentModel* ComplexTypeInfo::makeContentModel(const bool checkUPA, ContentS
     }
      else
     {
-        ThrowXML(RuntimeException, XMLExcepts::CM_MustBeMixedOrChildren);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_MustBeMixedOrChildren, fMemoryManager);
     }
 
     return cmRet;
 }
 
-
-
 // ---------------------------------------------------------------------------
 //  SchemaElementDecl: Private helper methods
 // ---------------------------------------------------------------------------
 XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, const bool isMixed)
 {
     if(!specNode)
-        ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
 
     ContentSpecNode::NodeTypes specType = specNode->getType();
     //
@@ -520,7 +693,7 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
     //
     if (specNode->getElement()) {
         if (specNode->getElement()->getURI() == XMLElementDecl::fgPCDataElemId)
-            ThrowXML(RuntimeException, XMLExcepts::CM_NoPCDATAHere);
+            ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_NoPCDATAHere, fMemoryManager);
     }
 
     //
@@ -561,8 +734,8 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
             , fMemoryManager
         );
     }
-     else if ((specType == ContentSpecNode::Choice)
-          ||  (specType == ContentSpecNode::Sequence))
+     else if (((specType & 0x0f) == ContentSpecNode::Choice)
+          ||  ((specType & 0x0f) == ContentSpecNode::Sequence))
     {
         //
         //  Lets see if both of the children are leafs. If so, then it has to
@@ -611,7 +784,7 @@ XMLContentModel* ComplexTypeInfo::createChildModel(ContentSpecNode* specNode, co
 
     else
     {
-        ThrowXML(RuntimeException, XMLExcepts::CM_UnknownCMSpecType);
+        ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::CM_UnknownCMSpecType, fMemoryManager);
     }
 
     // Its not any simple type of content, so create a DFA based content model
@@ -650,9 +823,9 @@ ComplexTypeInfo::convertContentSpecTree(ContentSpecNode* const curNode,
     {
         retNode =  expandContentModel(curNode, minOccurs, maxOccurs);
     }
-    else if ((curType == ContentSpecNode::Choice)
+    else if (((curType & 0x0f) == ContentSpecNode::Choice)
         ||   (curType == ContentSpecNode::All)
-        ||   (curType == ContentSpecNode::Sequence))
+        ||   ((curType & 0x0f) == ContentSpecNode::Sequence))
     {
         ContentSpecNode* childNode = curNode->getFirst();
         ContentSpecNode* leftNode = convertContentSpecTree(childNode, checkUPA);
@@ -880,6 +1053,118 @@ void ComplexTypeInfo::resizeContentSpecOrgURI() {
     fContentSpecOrgURISize = newSize;
 }
 
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(ComplexTypeInfo)
+
+void ComplexTypeInfo::serialize(XSerializeEngine& serEng)
+{
+   
+    if (serEng.isStoring())
+    {    
+        serEng<<fAnonymous;
+        serEng<<fAbstract;
+        serEng<<fAdoptContentSpec;
+        serEng<<fAttWithTypeId;
+        serEng<<fPreprocessed;
+        serEng<<fDerivedBy;
+        serEng<<fBlockSet;
+        serEng<<fFinalSet;
+        serEng<<fScopeDefined;
+        serEng<<fElementId;
+        serEng<<fContentType;
+
+        serEng.writeString(fTypeName);
+        serEng.writeString(fTypeLocalName);
+        serEng.writeString(fTypeUri);
+
+        DatatypeValidator::storeDV(serEng, fBaseDatatypeValidator);
+        DatatypeValidator::storeDV(serEng, fDatatypeValidator);
+
+        serEng<<fBaseComplexTypeInfo;
+        serEng<<fContentSpec;
+        serEng<<fAttWildCard;
+        serEng<<fAttList;
+
+        /***
+         * 
+         * Serialize RefVectorOf<SchemaElementDecl>*    fElements;
+         * Serialize RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
+         ***/
+        XTemplateSerializer::storeObject(fElements, serEng);
+        XTemplateSerializer::storeObject(fAttDefs, serEng);
+
+         /***
+          *   Don't serialize 
+          *
+          *   fContentModel;
+          *   fFormattedModel;
+          *   fLocator;
+          *   fSpecNodesToDelete
+          * 
+          *   fContentSpecOrgURI:     start of the array
+          *   fContentSpecOrgURISize: size of the array
+          *   fUniqueURI:             the current last element in the array
+          ***/
+    }
+    else
+    {
+        serEng>>fAnonymous;
+        serEng>>fAbstract;
+        serEng>>fAdoptContentSpec;
+        serEng>>fAttWithTypeId;
+        serEng>>fPreprocessed;
+        serEng>>fDerivedBy;
+        serEng>>fBlockSet;
+        serEng>>fFinalSet;
+        serEng>>fScopeDefined;
+        serEng>>fElementId;
+        serEng>>fContentType;
+
+        serEng.readString(fTypeName);
+        serEng.readString(fTypeLocalName);
+        serEng.readString(fTypeUri);
+
+        fBaseDatatypeValidator = DatatypeValidator::loadDV(serEng);
+        fDatatypeValidator     = DatatypeValidator::loadDV(serEng);
+
+        serEng>>fBaseComplexTypeInfo;
+        serEng>>fContentSpec;
+        serEng>>fAttWildCard;
+        serEng>>fAttList;
+
+        /***
+         * 
+         * Deserialize RefVectorOf<SchemaElementDecl>*    fElements;
+         * Deserialize RefHash2KeysTableOf<SchemaAttDef>* fAttDefs;
+         ***/
+        XTemplateSerializer::loadObject(&fElements, 8, false, serEng);
+        XTemplateSerializer::loadObject(&fAttDefs, 29, true, serEng);
+
+         /***
+          *   Don't deserialize 
+          *
+          *   fFormattedModel;
+          *   fLocator;
+          *   fSpecNodesToDelete
+          * 
+          *   fContentSpecOrgURI:     start of the array
+          *   fContentSpecOrgURISize: size of the array
+          *   fUniqueURI:             the current last element in the array
+          ***/
+
+         fFormattedModel = 0;
+         fLocator = 0;
+         fSpecNodesToDelete = 0;
+         fContentSpecOrgURI = 0;
+         fContentSpecOrgURISize = 0;
+         fUniqueURI = 0;
+    }
+}
+
+
 XERCES_CPP_NAMESPACE_END
 
 /**
