@@ -56,6 +56,30 @@
 
 /*
  * $Log: GeneralAttributeCheck.cpp,v $
+ * Revision 1.21  2004/01/29 11:52:31  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.20  2004/01/13 16:17:09  knoaman
+ * Fo sanity, use class name to qualify method
+ *
+ * Revision 1.19  2004/01/09 22:41:58  knoaman
+ * Use a global static mutex for locking when creating local static mutexes instead of compareAndSwap
+ *
+ * Revision 1.18  2003/12/17 00:18:40  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.17  2003/12/11 19:26:27  knoaman
+ * Store non schema attributes from parent in XSAnnotation
+ *
+ * Revision 1.16  2003/11/12 20:35:31  peiyongz
+ * Stateless Grammar: ValidationContext
+ *
+ * Revision 1.15  2003/10/20 15:57:22  knoaman
+ * Fix multithreading problem.
+ *
+ * Revision 1.14  2003/10/01 16:32:41  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
  * Revision 1.13  2003/05/15 18:57:27  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -162,6 +186,7 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <xercesc/validators/datatype/DatatypeValidatorFactory.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -195,6 +220,14 @@ const XMLCh fgGlobal[] =
 };
 
 
+// ---------------------------------------------------------------------------
+//  Static local data
+// ---------------------------------------------------------------------------
+static bool sGeneralAttCheckMutexRegistered = false;
+static XMLMutex* sGeneralAttCheckMutex = 0;
+static XMLRegisterCleanup sGeneralAttCheckCleanup;
+
+
 // ---------------------------------------------------------------------------
 //  Static member data initialization
 // ---------------------------------------------------------------------------
@@ -204,19 +237,13 @@ DatatypeValidator*                GeneralAttributeCheck::fNonNegIntDV = 0;
 DatatypeValidator*                GeneralAttributeCheck::fBooleanDV = 0;
 DatatypeValidator*                GeneralAttributeCheck::fAnyURIDV = 0;
 
-// ---------------------------------------------------------------------------
-//  Static local data
-// ---------------------------------------------------------------------------
-static XMLMutex* sGeneralAttCheckMutex = 0;
-static XMLRegisterCleanup GeneralAttCheckCleanup;
-
 
 // ---------------------------------------------------------------------------
 //  GeneralAttributeCheck: Constructors and Destructor
 // ---------------------------------------------------------------------------
 GeneralAttributeCheck::GeneralAttributeCheck(MemoryManager* const manager)
     : fMemoryManager(manager)
-    , fIDRefList(0)
+    , fValidationContext(0)
     , fIDValidator(manager)
 {
     mapElements();
@@ -242,27 +269,33 @@ void GeneralAttributeCheck::setUpValidators() {
     // TO DO - add remaining valdiators
 }
 
-void GeneralAttributeCheck::mapElements() {
-
-    if (!sGeneralAttCheckMutex)
+void GeneralAttributeCheck::mapElements()
+{
+    if (!sGeneralAttCheckMutexRegistered)
     {
-        XMLMutex* tmpMutex = new XMLMutex;
-        if (XMLPlatformUtils::compareAndSwap((void**)&sGeneralAttCheckMutex, tmpMutex, 0))
+        if (!sGeneralAttCheckMutex)
         {
-            // Some other thread beat us to it, so let's clean up ours.
-            delete tmpMutex;
-        }
-        else
-        {
-            //
-            // the thread who creates the mutex succesfully, to
-            // initialize the followings
-            //
-            setUpValidators();
-            mapAttributes();
+            XMLMutexLock lock(XMLPlatformUtils::fgAtomicMutex);
 
-            // This is the real mutex.  Register it for cleanup at Termination.
-            GeneralAttCheckCleanup.registerCleanup(reinitGeneralAttCheck);
+            if (!sGeneralAttCheckMutex)
+                sGeneralAttCheckMutex = new XMLMutex;
+        }
+
+        // Use a faux scope to synchronize while we do this
+        {
+            XMLMutexLock lock(sGeneralAttCheckMutex);
+
+            // If we got here first, then register it and set the registered flag
+            if (!sGeneralAttCheckMutexRegistered)
+            {
+                // initialize
+                setUpValidators();
+                mapAttributes();
+
+                // register for cleanup at Termination.
+                sGeneralAttCheckCleanup.registerCleanup(GeneralAttributeCheck::reinitGeneralAttCheck);
+                sGeneralAttCheckMutexRegistered = true;
+            }
         }
     }
 }
@@ -330,6 +363,7 @@ GeneralAttributeCheck::reinitGeneralAttCheck() {
 
     delete sGeneralAttCheckMutex;
     sGeneralAttCheckMutex = 0;
+    sGeneralAttCheckMutexRegistered = false;
 
     delete fAttMap;
     delete fFacetsMap;
@@ -345,11 +379,14 @@ void
 GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
                                        const unsigned short elemContext,
                                        TraverseSchema* const schema,
-                                       const bool isTopLevel) {
+                                       const bool isTopLevel,
+                                       ValueVectorOf<DOMNode*>* const nonXSAttList)
+{
+    if (nonXSAttList)
+        nonXSAttList->removeAllElements();
 
-    if (elem == 0 || !fAttMap || elemContext>=E_Count) {
+    if (elem == 0 || !fAttMap || elemContext>=E_Count)
         return;
-    }
 
     const XMLCh* elemName = elem->getLocalName();
     if (!XMLString::equals(SchemaSymbols::fgURI_SCHEMAFORSCHEMA, elem->getNamespaceURI())) {
@@ -374,10 +411,20 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
         DOMNode*     attribute = eltAttrs->item(i);
         const XMLCh* attName = attribute->getNodeName();
 
+        // skip namespace declarations
+        if (XMLString::equals(attName, XMLUni::fgXMLNSString)
+            || XMLString::startsWith(attName, XMLUni::fgXMLNSColonString))
+            continue;
+
         // Bypass attributes that start with xml
+        // add this to the list of "non-schema" attributes
         if ((*attName == chLatin_X || *attName == chLatin_x)
            && (*(attName+1) == chLatin_M || *(attName+1) == chLatin_m)
            && (*(attName+2) == chLatin_L || *(attName+2) == chLatin_l)) {
+
+           if (nonXSAttList)
+                nonXSAttList->addElement(attribute);
+
             continue;
         }
 
@@ -394,28 +441,10 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
 
                 schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain,
                     XMLErrs::AttributeDisallowed, attName, contextStr, elemName);
-            } else {
-
-                // Try for a "lax" validation
-                DatatypeValidator* dv = schema->getDatatypeValidator(attrURI, attribute->getLocalName());
-
-                if (dv) {
-
-                    const XMLCh* attrVal = attribute->getNodeValue();
-
-                    try {
-                        dv->validate(attrVal);
-                    }
-                    catch(const XMLException& excep) {
-                        schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain, XMLErrs::DisplayErrorMessage, excep.getMessage());
-                    }
-                    catch(...) {
-                        schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain, XMLErrs::InvalidAttValue, attrVal, attName);
-                    }
-                }
-                // REVISIT:
-                // If no dv found, store attribute info for a "lax" validation
-                // after schema traversal ?? - KN
+            }
+            else if (nonXSAttList)
+            {
+                nonXSAttList->addElement(attribute);
             }
 
             continue;
@@ -425,7 +454,11 @@ GeneralAttributeCheck::checkAttributes(const DOMElement* const elem,
         attName = attribute->getLocalName();
 
         try {
-            attNameId= fAttMap->get(attName);
+            attNameId= fAttMap->get(attName, fMemoryManager);
+        }
+        catch(const OutOfMemoryException&)
+        {
+            throw;
         }
         catch(...) {
 
@@ -529,21 +562,24 @@ void GeneralAttributeCheck::validate(const DOMElement* const elem,
         dv = fAnyURIDV;
         break;
     case DV_ID:
-        if (fIDRefList) {
-
+        if (fValidationContext)
+        {
             dv = &fIDValidator;
-            ((IDDatatypeValidator*) dv)->setIDRefList(fIDRefList);
         }
         break;
     }
 
     if (dv) {
         try {
-            dv->validate(attValue);
+            dv->validate(attValue, fValidationContext, fMemoryManager);
         }
         catch(const XMLException& excep) {
             schema->reportSchemaError(elem, XMLUni::fgXMLErrDomain, XMLErrs::DisplayErrorMessage, excep.getMessage());
         }
+        catch(const OutOfMemoryException&)
+        {
+            throw;
+        }
         catch(...) {
             isInvalid = true;
         }
@@ -671,7 +707,7 @@ void GeneralAttributeCheck::initCharFlagTable()
     attList[E_ElementGlobal][A_Default] = Att_Optional;
     attList[E_ElementGlobal][A_Final] = Att_Optional;
     attList[E_ElementGlobal][A_Fixed] = Att_Optional;
-    attList[E_ElementGlobal][A_ID] = Att_Optional | DV_ID;;
+    attList[E_ElementGlobal][A_ID] = Att_Optional | DV_ID;
     attList[E_ElementGlobal][A_Name] = Att_Required;
     attList[E_ElementGlobal][A_Nillable] = Att_Optional | DV_Boolean;
     attList[E_ElementGlobal][A_SubstitutionGroup] = Att_Optional;
