@@ -56,6 +56,26 @@
 
 /*
  * $Log: SchemaAttDef.hpp,v $
+ * Revision 1.12  2004/01/29 11:52:31  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.11  2003/12/24 17:42:03  knoaman
+ * Misc. PSVI updates
+ *
+ * Revision 1.10  2003/12/17 20:50:35  knoaman
+ * PSVI: fix for annotation of attributes in attributeGroup/derived types
+ *
+ * Revision 1.9  2003/11/24 05:13:41  neilg
+ * update method documentation
+ *
+ * Revision 1.8  2003/11/21 22:34:46  neilg
+ * More schema component model implementation, thanks to David Cargill.
+ * In particular, this cleans up and completes the XSModel, XSNamespaceItem,
+ * XSAttributeDeclaration and XSAttributeGroup implementations.
+ *
+ * Revision 1.7  2003/10/10 16:25:40  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
  * Revision 1.6  2003/05/18 14:02:07  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -102,11 +122,12 @@
 #include <xercesc/validators/datatype/DatatypeValidator.hpp>
 #include <xercesc/validators/datatype/UnionDatatypeValidator.hpp>
 #include <xercesc/validators/schema/PSVIDefs.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 class DatatypeValidator;
 class QName;
-
+class ComplexTypeInfo;
 //
 //  This class is a derivative of the core XMLAttDef class. This class adds
 //  any Schema specific data members and provides Schema specific implementations
@@ -155,7 +176,7 @@ public :
     // ----------------------------------------------------------------------
     // Partial implementation of PSVI
     // The values these methods return are only accurate until the DOMAttr
-    // is created that uses the values. After this a celan up method is called
+    // is created that uses the values. After this a clean up method is called
     // and the SchemaAttDef may be used again.
     // note that some of this information has dependancies. For example,
     // if something is not valid then the information returned by the other 
@@ -169,6 +190,7 @@ public :
      * 1.1 If it was valid as defined by Attribute Locally Valid (3.2.4), then valid;
      * 1.2 otherwise invalid.
      * 2 otherwise notKnown.
+     * @deprecated
      */
     PSVIDefs::Validity getValidity() const;
 
@@ -176,26 +198,31 @@ public :
      * The appropriate case among the following:
      * 1 If it was strictly assessed, then full;
      * 2 otherwise none.
+     * @deprecated
      */
     PSVIDefs::Validation getValidationAttempted() const;
 
     /**
      * @return the complexity. Always simple for attrs
+     * @deprecated
      */
     PSVIDefs::Complexity getTypeType() const;
 
     /**
      * The target namespace of the type definition.
+     * @deprecated
      */
     const XMLCh* getTypeUri() const;
 
     /**
      * The {name} of the type definition, if it is not absent. 
+     * @deprecated
      */
     const XMLCh* getTypeName() const;
 
     /**
      * true if the {name} of the type definition is absent, otherwise false.
+     * @deprecated
      */
     bool getTypeAnonymous() const;
 
@@ -203,25 +230,35 @@ public :
      * If this method returns true and validity is VALID then the next three 
      * produce accurate results
      * @return true if the element is validated using a union type
+     * @deprecated
      */
     bool isTypeDefinitionUnion() const;
 
     /**
      * The {target namespace} of the actual member type definition.
+     * @deprecated
      */
     const XMLCh* getMemberTypeUri() const;
 
     /**
      * @return true if the {name} of the actual member type definition is absent, otherwise false.
+     * @deprecated
      */
     bool getMemberTypeAnonymous() const;
 
     /**
      * @return the {name} of the actual member type definition, if it is not absent. 
+     * @deprecated
      */
     const XMLCh* getMemberTypeName() const;
 
+    /*
+     * @deprecated
+     */
     virtual const XMLCh* getDOMTypeInfoUri() const;
+    /*
+     * @deprecated
+     */
     virtual const XMLCh* getDOMTypeInfoName() const;
     
     
@@ -233,6 +270,14 @@ public :
     QName* getAttName() const;
     DatatypeValidator* getDatatypeValidator() const;
     ValueVectorOf<unsigned int>* getNamespaceList() const;
+    const SchemaAttDef* getBaseAttDecl() const;
+    SchemaAttDef* getBaseAttDecl();
+    PSVIDefs::PSVIScope getPSVIScope() const;
+
+    /*
+     * @deprecated
+     */
+    ComplexTypeInfo* getEnclosingCT() const;
 
     // -----------------------------------------------------------------------
     //  Setter methods
@@ -246,13 +291,36 @@ public :
     );
     void setDatatypeValidator(DatatypeValidator* newDatatypeValidator);
     void setAnyDatatypeValidator(DatatypeValidator* newDatatypeValidator);
+    void setBaseAttDecl(SchemaAttDef* const attDef);
+    void setPSVIScope(const PSVIDefs::PSVIScope toSet);
+
+    /*
+     * @deprecated
+     */
     void setMembertypeValidator(const DatatypeValidator* newDatatypeValidator);
     void setNamespaceList(const ValueVectorOf<unsigned int>* const toSet);
     void resetNamespaceList();
+    /*
+     * @deprecated
+     */
     void setValidity(PSVIDefs::Validity valid);
+    /*
+     * @deprecated
+     */
     void setValidationAttempted(PSVIDefs::Validation validation);
+    void setEnclosingCT(ComplexTypeInfo* complexTypeInfo);
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(SchemaAttDef)
 
 private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    SchemaAttDef(const SchemaAttDef&);
+    SchemaAttDef& operator=(const SchemaAttDef&);
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -284,6 +352,13 @@ private :
     //  fValidation
     //      The type of validation that happened to this attr
     //
+    //  fBaseAttDecl
+    //      The base attribute declaration that this attribute is based on
+    //      NOTE: we do not have a notion of attribute use, so in the case
+    //      of ref'd attributes and inherited attributes, we make a copy
+    //      of the actual attribute declaration. The fBaseAttDecl stores that
+    //      declaration, and will be helpful when we build the XSModel (i.e
+    //      easy access the XSAnnotation object).
     // -----------------------------------------------------------------------
     unsigned int                 fElemId;
     QName*                       fAttName;
@@ -293,6 +368,8 @@ private :
     ValueVectorOf<unsigned int>* fNamespaceList;
     PSVIDefs::Validity           fValidity;
     PSVIDefs::Validation         fValidation;
+    PSVIDefs::PSVIScope          fPSVIScope;
+    SchemaAttDef*                fBaseAttDecl;
 };
 
 
@@ -425,6 +502,25 @@ SchemaAttDef::getNamespaceList() const {
     return fNamespaceList;
 }
 
+inline ComplexTypeInfo* SchemaAttDef::getEnclosingCT() const
+{
+    return 0;
+}
+
+inline SchemaAttDef* SchemaAttDef::getBaseAttDecl()
+{
+    return fBaseAttDecl;
+}
+
+inline const SchemaAttDef* SchemaAttDef::getBaseAttDecl() const
+{
+    return fBaseAttDecl;
+}
+
+inline PSVIDefs::PSVIScope SchemaAttDef::getPSVIScope() const
+{
+    return fPSVIScope;
+}
 
 // ---------------------------------------------------------------------------
 //  SchemaAttDef: Setter methods
@@ -483,6 +579,20 @@ inline void SchemaAttDef::reset() {
     fValidation = PSVIDefs::NONE;    
 }
 
+inline void SchemaAttDef::setEnclosingCT(ComplexTypeInfo*)
+{
+}
+
+inline void SchemaAttDef::setBaseAttDecl(SchemaAttDef* const attDef)
+{
+    fBaseAttDecl = attDef;
+}
+
+inline void SchemaAttDef::setPSVIScope(const PSVIDefs::PSVIScope toSet)
+{
+    fPSVIScope = toSet;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 #endif
