@@ -56,6 +56,18 @@
 
 /*
  * $Log: XercesAttGroupInfo.cpp,v $
+ * Revision 1.8  2003/11/20 17:06:05  knoaman
+ * PSVI: store name and namespace information
+ *
+ * Revision 1.7  2003/11/13 23:20:47  peiyongz
+ * initSize
+ *
+ * Revision 1.6  2003/10/17 21:17:12  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.5  2003/10/14 15:22:28  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
  * Revision 1.4  2003/05/15 18:57:27  knoaman
  * Partial implementation of the configurable memory manager.
  *
@@ -88,6 +100,8 @@
 #include <xercesc/validators/schema/XercesAttGroupInfo.hpp>
 #include <xercesc/util/QName.hpp>
 
+#include <xercesc/internal/XTemplateSerializer.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
@@ -95,6 +109,8 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 XercesAttGroupInfo::XercesAttGroupInfo(MemoryManager* const manager)
     : fTypeWithId(false)
+    , fNameId(0)
+    , fNamespaceId(0)
     , fAttributes(0)
     , fAnyAttributes(0)
     , fCompleteWildCard(0)
@@ -103,6 +119,19 @@ XercesAttGroupInfo::XercesAttGroupInfo(MemoryManager* const manager)
 
 }
 
+XercesAttGroupInfo::XercesAttGroupInfo(unsigned int attGroupNameId,
+                                       unsigned int attGroupNamespaceId,
+                                       MemoryManager* const manager)
+    : fTypeWithId(false)
+    , fNameId(attGroupNameId)
+    , fNamespaceId(attGroupNamespaceId)
+    , fAttributes(0)
+    , fAnyAttributes(0)
+    , fCompleteWildCard(0)
+    , fMemoryManager(manager)
+{
+
+}
 
 XercesAttGroupInfo::~XercesAttGroupInfo()
 {
@@ -162,6 +191,63 @@ const SchemaAttDef* XercesAttGroupInfo::getAttDef(const XMLCh* const baseName,
     return 0;
 }
 
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(XercesAttGroupInfo)
+
+void XercesAttGroupInfo::serialize(XSerializeEngine& serEng)
+{
+
+    if (serEng.isStoring())
+    {
+        serEng<<fTypeWithId;
+        serEng<<fNameId;
+        serEng<<fNamespaceId;
+
+        /***
+         *
+         * Serialize RefVectorOf<SchemaAttDef>* fAttributes;
+         *
+         ***/
+        XTemplateSerializer::storeObject(fAttributes, serEng);
+
+        /***
+         *
+         * Serialize RefVectorOf<SchemaAttDef>* fAnyAttributes;
+         *
+         ***/
+        XTemplateSerializer::storeObject(fAnyAttributes, serEng);
+
+        serEng<<fCompleteWildCard;
+    }
+    else
+    {
+        serEng>>fTypeWithId;
+        serEng>>fNameId;
+        serEng>>fNamespaceId;
+
+        /***
+         *
+         * Deserialize RefVectorOf<SchemaAttDef>* fAttributes;
+         *
+         ***/
+        XTemplateSerializer::loadObject(&fAttributes, 4, true, serEng);
+
+        /***
+         *
+         * Deserialize RefVectorOf<SchemaAttDef>* fAnyAttributes;
+         *
+         ***/
+
+        XTemplateSerializer::loadObject(&fAnyAttributes, 2, true, serEng);
+
+        serEng>>fCompleteWildCard;
+    }
+
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
