@@ -56,6 +56,15 @@
 
 /*
  * $Log: ValueStore.cpp,v $
+ * Revision 1.10  2004/01/29 11:52:32  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.9  2003/12/17 00:18:41  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
+ * Revision 1.8  2003/12/16 18:41:15  knoaman
+ * Make IC_Field stateless
+ *
  * Revision 1.7  2003/05/18 14:02:09  knoaman
  * Memory manager implementation: pass per instance manager.
  *
@@ -94,6 +103,7 @@
 #include <xercesc/internal/XMLScanner.hpp>
 #include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/validators/datatype/DatatypeValidator.hpp>
+#include <xercesc/validators/schema/identity/FieldActivator.hpp>
 #include <xercesc/validators/schema/identity/ValueStore.hpp>
 #include <xercesc/validators/schema/identity/IC_Field.hpp>
 #include <xercesc/validators/schema/identity/IC_KeyRef.hpp>
@@ -128,11 +138,18 @@ ValueStore::~ValueStore()
 // ---------------------------------------------------------------------------
 //  ValueStore: Helper methods
 // ---------------------------------------------------------------------------
-void ValueStore::addValue(IC_Field* const field,
+void ValueStore::addValue(IC_Field* const,
+                          DatatypeValidator* const,
+                          const XMLCh* const) {
+
+}
+
+void ValueStore::addValue(FieldActivator* const fieldActivator,
+                          IC_Field* const field,
                           DatatypeValidator* const dv,
                           const XMLCh* const value) {
 
-    if (!field->getMayMatch() && fDoReportError) {
+    if (!fieldActivator->getMayMatch(field) && fDoReportError) {
         fScanner->getValidator()->emitError(XMLValid::IC_FieldMultipleMatch);
     }
 
@@ -304,7 +321,7 @@ bool ValueStore::isDuplicateOf(DatatypeValidator* const dv1, const XMLCh* const
     // are the validators equal?
     // As always we are obliged to compare by reference...
     if (dv1 == dv2) {
-        return ((dv1->compare(val1, val2)) == 0);
+        return ((dv1->compare(val1, val2, fMemoryManager)) == 0);
     }
 
     // see if this.fValidator is derived from value.fValidator:
@@ -312,14 +329,14 @@ bool ValueStore::isDuplicateOf(DatatypeValidator* const dv1, const XMLCh* const
     for(; !tempVal || tempVal == dv2; tempVal = tempVal->getBaseValidator()) ;
 
     if (tempVal) { // was derived!
-        return ((dv2->compare(val1, val2)) == 0);
+        return ((dv2->compare(val1, val2, fMemoryManager)) == 0);
     }
 
     // see if value.fValidator is derived from this.fValidator:
     for(tempVal = dv2; !tempVal || tempVal == dv1; tempVal = tempVal->getBaseValidator()) ;
 
     if(tempVal) { // was derived!
-        return ((dv1->compare(val1, val2)) == 0);
+        return ((dv1->compare(val1, val2, fMemoryManager)) == 0);
     }
 
     // if we're here it means the types weren't related.  Must fall back to strings:
