@@ -55,11 +55,11 @@
  */
 
 /*
- * $Id: XPathSymbols.hpp,v 1.2 2002/11/04 14:47:41 tng Exp $
+ * $Id: XPathSymbols.hpp,v 1.4 2004/01/29 11:52:32 cargilld Exp $
  */
 
-#if !defined(SCHEMASYMBOLS_HPP)
-#define SCHEMASYMBOLS_HPP
+#if !defined(XPATHSYMBOLS_HPP)
+#define XPATHSYMBOLS_HPP
 
 #include <xercesc/util/XercesDefs.hpp>
 
@@ -97,6 +97,11 @@ public :
     static const XMLCh fgSYMBOL_PRECEDING_SIBLING[];
     static const XMLCh fgSYMBOL_SELF[];
 
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    XPathSymbols();
 };
 
 XERCES_CPP_NAMESPACE_END
