@@ -56,6 +56,12 @@
 
 /*
  * $Log: XercesXPath.cpp,v $
+ * Revision 1.13  2004/01/29 11:52:32  cargilld
+ * Code cleanup changes to get rid of various compiler diagnostic messages.
+ *
+ * Revision 1.12  2003/12/17 00:18:41  cargilld
+ * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
+ *
  * Revision 1.11  2003/10/17 21:18:04  peiyongz
  * using XTemplateSerializer
  *
@@ -188,7 +194,7 @@ void XercesNodeTest::serialize(XSerializeEngine& serEng)
     }
 }
 
-XercesNodeTest::XercesNodeTest(MemoryManager* const manager)
+XercesNodeTest::XercesNodeTest(MemoryManager* const)
 :fType(UNKNOWN)
 ,fName(0)
 {
@@ -295,7 +301,7 @@ void XercesStep::serialize(XSerializeEngine& serEng)
     }
 }
 
-XercesStep::XercesStep(MemoryManager* const manager)
+XercesStep::XercesStep(MemoryManager* const)
 :fAxisType(UNKNOWN)
 ,fNodeTest(0)
 {
@@ -356,7 +362,7 @@ void XercesLocationPath::serialize(XSerializeEngine& serEng)
     }
 }
 
-XercesLocationPath::XercesLocationPath(MemoryManager* const manager)
+XercesLocationPath::XercesLocationPath(MemoryManager* const)
 :fSteps(0)
 {
 }
@@ -444,7 +450,7 @@ void XercesXPath::checkForSelectedAttributes() {
 
         if (stepSize) {
             if (locPath->getStep(stepSize - 1)->getAxisType() == XercesStep::ATTRIBUTE) {
-                ThrowXML(XPathException, XMLExcepts::XPath_NoAttrSelector);
+                ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoAttrSelector, fMemoryManager);
             }
 		}
     }
@@ -480,13 +486,13 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
         case  XercesXPath::EXPRTOKEN_OPERATOR_UNION:
             {
                 if (i == 0) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoUnionAtStart);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoUnionAtStart, fMemoryManager);
                 }
 
                 int stepsSize = stepsVector->size();
 
                 if (stepsSize == 0) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoMultipleUnion);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoMultipleUnion, fMemoryManager);
                 }
 
                 fLocationPaths->addElement(new (fMemoryManager) XercesLocationPath(stepsVector));
@@ -505,7 +511,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
             {
                 // consume QName token
                 if (i == tokenCount - 1) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_MissingAttr);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_MissingAttr, fMemoryManager);
                 }
 
                 aToken = tokens.elementAt(++i);
@@ -513,7 +519,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                 if (aToken != XercesXPath::EXPRTOKEN_NAMETEST_QNAME
                     && aToken!= XercesXPath::EXPRTOKEN_NAMETEST_ANY
                     && aToken!= XercesXPath::EXPRTOKEN_NAMETEST_NAMESPACE) {
-                        ThrowXML(XPathException, XMLExcepts::XPath_ExpectedToken1);
+                        ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_ExpectedToken1, fMemoryManager);
                 }
 
                 bool isNamespaceAtt=false;
@@ -545,7 +551,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                         }
 
                         if (aToken != -1 && scopeContext && uri == fEmptyNamespaceId) {
-                            ThrowXML1(XPathException, XMLExcepts::XPath_PrefixNoURI, prefix);
+                            ThrowXMLwithMemMgr1(XPathException, XMLExcepts::XPath_PrefixNoURI, prefix, fMemoryManager);
                         }
 
                         if (isNamespaceAtt) {
@@ -576,7 +582,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
         case XercesXPath::EXPRTOKEN_DOUBLE_COLON:
             {
                 // should never have a bare double colon
-                ThrowXML(XPathException, XMLExcepts::XPath_NoDoubleColon);
+                ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoDoubleColon, fMemoryManager);
             }
         case XercesXPath::EXPRTOKEN_AXISNAME_CHILD:
             {
@@ -584,7 +590,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                 i++;
 
                 if (i == tokenCount - 1) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_ExpectedStep1);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_ExpectedStep1, fMemoryManager);
                 }
 
                 firstTokenOfLocationPath=false;
@@ -617,7 +623,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                 }
 
                 if (aToken != -1 && scopeContext && uri == fEmptyNamespaceId) {
-                    ThrowXML1(XPathException, XMLExcepts::XPath_PrefixNoURI, prefix);
+                    ThrowXMLwithMemMgr1(XPathException, XMLExcepts::XPath_PrefixNoURI, prefix, fMemoryManager);
                 }
 
                 if (isNamespace) {
@@ -654,7 +660,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                     if (aToken == XercesXPath::EXPRTOKEN_OPERATOR_DOUBLE_SLASH){
 
                         if (++i == tokenCount - 1) {
-                            ThrowXML(XPathException, XMLExcepts::XPath_ExpectedStep2);
+                            ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_ExpectedStep2, fMemoryManager);
                         }
 
                         if (i+1 < tokenCount)	{
@@ -662,7 +668,7 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
                             aToken = tokens.elementAt(i+1);
 
                             if (aToken == XercesXPath::EXPRTOKEN_OPERATOR_SLASH) {
-                                ThrowXML(XPathException, XMLExcepts::XPath_NoForwardSlash);
+                                ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoForwardSlash, fMemoryManager);
                             }
                         }
                         // build step
@@ -676,21 +682,21 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
             }
         case XercesXPath::EXPRTOKEN_OPERATOR_DOUBLE_SLASH:
             {
-                ThrowXML(XPathException, XMLExcepts::XPath_NoDoubleForwardSlash);
+                ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoDoubleForwardSlash, fMemoryManager);
             }
         case XercesXPath::EXPRTOKEN_OPERATOR_SLASH:
             {
                 if (i == 0) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoForwardSlashAtStart);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoForwardSlashAtStart, fMemoryManager);
                 }
 
                 // keep on truckin'
                 if (firstTokenOfLocationPath) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_NoSelectionOfRoot);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoSelectionOfRoot, fMemoryManager);
                 }
 
                 if (i == tokenCount - 1) {
-                    ThrowXML(XPathException, XMLExcepts::XPath_ExpectedStep3);
+                    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_ExpectedStep3, fMemoryManager);
                 }
 
                 firstTokenOfLocationPath=false;
@@ -705,10 +711,10 @@ void XercesXPath::parseExpression(XMLStringPool* const stringPool,
 
     if (stepsSize == 0) {
         if (!fLocationPaths || fLocationPaths->size() == 0) {
-            ThrowXML(XPathException, XMLExcepts::XPath_EmptyExpr);
+            ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_EmptyExpr, fMemoryManager);
         }
         else {
-            ThrowXML(XPathException, XMLExcepts::XPath_NoUnionAtEnd);
+            ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_NoUnionAtEnd, fMemoryManager);
         }
     }
 
@@ -921,7 +927,7 @@ bool XPathScanner::scanExpression(const XMLCh* const data,
                     break;
                 }
             } else {
-                ThrowXML(XPathException, XMLExcepts::XPath_InvalidChar);
+                ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_InvalidChar, tokens->getMemoryManager());
             }
 
             break;
@@ -1415,8 +1421,6 @@ int XPathScanner::scanNumber(const XMLCh* const data,
 
         if (++currentOffset < endOffset) {
 
-            int start = currentOffset;
-
             ch = data[currentOffset];
 
             while (ch >= chDigit_0 && ch <= chDigit_9) {
@@ -1431,7 +1435,7 @@ int XPathScanner::scanNumber(const XMLCh* const data,
             }
 
             if (part != 0) {
-                ThrowXML(RuntimeException, XMLExcepts::XPath_FindSolution);
+                ThrowXMLwithMemMgr(RuntimeException, XMLExcepts::XPath_FindSolution, tokens->getMemoryManager());
             }
         }
     }
@@ -1476,7 +1480,7 @@ void XPathScannerForSchema::addToken(ValueVectorOf<int>* const tokens,
         return;
     }
 
-    ThrowXML(XPathException, XMLExcepts::XPath_TokenNotSupported);
+    ThrowXMLwithMemMgr(XPathException, XMLExcepts::XPath_TokenNotSupported, tokens->getMemoryManager());
 }
 
 XERCES_CPP_NAMESPACE_END
