@@ -79,6 +79,8 @@
 #include <xercesc/dom/DOM.hpp>
 #include <stdio.h>
 
+XERCES_CPP_NAMESPACE_USE
+
 static int gTestsFailed = 0;
 static int gTestsRun    = 0;
 static XercesDOMParser* parser = 0;
@@ -112,8 +114,8 @@ void ParseErrorHandler::error(const SAXParseException& e)
         systemId, e.getLineNumber(),
         e.getColumnNumber(), message);
 
-    delete [] systemId;
-    delete [] message;
+    XMLString::release(&systemId);
+    XMLString::release(&message);
     throw e;
 
 };
@@ -127,8 +129,8 @@ void ParseErrorHandler::fatalError(const SAXParseException& e)
         systemId, e.getLineNumber(),
         e.getColumnNumber(), message);
 
-    delete [] systemId;
-    delete [] message;
+    XMLString::release(&systemId);
+    XMLString::release(&message);
     throw e;
 };
 
@@ -141,8 +143,8 @@ void ParseErrorHandler::warning(const SAXParseException& e)
         systemId, e.getLineNumber(),
         e.getColumnNumber(), message);
 
-    delete [] systemId;
-    delete [] message;
+    XMLString::release(&systemId);
+    XMLString::release(&message);
     throw e;
 
 };
