@@ -55,14 +55,20 @@
  */
 
 /*
- * $Id: InitTermTest.hpp,v 1.6 2003/02/05 18:55:21 tng Exp $
+ * $Id: InitTermTest.hpp,v 1.7 2003/05/30 13:08:25 gareth Exp $
  */
 
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/sax/ErrorHandler.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+
+#if defined(XERCES_NEW_IOSTREAMS)
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 
 XERCES_CPP_NAMESPACE_USE
 
@@ -159,7 +165,7 @@ private :
     char*   fLocalForm;
 };
 
-inline ostream& operator<<(ostream& target, const StrX& toDump)
+inline XERCES_STD_QUALIFIER ostream& operator<<(XERCES_STD_QUALIFIER ostream& target, const StrX& toDump)
 {
     target << toDump.localForm();
     return target;
