@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ThreadTest.cpp,v 1.30 2003/03/13 22:13:28 tng Exp $
+ * $Id: ThreadTest.cpp,v 1.33 2004/02/13 13:23:17 cargilld Exp $
  *
  * @author Andy Heninger, IBM
  */
@@ -122,7 +122,7 @@ void ThreadFuncs::startThread(ThreadFunc func, void *param)
 }
 
 
-#elif defined (AIX) || defined(SOLARIS) || defined(LINUX) || defined(HPUX) || defined (OS390) || defined(FREEBSD) || defined(__CYGWIN__) || defined(__QNXNTO__)
+#elif defined (AIX) || defined(SOLARIS) || defined(LINUX) || defined(HPUX) || defined (OS390) || defined(MACOSX) || defined(FREEBSD) || defined(__CYGWIN__) || defined(__QNXNTO__)
 #include <pthread.h>
 #include <unistd.h>
 #include <errno.h>
@@ -234,6 +234,7 @@ struct RunInfo
     bool        doSchema;
     bool        schemaFullChecking;
     bool        doNamespaces;
+    bool        doInitialParse;
     int         totalTime;
     int         numInputFiles;
     InFileInfo  files[MAXINFILES];
@@ -409,6 +410,7 @@ int ThreadParser::parse(int fileNum)
     {
         if (gRunInfo.dom) {
             // Do a DOM parse
+            fXercesDOMParser->resetDocumentPool();
             if (gRunInfo.inMemory)
                 fXercesDOMParser->parse(*mbis);
             else
@@ -645,6 +647,7 @@ void parseCommandLine(int argc, char **argv)
     gRunInfo.doSchema = false;
     gRunInfo.schemaFullChecking = false;
     gRunInfo.doNamespaces = false;
+    gRunInfo.doInitialParse = false;
     gRunInfo.dom = false;
     gRunInfo.reuseParser = false;
     gRunInfo.inMemory = false;
@@ -681,6 +684,8 @@ void parseCommandLine(int argc, char **argv)
                 else
                     throw 1;
             }
+            else if (strcmp(argv[argnum], "-init") == 0)
+                gRunInfo.doInitialParse = true;
             else if (strcmp(argv[argnum], "-reuse") == 0)
                 gRunInfo.reuseParser = true;
             else if (strcmp(argv[argnum], "-dump") == 0)
@@ -748,7 +753,8 @@ void parseCommandLine(int argc, char **argv)
             "     -threads nnn   Number of threads.  Default is 2.\n"
             "     -time nnn      Total time to run, in seconds.  Default is forever.\n"
             "     -dump          Dump DOM tree on error.\n"
-            "     -mem           Read files into memory once only, and parse them from there.\n\n"
+            "     -mem           Read files into memory once only, and parse them from there.\n"
+            "     -init          Perform an initial parse of the file(s) before starting up the individual threads.\n\n"
             );
         exit(1);
     }
@@ -866,6 +872,15 @@ void threadMain (void *param)
         int checkSum = 0;
         checkSum = thParser->parse(docNum);
 
+        // For the case where we skip the preparse we will have nothing to
+        // compare the first parse's results to ... so if this looks like first
+        // parse move the checkSum back into the gRunInfo data for this file.
+
+        if (gRunInfo.files[docNum].checkSum == 0)
+        {
+           gRunInfo.files[docNum].checkSum = checkSum;
+        }
+
         if (checkSum != gRunInfo.files[docNum].checkSum)
         {
             fprintf(stderr, "\nThread %d: Parse Check sum error on file  \"%s\".  Expected %x,  got %x\n",
@@ -939,45 +954,53 @@ int main (int argc, char **argv)
     //
     ReadFilesIntoMemory();
 
+    // Initialize checksums to zero so we can check first parse and if
+    // zero then we need to move first parse's checksum into array. This
+    // is for the cse where we skip the initial parse.
+    for (int n = 0; n < gRunInfo.numInputFiles; n++)
+    {
+        gRunInfo.files[n].checkSum = 0;
+    }
 
+    if (gRunInfo.doInitialParse)
+    {
     //
     // While we are still single threaded, parse each of the documents
     //  once, to check for errors, and to note the checksum.
     // Blow off the rest of the test if there are errors.
     //
-    ThreadParser *mainParser = new ThreadParser;
-    int     n;
-    bool    errors = false;
-    int     cksum;
+        ThreadParser *mainParser = new ThreadParser;
+        int     n;
+        bool    errors = false;
+        int     cksum;
 
-
-    for (n = 0; n < gRunInfo.numInputFiles; n++)
-    {
-        char *fileName = gRunInfo.files[n].fileName;
-        if (gRunInfo.verbose)
-            printf("%s checksum is ", fileName);
-
-        cksum = mainParser->parse(n);
-
-        if (cksum == 0)
+        for (n = 0; n < gRunInfo.numInputFiles; n++)
         {
-            fprintf(stderr, "An error occurred while initially parsing %s\n",
-                fileName);
-            errors = true;
-        };
+            char *fileName = gRunInfo.files[n].fileName;
+            if (gRunInfo.verbose)
+                printf("%s checksum is ", fileName);
+
+            cksum = mainParser->parse(n);
+
+            if (cksum == 0)
+            {
+                fprintf(stderr, "An error occurred while initially parsing %s\n",
+                    fileName);
+                errors = true;
+            };
+
+            gRunInfo.files[n].checkSum = cksum;
+            if (gRunInfo.verbose )
+                printf("%x\n", cksum);
+            if (gRunInfo.dumpOnErr && errors) {
+                if (gRunInfo.dom)
+                    mainParser->domPrint();
+            }
 
-        gRunInfo.files[n].checkSum = cksum;
-        if (gRunInfo.verbose )
-            printf("%x\n", cksum);
-        if (gRunInfo.dumpOnErr && errors) {
-           if (gRunInfo.dom)
-              mainParser->domPrint();
         }
-
+        delete mainParser;
     }
 
-    delete mainParser;
-
     //
     //  Fire off the requested number of parallel threads
     //
